#pragma TextEncoding = "Windows-1252"
#pragma rtGlobals=3		// Use modern global access method.
//#pragma version=1.17d// decided to not use pragma but a string constant 
#pragma igorVersion=6.3

//ToF-AMS data processing kit for ToF-AMS analysis Toolkit
//Copyright (C) 2005, 2006, 2007,2008, 2009,2010,2011, 2012, 2013, 2014, 2015, 2016, 2017
//James Allan (SEAES, University of Manchester, UK; DIAC, NCAS, UK)
//Jonathan Crosier (SEAES, University of Manchester, UK)
//Edward Dunlea (CIRES, University of Colorado at Boulder, CO, USA)
//Donna Sueper (CIRES, University of Colorado at Boulder, CO, USA; Aerodyne Research Inc., MA, USA)
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

///////////////////////////////////////////////////////////////////////////////
// CURRENT STATUS
// see Read me file for details.

//Constant PostAvgFlag=0	// either we do not have ePToF data OR we have deconvoled in pre process step
//Constant PostAvgFlag=1	// avg across runs, deconvolve THEN apply frag ............ cannot have time dependent frags!
//Constant PostAvgFlag=2	// apply frag THEN deconvolve

constant LN_10=2.302585092994046 // Natural log of 10
static constant SQUIRREL_MAXDATA_DEF=0x6400000 	// Maximum acorn size, in bytes

// ************************** sq_MS_concentration **************************
// Master function by James that deals with the conversion of raw or stick data to end products.
// Different products identified by their product type:
// 0 = Speciated mass concentration (average)
// 1 = Speciated mass concentration vs time
// 2 = Speciated mass spectrum vs m/z (an averaged mass spectrum)
// 3 = Speciated mass spectrum vs time and m/z (what Qi's analysis uses)
// 4 = Speciated mass concentration vs diameter (averaged size distribution)
// 5 = Speciated mass concentration vs time and diameter (image plots OR banded mass concentrations)
// 6 = Speciated, banded, averaged mass spectrum vs m/z and diameter (what Silke's code can do)
// 7 = Speciated, banded mass spectrum vs Time, m/z and diameter (For a size-resolved version of Qi's analysis)
//
// There is a system to the above numbering convention - the logic goes like this:
// bit 0 = uses time; bit 1 = uses m/z; bit 2 = uses diameter
//
// The rank of the destination wave is 1 for types 1 and 2; 2 for types 3, 4, 5, and 6; and 3 for type 7
// The dimensions of the destinations are, in order of priority; time, mz and diameter
//
// Types 6 and 7 are impossible with qAMS data. In the 'Deluxe' version of the code, types 1, 2, 4 and 5 were
// handled by four sepearate functions, with 3 being handled by a plugin.
//
// Function outline is as follows:
// Step 1. Identify & set global and local variables
// identify data waves to be worked on
// identify global waves 
// velocity calibration waves
// parse important flags, entities from paramsStr
//  important product type variable
// still more flags
// m/z calibration waves
// clean up, reset variables extracted from paramStr
// declare local waves possibly created in earlier acorns
// declare locall variables used in calculations
//
// Step 2. Create waves needed for calculations :
// destination waves,  weighting waves, error waves conversion waves
// if (acorn=0)
// 	for (each species) make one destination, weighting, error, conversion wave	
// prep custom time base
// prep custom m/z base
// prep custom diameter base
//
// Step 3. Do some preliminary data maniuplations
// converting to Hz, generating the squared error matrix 
// smooth ToF data, if desired
// prep the 'ugfac' data according to the operation and options 
// if (acorn==0)
// 	ugfac_wav_tmp for all product types has all the unit conversion factors except RIE, CE
//
// Step 4. Generate data product
// for (each species)
//	if (stick data)
// 		if (species has a time dependency)
//			for(each entry associated with dependency)
//		identify the time-dependent, fragconverted op wave
// 		identify RIE, CE
//		apply frag-converted op wave
//			if (MS)
//			else (PToF Data)
//	else (raw spectra)
// 		identify RIE, CE
//			if (MS)
//				deal with possible raw background spectra 
//			else (PToF Data)
// 	distribute the data to the destination wave accordingly.
//	for (each run in current acorn ...index_pos)
//		if (we have real data to work with)
//			p1 identifies point number for this run		
// 			find ugfac conversion for this run
//			find t_step for possible weighting
// 			if (PToF dat and results to be in DVa) make diameter map, d_map
// 			if (mapping to new m/z space) make mz_map
//			if (weighting to conserve peak area instead of height)
//			set p2 = p1
// 			switch (product type) .... assign result according to product type
// 			for each case consider possible combinations of d_base_flag, t_base_flag,  mz_base_flag
//			case 0: Speciated mass concentration (average)
//			case 1: Speciated mass concentration vs time
//			case 2: Speciated mass spectrum vs m/z (an averaged mass spectrum)
//			case 3: Speciated mass spectrum vs time and m/z (what Qi's analysis uses)
//			case 4: Speciated mass concentration vs diameter (averaged size distribution)
// 			case 5: Speciated mass concentration vs time and diameter (image plots OR banded mass concentrations)
// 			case 6: Speciated, banded, averaged mass spectrum vs m/z and diameter (what Silke's code can do)
// 			case 7: Speciated, banded mass spectrum vs Time, m/z and diameter (For a size-resolved version of Qi's analysis)

// 			bit 0 avg/sum in m/z space (0= one value,1= time series)
// 			bit 1 avg/sum in run space  (2= avg mass spec, 3 = 2D time series mass spec)
// 			bit 2  avg/sum in run and mz space (4 = size distrib, 5= image plots, 6 = avg mass spec in size regions, 7 = 3D size, time series, mz dimension
// 			bit 3 2D time and raw spectra on the same mz axis
//
// Step 5. Finalize destination wave if at end of acorn
// Step 6. Delete temporary waves
 
Function sq_MS_concentration(index_pos,paramstr,acorn,n_acorns)
	wave index_pos 	//& 1D wave of positions in 'run_index' that are being operated on
	string paramstr 	//& Keyword-identified parameter string
	variable acorn 	//& Number of the 'acorn' being passed
	variable n_acorns 	//& The total number of 'acorns' that will be passed
	
//  Step 1. Identify & set global and local variables
	// identify data waves to be worked on
	wave source=$stringbykey("acorn_0",paramstr)		//^ can be stick or raw spectra
	wave /z source_err=$stringbykey("acorn_1",paramstr)	
	
	// identify global waves 
	wave series_index=root:index:series_index
	wave t_series=root:index:t_series
	wave imt_series=root:index:imt_series
	
	wave/wave TimeWaves=sq_GetModifiedTimes()
	wave TimeMSOpen = TimeWaves[0]
	wave TimeMSClosed = TimeWaves[1]
	wave TimePToF = TimeWaves[2]
	
	wave/z FastModeAvgTimeSec=root:diagnostics:FastModeAvgTimeSec 
		
	wave /z corr_fact=root:diagnostics:corr_fact
	wave/z ioneff_wave=root:diagnostics:ioneff		//1.34
	if(waveexists(ioneff_wave)==0)
		wave ioneff_wave=root:diagnostics:ioneff_logged	//1.34//^ use the wave from the diagnostics, not the one calculated through the airbeam correction section 
	endif
	wave/z flowrate=root:diagnostics:flowrate
	if(waveexists(flowrate)==0)
		wave flowrate=root:diagnostics:flowrate_logged	//1.38C
	endif	
	
	// velocity calibration waves		//1.59E  moved to sq_GetPToFParams(DVa_wave, runPos)
	wave/z vl=root:diagnostics:ptof_vl	
	wave/z vg=root:diagnostics:ptof_vg
	wave/z b=root:diagnostics:ptof_b	
	wave/z dx=root:diagnostics:ptof_dx
	if (!WaveExists(vl))
		wave vl=root:diagnostics:ptof_vl_logged	//^ 1.35
	endif
	if (!WaveExists(vg))
		wave vg=root:diagnostics:ptof_vg_logged	//^ 1.35
	endif
	if (!WaveExists(b))
		wave b=root:diagnostics:ptof_b_logged	//^ 1.35
	endif
	if (!WaveExists(dx))
		wave dx=root:diagnostics:ptof_dx_logged	//^ 1.35
	endif
	wave ptof_step=root:diagnostics:ptof_step
	wave ptof_t0=root:diagnostics:ptof_t0
	wave ChopperFreq = root:diagnostics:ChopperFreq		
	 
	// parse important flags, entities from paramsStr
	wave/z t_base=$stringbykey("t_base",paramstr) // Time base for destination data (null if t_series used)
	wave/z d_base=$stringbykey("d_base",paramstr) // Diameter base for destination data (null if dsig used)
	wave/z mz_base=$stringbykey("mz_base",paramstr) // m/z base for destination data (raw mass spec only)

	string speclist=stringbykey("speclist",paramstr) // Slash-seperated list of keywords
	string fraglist=stringbykey("fraglist",paramstr) // Slash-seperated list of frag lists
	string destlist=stringbykey("destlist",paramstr) // Slash-seperated list of destination waves
	string rie_list=stringbykey("rie_list",paramstr) // Slash-seperated list of RIEs
	string ce_list=stringbykey("ce_list",paramstr) // Slash-seperated list of CEs
	string spec_corr=stringbykey("spec_corr",paramstr) // Slash-seperated list of species-specific correction factors
	string frag_exclude=stringbykey("frag_exclude",paramstr) // Frag waves to exclude		// 1.41B by James
	string CE_corr=stringbykey("CE_corr",paramstr) // Slash-seperated list of species-specific CE correction factors

	//  important product type variable
	variable product_type=numberbykey("product_type",paramstr) // The type of data being produced

	// still more flags
	variable lc=numberbykey("lc",paramstr) // Chamber length
	variable corr_op=numberbykey("corr_op",paramstr) // Use correction factor or not
	variable aw=numberbykey("aw",paramstr) // Atomic mass of calibration material (nominally 62)
	variable ug_op=numberbykey("ug_op",paramstr) // Convert from Hz to g m-3
	variable cache_op=numberbykey("cache_op",paramstr)==1 // Use existing matrices/vectors
	variable filter_op=numberbykey("filter_op",paramstr)*2 // Filter negatives		//1.46E  in practice this should now always be 0. 
	variable dlog_op=numberbykey("dlog_op",paramstr)==1 // Whether to convert to dMdlogDp or not
	variable chop_duty=numberbykey("chop_duty",paramstr) // Chopper duty cycle
	variable raw_flag=numberbykey("raw_flag",paramstr)==1 // Data being handled are raw (not stick) MS
	variable smooth_pnts=numberbykey("smooth_pnts",paramstr) // Diameter smoothing points
	variable boxcar_op=numberbykey("boxcar_op",paramstr)==1 // Use boxcar (instead of binomial) smoothing
	variable mz_w_op=numberbykey("mz_w_op",paramstr)==1 // Use weighting to conserve peak area (rather than peak height) of mass spectra
	variable hz_op=numberbykey("hz_op",paramstr)==1 // Convert raw data to Hz (eliminates need to preprocess)
	variable findBaseline_op=numberbykey("findBaseline_op",paramstr)==1 // for subtracting raw spectra from baseline
	variable rawBaseline_flag=numberbykey("rawBaseline_flag",paramstr)==1 // 	// added in 1.46, for finding/saving the raw AND the baseline spectra
	variable timeweight_op=numberbykey("timeweight_op",paramstr)
	
	variable ePToF_flag=numberbykey("ePToF_flag",paramstr) //1.53N
	variable PostAvgFlag=numberbykey("PostAvgFlag",paramstr) //1.53Zq
	PostAvgFlag = numtype(PostAvgFlag) != 0 ? 0 : PostAvgFlag

	variable ePToF_twflag=numberbykey("ePToF_twflag",paramstr)		//1.61A

	variable chopperDutyWave_op=numberbykey("chopperDutyWave_op",paramstr)
	if (chopperDutyWave_op)
		wave chopperDutyWave = root:diagnostics:chopperDutyWave
	endif
	
	nvar/z PToF_DCgr_ePToFSumDiffScaling = root:panel:PToF_DCgr_ePToFSumDiffScaling

	// m/z calibration waves
	variable new_mz_cal=numberbykey("new_mz_cal",paramstr)==1 // Use new m/z calibration (if available)
	if (new_mz_cal)		// 1.56E  for open and closed mz params
		wave/z mzSlope=$ReplaceString("/", stringbykey("mzSlope",paramstr), ":") 
		wave/z mzInt=$ReplaceString("/", stringbykey("mzInt",paramstr) , ":")
		wave/z mzPower=$ReplaceString("/", stringbykey("mzPower",paramstr), ":") 
		if (!WaveExists(mzSlope))
			wave/z mzSlope=root:mzFitting:mz_FitSlope		//^ for cleaner debugging
			wave/z mzInt=root:mzFitting:mz_FitIntercept
			wave/z mzPower=root:mzFitting:mz_FitPower
			if (!WaveExists(mzSlope) || !WaveExists(mzInt) || !WaveExists(mzPower))
				print "//The Use new m/z Fit params was checked but the waves could not be found.  Using Daq values."
			endif
		endif		
	endif
	if (!new_mz_cal || !waveexists(mzslope) ||  !waveexists(mzInt) ||  !waveexists(mzPower))
		wave /z mzSlope=root:diagnostics:mzSlope
		wave /z mzInt=root:diagnostics:mzint
		wave /z mzPower = root:diagnostics:mzPower
		if (!WaveExists(mzPower))
			make/o/n=(numpnts(mzSlope)) root:diagnostics:mzPower=0.5
			wave mzPower = root:diagnostics:mzPower
		endif
	endif
	
	variable daqPToFStick_op=numberbykey("daqPToFStick",paramstr) // 1.51Q	
	if (daqPToFStick_op)			//1.51Q
		sq_insert_Matrix(index_pos,paramstr,acorn,n_acorns)
		wave source=$stringbykey("acorn_0",paramstr)		//^ can be stick or raw spectra
	endif

	//  clean up, reset variables extracted from paramStr
	aw=numtype(aw)?(62):(aw) // Defaults aw to 62 if omitted
	filter_op=numtype(filter_op)?(0):(filter_op)
	raw_flag=numtype(raw_flag)?(0):(raw_flag)
	
	if (strlen(destlist)==0)
		destlist=speclist // Defaults the destination to the species name if omitted
	endif
		
	variable dimSource0=dimsize(source,0), dimSource1=dimsize(source,1), dimSource2=dimsize(source,2)	//1.19E

	variable t_base_flag=waveexists(t_base) && (product_type&1)
	variable d_base_flag=waveexists(d_base) && (product_type&4) && dlog_op
	variable mz_base_flag=waveexists(mz_base) && raw_flag && (product_type&2)
	variable error_flag=waveexists(source_err)
	variable matrix_flag=product_type&2?(1):(0)
//	string targsuff=selectstring(matrix_flag,"_vec","_mat")		// do this differently in 1.46A+
	string targsuff=selectstring(matrix_flag,"_vec","_sparse") // 1.46A  JDA edit
	variable time_points=t_base_flag?(numpnts(t_base)-1):(numpnts(t_series))
//	variable n_t=dimsize(source,1)		// for MS sticks, number of mz, for PToF sticks number of PToF bins
	variable d_points=d_base_flag?(numpnts(d_base)-1):(dimSource1)		// diameter base
	variable numMzSticks=dimsize(source, product_type<4?(1):(2)) // Assume fixed //  for MS OR PToF number of mz
	variable mz_points=mz_base_flag?(numpnts(mz_base)):(numMzSticks)	// raw spectra averaging
	// for MS sticks n_t = n_it = mzpoints  for PToF sticks n_it = mz_points but n_t  is original number of ptof bins 
	// for MS raw n_t = n_it = num raw spectra points  for PToF raw n_it = num raw spectra points but n_t  is original number of ptof bins 

	//1.51Q
	variable ignoreChop = numberbykey("ignoreChop",paramstr) 
	ignoreChop = numtype(ignoreChop)==0 ? ignoreChop : 0
	
	// declare local waves possibly created in earlier fetch calls ...(acorns!=0)
	wave /z mpos
	wave /z ms_conc_tmp
	if ((!waveexists(ms_conc_tmp) || !waveexists(mpos)) && (!(product_type&1) || t_base_flag || d_base_flag || mz_base_flag))
		make /o/n=(1,1,1) ms_conc_tmp
		make /o/n=(2,3) mpos
	endif
	
	// declare local variables used in calculations
	variable i,n_spec,j,p1,p2,k,invpower,l, n_dep
	variable rie,ce,t_step,ugfac,t1,t2,opnote_n,opfaclist_n		// ,opfac_r,opfac_c  //1.45 addition from James
	string target,deststr,opnote,opnoteitem,opfaclist,opfacitem
	variable numUserDef, postAvgIdex			//1.51K
	
//	variable PostAvgFlag=numberbykey("PostAvgFlag",paramstr) 		//1.57L
//	PostAvgFlag = numtype(PostAvgFlag) == 2 ? 0 : PostAvgFlag
	variable osFactor=numberbykey("osFactor",paramstr) 
	osFactor=numtype(osFactor) == 2 ? 2 : osFactor
	string deststrePToF 
	variable RunHasData
					
	variable numRunsInThisAcorn=numpnts(index_pos)		// number of runs in this acorn
	n_spec=raw_flag?(1):(itemsinlist(speclist,"/"))		// number of species

	variable dimDest0, dimDest1, dimDest2

	//1.45 addition from James
//	bigop_flag=!matrix_flag || mz_points*mz_points*n_vals<numvarordefault("root:squirrel_maxdata",SQUIRREL_MAXDATA_DEF) // Allow 'big' operators - faster but memory inefficient
	
// 1.51J  The addition of unity as a valid entry in the frag table means these things:
// Firstly, the ms_mats matricies need to have an extra column that corresponds to the max column value
// The user needs to check that nany frag wave referencing this mz column be set to zero in the frag wave
//  		i.e is the max mz is 400 we need to make sure frag_organic[401]=nan
// The source and possibly error matricies need to be redimensioned with one extra column corresonding to that mz
// The final product needs to be resized

// Step 2. Create waves needed for calculations:
	// destination waves,  weighting waves, error waves conversion waves
	if (acorn==0)
		sq_ams_stat_upd("Generating destination waves")
		gen_removeSpacesTextWaves("root:frag", "frag_")		// added in 1.47E
	
		for(i=0;i<n_spec;i+=1)
			deststr=stringfromlist(i, destlist,"/")

			switch(product_type)
				case 0:		// added by James? 1.42E - yeah, used for normalisations; a zero-order data product, which is effectively an average
					make /o/n=(1) $deststr=0
				break
				case 1:		// time series
					make /o/n=(time_points) $deststr=0
				break
				case 2:	 // avg mz
					make /o/n=(mz_points) $deststr=0
				break
				case 3:	// 2D time series and mz
					make /o/n=(time_points,mz_points) $deststr=0
				break
				case 4:	// size distr
					make /o/n=(d_points) $deststr=0
				break
				case 5:	// regular PToF image plot
					make /o/n=(time_points,d_points) $deststr=0
				break
				case 6:	// 2D avg mass spectrum (with dimension)  // mz x diam
					make /o/n=(mz_points,d_points) $deststr=0
				break
				case 7:	// 3D time, mz, size
					make /o/n=(time_points,mz_points,d_points) $deststr=0
				break
			endswitch		// endswitch(product type)
			
			// Donna modified to facilitate the UNITY usage in the frag table.  We always presume a unity exists to prevent unneccessary cluttered if statements
			// Create conversion matrices if needed
			target="root:ms_mats:"+stringfromlist(i,speclist,"/")
			if (!raw_flag && (cache_op!=1 || !waveexists($(target+targsuff)) || dimsize($(target+targsuff),0)!=mz_points)) //JDA 1.43d
				sq_ams_stat_upd("Generating "+target+" matrices")
				sq_fraglist_convert(stringfromlist(i,fraglist,"/"),"vec_tmp",mz_points,frag_exclude)
				wave/t vec_tmp = $"vec_tmp"		//1.51K UserDefHz
//				vec_tmp[numMzSticks,]=""		//   UserDefHz  vec_tmp counts from zero. 
				//  If nummz = 200, stick matrix goes from mz1 to mz200 (200 points at positions 0 thru 199) this puts nans in row 200 - 209 = mz 201-210
				sq_frag_matrix($"vec_tmp",target,filter_op)
				wave/z spec_mat = $"root:ms_mats:"+stringfromlist(i,speclist,"/")+"_mat"		// 1.51L for raw spectra we don't need these
				wave/z spec_vec = $"root:ms_mats:"+stringfromlist(i,speclist,"/")+"_vec"		// 1.51L for raw spectra we don't need these			if (WaveExists(spec_mat) &&  WaveExists(spec_vec) )	//1.51L
//				Deletepoints numMzSticks+numUserDef, (10-numUserDef), spec_vec
//				Deletepoints/M=0 numMzSticks+numUserDef, (10-numUserDef), spec_mat
//				Deletepoints/M=1 numMzSticks+numUserDef, (10-numUserDef), spec_mat 
			endif

			//& Create weighting wave, if needed
			if (!(product_type&1) || t_base_flag || d_base_flag || mz_base_flag)
				duplicate /o $deststr, $("conc_weight_"+num2str(i))
			else
				fastop $deststr=(nan)
			endif
			
			//& Create error destination wave
			if (error_flag)
				duplicate /o $deststr,$(deststr+"_err")
			endif
			
			//& if finding average raw spectra and average baseline raw spectra
			if (rawBaseline_flag)
				duplicate /o $deststr,$(deststr+"_BL")
			endif
			
			if (PostAvgFlag==2)		// hold a space for the intermediate result  we keep the same PTOF dimension order as source until we apply frags
				deststrePToF = deststr+"X"
				switch(product_type)
				case 4:		// size distribution   // i.e. Org product is 1D in PToF dimension
					make/o/n=(1, dimSource1) $deststrePToF = 0		// temporary place holders  1 x PToF   for PostAvgFlag==0 we may average in PToF dim
				break
				case 5:	// image plot    // i.e. Org product is 2D in time and PToF dimension
					make/o/n=(time_points, dimSource1 ) $deststrePToF = 0		// temporary place holders    run time x PToF  for PostAvgFlag==0 we may average in PToF dim 
				break
				case 6:	// avg mass spec   // i.e. Org product is several 1D size waves with mz
					make/o/n=(1, dimSource1, dimSource2) $deststrePToF = 0		// temporary place holders run x PToF x mz for PostAvgFlag==0 we may average in PToF dim
				break
				case 7:	// avg mass spec with time
					make/o/n=(time_points,dimSource1, dimSource2) $deststrePToF = 0		// temporary place holders  run x PToF x mz for PostAvgFlag==0 we may average in PToF dim
				break
				endswitch
				
				//& Create weighting wave, if needed... this will help nan out values at the end
				// if weighting averages in run time, DVa or mz dataTemp_n will keep track of weights
				if (!(product_type&1) || t_base_flag || mz_base_flag)		// || d_base_flag   
					make/o/n=(dimsize($deststrePToF,0))  $deststrePToF+"n"=0// duplicate /o $deststrePToF, $deststrePToF+"n"  // we only average in the run time dimension... no need for this to be multidimensional
				else
					fastop $deststrePToF=(nan)
				endif
			endif
		endfor	// endfor each species in spec_list

		// 1.51K 
		// By default the sq_fragList_convert makes the frag waves to be 10 (=maxUserDefined) bigger than they need to be.
		// Once we know of all the userdefined waves, we trim all the converted frag waves back to the correct size
//		for(i=0;i<n_spec;i+=1)
//			target=stringfromlist(i, speclist,"/")
//			wave/z spec_mat = $"root:ms_mats:"+target+"_mat"		// 1.51L for raw spectra we don't need these
//			wave/z spec_vec = $"root:ms_mats:"+target+"_vec"		// 1.51L for raw spectra we don't need these			if (WaveExists(spec_mat) &&  WaveExists(spec_vec) )	//1.51L
//			Deletepoints numMzSticks+numUserDef, (10-numUserDef), spec_vec
//			Deletepoints/M=0 numMzSticks+numUserDef, (10-numUserDef), spec_mat
//			Deletepoints/M=1 numMzSticks+numUserDef, (10-numUserDef), spec_mat 
//		endfor	// endfor each species in spec_list			
//		

	endif		// endif acorn ==0
		
	// Prep custom time base
	if (t_base_flag)
		make /o/d/n=(numRunsInThisAcorn,2) t_source=(index_pos[p]<0 ? 0 :imt_series[series_index[index_pos[p]]+q])
		sq_make_map(t_source,t_base,"t_map")
		wave t_map
	endif

	// Prep custom m/z base
	if (mz_base_flag)
		if (acorn==0)
			make/o/n=(mz_points+1) mz2_base_tmp		//^ this is a weighting wave, is constant for all runs
			// sort of defines left and right 'edges' of thein ns space; uses the average of the square root of the mz_base values
			// is used in the mapping of one raw spectra to another of different m/z base, regardless as to whether athe "conserve area' checkbox is checked.
			mz2_base_tmp[1,mz_points-1]=0.5*(sqrt(mz_base[p])+sqrt(mz_base[p-1]))
			mz2_base_tmp[0]=(1.5*sqrt(mz_base[0]))-(0.5*sqrt(mz_base[1]))
//			mz2_base_tmp[mz_points]=(1.5*sqrt(mz_base[mz_points]))-(0.5*sqrt(mz_base[mz_points-1]))
			mz2_base_tmp[mz_points]=mz2_base_tmp[mz_points-1]+(mz2_base_tmp[mz_points-1]-mz2_base_tmp[mz_points-2])		/// 1.59E  not sure if this is completely correct.  But it is only one point
		else
			wave mz2_base_tmp
		endif
	endif

	// Prep custom diameter base
	if (d_base_flag)
		if (acorn==0)
			make /o/n=(d_points+1) log_d_base_tmp=log(d_base)
		else
			wave log_d_base_tmp
		endif
	endif

// Step 3. Do some preliminary data manipulations
	// convert to Hz, generate the squared error matrix 
	// Need to insert something to apply corrections to unprocessed data - insert points etc  ??
//	if (hz_op && raw_flag) //& Using raw (non-processed) MS data
//		sq_Hz(index_pos,source)
//	endif
//	if (NonNsSampling_op)
//		sq_correctSamplingTimeUnit(index_pos,source) 		// convert to units correctly  1.54D
//	endif
//	if (dutycycle_op)
//		sq_dutyCycleCorr(source, acorn==n_acorns-1)
//	endif
//	sq_insert_Matrix(index_pos,paramstr,acorn,n_acorns)
	
	if (error_flag)
		matrixop /o source_err2=powr(source_err,2)
	endif
	
	// smooth ToF data, if desired
	// Not mathematically kosher to use in conjunction with rebinning
	if (smooth_pnts>0 && product_type&4)
		sq_ams_stat_upd("Smoothing data")
		if (boxcar_op)
			smooth_pnts=(floor(smooth_pnts*0.5)*2)+1
			make /o/n=(1,smooth_pnts,1) smooth_tmp=1/smooth_pnts
		else
			make /o/n=(1,(2*smooth_pnts)+1,1) smooth_tmp=binomial(2*smooth_pnts,q)
			smooth_pnts=sum(smooth_tmp)
			smooth_tmp/=smooth_pnts
		endif
		matrixconvolve smooth_tmp,source
		wave m_convolution
		fastop source=m_convolution
		if (error_flag)
			smooth_tmp=smooth_tmp^2
			matrixconvolve smooth_tmp,source_err2
			fastop source_err2=m_convolution
		endif
	endif

	// prep the ugfac_wav_tmp according to the operation and options
	// ugfac_wav_tmp  has all the unit conversion factors except RIE, CE, species correction waves
	if (acorn==0)
		make /o/n=(numpnts(t_series)) ugfac_wav_tmp
		if (ug_op)
			if (product_type&4) // PToF: dM/dlogD (g m-3) or dM/dPToF (g m-3 s-1)
				if (ePToF_flag)		//1.53N
					ugfac_wav_tmp=((aw/AVOGADRO_NUM)*1e12/(ioneff_wave[p]*flowrate[p])  )*(1/ePToFFractionOpen)*ChopperFreq 	//1.55B	//1.53M  2 is cuz we are in open half the time
				else
					if (chopperDutyWave_op)		//1.53E
						ugfac_wav_tmp=(aw/AVOGADRO_NUM)*1e12*(ChopperFreq[p]/chopperdutyWave[p])/(ioneff_wave[p]*flowrate[p]) 
					else
						ugfac_wav_tmp=(aw/AVOGADRO_NUM)*1e12*(ChopperFreq[p]/chop_duty)/(ioneff_wave[p]*flowrate[p]) 
					endif
				endif
			else // MS g m-3
				ugfac_wav_tmp=(aw/AVOGADRO_NUM)*1e12/(ioneff_wave[p]*flowrate[p])
			endif
		elseif (product_type>=4 && !ignoreChop)		//1.43G		//1.51P  I don't know why this was commented out in 1.43G
			if (ePToF_flag)		//1.53N
				fastop ugfac_wav_tmp=(1/ePToFFractionOpen)*ChopperFreq // we are in open half the time  //1.55B  twas 2
			elseif (chopperDutyWave_op)  // this really shouldn't happen.  we shouldn't mix nonEPToF and PTOF
				wave chopperdutyWave = root:diagnostics:chopperdutyWave
				fastop ugfac_wav_tmp = ChopperFreq/chopperdutyWave
			else
				fastop ugfac_wav_tmp=(1/chop_duty)*ChopperFreq	// regular PToF
			endif
		else // MS Hz or PToF dI/dPToF (Hz)
			fastop ugfac_wav_tmp=(1)
		endif
		
		if (corr_op) //& apply correction factor
			ugfac_wav_tmp*=corr_fact[p]
		endif
		
		// notice that we only make one dataTemp wave, regardless as to how many species are requested
		// also, all dataTemp waves retain the original order of dimensions... run x PToF x mz
		if (PostAvgFlag==1)		// hold a space for the intermediate result  we keep the same dimension order as source until we apply frags
			switch(product_type)
			case 4:		// size distribution   // i.e. Org product is 1D in PToF dimension
				make/o/n=(1, dimSource1,dimSource2) dataTemp = 0		// temporary place holders
			break
			case 5:	// image plot    // i.e. Org product is 2D in time and PToF dimension
				make/o/n=(time_points, dimSource1, dimSource2) dataTemp = 0		// temporary place holders
			break
			case 6:	// avg mass spec   // i.e. Org product is several 1D size waves with mz
				make/o/n=(1, dimSource1, dimSource2) dataTemp = 0		// temporary place holders  1 x mz x PToF
			break
			case 7:	// avg mass spec with time
				make/o/n=(time_points, dimSource1, dimSource2) dataTemp = 0		// temporary place holders
			break
			endswitch
			
			//& Create weighting wave, if needed... this will help nan out values at the end
			// if weighting averages in run time, DVa or mz dataTemp_n will keep track of weights
			If (!(product_type&1) || t_base_flag || mz_base_flag)		// || d_base_flag   
				make/o/n=(dimsize(dataTemp,0)) dataTemp_n   // duplicate /o dataTemp, dataTemp_n
			else
				fastop dataTemp=(nan)
			endif

			make/o/n=(time_points) pseudoIndex_pos = series_index[index_pos[0]]		// position of first run of first acorn
			
		// used when we apply frags
		endif	
		
	else		// acorn = 0
		if (PostAvgFlag==1)
			wave dataTemp	// has dimensions of runs x PToF x mz
			wave/z dataTemp_n
			wave pseudoIndex_pos
		endif		
		wave ugfac_wav_tmp
	endif	// endif acorn ==0

// Step 4. Generate data product
//& Comment from Donna as a proxy for James - Sparse matricies are now employed! June, 2009
//& Considerably changed in 1.46A
//& 19 July 2007 - handled the matrix maths differently, with time as layers
//& This allows for the use of time-dependent frag waves without writing specific code
//& to handle them in this section. All subsequent sections are ignorant of whether they
//& were used or not.
//& Upshot - version 5.05 is now required, as this sorted out a bug that would have created
//& lots of waves with zero-length dimensions

	for (i=0;i<n_spec;i+=1)		// typically for each species   

		sq_ams_prog_upd(100*(acorn+(i/n_spec))/n_acorns)

		deststr=stringfromlist(i, destlist,"/")
		wave dest=$deststr
		wave /z dest_n=$("conc_weight_"+num2str(i)),dest_err=$(deststr+"_err")		

		if (PostAvgFlag==2)		// hold a space for the intermediate result  we keep the same dimension order as source until we apply frags
			deststrePToF = deststr+"X"
			wave dataTemp = $deststrePToF
			wave/z dataTemp_n = $deststrePToF+"n"
		endif

		if (PostAvgFlag==0 || PostAvgFlag==2)		// we either deconvolved already or we apply frag BEFORE deconvolution
			wave /z spec_corr_wav=$stringfromlist(i,spec_corr,"/")
			wave /z CE_corr_wav=$stringfromlist(i,CE_corr,"/")
			wave /z dest_BL = $(deststr+"_BL")		//1.46
		endif	
					
		if (!raw_flag)		// stick data
			// identify RIE, CE
			if ( (strlen(rie_list)>0 && strlen (ce_list)>0  && ug_op) && (PostAvgFlag==0 || PostAvgFlag==2) )			// 1.41B change to include ug_op
				rie=str2num(stringfromlist(i, rie_list,"/"))
				if (WaveExists(CE_corr_wav))				//1.53M
					ce=1  // ignore scalar values!  1.53M  str2num(stringfromlist(i, ce_list,"/"))
				else
					ce=str2num(stringfromlist(i, ce_list,"/"))
				endif
			else			//  lists are empty OR (PostAvgFlag==1)
				rie=1
				ce=1
			endif
				
		 	// when applying frag waves we input source and come out with mass_tmp which may have different dimensions
			
			if (PostAvgFlag==0 || PostAvgFlag==2)	// apply frag waves here  THIS COLLAPSES the m/z dimension, except when result still has mz dimension
				if (error_flag)
					wave/wave waveRefs = sq_ApplyFragWaves(source, i, destlist, speclist, product_type, error_flag, "root:ms_mats:", numMzSticks, numRunsInThisAcorn, dimSource1, index_pos = index_pos, source_err2 = source_err2)
				else
					wave/wave waveRefs = sq_ApplyFragWaves(source, i, destlist, speclist, product_type, error_flag, "root:ms_mats:", numMzSticks, numRunsInThisAcorn, dimSource1, index_pos = index_pos)
				endif
				// if result doesn't have m/z dimension sq_ApplyFragWaves collapses the m/z dimension.  
				// So if MS data Org time series product, mass_tmp now 1D of  acorn-runs of points.  If  (nonePToF) PToF data Org size distribution or image plot mass_tmp is 2D of acorn-runs x PToF bins 
				// So if MS data Org MS product, mass_tmp now 2D of  acorn-runs x max mz.  If (nonePToF) PToF data Org size distribution mass_tmp is 3D of acorn-runs x PToF bins x max mz 

				wave mass_tmp = waveRefs[0]			// 	
				// mass_tmp  runs x mz x (PToF) for results with mz dimension
				// mass_tmp  runs x (PToF) for results without mz dimension	
				if (error_flag)
					wave mass_tmp_err = waveRefs[1]
				endif
			else // PostAvgFlag==1		// we wait until the end to apply species specific things   spec_corr_wav
				// we don't do anything because we apply Frag at the end.... subsequent waves used are the source wave, not mass_tmp or dataTemp			
			endif		
			// at this point the waves used are
//			PostAvgFlag==0 mass_tmp
//			PostAvgFlag==1 dataTemp
//			PostAvgFlag==2 mass_tmp
			
		else		// raw spectra
			rie=1
			ce=1
			target=deststr
			if (product_type<4 ) // Raw MS Data   //1.60Q
				duplicate /o source,mass_tmp
				if (error_flag)
					duplicate /o source_err2,mass_tmp_err
				endif
				if (findBaseline_op || rawBaseline_flag)	//1.40  find the baseline matrix and subtract from raw, modified in 1.46E
					duplicate /o source,bk_tmp
					bk_CreateBackgroundValues(index_pos, bk_tmp,  mzSlope, mzInt, mzPower, deststr)
					if (findBaseline_op)		// need to be backwards compatible
						mass_tmp-=bk_tmp
					else // rawBaseline_flag,  we want to save both raw and baseline - re: Jose's insistence on not ever showing altered raw spectra
						duplicate /o bk_tmp,mass_tmp_BL
					endif
				endif
			else // Raw PToF data
				imagetransform /g=1 transposevol source		// swap cols and layers
				wave M_VolumeTranspose				
				duplicate /o M_VolumeTranspose,mass_tmp		//1.51L
				if (error_flag)
					imagetransform /g=1 transposevol source_err2
					duplicate /o m_volumetranspose,mass_tmp_err
				endif
				if (findBaseline_op || rawBaseline_flag)	//1.40  find the baseline matrix and subtract from raw, modified in 1.46E
					duplicate /o mass_tmp,bk_tmp
					bk_CreateBackgroundValues(index_pos, bk_tmp,  mzSlope, mzInt, mzPower, deststr)
					if (findBaseline_op)		// need to be backwards compatible
						mass_tmp-=bk_tmp
					else // rawBaseline_flag,  we want to save both raw and baseline - re: Jose's insistence on not ever showing altered raw spectra
						duplicate /o bk_tmp,mass_tmp_BL
					endif
				endif

			endif
		endif	// stick or raw data

		// still within for (each species) loop
		// Distribute the data to the destination wave accordingly.
		//& Very complicated but deals with all the permutations
		//& Lots of duplication; not ideal, but I couldn't get them all to match up easily
		sq_ams_stat_upd("Assigning "+deststr+" data to destination")
		p2=-1
		
		for (j=0;j<numRunsInThisAcorn;j+=1)	// (each run in current acorn ...index_pos)   // still within each species loop
			sq_ams_prog_upd(100*(acorn+((i+(j/numRunsInThisAcorn))/n_spec))/n_acorns)
			RunHasData = dimsize(source,2) == 0 ?  numtype(source[j][0])==0  :  numtype(source[j][0][0])==0 		
			if (index_pos[j]>-1 && RunHasData)		//^ if(we have real data to work with) mass_tmp may be a 2-d wave
				// p1 identifies point number for this run
				p1=series_index[index_pos[j]]
					
				// find ugfac conversion for this run, species
				ugfac=ugfac_wav_tmp[p1]/(rie*ce)					
				if (waveexists(spec_corr_wav))
					ugfac*=spec_corr_wav[p1]
				endif				
				if (waveexists(CE_corr_wav))	// note that in 1.53M this above was effectively changed so that ce above is 1
					ugfac/=CE_corr_wav[p1]
				endif

				// find t_step for possible weighting			//1.48E addition by James
				switch (timeweight_op)
					case 1: // Would be used for MSOpen data
						t_step=TimeMSOpen[p1]
						break
					case 2: // Would be used for MSClosed data
						t_step=TimeMSClosed[p1]
						break
					case 3: // Would be used for MSDiff data
						t_step=TimeMSOpen[p1]+TimeMSClosed[p1]
						break
					case 4: // Would be used for pToF data
						t_step=TimePToF[p1]
						break
					case 5: // Would be used for FMS data
						t_step=FastModeAvgTimeSec[p1]		//1.57C
						break
					default: // Old method
						t_step=imt_series[p1+1]-imt_series[p1]
				endswitch
				if (numtype(t_step)!=0)		// there are cases in old DAQ versions where these values aren't saved.
					t_step=imt_series[p1+1]-imt_series[p1]
				endif					
				
				// prepare to deal with averaging in any dimension, mz, time, DVa
				// prepare to deal with PToF dimension...   make dtdlogdp_tmp and possible diameter map wave d_map
				// we do this here because in theory we could be averaging PToF data across runs with different DVa parameters
				if (product_type& 4 && dlog_op) 	// (PToF data and results to be in DVa) make diameter map  // in this case we don't care what PostAvgFlag is....we always use the last one
					// prepare dtdlogdp_tmp for this run, and if necessary , d_map
					// This handles cases where the user has different DVa parameters within the same todo.  
					// First time through waves dtdlogdp_tmp and d_map are set.  Subsequent runs we check to see if anything has changed
//  Reworked in 1.61G
					if (p2==-1 || vl[p2]!=vl[p1] || vg[p2]!=vg[p1] || b[p2]!=b[p1] || dx[p2]!=dx[p1] || ptof_step[p2]!=t2 || ptof_t0[p2]!=t1)
						// first go around, or something changed
						t1 = ptof_t0[p1] // Assumes that the time point represents the mean time of sampling - need to check
						t2 = ptof_step[p1]
						make /o/n=(dimSource1) dtdlogdp_tmp=sq_dtdlogdp(lc,vl[p1],vg[p1],b[p1],t1+(p*t2))
						if (d_base_flag)
							make /o/n=(dimSource1+1) log_dsig_tmp=log(sq_ptof2dva(lc,vl[p1],vg[p1],b[p1],dx[p1],t1+((p-0.5)*t2)))
							sq_make_map(log_dsig_tmp,log_d_base_tmp,"d_map")
							wave d_map
						endif
					else
						wave dtdlogdp_tmp
						wave/z d_map
					endif

					if (PostAvgFlag==0)	
						if (dimsize(mass_tmp,2)==0)		// result does not have m/z dimension, just run and PToF
							mass_tmp[j][]*=dtdlogdp_tmp[q]
							if (error_flag)
								mass_tmp_err[j][]*=dtdlogdp_tmp[q]^2
							endif
						else  // result retains m/z dimension... PToF is the [2] dimension
							mass_tmp[j][][]*=dtdlogdp_tmp[r]
							if (error_flag)
								mass_tmp_err[j][][]*=dtdlogdp_tmp[r]^2
							endif
						endif
					endif  // endif (PostAvgFlag==0)
				endif // endif (PToF data and results to be in DVa)
				
				if (mz_base_flag)  // dealing with mapping to new m/z space
					if (p2==-1 || mzslope[p2]!=mzslope[p1] || mzint[p2]!=mzint[p1] || mzpower[p2]!=mzpower[p1])
						if (mz_w_op) //& Conserve peak area, if desired
							make /o/n=(numMzSticks) dtdmz_tmp=mzpower[p1]*(mzslope[p1]^(1/mzpower[p1]))*((p-mzint[p1])^(1-(1/mzpower[p1])))  //^ the first derivative of the mz(t) function
						endif
						make /o/n=(numMzSticks+1) mz_tmp=(((p-0.5)-mzint[p1])/mzslope[p1])^(0.5/mzpower[p1])		//^ the square root of the m/z wave for this run
						sq_make_map(mz_tmp,mz2_base_tmp,"mz_map")
						wave mz_map
					endif
				endif	// endif dealing with mapping to new m/z space
				
				if (mz_w_op)	// m/z weighting to conserve peak area instead of height
					if (p2==-1 && !mz_base_flag)
						make /o/n=(numMzSticks) dtdmz_tmp=mzpower[p1]*(mzslope[p1]^(1/mzpower[p1]))*((p-mzint[p1])^(1-(1/mzpower[p1])))
					endif	
					if (dimsize(mass_tmp,2)==0)	// MS
						mass_tmp[j][]*=dtdmz_tmp[q]
						if (error_flag)
							mass_tmp_err[j][]*=dtdmz_tmp[q]^2
						endif
						if (rawBaseline_flag)		//1.46E  - if we want to get both the average raw spec and the average baseline
							mass_tmp_bl[j][]*=dtdmz_tmp[q]
						endif	
					else						// PToF
						mass_tmp[j][][]*=dtdmz_tmp[q]
						if (error_flag)
							mass_tmp_err[j][][]*=dtdmz_tmp[q]^2
						endif
						if (rawBaseline_flag)		//1.46E  - if we want to get both the average raw spec and the average baseline
							mass_tmp_bl[j][][]*=dtdmz_tmp[q]
						endif
					endif
				endif
								
				p2=p1  // set p2 = p1 in prep for later potential averaging in PToF dim
				
				//1.19E
				dimDest0 = dimsize(dest,0)-1
				dimDest1 = dimsize(dest,1)-1
				dimDest2 = dimsize(dest,2)-1				
				
				// assign result according to product type
				switch (product_type)
					case 0: // Speciated mass concentration (average)		// added by James 1.42x
						ugfac*=t_step
						dest[0]+=mass_tmp[j]*ugfac
						dest_n[0]+=t_step
						if (error_flag)
							dest_err[0]+=mass_tmp_err[j]*ugfac*ugfac
						endif
					break
					case 1: // Speciated mass concentration vs time
						if (t_base_flag)
							ugfac*=t_step
							sq_map2dist(j,0,0,ms_conc_tmp,mpos,map1=t_map)
							dest[mpos[0][0],mpos[1][0]]+=mass_tmp[j]*ugfac*ms_conc_tmp[p-mpos[0][0]][0][0]  //1.59E
							dest_n[mpos[0][0],mpos[1][0]]+=t_step*ms_conc_tmp[p-mpos[0][0]][0][0]
							if (error_flag)
								dest_err[mpos[0][0],mpos[1][0]]+=mass_tmp_err[j]*((ugfac*ms_conc_tmp[p-mpos[0][0]][0][0])^2)
							endif
						else		// else !t_base_flag... we do no averagin
							dest[p1]=mass_tmp[j]*ugfac
							if (error_flag)
								dest_err[p1]=mass_tmp_err[j]*ugfac*ugfac
							endif
						endif	// endif(t_base_flag)
					break
					case 2:  // Speciated mass spectrum vs m/z (an averaged mass spectrum)
						ugfac*=t_step
						if (mz_base_flag)
							for (k=0;k<numMzSticks;k+=1) //1.59E   took out <=
								sq_map2dist(k,0,0,ms_conc_tmp,mpos,map1=mz_map)
								dest[mpos[0][0],mpos[1][0]]+=mass_tmp[j][k]*ugfac*ms_conc_tmp[p-mpos[0][0]][0][0]		//1.59E
								dest_n[mpos[0][0],mpos[1][0]]+=t_step*ms_conc_tmp[p-mpos[0][0]][0][0]
								if (error_flag)
									dest_err[mpos[0][0],mpos[1][0]]+=mass_tmp_err[j][k]*((ugfac*ms_conc_tmp[p-mpos[0][0]][0][0])^2) 	//1.59E
								endif
								if (rawBaseline_flag)		//1.46E  - if we want to get both the average raw spec and the average baseline
									dest_bl[mpos[0][0],mpos[1][0]]+=mass_tmp_bl[j][k]*ugfac*ms_conc_tmp[p-mpos[0][0]][0][0]  	//1.59E
								endif
							endfor
						else		//   else !mz_base_flag
							dest+=mass_tmp[j][p]*ugfac
							dest_n+=t_step
							if (error_flag)
								dest_err+=mass_tmp_err[j][p]*ugfac*ugfac
							endif
							if (rawBaseline_flag)		//1.46E  - if we want to get both the average raw spec and the average baseline
								dest_bl+=mass_tmp_bl[j][p]*ugfac
							endif
						endif 	// endif(mz_base_flag)
					break
					case 3: // Speciated mass spectrum vs time and m/z (what Qi's analysis uses)  2D product of time and mz
						if (t_base_flag || mz_base_flag)
							ugfac*=t_step
							if (!mz_base_flag)		// if t_base_flag only
								sq_map2dist(j,0,0,ms_conc_tmp,mpos,map1=t_map)
								dest[mpos[0][0],mpos[1][0]][]+=mass_tmp[j][q]*ugfac*ms_conc_tmp[p-mpos[0][0]][0][0]	//1.59E
								dest_n[mpos[0][0],mpos[1][0]][]+=t_step*ms_conc_tmp[p-mpos[0][0]][0][0]
								if (error_flag)
									dest_err[mpos[0][0],mpos[1][0]][]+=mass_tmp_err[j][q]*((ugfac*ms_conc_tmp[p-mpos[0][0]][0][0])^2)  //1.59E
								endif
							else			// if mz_base_flag
								for (k=0;k<numMzSticks;k+=1)   // also used for raw spectra  1.60Q
									if (t_base_flag)  // if mz_base_flag and t_base_flag
										sq_map2dist(j,k,0,ms_conc_tmp,mpos,map1=t_map,map2=mz_map)  // sets rows and cols of mpos
									else  // if mz_base_flag and !t_base_flag
										sq_map2dist(0,k,0,ms_conc_tmp,mpos,map2=mz_map)  // sets col 1 of mpos
										mpos[][0]=p1											// sets cols 0 of mpos
									endif
									dest[mpos[0][0],mpos[1][0]][mpos[0][1],mpos[1][1]]+=mass_tmp[j][k]*ugfac*ms_conc_tmp[p-mpos[0][0]][q-mpos[0][1]][0]   //1.60Q  fixed typo
									dest_n[mpos[0][0],mpos[1][0]][mpos[0][1],mpos[1][1]]+=t_step*ms_conc_tmp[p-mpos[0][0]][q-mpos[0][1]][0]
									if (error_flag)
										dest_err[mpos[0][0],mpos[1][0]][mpos[0][1],mpos[1][1]]+=mass_tmp_err[j][q]*((ugfac*ms_conc_tmp[p-mpos[0][0]][q-mpos[0][1]][0])^2)
									endif
								endfor
							endif  //  if (!mz_base_flag)	
						else	// neither t_base_flag nor mz_base_flag
							dest[p1][]=mass_tmp[j][q]*ugfac
							if (error_flag)
								dest_err[p1][]=mass_tmp_err[j][q]*ugfac*ugfac
							endif
						endif
					break
					case 4: // Speciated mass concentration vs diameter (averaged size distribution)   
//			PostAvgFlag==0  use mass_tmp...  is runs x numPToF
				// mass_tmp  runs x mz x (PToF) for results with mz dimension
				// mass_tmp  runs x (PToF) for results without mz dimension	
//			PostAvgFlag==1 use source  ... dataTemp is 1 x numPToF x mz 
//			PostAvgFlag==2 use mass_tmp... dataTemp is runs x numPToF
						ugfac*=t_step
						if (PostAvgFlag==0)	//  if ePToF, then data has already been deconvolved.... treat it just like regular PToF 
							if (d_base_flag)
								for (k=0;k<dimSource1;k+=1)
									sq_map2dist(k,0,0,ms_conc_tmp,mpos,map1=d_map)
									dest[mpos[0][0],mpos[1][0]]+=mass_tmp[j][k]*ugfac*ms_conc_tmp[p-mpos[0][0]][0][0]
									dest_n[mpos[0][0],mpos[1][0]]+=t_step*ms_conc_tmp[p-mpos[0][0]][0][0]
									if (error_flag)
										dest_err[mpos[0][0],mpos[1][0]]+=mass_tmp_err[j][k]*((ugfac*ms_conc_tmp[p-mpos[0][0]][0][0])^2)
									endif
								endfor
							else		// (!d_base_flag)
								dest+=mass_tmp[j][p]*ugfac  // dest is 1D wave d_points
								dest_n+=t_step
								if (error_flag)
									dest_err+=mass_tmp_err[j][p]*ugfac*ugfac
								endif
							endif	// if(d_base_flag)
						elseif (PostAvgFlag == 1)			// we still don't know if we are dealing with a species like Org or a simple mz like 55
							dataTemp[0][][] += source[j][q][r]*ugfac		// dataTemp has dims (1,dimsize(source, 1), dimsize(source, 2)) dataTemp  we deliberately use source here
							dataTemp_n+= t_step
						elseif (PostAvgFlag==2)	// we have applied frags	dataTemp has dims 1 x dimSource1 and mass_tmp has dims runs x dimSource1
							dataTemp[0][]+=mass_tmp[j][q]*ugfac		// we don't use dest because final dest may be averaged in PToF space..  dataTemp dimensions are (1, dimSource1)
							dataTemp_n+=t_step
						endif   //(!PostAvgFlag)		//1.57L
					break
					case 5: // Speciated mass concentration vs time and diameter , image plots...   mass_tmp is run x PToF                image plots
//			PostAvgFlag==0  use mass_tmp...  is runs x numPToF
				// mass_tmp  runs x mz x (PToF) for results with mz dimension
				// mass_tmp  runs x (PToF) for results without mz dimension	
//			PostAvgFlag==1 use source  ... dataTemp is 1 x numPToF x mz 
//			PostAvgFlag==2 use mass_tmp... dataTemp is runs x numPToF
						if (PostAvgFlag==0)			//  if ePToF, then data has already been deconvolved.... treat it just like regular PToF 
							if (d_base_flag || t_base_flag)
								ugfac*=t_step
								if (!d_base_flag)	// t_base_flag && !d_base_flag 
									sq_map2dist(j,0,0,ms_conc_tmp,mpos,map1=t_map)
									dest[mpos[0][0],mpos[1][0]][]+=mass_tmp[j][q]*ugfac*ms_conc_tmp[p-mpos[0][0]][0][0]
									dest_n[mpos[0][0],mpos[1][0]]+=t_step*ms_conc_tmp[p-mpos[0][0]][0][0]
									if (error_flag)
										dest_err[mpos[0][0],mpos[1][0]][]+=mass_tmp_err[j][q]*((ugfac*ms_conc_tmp[p-mpos[0][0]][0][0])^2)
									endif
								else		//  d_base_flag 
									for (k=0;k<dimSource1;k+=1)
										if (t_base_flag)  // d_base_flag && t_base_flag
											sq_map2dist(j,k,0,ms_conc_tmp,mpos,map1=t_map,map2=d_map)
										else  // d_base_flag && !t_base_flag
											sq_map2dist(0,k,0,ms_conc_tmp,mpos,map2=d_map)	// results deal with the second dimension, size
											mpos[][0]=p1
										endif
										dest[mpos[0][0],mpos[1][0]][mpos[0][1],mpos[1][1]]+=mass_tmp[j][k]*ugfac*ms_conc_tmp[p-mpos[0][0]][q-mpos[0][1]][0]
										dest_n[mpos[0][0],mpos[1][0]][mpos[0][1],mpos[1][1]]+=t_step*ms_conc_tmp[p-mpos[0][0]][q-mpos[0][1]][0]
										if (error_flag)
											dest_err[mpos[0][0],mpos[1][0]][mpos[0][1],mpos[1][1]]+=mass_tmp_err[j][k]*((ugfac*ms_conc_tmp[p-mpos[0][0]][q-mpos[0][1]][0])^2)
										endif
									endfor
								endif	// endif dbase_flag
							else		//  else !(d_base_flag || t_base_flag)
								dest[p1][]=mass_tmp[j][q]*ugfac
								if (error_flag)
									dest_err[p1][]=mass_tmp_err[j][q]*ugfac*ugfac
								endif
							endif  // endif (d_base_flag || t_base_flag)
						elseif(PostAvgFlag==1)   // use source  // frag NOT applied, we will deconvolve later//1.57L  image plots OR banded mass concentrations
							if (t_base_flag)
								sq_map2dist(j,0,0,ms_conc_tmp,mpos,map1=t_map)
								dataTemp[mpos[0][0],mpos[1][0]][][]+=source[j][q][r]*ugfac*ms_conc_tmp[p-mpos[0][0]][0][0]
								dataTemp_n[mpos[0][0],mpos[1][0]]+=t_step*ms_conc_tmp[p-mpos[0][0]][0][0]
							else  // PostAvgFlag	and ! tbaseflag
								dataTemp[p1][][] = source[j][q][r]*ugfac
							endif   //  t_base_flag		//1.57L
						elseif (PostAvgFlag==2)   // use mass_tmp with dims runs PToF // frag applied, we will deconvolve later
							if (t_base_flag)
								sq_map2dist(j,0,0,ms_conc_tmp,mpos,map1=t_map)
								dataTemp[mpos[0][0],mpos[1][0]][]+=mass_tmp[j][q]*ugfac*ms_conc_tmp[p-mpos[0][0]][0][0]
								dataTemp_n[mpos[0][0],mpos[1][0]]+=t_step*ms_conc_tmp[p-mpos[0][0]][0][0]
							else  // PostAvgFlag	and ! tbaseflag
								dataTemp[p1][] = mass_tmp[j][q]*ugfac
							endif   //  t_base_flag		//1.57L
						endif //  (!PostAvgFlag)		//1.57L
					break
					case 6: // Speciated, banded, averaged mass spectrum vs m/z and diameter
							// mz  mass_tmp has dims = time x mz x PToF
						ugfac*=t_step
						if (PostAvgFlag==0)			//  if ePToF, then data has already been deconvolved.... treat it just like regular PToF 
							if (d_base_flag && mz_base_flag)
								for (k=0;k<numMzSticks;k+=1)  // k ranges over mz_base
									for (l=0;l<dimSource1;l+=1)   // k ranges over d_base
										sq_map2dist(k,l,0,ms_conc_tmp,mpos,map1=mz_map,map2=d_map)
										dest[mpos[0][0],mpos[1][0]][mpos[0][1],mpos[1][1]]+=mass_tmp[j][k][l]*ugfac*ms_conc_tmp[p-mpos[0][0]][q-mpos[0][1]][0]
										dest_n[mpos[0][0],mpos[1][0]][mpos[0][1],mpos[1][1]]+=t_step*ms_conc_tmp[p-mpos[0][0]][q-mpos[0][1]][0]
										if (error_flag)
											dest_err[mpos[0][0],mpos[1][0]][mpos[0][1],mpos[1][1]]+=mass_tmp_err[j][k][l]*((ugfac*ms_conc_tmp[p-mpos[0][0]][q-mpos[0][1]][0])^2)
										endif
									endfor
								endfor
							elseif (d_base_flag)	// && !mz_base_flag
								//  kinda tricky.  Rows are the size dimensions and Cols are the mz dimension
								for (k=0;k<dimSource1;k+=1)
									sq_map2dist(k,0,0,ms_conc_tmp,mpos,map1=d_map)
									dest[][mpos[0][0],mpos[1][0]]+=mass_tmp[j][p][k]*ugfac*ms_conc_tmp[q-mpos[0][0]][0][0]
									dest_n[][mpos[0][0],mpos[1][0]]+=t_step*ms_conc_tmp[q-mpos[0][0]][0][0]
									if (error_flag)
										dest_err[][mpos[0][0],mpos[1][0]]+=mass_tmp_err[j][p][k]*((ugfac*ms_conc_tmp[q-mpos[0][0]][0][0])^2)
									endif
								endfor
							elseif (mz_base_flag)  // && !d_base_flag
								for (k=0;k<numMzSticks;k+=1)
									sq_map2dist(k,0,0,ms_conc_tmp,mpos,map1=mz_map)
									dest[mpos[0][0],mpos[1][0]][]+=mass_tmp[j][k][q]*ugfac*ms_conc_tmp[p-mpos[0][0]][0][0]
									dest_n[mpos[0][0],mpos[1][0]][]+=t_step*ms_conc_tmp[p-mpos[0][0]][0][0]
									if (error_flag)
										dest_err[mpos[0][0],mpos[1][0]][]+=mass_tmp_err[j][k][q]*((ugfac*ms_conc_tmp[p-mpos[0][0]][0][0])^2)
									endif
								endfor
							else //  !d_base_flag && !mz_base_flag  this rarely happens that we want an average mass spec for each PToF bin
								// mass_tmp  runs x mz x (PToF) for results with mz dimension
								dest+= mass_tmp[j][p][q]*ugfac
								dest_n+=t_step
								if (error_flag)
									dest_err+= mass_tmp_err[j][p][q]*ugfac*ugfac  //    mass_tmp_err[j][p][q]*ugfac*ugfac
								endif
							endif  // end if d_base_flag, mz_base_flag combinations
						elseif (PostAvgFlag==1)	// frag NOT applied, we will deconvolve later 
							// be careful  dataTemp has dims (1, dimSource1, dimSource2)	
							dataTemp[0][][] +=source[j][q][r]*ugfac
							dataTemp_n +=t_step
						elseif (PostAvgFlag==2)	// frag applied, deconvolve later 
							// be careful  dataTemp has dims (1, dimSource1, dimSource2)	
							// mass_tmp  runs x mz x (PToF) for results with mz dimension  // this is why right side has [q][p]
							dataTemp[0][][]+=mass_tmp[j][r][q]*ugfac
							dataTemp_n+=t_step
						endif		// if (!PostAvgFlag)		//1.57L
					break
					case 7: // Speciated, banded mass spectrum vs Time, m/z and diameter
						if (PostAvgFlag==0)		//  if ePToF, then data has already been deconvolved.... treat it just like regular PToF 
							// recall dest has dims time_points x mz_points x d_points 							
							if (d_base_flag || t_base_flag || mz_base_flag) // Only multiply by time if weighting is used
								ugfac*=t_step
							endif
							if (d_base_flag && mz_base_flag)
								for (k=0;k<numMzSticks;k+=1)
									for (l=0;l<dimSource1;l+=1)
										if (t_base_flag)    // d_base_flag && mz_base_flag && t_base_flag
											sq_map2dist(j,k,l,ms_conc_tmp,mpos,map1=t_map,map2=mz_map,map3=d_map)
										else				   // d_base_flag && mz_base_flag && !t_base_flag
											sq_map2dist(0,k,l,ms_conc_tmp,mpos,map2=mz_map,map3=d_map)
											mpos[][0]=p1
										endif
										dest[mpos[0][0],mpos[1][0]][mpos[0][1],mpos[1][1]][mpos[0][2],mpos[1][2]]+=mass_tmp[j][k][l]*ugfac*ms_conc_tmp[p-mpos[0][0]][q-mpos[0][1]][r-mpos[0][2]]
										dest_n[mpos[0][0],mpos[1][0]][mpos[0][1],mpos[1][1]][mpos[0][2],mpos[1][2]]+=t_step*ms_conc_tmp[p-mpos[0][0]][q-mpos[0][1]][r-mpos[0][2]]
										if (error_flag)
											dest_err[mpos[0][0],mpos[1][0]][mpos[0][1],mpos[1][1]][mpos[0][2],mpos[1][2]]+=mass_tmp_err[j][k][l]*((ugfac*ms_conc_tmp[p-mpos[0][0]][q-mpos[0][1]][r-mpos[0][2]])^2)
										endif
									endfor
								endfor
							elseif (d_base_flag)		// elseif d_base_flag  && !mz_base_flag
								for (k=0;k<dimSource1;k+=1)
									if (t_base_flag)	//	d_base_flag  && !mz_base_flag && t_base_flag
										sq_map2dist(j,k,0,ms_conc_tmp,mpos,map1=t_map,map2=d_map)
									else				//	d_base_flag  && !mz_base_flag && !t_base_flag
										sq_map2dist(0,k,0,ms_conc_tmp,mpos,map2=d_map)
										mpos[][0]=p1
									endif
									dest[mpos[0][0],mpos[1][0]][][mpos[0][1],mpos[1][1]]+=mass_tmp[j][q][k]*ugfac*ms_conc_tmp[p-mpos[0][0]][r-mpos[0][1]][0]
									dest_n[mpos[0][0],mpos[1][0]][][mpos[0][1],mpos[1][1]]+=t_step*ms_conc_tmp[p-mpos[0][0]][r-mpos[0][1]][0]
									if (error_flag)
										dest_err[mpos[0][0],mpos[1][0]][][mpos[0][1],mpos[1][1]]+=mass_tmp_err[j][q][k]*((ugfac*ms_conc_tmp[p-mpos[0][0]][r-mpos[0][1]][0])^2)
									endif
								endfor
							elseif (mz_base_flag)	 // elseif !d_base_flag  && mz_base_flag
								for (k=0;k<numMzSticks;k+=1)
									if (t_base_flag)   // !d_base_flag  && mz_base_flag && t_base_flag
										sq_map2dist(j,k,0,ms_conc_tmp,mpos,map1=t_map,map2=mz_map)
									else				 // !d_base_flag  && mz_base_flag && !t_base_flag
										sq_map2dist(0,k,0,ms_conc_tmp,mpos,map2=mz_map)
										mpos[][0]=p1
									endif
									dest[mpos[0][0],mpos[1][0]][mpos[0][1],mpos[1][1]][]+=mass_tmp[j][k][r]*ugfac*ms_conc_tmp[p-mpos[0][0]][q-mpos[0][1]][0]
									dest_n[mpos[0][0],mpos[1][0]][mpos[0][1],mpos[1][1]][]+=t_step*ms_conc_tmp[p-mpos[0][0]][q-mpos[0][1]][0]
									if (error_flag)
										dest_err[mpos[0][0],mpos[1][0]][mpos[0][1],mpos[1][1]][]+=mass_tmp_err[j][k][r]*((ugfac*ms_conc_tmp[p-mpos[0][0]][q-mpos[0][1]][0])^2)
									endif
								endfor
							elseif (t_base_flag)	//  !d_base_flag && !mz_base_flag  t_base_flag
								sq_map2dist(j,0,0,ms_conc_tmp,mpos,map1=t_map)
								dest[mpos[0][0],mpos[1][0]][][]+=mass_tmp[j][q][r]*ugfac*ms_conc_tmp[p-mpos[0][0]][0][0]
								dest_n[mpos[0][0],mpos[1][0]][][]+=t_step*ms_conc_tmp[p-mpos[0][0]][0][0]
								if (error_flag)
									dest_err[mpos[0][0],mpos[1][0]][][]+=mass_tmp_err[j][q][r]*((ugfac*ms_conc_tmp[p-mpos[0][0]][0][0])^2)
								endif
							else					//  !d_base_flag && !mz_base_flag && !t_base_flag 
								dest[p1][][]=mass_tmp[j][q][r]*ugfac
								if (error_flag)
									dest_err[p1][][]=mass_tmp_err[j][q][r]*ugfac*ugfac
								endif
							endif	// combinations of d_base_flag, t_base_flag,  mz_base_flag
						elseif (PostAvgFlag==1)  	// frag NOT applied, we will deconvolve later 
							if (t_base_flag) // Only multiply by time if weighting is used
								ugfac*=t_step
								sq_map2dist(j,0,0,ms_conc_tmp,mpos,map1=t_map)
								dataTemp[mpos[0][0],mpos[1][0]][][]+=source[j][q][r]*ugfac*ms_conc_tmp[p-mpos[0][0]][0][0]
								dataTemp_n[mpos[0][0],mpos[1][0]]+=t_step*ms_conc_tmp[p-mpos[0][0]][0][0]
							else
								dataTemp[p1][][]=source[j][q][r]*ugfac
							endif
						elseif (PostAvgFlag==2)  	// frag applied, deconvolve later 
							// mass_tmp  runs x mz x (PToF) for results with mz dimension
							// dataTemp has dims time_points,dimSource1, dimSource2)  
							 // this is why right side has [q][p]
							if (t_base_flag) // Only multiply by time if weighting is used
								ugfac*=t_step
								sq_map2dist(j,0,0,ms_conc_tmp,mpos,map1=t_map)
								dataTemp[mpos[0][0],mpos[1][0]][][]+=mass_tmp[j][r][q]*ugfac*ms_conc_tmp[p-mpos[0][0]][0][0]
								dataTemp_n[mpos[0][0],mpos[1][0]]+=t_step*ms_conc_tmp[p-mpos[0][0]][0][0]
							else
								dataTemp[p1][][]=mass_tmp[j][r][q]*ugfac
							endif							
						endif  //  (!PostAvgFlag)		//1.57L
					break
	
				endswitch	// product type
			endif		// if(we have real data to work with, index_pos>=0)
		endfor		// for (each run)  still within for loop of each species

// Step 5. Finalize destination wave if at end of all acorns... divide by any weighting factors derived from customized mz, time, diameter
		if (acorn==n_acorns-1)
			sq_ams_stat_upd("Finalising data")
			
			if (PostAvgFlag==1 || PostAvgFlag==2 )		// deconvolve and apply frag waves here

//				for (i=0;i<n_spec;i+=1)		// typically for each species   
				i = n_spec// we get out of first loop
				  
			 	if (!(product_type&1) || t_base_flag )			// || d_base_flag   || mz_base_flag
//					wave dataTemp_n
					dataTemp/= dataTemp_n[p]		// all averaging in the run time dimension has been taken care of
				endif
				if ( PostAvgFlag==2 && dimsize(dataTemp,2)==0)	
					redimension/n=(dimsize(dataTemp,0), dimsize(dataTemp,1), 1) dataTemp
				endif
				
				sq_SeqExpand(osFactor)		// creates  root:PToF:SeqOutExp
				wave SeqOutExp =  root:PToF:SeqOutExp
				duplicate/o dataTemp deconvolved3DData
				 //  deconvolve the whole beast
				make/free/o/n=(dimSource2) confidLim1D		// mz
				make/free/o/n=(dimSource0, dimSource2) confidLimitMx		// run x mz
				// sq_EPToFDeconvolve3D relies on dim order to be runs, PToF, mz
				sq_EPToFDeconvolve3D(dataTemp, osFactor, deconvolved3DData, confidLimitMx, confidLim1D, SeqOutExp, 1, 0)
				// 1.61A addition  The deconvol will insert zeros if dataTemp is nan 
				variable idex
				for(idex=0;idex<dimsize(dataTemp,0); idex+=1)
					ImageTransform/pTyp=2/P=(idex) getPlane deconvolved3DData
					wave M_ImagePlane
					if (WaveMax(M_ImagePlane)==0 && WaveMin(M_ImagePlane)==0)
						deconvolved3DData[idex][][]=nan
					endif
				endfor
				nvar deconvolveJoel = root:panel:deconvolveJoel
				if (deconvolveJoel)//1.59F
					// we fake the index_pos values  1.60X
					wave squirrel_index = root:index:squirrel_index
					variable layNum = FindDimLabel(squirrel_index, 2,"PTof_stick")
					make/o/free/n=(dimsize(squirrel_index,0)) col0 = squirrel_index[p][0][layNum]
					wavestats/q/m=1 col0
					index_pos=V_maxloc
					redimension/n=(dimsize(deconvolved3DData,0)) index_pos
					sq_ApplyDCmarkers(deconvolved3DData,index_pos,1,nan,nan,nan,nan,"","", 1, 2,0, PossibleTimeAvg=1)		// moved to this location 1.59D		
				endif
				// apply frag waves, RIE, CE
				if (  (product_type==4 || product_type==5) && ePToF_twflag & PToF_DCgr_ePToFSumDiffScaling)
					wave ePToFSumDiffAvg_temp = $"ePToFSumDiffAvg_temp"
					if (!WaveExists(ePToFSumDiffAvg_temp) )
						print "// Alert!  The TW deconvolution did not do a scaling."
					endif
					make/free/n=(0,0) AcornMx
					variable unDoChopper = (1/ePToFFractionOpen)*ChopperFreq[p1] // we are in open half the time  //1.55B  twas 2
					sq_EPToFScaling(deconvolved3DData, ePToFSumDiffAvg_temp, AcornMx, index_pos, acorn==n_acorns-1, AvgFlag=1 )
					print "// The TW scaling values for each mz can be found in root:TWePTOF:AvgDeconvScalMz"
				endif				
							
				if (dlog_op)
					deconvolved3DData[][][]*=dtdlogdp_tmp[q]		// was dtdlogdp_tmp[r]  fix in 1.57N	
				endif

				for (postAvgIdex=0;postAvgIdex<n_spec;postAvgIdex+=1)		// typically for each species   can be confusing cuz we are using i as an index for species elsewhere
					deststr=stringfromlist(postAvgIdex, destlist,"/")
					wave dest=$deststr
					wave /z dest_n=$"conc_weight_"+num2str(postAvgIdex)
					
					if (PostAvgFlag==1)	
						if (strlen(rie_list)>0 && strlen (ce_list)>0  && ug_op)			// 1.41B change to include ug_op
							rie=str2num(stringfromlist(postAvgIdex, rie_list,"/"))
							if (WaveExists(CE_corr_wav))				//1.53M
								ce=1  // ignore scalar values!  1.53M  str2num(stringfromlist(i, ce_list,"/"))
							else
								ce=str2num(stringfromlist(postAvgIdex, ce_list,"/"))
							endif
						else
							rie=1
							ce=1
						endif
						wave/wave waveRefs = sq_ApplyFragWaves(deconvolved3DData, postAvgIdex, destlist, speclist, product_type, error_flag, "root:ms_mats:", numMzSticks, dimsize(deconvolved3DData,0), dimSource1, index_pos = pseudoIndex_pos)
						// if result doesn't have m/z dimension sq_ApplyFragWaves collapses the m/z dimension.  
						//  Org size distribution or image plot mass_tmp is 2D of acorn-runs x PToF bins 
						//  Org mass spec mass_tmp is  3D of acorn-runs x PToF bins x max mz 					
						wave mass_tmp = waveRefs[0]
						mass_tmp /=(rie*ce)
					endif
						
					// then we do any diameter averaging  our results are in dataTemp not mass_temp
					if (PostAvgFlag==1  )		//we JUST applied the frag
						wave AvgInDim = mass_tmp
					else   // ( PostAvgFlag==2 )	
						wave AvgInDim = deconvolved3DData
					endif
					switch (product_type)
						case 4: // Speciated mass concentration vs diameter (averaged size distribution)
							// if PostAvgFlag==1  AvgInDim  has dims 1, dimSource1
							// if PostAvgFlag==2  AvgInDim  has dims 1, dimSource1
							if (d_base_flag)		// we are presuming the same DVa mapping for all runs 
								for (k=0;k<dimSource1;k+=1) // for each PToF bin........ we are averaging in the 2nd dimension
									sq_map2dist(k,0,0,ms_conc_tmp,mpos,map1=d_map)
									dest[mpos[0][0],mpos[1][0]]+=AvgInDim[0][k]*ms_conc_tmp[p-mpos[0][0]][0][0]
									dest_n[mpos[0][0],mpos[1][0]][mpos[0][1],mpos[1][1]]+=ms_conc_tmp[p-mpos[0][0]][q-mpos[0][1]][0]
								endfor
								dest/=dest_n
							else
								dest = AvgInDim[0][p]		// in the case of PostAvgFlag==2 this could be a 3D beast
							endif								
						break
						case 5: // Speciated mass concentration vs time and diameter (image plots)
							// if PostAvgFlag==1  AvgInDim  = mass_tmp has dims time x dimSource1
							// if PostAvgFlag==2  AvgInDim = dataTemp has dims time x dimSource1 
							if (d_base_flag)		// we are presuming the same DVa mapping for all runs 	
								for (k=0;k<dimSource1;k+=1) // for each PToF bin........ we are averaging in the 2nd dimension
									sq_map2dist(k,0,0,ms_conc_tmp,mpos,map1=d_map)
									dest[][mpos[0][0],mpos[1][0]]+=AvgInDim[p][k]*ms_conc_tmp[q-mpos[0][0]][0][0]
									dest_n[][mpos[0][0],mpos[1][0]]+=ms_conc_tmp[q-mpos[0][0]][0][0]
								endfor
								dest/=dest_n
							else
								dest = AvgInDim[p][q]	 // in the case of PostAvgFlag==2 this could be a 3D beast	// note that AvgInDim is double precision, dest is SP
							endif
						break
						case 6: // Speciated, banded, averaged mass spectrum vs m/z and diameter // size resolved average MS
							// handled a bit differently because this is the ony product where rows are mz
							// if PostAvgFlag==1  AvgInDim  = mass_tmp has runs x mz x dimSource1
							// if PostAvgFlag==2  AvgInDim = deconvolved3DData has runs x dimSource1 x dimSource2 
							if (PostAvgFlag==1)
								ImageTransform/g=4 transposevol mass_tmp //  M_VolumeTranspose=imageMatrix [q][r][p]
							elseif (PostAvgFlag==2)
								ImageTransform/g=3 transposevol deconvolved3DData //  M_VolumeTranspose=imageMatrix [r][q][p]
							endif
							wave M_VolumeTranspose
							wave AvgInDim = 	M_VolumeTranspose
						  // i.e. Org product is 2D sized matrix with mz rows, #size dims columns //  dest has dims (mz_points,d_points) 
						  // dataTemp has dims (1, dimSource1, dimSource2) dataTemp  run x PToF x mz
							if (d_base_flag)		// we are presuming the same DVa mapping for all runs 
								for (k=0;k<dimSource1;k+=1) // for each PToF bin........ we are averaging in the 2nd dimension
									sq_map2dist(k,0,0,ms_conc_tmp,mpos,map1=d_map)
									dest[][mpos[0][0],mpos[1][0]]+=AvgInDim[p][k][0]*ms_conc_tmp[q-mpos[0][0]][0][0]
									dest_n[][mpos[0][0],mpos[1][0]]+=ms_conc_tmp[q-mpos[0][0]][0][0]
								endfor
								dest/=dest_n
							else
								dest =  AvgInDim[p][q][0]	//AvgInDim[0][q][p]		// rarely used because this would mean many mass spectra
							endif
						break
						case 7: // Speciated, banded mass spectrum vs Time, m/z and diameter (For a size-resolved version of Qi's analysis)
							// if PostAvgFlag==1  AvgInDim  = mass_tmp has dims time x mz x dimSource1
							// if PostAvgFlag==2  AvgInDim = deconvolved3DData has dims time x dimSource1 x dimSource2 
							if (PostAvgFlag==2)
								ImageTransform/g=1 transposevol deconvolved3DData //  M_VolumeTranspose=imageMatrix [r][q][p]
								wave M_VolumeTranspose
								wave AvgInDim = M_VolumeTranspose
							endif
							if (d_base_flag)		// we are presuming the same DVa mapping for all runs 
								for (k=0;k<dimSource1;k+=1) // for each PToF bin........ we are averaging in the 2nd dimension
									sq_map2dist(k,0,0,ms_conc_tmp,mpos,map1=d_map)
									dest[][][mpos[0][0],mpos[1][0]]+=AvgInDim[p][q][k]*ms_conc_tmp[q-mpos[0][0]][0][0]
									dest_n[][][mpos[0][0],mpos[1][0]]+=ms_conc_tmp[q-mpos[0][0]][0][0]
								endfor
								dest/=dest_n
							else
								dest = mass_tmp[p][q][r]
							endif
						break
					endswitch	// product type
				
				endfor   // for each species
//			endif  // PostAvgFlag is true	
			
			elseif (!(product_type&1) || t_base_flag || d_base_flag || mz_base_flag)
				if (dimSize(dest,2)==0	)	// result is a 2D beast, typical
					matrixop /o dest=dest/dest_n
					if (error_flag)
						matrixop /o dest_err=powr(dest_err,0.5)/dest_n
					endif
					if (rawBaseline_flag)		//1.46E  - if we want to get both the average raw spec and the average baseline
						matrixop /o dest_bl = dest_bl/dest_n
					endif
					//                if (dimsize(dest,1)==0 && d_base_flag)    //^ 1.36G, 1.37A Igor changes dimensionality when the magnitude of a dimension is 1
					if (dimsize(dest_n,1)==1 && dimsize(dest_n,2)==0)
						Redimension/N=(-1,1) dest
						if (error_flag)
							redimension/n=(-1,1) dest_err
						endif
					elseif (dimsize(dest_n,2)==1)
						Redimension/N=(-1,-1,1) dest
						if (error_flag)
							redimension/n=(-1,-1,1) dest_err
						endif
					endif
					killwaves /z dest_n				
				else		// result is a 3D beast, i.e. PToF run, Mz & size matrix
					dest/=dest_n
					if (dimsize(dest_n,2)==1)
						Redimension/N=(-1,-1,1) dest
					endif
				endif
				
				killwaves /z dest_n				
				
			elseif (error_flag) 
				matrixop /o dest_err=powr(dest_err,0.5)
				if (dimsize(dest,1)==1 && dimsize(dest,2)==0)
					redimension /n=(-1,1) dest_err
				elseif (dimsize(dest,2)==1)
					redimension /n=(-1,-1,1) dest_err
				endif
			endif	// we need to deal with weighting average for run, mz, diameter dimension OR errors
		endif	// and the end of an acorn

	endfor  		// for each species 

	// Step 6. Delete temporary waves
	if (acorn==n_acorns-1)
		sq_ams_stat_upd("Deleting temporary data")
		killwaves /z mass_tmp,mass_tmp_err,vec_tmp,dtdlogdp_tmp,d_map,t_map,log_d_base_tmp,log_dsig_tmp, ms_conc_tmp,mz_tmp
		killwaves /z m_volumetranspose,m_convolution,smooth_tmp,t_source,mz2_base_tmp,dtdmz_tmp, mpos, ugfac_wav_tmp //,mz_map
		killwaves /z op_tmp,opfac_tmp, dummyWave
		killwaves /z op_dep1,op_dep2, source_err2  //1.45 addition from James
		killwaves /z mass_tmp_bl, bk_tmp, samptimeClosedModified
		speclist= WaveList("conc_weight_*", ";", "")
		for (i=0;i<itemsinlist(speclist); i+=1 )
			killwaves /z $stringfromList(i, specList, ";")
		endfor
		killwaves /z waveRefs
		killwaves/z root:M_VolumeTranspose,  root:hoard:M_VolumeTranspose
		killwaves/z  dataTemp, dataTemp_n
		killwaves/z  pseudoIndex_pos
		KillWaves/z deconvolved3DData,confidLim1D,confidLimitMx, M_ImagePlane		/// important!  not temp_PToFmatrix
		KillWaves/z ePToFsamptimeClosedModified, fastsamptimeClosedModified
		killwaves/z Timewaves
	endif

End


// modified in 1.59E
//& Make a mapping wave, used to assign data from one axis to another.
Function sq_make_map(source,dest,mapstr)
wave source,dest //Souce and destination axes
string mapstr

	variable n,m,n_map,p1,p2,i,fp1,fp2
	variable rangeflag,lo_range=dest[0],hi_range=dest[numpnts(dest)-1]  //JDA 260417
	
	n=dimsize(source,0)
	m=dimsize(source,1)*0.5                              // a flag for a 2D input.  For now used only when making a time series map
	n_map=n+m-1
	make /o/n=(n_map,4) $mapstr
	wave map=$mapstr
	
	if (!m)
		p2=binarysearchinterp(dest,source[0])
		fp2=floor(p2)
	endif
	
	for (i=0;i<n_map;i+=1)
	
		if (m)
			p1=binarysearchinterp(dest,source[i][0])
			fp1=floor(p1)
			p2=binarysearchinterp(dest,source[i][1])
			rangeflag=(source[i][1]>lo_range) && (source[i][0]<hi_range)  //JDA 260417
		else
			p1=p2
			fp1=floor(p2) //JDA 120417
			p2=binarysearchinterp(dest,source[i+1])
			rangeflag=(source[i+1]>lo_range) && (source[i]<hi_range)  //JDA 260417
		endif
			fp2=ceil(p2)-1 //JDA 120417
		//                           if (numtype(p1) && numtype (p2)) // No overlap
		if (!rangeflag) // No overlap //JDA 260417
			map[i][]=0
		else
			if (numtype(p1)) // Clips beginning
				p1=0
				fp1=0
			elseif (numtype(p2)) // Clips end
				p2=numpnts(dest)-1
				fp2=p2-1
			endif
			if (fp1==fp2) // Data to be allocated to a single point
				map[i][0]=fp1
				map[i][1]=p2-p1
				map[i][2]=fp1
				map[i][3]=0
			else // Data to be allocated to multiple points
				map[i][0]=fp1
				map[i][1]=fp1+1-p1
				map[i][2]=fp2
				map[i][3]=p2-fp2
			endif
		endif
	endfor
 
End



//& Returns the dtdlogdp value for specific PToF value - used for conversion to dMdLogDp
Function sq_dtdlogdp(lc,vl,vg,b,t)
	variable lc,vl,vg,b,t

	return b*LN_10*((-lc/(vg-vl)) + (t*(1+(2*vl/(vg-vl)))) - (t*t*vl*(1+(vl/(vg-vl)))/lc))

End


//& Returns the vacuum aerodynamic diamter for a given PToF value
Function sq_ptof2dva(lc,vl,vg,b,dx,t)
	variable lc,vl,vg,b,dx,t
	
	variable vp=lc/t
	
	if (t<0) 		//1.43F fix
		return 0
	else
		if (vp<vl)
			return inf
		elseif (vp>vg)
			return 0
		else
			return dx*((((vg - vl) / (vp - vl)) - 1) ^ (1/b))
		endif
	endif
End


//& Make a footprint matrix, based on mapping waves in up to 3 dimensions.
//& No performance benifit over the previous version of the code, but it makes it so much tider.
// mpos and distwav are filled in
// mpos contains index values, i.e. row or p values is always a 2D  2x3 wave....each column is run, PToF, Mz , first row is start, second row is end
// distwav contains the weighting values, is initialized as a 3D 1x1x1 wave gets possibly redimensioned here
// distwav gets redimensioned if averagingis mapped onto more than one spot
// mapX is a 2D wave alwyas with 4 columns... columns 0 and 2 are point values, 1 and 3 are weighting values
Function sq_map2dist(i1,i2,i3,distwav,mpos,[map1,map2,map3])
	wave distwav,mpos,map1,map2,map3
	variable i1,i2,i3			// index 1,2,3 are used only for map1, map2, map3
	
//	variable mw1_1=1,mw2_1=0,mw1_2=1,mw2_2=0,mw1_3=1,mw2_3=0,md_1=0,md_2=0,md_3=0
// reorganizing to decipher... the second number, mwx_y, i.e. y is the map type: map1, map2, map3

	variable mw1_1=1,mw1_2=1,mw1_3=1
	variable mw2_1=0,mw2_2=0,mw2_3=0	
	variable md_1=0,  md_2=0,   md_3=0

//& Initialise variables dimension at a time and define locations to map to
// initialize mw1_y, mw2_y, mpos[0][0], mpos[1][0], md_y
	if (waveexists(map1))  /// mpos uses COLUMN 0
		mw1_1=map1[i1][1]
		mw2_1=map1[i1][3]
		mpos[0][0]=map1[i1][0]
		mpos[1][0]=map1[i1][2]
		md_1=mpos[1][0]-mpos[0][0]
		if (md_1>=dimsize(distwav,0))
			insertpoints /m=0 dimsize(distwav,0),(1+md_1-dimsize(distwav,0)), distwav
		endif
	endif
	if (waveexists(map2))   /// mpos uses COLUMN 1
		mw1_2=map2[i2][1]
		mw2_2=map2[i2][3]
		mpos[0][1]=map2[i2][0]
		mpos[1][1]=map2[i2][2]
		md_2=mpos[1][1]-mpos[0][1]
		if (md_2>=dimsize(distwav,1))
			insertpoints /m=1 dimsize(distwav,1),(1+md_2-dimsize(distwav,1)), distwav
		endif
	endif
	if (waveexists(map3))   /// mpos uses COLUMN 2
		mw1_3=map3[i3][1]
		mw2_3=map3[i3][3]
		mpos[0][2]=map3[i3][0]
		mpos[1][2]=map3[i3][2]
		md_3=mpos[1][2]-mpos[0][2]
		if (md_3>=dimsize(distwav,2))
			insertpoints /m=2 dimsize(distwav,2),(1+md_3-dimsize(distwav,2)), distwav
		endif
	endif

//& Initialise to 0
	fastop distwav=0
//& Assign values dimension at a time

	if (md_1>0)
		distwav[1,md_1][0][0]=p<md_1?(1):(mw2_1)
	endif
	distwav[0][0][0]=mw1_1

	if (md_2>0)
		distwav[0,md_1][1,md_2][0]=(q<md_2?(1):(mw2_2))*distwav[p][0][0]		 // JDA Fix 14/04/11 mw2_2 was incorrectly mw2_1  // 1.51C
	endif
	distwav[0,md_1][0][0]*=mw1_2
	
	if (md_3>0)
		distwav[0,md_1][0,md_2][1,md_3]=(r<md_3?(1):(mw2_3))*distwav[p][q][0]
	endif
	distwav[0,md_1][0,md_2][0]*=mw1_3

End


//^ was originally in SQ err WIP.ipf that James sent on 3/31/2007  // 1.38B
//& Function to calculate errors based on data in Hz
//& Includes only Poisson Stats at the moment.
//& call by:
// squirrel_fetch(all,sq_err_calc,"dest0:mssclosed_p_err;dest1:mssdiff_p_err;samptime0:root:diagnostics:timemsclosed;samptime1:root:diagnostics:timemsopen;","mssclosed_p;mssdiff_p;")
//^  I think James' comments below are dated - 1.38C
//& NEED TO INCLUDE:
//& MS baselines (Poisson)
//& PTOF baselines (Poisson)
//& Electronic noise (integration window dependence)
Function sq_err_calc(index_pos,paramstr,acorn,n_acorns)
	wave index_pos
	string paramstr
	variable acorn,n_acorns
	
	wave source0=$stringbykey("acorn_0",paramstr)			//^ Closed or PToF  	//OPEN
	wave /z source1=$stringbykey("acorn_1",paramstr)		// Diff or nothing		/CLOSED
	string deststr0=stringbykey("dest0",paramstr)			// Closed or PToF 	 // MSSClosed_p_err
	string sampTimeClosedStr=stringbykey("sampTimeClosed",paramstr)	// Closed or PToF 	// timeMSClosed
	string deststr1=stringbykey("dest1",paramstr)			// Diff or never exists   // MSSDiff_p_err
	string sampTimeOpenStr=stringbykey("sampTimeOpen",paramstr)	// Open or nothing 	// timeMSOpen
	wave /z source2=$stringbykey("acorn_2",paramstr)		// Open		// MSSOpenBaseL_p
	wave /z source3=$stringbykey("acorn_3",paramstr)		// Closed		// MSSClosedBaseL_p

//	wave nSecMSsamplingInterval= root:diagnostics:nSecMSsamplingInterval		//1.54D

//	variable PutInMemory=numtype(numberbykey("PutInMemory",paramstr) == 0) ? 0 : 1		// 1.47E
	variable PutInMemory=numberbykey("PutInMemory",paramstr)
	PutInMemory = numtype(PutInMemory)==0 ? 0 : 1			// GGGRRR bug in 1.49B   ) in the wrong place!
	// if this param is not sent then presume that we do the usual thing and keep file in memory

	wave series_index = root:index:series_index
	wave /z corr_fact= root:diagnostics:corr_fact
	
	wave /z ionSingleStr_wave=root:diagnostics:ionSingleStr
	if (!waveexists(ionSingleStr_wave) || numpnts(ionSingleStr_wave)==0)
		wave ionSingleStr_wave=root:diagnostics:ionSingleStr_logged
	endif

	wave ToFPulserInHz=root:diagnostics:ToFPulserInHz

	variable sigma=numberbykey("sigma",paramstr)^2		//Sigma is the sqrt(1+dSI2) from the formula in James' Word doc
	variable noise=numberbykey("noise",paramstr)
					
//	wave samptimeOpen=$ReplaceString("/", sampTimeOpenStr, ":")
//	wave samptimeClosedTemp = $ReplaceString("/", sampTimeClosedStr, ":")
//	Wave samptimeClosed = sq_AdjustClosedTimeForErrCalcs(sampTimeClosedTemp, sampTimeOpen)
	wave/wave TimeWaves=sq_GetModifiedTimes()
	wave samptimeOpen = TimeWaves[0]
	wave samptimeClosed = TimeWaves[1]
	wave TimePToF = TimeWaves[2]

	//1.44B	// Sanna had issues that the dimensions of these two matricies were off by 1
	// we assume that the very last m/z column doesn't matter much....
	variable numCols0 = dimSize(source0, 1)
	variable numCols1 = dimSize(source1, 1)

	if (numCols0 > numCols1)
		insertpoints/m=1 numCols1, (numCOls0 - numCols1 ), source1
	elseif (numCols0 < numCols1)
		deletepoints/m=1 numCols0, numCOls1, source1
//		insertpoints/m=1 numCols0, (numCOls1 - numCols0), source0  //1.59E
	endif

	sq_findIntegrationWidthNs(dimsize(source0,0),dimsize(source0,1),dimsize(source0,2), index_pos)		//^
	wave IntegrationWidthNs = root:IntegrationWidthNs
	
//	if (!waveexists(source1))		//PToF data
//		wave samptime=$samptimestr0
//		matrixop /o err_tmpPToF=abs(source0) //^ PTOF data
//		wave err_tmpPToF
//		err_tmpPToF *= sigma/samptime[series_index[index_pos[p]]]
//		if (waveexists(corr_fact))
//			err_tmpPToF /=corr_fact[series_index[index_pos[p]]]
//		endif
//		err_tmpPToF *= sqrt(28/(r+1)) 
//		err_tmpPToF += noise*noise*IntegrationWidthNs[p][q][r]*28/((ion_wave[series_index[index_pos[p]]]^2)*ToFPulserInHz[series_index[index_pos[p]]]*(r+1))
//		err_tmpPToF=sqrt(err_tmpPToF)		
//		//1.14M  PToF sticks, should be already trimmed, UMR
//		squirrel_hdf_write(index_pos,err_tmpPToF,deststr0)
//	else //& MSSDiff data - calculate for open
//		wave samptimeClosed=$samptimestr0
//		wave samptimeOpen=$samptimestr1

		matrixop /o err_tmpClosed=abs(source0)		
		wave err_tmpClosed
		matrixop /o err_tmpOpen=abs(source1+source0)		//^ reconstruct open by adding diff and closed
		wave err_tmpOpen
		// 1.50K addition begin
		matrixop /o err_tmpClosedBase=abs(source3)		//1.153Err	
		wave err_tmpClosedBase
		matrixop /o err_tmpOpenBase=abs(source2)		//^ reconstruct open by adding diff and closed
		wave err_tmpOpenBase
		// 1.50K addition end
		err_tmpClosed*=(index_pos[p]<0 ? 0 : sigma/samptimeClosed[series_index[index_pos[p]]])
		err_tmpOpen*=(index_pos[p]<0 ? 0 : sigma/samptimeOpen[series_index[index_pos[p]]])
		// 1.50K addition begin
		err_tmpClosedBase*=(index_pos[p]<0 ? 0 : sigma/samptimeClosed[series_index[index_pos[p]]] )
		err_tmpOpenBase*=(index_pos[p]<0 ? 0 : sigma/samptimeOpen[series_index[index_pos[p]]] )
		// 1.50K addition end
		
		if (waveexists(corr_fact))
			err_tmpClosed/=(index_pos[p]<0 ? 0 : corr_fact[series_index[index_pos[p]]] )
			err_tmpOpen/=(index_pos[p]<0 ? 0 : corr_fact[series_index[index_pos[p]]] )
			// 1.50K addition begin
			err_tmpClosedBase/=(index_pos[p]<0 ? 0 : corr_fact[series_index[index_pos[p]]] )
			err_tmpOpenBase/=(index_pos[p]<0 ? 0 : corr_fact[series_index[index_pos[p]]] )
			// 1.50K addition end
		endif
		
		err_tmpClosed*=sqrt(28/(q+1)) 		// we start our m/z matrix dimension at m/z 1.  When q = 0, m/z = 1 
		err_tmpOpen*=sqrt(28/(q+1))
		// 1.50K addition begin
		err_tmpClosedBase*=sqrt(28/(q+1)) 		// we start our m/z matrix dimension at m/z 1.  When q = 0, m/z = 1 
		err_tmpOpenBase*=sqrt(28/(q+1))
		// 1.50K addition end
		duplicate/o err_tmpClosed err_tmp
		wave err_tmp
		err_tmp = (index_pos[p]<0 ? 0 : noise*noise*IntegrationWidthNs[p][q]*28/((ionSingleStr_wave[series_index[index_pos[p]]]^2)*ToFPulserInHz[series_index[index_pos[p]]]*(q+1))  )
		err_tmpClosed+=err_tmp
		err_tmpOpen+= err_tmp
		// 1.50K addition begin
		err_tmpClosedBase+=err_tmp
		err_tmpOpenBase+= err_tmp
		// 1.50K addition end
		// below, more officially,  it is sqrt(   sqrt(err_tmpOpen)^2 + sqrt(err_tmpClosed)^2 ) 
//		err_tmp=sqrt(err_tmpOpen+err_tmpClosed)	// now the use of err_tmp changes from Electronic error to Diff error	
		err_tmp=sqrt(err_tmpOpen+err_tmpClosed+err_tmpClosedBase+err_tmpOpenBase)	// now the use of err_tmp changes from Electronic error to Diff error	
		// 1.50K addition end
		if (PutInMemory==1)		//1.47E
			squirrel_mem_write(index_pos,err_tmp,deststr1,deststr1)		// Diff
		else
			//1.14M  UMR
			squirrel_hdf_write(index_pos,err_tmp,"MSSDiff_p_err")
		endif
		err_tmpClosed=sqrt(err_tmpClosed+err_tmpClosedBase)
		// change in 1.46E, don't put the closed err in memory, write to hdf
		//1.14M  UMR
		squirrel_hdf_write(index_pos,err_tmpClosed,"MSSClosed_p_err")
		// added in 1.46E
		err_tmpOpen=sqrt(err_tmpOpen+err_tmpOpenBase)
		//1.14M  UMR
		squirrel_hdf_write(index_pos,err_tmpOpen,"MSSOpen_p_err")
//	endif
		
// Need to include some lines of code to add the baseline values to the ion signal waves 
//^This is true for both MS (via saving some baseline parameters info for each run) and PToF (via saving some DC marker values for each run)
	
	if (acorn==n_acorns-1)
		killwaves /z err_tmpClosed,err_tmpOpen, err_tmpPToF, err_tmp, IntegrationWidthNs, err_tmpOpenBase,err_tmpClosedBase, samptimeClosedModified	
		killwaves/z Timewaves
	endif

End

	
//^ calculate the integration width in ns.  Is there an easier way? Do we really need to do this on a run-by-run basis? Yup because of V/W, menu switching.
// we only need source0 for the dimensions.
// this may be obsolete in 1.44B - see sq_stickCalcGetRegions(maxMz)
Function sq_findIntegrationWidthNs(numrows, numcols, numlays, index_pos)
wave  index_pos
variable numrows, numcols, numlays

	wave series_index = root:index:series_index
	wave mStart = root:diagnostics:mStart
	wave mFinish = root:diagnostics:mFinish
	wave/z nSecMSsamplingInterval =  root:diagnostics:nSecMSsamplingInterval		//1.53H  1.12H
	if (!WaveExists(nSecMSsamplingInterval)   || !waveMax(nSecMSsamplingInterval) >0 )  //1.55G
		make/o/n=(numpnts(mStart))  root:diagnostics:nSecMSsamplingInterval=1		// default - old card always at 1ns
		wave nSecMSsamplingInterval =  root:diagnostics:nSecMSsamplingInterval		//1.53H  1.12H
	endif
		
	variable peakIntegrationDelta
	
	make/o/n=(numrows,numcols, numlays)  IntegrationWidthNs		// resides in root folder; should  be killed later in sq_err_calc
		
	make/o/n=(numpnts(index_pos)) minMz, MaxMz
	
	minMz = (index_pos[p]<0 ? waveMin(mStart) : mStart[series_index[index_pos[p]]] )
	maxMz = (index_pos[p]<0 ? waveMax(mFinish) : mFinish[series_index[index_pos[p]]] )
	
	wavestats/q/m=1 mFinish
	make/free/o/n=(V_max) mz_end, mz_start, res_temp
	
	//^ just decide to use the logged values.  The correct was is to determine if the sticks were recalced or not (on a run-by run basis) with the new m/z fitting parameters.
	wave /z mzSlope=root:diagnostics:mzSlope
	wave /z mzInt=root:diagnostics:mzint
	wave /z mzPower = root:diagnostics:mzPower
	if (!WaveExists(mzPower))
		make/o/n=(numpnts(mzSlope)) root:diagnostics:mzPower=0.5
		wave mzPower = root:diagnostics:mzPower
	endif

	nvar/z resolution_R0 = root:diagnostics:resolution_R0
	nvar/z resolution_dm = root:diagnostics:resolution_dm
	nvar/z resolution_m0 = root:diagnostics:resolution_m0

	// if these parameters exist, they will be used.  User has to be careful *not* to leave these params in a *not-final* state in the background panel
	if (NVar_Exists(resolution_R0) == 0  )
		sq_InitializeResolutionParams()
		nvar resolution_R0 = root:diagnostics:resolution_R0
		nvar resolution_dm = root:diagnostics:resolution_dm
		nvar resolution_m0 = root:diagnostics:resolution_m0
	endif
	
	nvar/z bk_StickRegionMaxAmu = root:baselineFitting:bk_StickRegionMaxAmu

	if (NVar_Exists(bk_StickRegionMaxAmu) == 0  )
		peakIntegrationDelta = .8/2		// .8 is the default mas integration region  in units of amu
	else
		peakIntegrationDelta = bk_StickRegionMaxAmu/2
	endif

	// translates raw point number into resolution value
	res_temp=resolution_R0-(resolution_R0/(1+exp(((p+1)-resolution_m0)/resolution_dm)))	

	// get the integer amu start and stop value
	mz_start[0,minMz-2]=0		//1.59F	// -1 because mz1 is at row 0 and -1 because we want the value before the last good value
	mz_end[0,minMz-2]=0			//1.59F
	mz_start[minMz-1,]=(p+1)-min(peakIntegrationDelta,( (p+1)/(2*res_temp[p]) )    )
	mz_end[minMz-1,]=(p+1)+min(peakIntegrationDelta,( (p+1)/(2*res_temp[p]) )     )

	if (numlays>0)		//ptof data
		IntegrationWidthNs[][0][] = (index_pos[p]<0 ? 0 : mzSlope[series_index[index_pos[p]]]*(mz_end[r]^.5 - mz_start[r]^.5) )		//^ intercept values drop out
		IntegrationWidthNs[][][] = (index_pos[p]<0 ? 0 : IntegrationWidthNs[p][0][r] )
	else
		IntegrationWidthNs = (index_pos[p]<0 ? 0 : mzSlope[series_index[index_pos[p]]]*(mz_end[q]^.5 - mz_start[q]^.5) )		//^ intercept values drop out
	endif

	IntegrationWidthNs*=(index_pos[p]<0 ? 0 : nSecMSsamplingInterval[series_index[index_pos[p]]] )			//1.54D

	killwaves/z  mz_end, mz_start, res_temp, minMz, maxMz
	
End


// obsolete in 1.46, kept for posterity sake
Function sq_convertNmToTime(nmVal, todoWave)
variable nmVal
wave todoWave

	nvar  cor3_chamber = root:panel:cor3_chamber
	
	wave/z vl=root:diagnostics:ptof_vl	
	if (!WaveExists(vl))
		wave vl=root:diagnostics:ptof_vl_logged
		duplicate/o vl root:diagnostics:ptof_vl
	endif
	wave/z vg=root:diagnostics:ptof_vg
	if (!WaveExists(vg))
		wave vg=root:diagnostics:ptof_vg_logged
		duplicate/o vg root:diagnostics:ptof_vg
	endif
	wave/z b=root:diagnostics:ptof_b	
	if (!WaveExists(b))
		wave b=root:diagnostics:ptof_b_logged
		duplicate/o b root:diagnostics:ptof_b
	endif
	wave/z dx=root:diagnostics:ptof_dx
	if (!WaveExists(dx))
		wave dx=root:diagnostics:ptof_dx_logged
		duplicate/o dx root:diagnostics:ptof_dx
	endif

	wave rn_series=root:index:rn_series

	variable i, n, pos
	n = numpnts(todoWave)
	
	do	
		if (sq_doesPToFDataExist(todoWave[i]) )
			pos = sq_findRun(rn_series, todoWave[i])
			return  cor3_chamber / ((vg[pos] - vl[pos]) /  ((nmVal/dx[pos])^b[pos] +1) + vl[pos])
		endif
		i+=1
	while (i<n)

// 		vp=lc/t
//		return dx*((((vg - vl) / (vp - vl)) - 1) ^ (1/b))
//		((nmVal/dx)^b +1)   = (vg - vl) / (vp - vl)
//		 vp  = (vg - vl) /  ((nmVal/dx)^b +1) + vl

	return nan

End


// obsolete in 1.46
//1.45 addition from James
//Function sq_crosswithdep(wav,op,op_dep1,op_dep2,pow,deststr) // Cross product with a dependency
//	wave wav,op,op_dep1,op_dep2
//	variable pow
//	string deststr
//	
//	variable i,j,n=dimsize(wav,2),m=dimsize(op_dep1,0)
//	duplicate /o wav, $deststr
//	wave dest=$deststr
//	for (i=0;i<n;i+=1)
//		duplicate /o op,crossdep_tmp
//		for (j=0;j<m;j+=1)
//			crossdep_tmp[op_dep1[j][0]][op_dep1[j][1]]+=op_dep2[j][i]
//		endfor
//		if (pow==1)
//			matrixop /o crossdep_tmp = wav[][][i] x crossdep_tmp
//		else
//			matrixop /o crossdep_tmp = wav[][][i] x powr(crossdep_tmp,pow)
//		endif
//		imagetransform /p=(i)/d=crossdep_tmp setplane dest
//	endfor
//	killwaves/z crossdep_tmp //^ Donna added in 1.45
//End
//	

Function sq_sparsemult(op,source,dest,depmatStr,pow)
// Apply sparse operator 'op' to data 'source' and add result to 'dest'
// Source and dest: Rows = ptof, columns = m/z, layers = time
// Operator column 0 = source m/z
// Operator column 1 = dest m/z
// Operator column 2 = multiplier
// Time dependency: Colums = time
	wave op,source,dest
	variable pow
	string depmatStr
	
	variable n=(dimsize(op,0)),i, mzdest = dimsize(dest,1), mzsource = dimsize(source,1)
	wave/z depmat = $depmatStr
	if (waveexists(depmat))
		for (i=0;i<n;i+=1) // With dependency
			if (op[i][1] < mzdest  && op[i][0] < mzsource )				//1.60L  
				dest[][op[i][1]][]+=source[p][op[i][0]][r]*(depmat[i][r]^pow)
			endif
		endfor
	else
		for (i=0;i<n;i+=1) // Without dependency 
			if (op[i][1] < mzdest  && op[i][0] < mzsource )				//1.60L
				dest[][op[i][1]][]+=source[p][op[i][0]][r]*(op[i][2]^pow)
			endif
		endfor
	endif		
end


// The function below was copied from Igrid Ulbrich's pmf_errPrep_ToF-AMS_OneIOnEquiv_v2.3.ipf
CONSTANT PMF_AVOGADRO=6.0221367e23    //& Avagadro's number
CONSTANT PMF_AW_NO3 = 62  //atomic weight of nitrate, defaults to 62 (it is rounded, not exact)
////////////////////////////////////////////////////////////////////
// identical to pmf_err_minErr1ion_ugm3, except now returns the name of the created igor wave
Function/s Err_minErr1ion(conv2ugm3Flag, TimeMeasuringWave)
wave TimeMeasuringWave

	variable conv2ugm3Flag
	variable idex
	string resultStr
	
	// 0.  get current todo wave
	svar/z sq_toDoWvNm = root:panel:sq_toDoWvNm
	wave toDoWv = $sq_toDoWvNm

	wave rn_series = root:index:rn_series
	make/O/N=(numpnts(rn_series)) minErr1ion = 1/TimeMeasuringWave	

	// 2.  Hz to ug/m3 correction factor
	if(conv2ugm3Flag == 1)
		
		wave/Z ioneff = root:diagnostics:ioneff
		wave/Z flowrate = root:diagnostics:flowrate
		wave/Z corr_fact = root:diagnostics:corr_fact

		if (!WaveExists(ioneff))
			wave/Z ioneff = root:diagnostics:ioneff_logged
		endif
		make/FREE/o/n=(numpnts(rn_series)) ugfac_wav_tmp
		if(waveexists(ioneff) && waveexists(flowrate))
			ugfac_wav_tmp=(PMF_AW_NO3/PMF_AVOGADRO)*1e12/(ioneff[p]*flowrate[p])     // presumes ioneff exists; you did the AB corrections
			if(waveexists(corr_fact))
				ugfac_wav_tmp*=corr_fact[p]      // if you want to fold in the Airbeam correction factor
			else
				print "Airbeam correction factor was not applied."
			endif	

			// apply to hz wave		
//			duplicate/O minErr1ion_Hz_all, minErr1ion_ugm3_all
			minErr1ion *= ugfac_wav_tmp
//			minErr1ion_ugm3_all *= ugfac_wav_tmp
		else
			print "Conversion to ug/m3 could not be completed because ioneff or flowrate was not available."
		endif
		
		// apply to hz wave
	endif

	// extract current todo from all
	make/o/FREE maskwave      // will be redimensioned correctly
	sq_makeMaskFromTodofindRun(maskwave,toDoWv)   // maskwave makes a wave of 1s and 0s
//	Extract/o minErr1ion_Hz_all, minErr1ion_Hz, maskwave>=0  // hand extract function
//	if(waveexists(minErr1ion_ugm3_all))
//		Extract/o minErr1ion_ugm3_all, minErr1ion_ugm3, maskwave>=0
//	endif
	
	minErr1ion = SelectNumber(maskwave[p] , nan, minErr1ion[p])
	return "minErr1ion"
	
end


// when one wants to export t series ms over a larger time base for PMF output
Function sq_sumOrAvgOverTbase(t_base, wave2PutOnNewTWave, destStr, sumFlag)
	wave t_base, wave2PutOnNewTWave
	string destStr
	variable sumFlag
	
	wave series_index = root:index:series_index
	wave imt_series = root:index:imt_series
	variable numRuns = numpnts(series_index)
	variable p1, idex, t_step, j, tdelta
	
	make /o/n=(1,1,1) mapAvgCoef		// gets redimensioned if necessary by sq_map2dist
	make /o/n=(2,3) mpos				// column 0 of this wave is the point numbers for which this measurement is mapped
	make/o/d/n=(numRuns,2) t_source=imt_series[series_index[p]+q]
	sq_make_map(t_source,t_base,"t_map")
	wave t_map

	make/o/n=(numpnts(t_base) ) $destStr/wave=dest=0
	make/o/n=(numpnts(t_base) ) dest_n=0
	
	tdelta=t_base[1] - t_base[0]		// assume the same for all

	for(j=0;j<numRuns;j+=1)		// can be made simpler without series_index, but will keep it in to mimic what we don in MS_conc

		p1=series_index[j]
		t_step=imt_series[p1+1]-imt_series[p1]
		
		sq_map2dist(j,0,0,mapAvgCoef,mpos,map1=t_map)		// 
		if (sumFlag)
			dest[mpos[0][0],mpos[1][0]]+= wave2PutOnNewTWave[j]* (mapAvgCoef[p-mpos[0][0]][0][0]* (tdelta/t_step) )	
		else		// average
			dest[mpos[0][0],mpos[1][0]]+=wave2PutOnNewTWave[j]*t_step*mapAvgCoef[p-mpos[0][0]][0][0]
			dest_n[mpos[0][0],mpos[1][0]]+=t_step*mapAvgCoef[p-mpos[0][0]][0][0]
		endif
	endfor
	
	if (!sumFlag)
		dest/=dest_n
	endif
	
	killwaves/z t_map, t_source, dest_n
end



//1.57I
// source dimensions for MS are runs x mz 
// source dimensions for PToF are runs x PToF x mz 
// result  for MS will be runs x mz if product has mz dimension, else runs
// result  for MS will be runs x mz  x PToF if product has if product has mz dimension
// 

Function/wave sq_ApplyFragWaves(source, specDex, destlist, speclist, product_type, error_flag, matDFStr, numMzSticks, numRunsInThisAcorn, n_t,[index_pos, source_err2])
wave source, index_pos, source_err2
string destlist, speclist, matDFStr
variable specDex, product_type, error_flag, numMzSticks,numRunsInThisAcorn, n_t

	wave series_index = root:index:series_index
	variable  n_dep, depdex, opfaclist_n, opfacdex, runDex
	string destStr, specStr
	
	deststr=stringfromlist(specDex, destlist,"/")
	wave dest=$deststr
	wave /z dest_n=$("conc_weight_"+num2str(specdex)),dest_err=$(deststr+"_err")		
	specStr=stringfromlist(specdex,speclist,"/")
				
	Make/O/WAVE/N=(error_flag ? 2 : 1) waveRefs

	// apply frag-converted op wave to data

//	wave op_sparse=root:ms_mats:$(specStr+"_sparse")		// i.e. Org_sparse
//	wave op_vec=root:ms_mats:$(specStr+"_vec")			// i.e. Org_vec
	wave op_sparse=$matDFStr+specStr+"_sparse"		// i.e. Org_sparse
	wave op_vec=$matDFStr+specStr+"_vec"			// i.e. Org_vec

//	// Initialise dependencies 1.46A  JDA
	wave /z opdep=$matDFStr+specStr+"_dep"				// i.e. Org_dep  if using i.e. CO2FracWave then this has dims (10,2)
	wave /z/t opdepstr=$matDFStr+specStr+"_depstr"		// i.e. Org_depStr  if using i.e. CO2FracWave then this has dims (10)  .... all the affected frag entries in 'words'
	if (waveexists(opdep))
		if (!(product_type&2)) // Total product
			make /o/n=(numMzSticks,1,numRunsInThisAcorn) op_dep1=op_vec[p]		// i.e. if mz is 700 and runs are 120 then op_dep1 = 700x1x120 and looks like Org_vec (700) for every layer
		endif
		n_dep=dimsize(opdep,0)		// i.e. 10.. the number of mzs affected
		make /o/n=(n_dep,numRunsInThisAcorn) op_dep2  // i.e. 10x120  big enough to take care of this acorn
		for (depdex=0;depdex<n_dep;depdex+=1)	// for each mz affected
			opfaclist_n=itemsinlist(opdepstr[depdex],"*")-1	// the number of dependencies..i.e. 1.  in theory one could have a frag entry that looked like CO2FracWave*anotherwave*frag_air[28]
			op_dep2[depdex][]=str2num(stringfromlist(opfaclist_n,opdepstr[depdex],"*"))  // work from the end  CO2FracWave*0.00037*1.36*1.28*1.14*frag_air[28]  gets collapsed to  CO2FracWave*0.000734269
			for (opfacdex=0;opfacdex<opfaclist_n;opfacdex+=1)  // for each dependencies..i.e. 1
				wave /z opdepwav=$stringfromlist(opfacdex,opdepstr[depdex],"*")		//  CO2FracWave
				if (!waveexists(opdepwav))
					op_dep2[depdex][]=0
					print "// ***Dependency wave "+stringfromlist(depdex,opdepstr[depdex],"*")+" not found!!***"
					break
				endif
//				op_dep2[depdex][]*= index_pos[q]>=0 ? opdepwav[series_index[index_pos[q]]]	 : 0 //1.20X	// here is where we actually replace CO2FracWave with values within this wave
				for(runDex=0;runDex<numRunsInThisAcorn;rundex+=1 )  	//1.20X
					if (index_pos[runDex]>=0)
						op_dep2[depdex][runDex]*=opdepwav[series_index[index_pos[runDex]]]
					endif
				endfor
			endfor
			if (!(product_type&2))
				op_dep1[opdep[depdex][0]][0][]+=op_dep2[depdex][r]		// recall op_dep1 dims are (numMzSticks,1,numRunsInThisAcorn and op_dep2 dims are  (n_dep,numRunsInThisAcorn) 
			endif
		endfor
	endif  	// endif dependencies exist	
		
// Arrange source data so rows pToF, columns are m/z and layers are time
//	if (dimSource2==0) // MS Data   //1.21G
	variable MSDataFlag = dimsize(source,2)==0 
	
	if (MSDataFlag) // product_type<4) // MS Data  //1.22A
		redimension /n=(numRunsInThisAcorn,numMzSticks,1) source  // runs x mz x fakePTOF
		imagetransform /g=3 transposevol source		// M_VolumeTranspose=imageMatrix [r][q][p]
	else
		 //pTOF Data
		imagetransform /g=4 transposevol source		// M_VolumeTranspose=imageMatrix [q][r][p]
	endif
	wave m_volumetranspose 			
 	// m_volumetranspose for ms  fake PToF x mz x runs 	
 	// m_volumetranspose for PToF  PToF x mz x runs 	
 		
// Perform the operation
	if (product_type&2) // MS product
		make /o/n=(  (product_type&4 && (product_type!=11 && product_type!=14) ) ? (n_t):(1),numMzSticks,numRunsInThisAcorn) mass_tmp
		fastop mass_tmp=(0)
		sq_sparsemult(op_sparse,m_volumetranspose,mass_tmp,"",1)	//1.46A  
		if (waveexists(opdep)) // Apply dependency
			sq_sparsemult(opdep,m_volumetranspose,mass_tmp,NameOfWave(op_dep2),1)
		endif
	else // Total product
		if (waveexists(opdep)) // Dependency
			matrixop /o mass_tmp=m_volumetranspose x op_dep1
		else
			matrixop /o mass_tmp=m_volumetranspose x op_vec
		endif
		if (dimsize(mass_tmp,2)==0)
			redimension /n=(-1,-1,1) mass_tmp
		endif
	endif
		
// Arrange destination data to expected form
	if (product_type<4) // source data is MS
		imagetransform /g=3 transposevol mass_tmp		// /g=3 M_VolumeTranspose=imageMatrix [r][q][p]
		matrixop /o mass_tmp=m_volumetranspose[][][0]
		if (product_type<2)
			redimension /n=(numRunsInThisAcorn) mass_tmp
		endif
	else // source data is PTOF
		if (product_type<6)
			imagetransform /g=2 transposevol mass_tmp	// /g=2 M_VolumeTranspose=imageMatrix [r][p][q]
			matrixop /o mass_tmp=m_volumetranspose[][][0]
		else
			imagetransform /g=3 transposevol mass_tmp	// /g=3 M_VolumeTranspose=imageMatrix [r][q][p]
			duplicate /o m_volumetranspose,mass_tmp
		endif
	endif

	// mass_tmp  runs x mz x (PToF) for results with mz dimension
	// mass_tmp  runs x (PToF) for results without mz dimension

	waveRefs[0] = mass_tmp

	if (error_flag)
// Arrange source data so rows pToF, columns are m/z and layers are time
		if (product_type<4) // MS Data
			redimension /n=(numRunsInThisAcorn,numMzSticks,1) source_err2
			imagetransform /g=3 transposevol source_err2 	// /g=3 M_VolumeTranspose=imageMatrix [r][q][p]
		else //pTOF Data
			imagetransform /g=4 transposevol source_err2	// /g=4 M_VolumeTranspose=imageMatrix [q][r][p]
		endif
		wave m_volumetranspose				
			
// Perform the operation
		if (product_type&2) // MS product
			make /o/n=(product_type&4?(n_t):(1),numMzSticks,numRunsInThisAcorn) mass_tmp_err
			fastop mass_tmp_err=(0)
			sq_sparsemult(op_sparse,m_volumetranspose,mass_tmp_err,"",2)		// 1.46A		// imp bug Fix in 1.47H
			if (waveexists(opdep)) // Apply dependency
				sq_sparsemult(opdep,m_volumetranspose,mass_tmp_err,NameOfWave(op_dep2),2) //1.46A // imp bug Fix in 1.47H
			endif
		else // Total product
			if (waveexists(opdep)) // Dependency
				matrixop /o mass_tmp_err=m_volumetranspose x powr(op_dep1,2)  // imp bug Fix in 1.47H
			else
				matrixop /o mass_tmp_err=m_volumetranspose x powr(op_vec,2)  // imp bug Fix in 1.47H
			endif
 			endif
// Arrange destination data to expected form
		if (product_type<4) //MS
			imagetransform /g=3 transposevol mass_tmp_err 	// /g=3 M_VolumeTranspose=imageMatrix [r][q][p]
			matrixop /o mass_tmp_err=m_volumetranspose[][][0]
			if (product_type<2)
				redimension /n=(numRunsInThisAcorn) mass_tmp_err
			endif
		else // PTOF
			if (product_type<6)
				imagetransform /g=2 transposevol mass_tmp_err  	// /g=3 M_VolumeTranspose=imageMatrix [r][p][q]
				matrixop /o mass_tmp_err=m_volumetranspose[][][0]
			else
				imagetransform /g=3 transposevol mass_tmp_err  	// /g=3 M_VolumeTranspose=imageMatrix [r][q][p]
				duplicate /o m_volumetranspose,mass_tmp_err
			endif
		endif
		
		waveRefs[1] = mass_tmp_err

	endif

	if(MSDataFlag)
		redimension /n=(numRunsInThisAcorn,numMzSticks) source  // 1.22A
	endif
	
	return waveRefs

End


//1.58E
//Function/wave sq_AdjustClosedTimeForErrCalcs(sampTimeClosedTemp, samptimeOpen)
//wave sampTimeClosedTemp, samptimeOpen
//
//	// There are two disjoint situations:  ePToF only runs and fast mode runs.  In both cases we want to change the time spent in closed to be >0 (only for the purposes of calcing errors)
//	//  at the end of this section we should have the waves samptimeClosed and samptimeOpen be filled with values we want
//	// For fast mode, the wave root:diagnostics:fm_TimeClosedForErrs can be created and modified by the user (however they see fit) before this section to generate errors is called.
//	// For ePToF data the wave root:diagnostics:ePToF_TimeClosedForErrs can be created and modified by the user (however they see fit).
//
//	wave/z ePToF_enabled= root:diagnostics:ePToF_enabled
//	wave/z FastKey = root:diagnostics:FastKey
//	wave/z ePToF_oversampling= root:diagnostics:ePToF_oversampling
//	
//	wave SampTimeClosed = samptimeClosedTemp
//
//	if (waveExists(FastKey) && waveMax(FastKey)>=1)			// we have to worry about fast mode timing not being correct for fast open runs (zero time spent in fast closed)
//		wave/z fm_TimeClosedForErrs = root:diagnostics:fm_TimeClosedForErrs // can be created in fm_SaveTimeAvg
//		if (!WaveExists(fm_TimeClosedForErrs) )		//1.58D
//			duplicate/o samptimeClosedTemp  root:diagnostics:fm_TimeClosedForErrs
//			wave fm_TimeClosedForErrs = root:diagnostics:fm_TimeClosedForErrs
//			fm_ChangeFastClosedTimeInFastO(fm_TimeClosedForErrs, FastKey)
//		elseif (WaveMin(fm_TimeClosedForErrs)<=1)
//			fm_ChangeFastClosedTimeInFastO(fm_TimeClosedForErrs, FastKey)
//		endif
//		wave SampTimeClosed=fm_TimeClosedForErrs			
//	endif
//	// deal with case where we have ePToF and want to average the closed.
//	if (waveExists(ePToF_enabled) && waveMax(ePToF_enabled)>=1 && waveMax(ePToF_oversampling)>=2 )	//1.57K		// we have to worry about fast mode timing not being correct for fast open runs (zero time spent in fast closed)
//		wave/z ePToF_TimeClosedForErrs= root:diagnostics:ePToF_TimeClosedForErrs
//		if ( !WaveExists(ePToF_TimeClosedForErrs))		//1.57D
//			duplicate/o samptimeClosed root:diagnostics:ePToF_TimeClosedForErrs/wave=ePToF_TimeClosedForErrs
//			ePToF_ChangeFastClosedTime(ePToF_TimeClosedForErrs, ePToF_enabled, samptimeOpen)
//		endif
//		wave samptimeClosed=ePToF_TimeClosedForErrs
//	endif		
//
//	return SampTimeClosed
//
//End


// Added in 1.60Q and modified significantly in 1.60T
Function sq_2dMSRawExport(ctrlName) : ButtonControl
	String ctrlName

	wave squirrel_index=root:index:squirrel_index
	svar todoStr=root:panel:sq_toDoWvNm
	wave todo=$"root:"+todoStr		
	
	nvar ams_time_saved=root:panel:ams_time_saved
	nvar ms0_ion2mass=root:panel:ms0_ion2mass
	nvar ms0_corr=root:panel:ms0_corr
	nvar ms0_dataType=root:panel:ms0_dataType		//ms0_source	1.46B

	nvar ms0_mz_saved_regrid=root:panel:ms0_mz_saved_regrid
	nvar ms0_mz_area=root:panel:ms0_mz_area
	nvar ms0_mz_refit=root:panel:ms0_mz_refit		// use new m/z params
	nvar ms0_mz_saved=root:panel:ms0_mz_saved
	nvar ms0_mz_custom=root:panel:ms0_mz_custom
	nvar ms0_mz_reg_min=root:panel:ms0_mz_reg_min
	nvar ms0_mz_reg_max=root:panel:ms0_mz_reg_max
	nvar ms0_mz_reg_dm=root:panel:ms0_mz_reg_dm
	svar ms0_mz_custom_str=root:panel:ms0_mz_custom_str

	nvar ms0_ck_findAvgBaseline=root:panel:ms0_ck_findAvgBaseline
	nvar ms0_AvGraphType=root:panel:ms0_AvGraphType
	nvar ms0_appendErrorBars=root:panel:ms0_appendErrorBars
		
	wave/z allMS = root:allMS
	
	variable RawSpectraOnMzAxisFlag, firstrun, mz_int,mz_sl,mz_exp, npnts, idex, numIter, DAQMaxMz, mz_points, numRuns,RawPts,runPos,ResultMaxmz
	string tempStr, dataStr, prefixstr, msg, FileNameAndPath, dumpedMxName,destMxName, dumpedErrMxName,destErrMxName // , TimeStr
		
	if (ms0_mz_refit) 
		wave mzpower = root:mzFitting:mz_FitPower
		wave mzslope = root:mzFitting:mz_FitSlope
		wave mzint = root:mzFitting:mz_FitIntercept
	else
		wave mzpower = root:diagnostics:mzPower
		wave mzslope = root:diagnostics:mzSlope
		wave mzint = root:diagnostics:mzInt
	endif

	svar MSRawList = root:panel:MSRawList
	svar MSTimeAvgList = root:panel:MSTimeAvgList
	svar MSPrefixList = root:panel:MSPrefixList
	variable DiffFlag=0
	dataStr = stringFromList(ms0_dataType-1 ,MSRawList)	//	 ms0_dataType is one higher because of the popmenu

	if (ms0_ion2mass)
		DoAlert 1,  "The units of ug/m3 are not intuitive for raw spectra. Select Yes to uncheck and continue"//1.59Z2  fix		
		if (V_flag)
			ms0_ion2mass=0
		else
			abort
		endif
	endif

	if (strsearch(dataStr, "Diff", 0)>=0)		// require that all Diff generated matricies
		DiffFlag=1
		if (FindDimLabel(squirrel_index, 2, "MSOpen_p")<0)
			abort "Please preprocess the raw spectra for this todo wave to allow this to work correctly. If this is fast mode data, make sure this data type is valid. Aborting from sq_MSExport_avgRaw2D"//1.59Z2  fix		
		endif
		dataStr = "MSOpen_p"  // MSClosed_p;"		//1.60Q
	endif				
	
	RawPts = sq_findMaxMinSQIndex(todo, dataStr, 4, 1)		//1.60K
	if (numtype(RawPts)!=0)
		abort "Either the data set "+dataStr+" doesn't exist or wasn't preprocessed for the runs in todo "+NameofWave(todo)+".  Aborting from sq_MSExport_avgRaw2D"
	endif
	runPos = sq_findRun(root:index:rn_series,todo[0])
	make/free/n=(RawPts) mzBaseDAQ = ((p-mzint[runPos])/mzslope[runPos])^(1/(mzpower[runPos]))
		
	if (!ams_time_saved) //
		Abort "Please select 'As saved for the time grid.  It is at the bottom of the main squirrel panel."
	endif

	if (ms0_corr) //
		sq_ABSanity()
	endif

	if (WaveExists(allMS))
		sq_create_todo(todoStr+" and allMS","temptodo")
		wave todo = $"temptodo"
		if (numpnts(todo)==0)
			abort "There are no MS waves in this todo.  Aborting from sq_MSExport_avgRaw2D"
		endif
	endif	

	// all systems go, we are gonna do some calcs
	setdatafolder root:	
	if (ms0_mz_custom) //& Custom m/z base wave
		wave/z mzbase=$"root:"+ms0_mz_custom_str
		if (!waveexists(mzbase))
			abort ("The specified m/z base wave "+ms0_mz_custom_str+" cannot be found")
		endif
	else
		wave mzbase=$(sq_FindRawMzBase(str2num(tempStr), todo, ms0_mz_refit, ms0_mz_saved, mz_reg_dm=ms0_mz_reg_dm, mz_reg_min=ms0_mz_reg_min, mz_reg_max=ms0_mz_reg_max))
	endif

	ResultMaxmz =Round(WaveMax(mzbase)) 
	if (ResultMaxmz > Round(WaveMax(mzBaseDAQ)))
		abort "The maximum m/z value you are requesting is larger than the maximim m/z for which data was taken for this todo.  Reduce the maximum m/z.  Aborting from sq_MSExport_avgRaw2D"
	endif	
					
	msg = "Select the folder and the file name that will contain the 2D raw spectra"
	FileNameAndPath = gen_DoSaveFileDialog(msg, "*.h5")

	if (strlen(FileNameAndPath)==0 )
		return 0 // user cancelled
	elseif (!stringmatch("h5", ParseFilePath(4, FileNameAndPath, ":", 0, 0)))
		Abort "Please try again with a file name that has an .h5 extension."
	endif

	if (DiffFlag)
		dumpedMxName= "MSDiff_temp" 
		destMxName = "MSDiff_2D" 
		if (ms0_appendErrorBars)
			dumpedErrMxName= "MSDiffErr_temp" 
			destErrMxName = "MSDiffErr_2D" 
		endif
	else
		dumpedMxName=stringFromList(0, dataStr, "_") + "_temp" 
		destMxName = stringFromList(0, dataStr, "_") + "_2D" 
		if (ms0_appendErrorBars)
			dumpedErrMxName= stringFromList(0, dataStr, "_") + "Err_temp" 
			destErrMxName = stringFromList(0, dataStr, "_") + "Err_2D" 
		endif
	endif
	
	if (ms0_appendErrorBars)
		sq_MS_doExport_avgRaw2D(todo, mzbase, FileNameAndPath, dataStr, dumpedMxName,destMxName, ms0_mz_area, mzpower, mzslope, mzint, ms0_corr, ms0_ion2mass, ms0_ck_findAvgBaseline, dumpedErrMxStr = dumpedErrMxName, destErrMxStr =destErrMxName ) //, MaxRawPts, ams_time_saved, TimeW) 
	else
		sq_MS_doExport_avgRaw2D(todo, mzbase, FileNameAndPath, dataStr, dumpedMxName,destMxName, ms0_mz_area, mzpower, mzslope, mzint, ms0_corr, ms0_ion2mass, ms0_ck_findAvgBaseline) //, MaxRawPts, ams_time_saved, TimeW) 
	endif

End


Function sq_MS_doExport_avgRaw2D(maintodo,mzbase, FileNameAndPath, dataSetStr, dumpedMxName,destMxName, mz_w_op, mzpower, mzslope, mzint, ms0_corr, ug_op, FindBL, [dumpedErrMxStr, destErrMxStr]) //, MaxRawPts, ams_time_saved, TimeW)
wave maintodo,mzbase, mzpower, mzslope, mzint
string FileNameAndPath, dataSetStr, dumpedMxName,destMxName, dumpedErrMxStr, destErrMxStr
variable mz_w_op,ms0_corr, ug_op, FindBL

	nvar ams_sqprog=root:panel:ams_sqprog
	svar sq_str_status=root:panel:sq_str_status
	svar sq_toDoWvNm=root:panel:sq_toDoWvNm
	wave t_series = root:index:t_series
	wave run_index = root:index:run_index
				
	variable rowGroupNum, numRuns, numIter, fileID, idex, mz_points, kdex , ErrFlag = !ParamIsDefault(dumpedErrMxStr) && !ParamIsDefault(destErrMxStr), DiffFlag = stringMatch("D", destMxName[2])
	string mainTodoStr, r1, r2, paramStr
	
	rowGroupNum=50 // the number of runs we use in each iteration.  This can be changed for Long ToFs or runs with very small mz range.
	mainTodoStr=NameofWave(maintodo)
	numRuns = numpnts(maintodo)
	numIter=ceil(numRuns/rowGroupNum)
				
	mz_points = numpnts(mzbase)
	
	// wave for which we will map the native m/z axis onto all runs
	make/o/n=(mz_points+1) AvgSqrt_mzbase		//^ this is a weighting wave, is constant for all runs
	// sort of defines left and right 'edges' of their ns space; uses the average of the square root of the mz_base values
	// is used i n the mapping of one raw spectra to another of different m/z base, regardless as to whether athe "conserve area' checkbox is checked.
	AvgSqrt_mzbase[1,mz_points-1]=0.5*(sqrt(mzbase[p])+sqrt(mzbase[p-1]))
	AvgSqrt_mzbase[0]=(1.5*sqrt(mzbase[0]))-(0.5*sqrt(mzbase[1]))
	AvgSqrt_mzbase[mz_points]=AvgSqrt_mzbase[mz_points-1]+(AvgSqrt_mzbase[mz_points-1]-AvgSqrt_mzbase[mz_points-2])		/// 1.59E  not sure if this is completely correct.  But it is only one point

	if (ms0_corr)
		wave corr_fact=	root:diagnostics:corr_fact
	endif
	if (ug_op)
		wave/z ioneff_wave=root:diagnostics:ioneff		//1.34
		if(waveexists(ioneff_wave)==0)
			wave ioneff_wave=root:diagnostics:ioneff_logged	//1.34//^ use the wave from the diagnostics, not the one calculated through the airbeam correction section 
		endif
		wave/z flowrate=root:diagnostics:flowrate
		if(waveexists(flowrate)==0)
			wave flowrate=root:diagnostics:flowrate_logged	//1.38C
		endif	
		make/free/o/n=(numpnts(flowrate)) ugfac_wav= (62/AVOGADRO_NUM)*1e12/(ioneff_wave[p]*flowrate[p])  // defaults to 62 for atomic weight of calibr
	endif

	make/free/o/n=(numpnts(t_series) ) mask
	sq_makeMaskFromTodofindRun(mask,maintodo)	
	duplicate/o mask,maskRow
	maskRow *=p
	WaveTransform zapNans maskRow
	
	killwaves/z $dumpedMxName
	wave/z oldwave = $dumpedMxName
	if (WaveExists(oldwave))
//		Abort "The wave "+dumpedMxName+" must be killed before proceeding.  Aborting from sq_MSExport_avgRaw2D"
		dumpedMxName =UniqueName(dumpedMxName, 1, 0)
	endif
	killwaves/z $destMxName
	wave/z oldwave = $destMxName
	if (WaveExists(oldwave))
//		Abort "The wave "+destMxName+" must be killed before proceeding.  Aborting from sq_MSExport_avgRaw2D"
		destMxName =UniqueName(destMxName, 1, 0)
	endif

	print "// Export of the 2D raw spectra matrix named "+destMxName+" using todo "+sq_toDoWvNm+" into "+FileNameAndPath+" began at "+Date()+" "+time()

	HDF5CreateFile/O fileID  as FileNameAndPath
	HDF5CloseFile fileID
	
	for(idex=0;idex< numIter;idex+=1)
		sq_sqprog_upd(100*idex/numIter)
		// make the smaller subset of the main todo
		r1 = num2istr(maintodo[idex*rowGroupNum])
		r2 = num2istr(maintodo[min(rowGroupNum*(idex+1)-1,numRuns-1)])
		sq_create_todo(r1+"-"+r2+" and "+mainTodoStr,"partialTodo")
		wave partialTodo =$"partialTodo"
		make/free/o/n=(numpnts(partialTodo)) row_series= maskRow[idex*rowGroupNum+p] 
		make/free/o/n=(1, numpnts(partialTodo)) factorwave
				
		// call the dump spectra function
		squirrel_fetch(partialTodo,sq_dump2memory,"dest:"+dumpedMxName,dataSetStr)   // "max_cl:"+num2istr(MaxRawPts)+";
		wave w = $dumpedMxName
//		sq_Hz(index_pos,w)

		if (ErrFlag)
			Duplicate/o W $dumpedErrMxStr/wave= wErr
		else
			wave wErr=$""	// null wave
		endif

		if (FindBL)
			make/u/free/o/n=(numpnts(partialTodo)) index_pos
			for(kdex=0;kdex<numpnts(partialTodo); kdex+=1)
				FindValue/U=(partialTodo[kdex])/Z run_index
				index_pos[kdex] = V_Value
			endfor
			duplicate/free/o w,bk_tmp
			bk_CreateBackgroundValues(index_pos, bk_tmp,  mzSlope, mzInt, mzPower, dumpedMxName)
			matrixop/o w = w-bk_tmp
		endif

		if (DiffFlag)
			squirrel_fetch(partialTodo,sq_dump2memory,"dest:MSClosed_temp" ,"MSClosed_p")   // "max_cl:"+num2istr(MaxRawPts)+";
			wave wC = $"MSClosed_temp" 
			if (ErrFlag)
				matrixop/o wErr = sqrt(powR(wC,2)+powR(wErr,2))
			endif
			if (FindBL)
				duplicate/free/o wC,bk_tmp
				bk_CreateBackgroundValues(index_pos, bk_tmp,  mzSlope, mzInt, mzPower, dumpedMxName)
				matrixop/o wC = wC-bk_tmp
			endif
			MatrixOP/o w = w-wC
			killwaves wC		// I want an error thrown if this wave can't be killed
		else
			if (ErrFlag)
				matrixop/o wErr = sqrt(W)
			endif
		endif
		
		if (ms0_corr)
			w*=corr_fact[row_series[p]]
			if (ErrFlag)
				wErr *=corr_fact[row_series[p]]
			endif
		endif	

	// do the math for the m/z mapping
		sq_MzMapping(row_series,mzbase, w, AvgSqrt_mzbase, mz_w_op, mzpower, mzslope, mzint, destMxName, wErr=wErr, destErrMxStr=destErrMxStr)
		// FillWaveThreadAtATime(row_series, mzbase, wav,  AvgSqrt_mzbase, mz_w_op, mzpower, mzslope, mzint, destMxName)			
		wave d = $destMxName 

		// save to hdf 
		HDF5OpenFile fileID  as FileNameAndPath
		HDF5SaveData/o/IGOR=0/GZIP={9,0}/APND=0/Z/LAYO={2,dimsize(d,0),mz_points}/MAXD={numRuns,mz_points} d, fileId
		if (ErrFlag)
			wave dErr = $destErrMxStr
			HDF5SaveData/o/IGOR=0/GZIP={9,0}/APND=0/Z/LAYO={2,dimsize(d,0),mz_points}/MAXD={numRuns,mz_points} dErr, fileId
		endif
		HDF5CloseFile fileID
		killwaves w, d dErr wErr		// I want an error thrown if this wave can't be killed
		
	endfor
	
	// prep t_series wave to write	
	duplicate/o t_series, root:hoard:timeWave	/wave=timewave		// don't want to make free beacuse they will be named free in the hdf
	timeWave/=mask
	WaveTransform zapNans timeWave
	duplicate/o maintodo, root:hoard:runNumbers	/wave=runNumbers			// don't want to make free beacuse they will be named free in the hdf

	// write todo and t_series
	HDF5OpenFile fileID  as FileNameAndPath
	duplicate/o mzbase root:hoard:MzAxis/wave=MzAxis	
	HDF5SaveData/o/IGOR=0 MzAxis, fileId  // /GZIP={9,0}
	HDF5SaveData/o/IGOR=0 runNumbers, fileId  // /GZIP={9,0}
	HDF5SaveData/o/IGOR=0 timeWave, fileId  //GZIP={9,0}
	HDF5CloseFile fileID

	sq_sqprog_upd(100)
	sq_str_status = "Done"

	print "// Export of the 2D raw spectra matrix named "+destMxName+" using todo "+sq_toDoWvNm+" into "+FileNameAndPath+" finished at "+Date()+" "+time()

	Killwaves/z partialTodo, timeWave, runNumbers, MzAxis,w, temptodo, d, dest_n, dtdmz_tmp, mz_tmp, tempTodo, maskRow, AvgSqrt_mzbase, MZ_Base_tempTodo, dest_n,mz_map, Mzbase_2D

End


Function sq_MzMapping(row_series,mzbase, wav, mz2_base_tmp, mz_w_op, mzpower, mzslope, mzint, destStr, [wErr, destErrMxStr])
wave row_series,mzbase, wav, mz2_base_tmp, mzpower, mzslope, mzint, wErr
variable mz_w_op
string destStr, destErrMxStr

	make/free /o/n=(1,1,1) ms_conc_tmp
	make/free/o/n=(2,3) mpos

	variable rdex, idex, p1, k, numRuns= dimsize(wav,0), numMz = dimsize(wav,1), numNewMz = numpnts(mzbase), errFlag = Strlen(destErrMxStr)>0
	
	make/o/n=(numRuns,numNewMz) $destStr=0, dest_n=0
	wave dest=$destStr
	if(errFlag)
		make/o/n=(numRuns,numNewMz) $destErrMxStr=0
		wave Err = $destErrMxStr
	endif
	
	for(rdex=0;rdex<(numpnts(row_series));rdex+=1)
		p1 =row_series[rdex]
		if (mz_w_op) //& Conserve peak area, if desired
			make /o/n=(numMz) dtdmz_tmp=mzpower[p1]*(mzslope[p1]^(1/mzpower[p1]))*((p-mzint[p1])^(1-(1/mzpower[p1])))  //^ the first derivative of the
			wav[rdex][]*=dtdmz_tmp[q]  // in as saved m/z units
		endif
		
		make /o/n=(numMz+1) mz_tmp=(((p-0.5)-mzint[p1])/mzslope[p1])^(0.5/mzpower[p1])		//^ the square root of the m/z wave for this run
		sq_make_map(mz_tmp,mz2_base_tmp,"mz_map")  // both mz_tmp and mz2_base_tmp are in sqrt (mz) 'units'
		wave mz_map
		
		mpos[0][]=rdex
		for (k=0;k<numMz;k+=1)   // also used for raw spectra  1.60Q
			sq_map2dist(0,k,0,ms_conc_tmp,mpos,map2=mz_map) 
			dest[rdex][mpos[0][1],mpos[1][1]]+=wav[p][k]*ms_conc_tmp[p-mpos[0][0]][q-mpos[0][1]][0]
			dest_n[rdex][mpos[0][1],mpos[1][1]]+= ms_conc_tmp[p-mpos[0][0]][q-mpos[0][1]][0]
			if(errFlag)
				Err[rdex][mpos[0][1],mpos[1][1]]+=wErr[p][k]*((ms_conc_tmp[p-mpos[0][0]][0][0])^2)
			endif
		endfor
	endfor

	matrixop/o dest=dest/dest_n

	if (errFlag)
//		matrixop /o Err=powr(Err,0.5)/dest_n
		matrixop /o Err=Err/dest_n
	endif
End

// Not used in 1.60V.  First attempt at trying to make the remapping onto different mz axes faster
//ThreadSafe Function sq_MzMappingRow(wav, dest, dest_n,mz2_base_tmp, row, numMz, mz_w_op, mzpowerVal, mzslopeVal, mzintVal)
//wave wav, dest, dest_n, mz2_base_tmp
//variable row,numMz, mz_w_op, mzpowerVal, mzslopeVal, mzintVal
//
//	make/free /o/n=(1,1,1) ms_conc_tmp
//	make/free/o/n=(2,3) mpos
//
//	variable k
//	
//	if (mz_w_op) //& Conserve peak area, if desired
//		make /o/n=(numMz) dtdmz_tmp=mzpowerVal*(mzslopeVal^(1/mzpowerVal))*((p-mzintVal)^(1-(1/mzpowerVal)))  //^ the first derivative of the
//		wav[row][]*=dtdmz_tmp[q]  // in as saved m/z units
//	endif
//	
//	make /o/n=(numMz+1) mz_tmp=(((p-0.5)-mzintVal)/mzslopeVal)^(0.5/mzpowerVal)		//^ the square root of the m/z wave for this run
//	TS_sq_make_map(mz_tmp,mz2_base_tmp,"mz_map")  // both mz_tmp and mz2_base_tmp are in sqrt (mz) 'units'
//	wave mz_map
//
//	mpos[0][]=row
//	for (k=0;k<numMz;k+=1)   // also used for raw spectra  1.60Q
//		TS_sq_map2dist(0,k,0,ms_conc_tmp,mpos,map2=mz_map) 
//		dest[row][mpos[0][1],mpos[1][1]]+=wav[p][k]*ms_conc_tmp[p-mpos[0][0]][q-mpos[0][1]][0]
//		dest_n[row][mpos[0][1],mpos[1][1]]+= ms_conc_tmp[p-mpos[0][0]][q-mpos[0][1]][0]
//	endfor
//
//End
//
//
//Threadsafe Function TS_MzMappingChunk(row_series,mzbase, wav, mz2_base_tmp, mz_w_op, mzpower, mzslope, mzint, destStr)
//wave row_series,mzbase, wav, mz2_base_tmp, mzpower, mzslope, mzint
//variable mz_w_op
//string destStr
//
//	variable rdex, idex, p1, k, numRuns= dimsize(wav,0), numMz = dimsize(wav,1), numNewMz = numpnts(mzbase)
//	
//	make/o/n=(numRuns,numNewMz) $destStr=0, dest_n
//	wave dest=$destStr
//	
//	for(rdex=0;rdex<(numpnts(row_series));rdex+=1)
//		p1 =row_series[rdex]
//		if (mz_w_op) //& Conserve peak area, if desired
//			make /o/n=(numMz) dtdmz_tmp=mzpower[p1]*(mzslope[p1]^(1/mzpower[p1]))*((p-mzint[p1])^(1-(1/mzpower[p1])))  //^ the first derivative of the
//			wav[rdex][]*=dtdmz_tmp[q]  // in as saved m/z units
//		endif
//		
//		make /o/n=(numMz+1) mz_tmp=(((p-0.5)-mzint[p1])/mzslope[p1])^(0.5/mzpower[p1])		//^ the square root of the m/z wave for this run
//		TS_sq_make_map(mz_tmp,mz2_base_tmp,"mz_map")  // both mz_tmp and mz2_base_tmp are in sqrt (mz) 'units'
//		wave mz_map
//
////		mpos[0][]=rdex
//		for (k=0;k<numMz;k+=1)   // also used for raw spectra  1.60Q
//			TS_sq_map2dist(0,k,0,ms_conc_tmp,mpos,map2=mz_map) 
////			dest[rdex][mpos[0][1],mpos[1][1]]+=wav[p][k]*ms_conc_tmp[p-mpos[0][0]][q-mpos[0][1]][0]
////			dest_n[rdex][mpos[0][1],mpos[1][1]]+= ms_conc_tmp[p-mpos[0][0]][q-mpos[0][1]][0]
//		endfor
//	endfor
//
//	Matrixop	dest = dest/dest_n
//
//End

// based off of the WM example MTFillWaveThreadAtATime
		// FillWaveThreadAtATime(row_series, mzbase, wav,  AvgSqrt_mzbase, mz_w_op, mzpower, mzslope, mzint, destMxName)			
//Function FillWaveThreadAtATime(row_series, mzbase, wav,  AvgSqrt_mzbase, mz_w_op, mzpower, mzslope, mzint, destStr)	
//	WAVE row_series, mzbase, wav,  AvgSqrt_mzbase, mzpower, mzslope, mzint
//	variable mz_w_op
//	string destStr
//	
//	Variable numRuns= dimsize(wav,0)
//	Variable numMz = dimsize(wav,1)
//	variable numNewMz = numpnts(mzbase)
//	Variable row,nthreads= ThreadProcessorCount
//	Variable threadGroupID= ThreadGroupCreate(nthreads)
//	Variable dummy
//	
//	variable p1
//	make/o/n=(numRuns,numNewMz) $destStr=0
//	make/o/n=(numRuns,numNewMz)/free dest_n=0
//	wave dest=$destStr
//
//	for(row=0; row<numRuns; row+=1)
//		// Get index of a free thread - Requires Igor Pro 6.23 or later
//		Variable threadIndex = ThreadGroupWait(threadGroupID,-2) - 1
//		if (threadIndex < 0)
//			dummy = ThreadGroupWait(threadGroupID, 50)	// Let threads run a while
//			row -= 1				// Try again for the same column
//			continue				// No free threads yet
//		endif
//		p1 =row_series[row]
//		ThreadStart threadGroupID, threadIndex, TS_MzMappingRow(wav, dest, dest_n, AvgSqrt_mzbase, row, numMz, mz_w_op, mzpower[p1], mzslope[p1], mzint[p1])
//	endfor
//
//	// Wait for all threads to finish
//	do
//		Variable threadGroupStatus = ThreadGroupWait(threadGroupID,100)
//	while(threadGroupStatus != 0)
//
//	dummy = ThreadGroupRelease(threadGroupID)
//
//End


// modified in 1.59E
//& Make a mapping wave, used to assign data from one axis to another.
//ThreadSafe Function TS_sq_make_map(source,dest,mapstr)
//wave source,dest //Souce and destination axes
//string mapstr
//
//	variable n,m,n_map,p1,p2,i,fp1,fp2
//	variable rangeflag,lo_range=dest[0],hi_range=dest[numpnts(dest)-1]  //JDA 260417
//	
//	n=dimsize(source,0)
//	m=dimsize(source,1)*0.5                              // a flag for a 2D input.  For now used only when making a time series map
//	n_map=n+m-1
//	make /o/n=(n_map,4) $mapstr
//	wave map=$mapstr
//	
//	if (!m)
//		p2=binarysearchinterp(dest,source[0])
//		fp2=floor(p2)
//	endif
//	
//	for (i=0;i<n_map;i+=1)
//	
//		if (m)
//			p1=binarysearchinterp(dest,source[i][0])
//			fp1=floor(p1)
//			p2=binarysearchinterp(dest,source[i][1])
//			rangeflag=(source[i][1]>lo_range) && (source[i][0]<hi_range)  //JDA 260417
//		else
//			p1=p2
//			fp1=floor(p2) //JDA 120417
//			p2=binarysearchinterp(dest,source[i+1])
//			rangeflag=(source[i+1]>lo_range) && (source[i]<hi_range)  //JDA 260417
//		endif
//			fp2=ceil(p2)-1 //JDA 120417
//		//                           if (numtype(p1) && numtype (p2)) // No overlap
//		if (!rangeflag) // No overlap //JDA 260417
//			map[i][]=0
//		else
//			if (numtype(p1)) // Clips beginning
//				p1=0
//				fp1=0
//			elseif (numtype(p2)) // Clips end
//				p2=numpnts(dest)-1
//				fp2=p2-1
//			endif
//			if (fp1==fp2) // Data to be allocated to a single point
//				map[i][0]=fp1
//				map[i][1]=p2-p1
//				map[i][2]=fp1
//				map[i][3]=0
//			else // Data to be allocated to multiple points
//				map[i][0]=fp1
//				map[i][1]=fp1+1-p1
//				map[i][2]=fp2
//				map[i][3]=p2-fp2
//			endif
//		endif
//	endfor
// 
//End



//& Make a footprint matrix, based on mapping waves in up to 3 dimensions.
//& No performance benifit over the previous version of the code, but it makes it so much tider.
// mpos and distwav are filled in
// mpos contains index values, i.e. row or p values is always a 2D  2x3 wave....each column is run, PToF, Mz , first row is start, second row is end
// distwav contains the weighting values, is initialized as a 3D 1x1x1 wave gets possibly redimensioned here
// distwav gets redimensioned if averagingis mapped onto more than one spot
// mapX is a 2D wave alwyas with 4 columns... columns 0 and 2 are point values, 1 and 3 are weighting values
//Threadsafe Function TS_sq_map2dist(i1,i2,i3,distwav,mpos,[map1,map2,map3])
//	wave distwav,mpos,map1,map2,map3
//	variable i1,i2,i3			// index 1,2,3 are used only for map1, map2, map3
//	
////	variable mw1_1=1,mw2_1=0,mw1_2=1,mw2_2=0,mw1_3=1,mw2_3=0,md_1=0,md_2=0,md_3=0
//// reorganizing to decipher... the second number, mwx_y, i.e. y is the map type: map1, map2, map3
//
//	variable mw1_1=1,mw1_2=1,mw1_3=1
//	variable mw2_1=0,mw2_2=0,mw2_3=0	
//	variable md_1=0,  md_2=0,   md_3=0
//
////& Initialise variables dimension at a time and define locations to map to
//// initialize mw1_y, mw2_y, mpos[0][0], mpos[1][0], md_y
//	if (waveexists(map1))  /// mpos uses COLUMN 0
//		mw1_1=map1[i1][1]
//		mw2_1=map1[i1][3]
//		mpos[0][0]=map1[i1][0]
//		mpos[1][0]=map1[i1][2]
//		md_1=mpos[1][0]-mpos[0][0]
//		if (md_1>=dimsize(distwav,0))
//			insertpoints /m=0 dimsize(distwav,0),(1+md_1-dimsize(distwav,0)), distwav
//		endif
//	endif
//	if (waveexists(map2))   /// mpos uses COLUMN 1
//		mw1_2=map2[i2][1]
//		mw2_2=map2[i2][3]
//		mpos[0][1]=map2[i2][0]
//		mpos[1][1]=map2[i2][2]
//		md_2=mpos[1][1]-mpos[0][1]
//		if (md_2>=dimsize(distwav,1))
//			insertpoints /m=1 dimsize(distwav,1),(1+md_2-dimsize(distwav,1)), distwav
//		endif
//	endif
//	if (waveexists(map3))   /// mpos uses COLUMN 2
//		mw1_3=map3[i3][1]
//		mw2_3=map3[i3][3]
//		mpos[0][2]=map3[i3][0]
//		mpos[1][2]=map3[i3][2]
//		md_3=mpos[1][2]-mpos[0][2]
//		if (md_3>=dimsize(distwav,2))
//			insertpoints /m=2 dimsize(distwav,2),(1+md_3-dimsize(distwav,2)), distwav
//		endif
//	endif
//
////& Initialise to 0
//	fastop distwav=0
////& Assign values dimension at a time
//
//	if (md_1>0)
//		distwav[1,md_1][0][0]=p<md_1?(1):(mw2_1)
//	endif
//	distwav[0][0][0]=mw1_1
//
//	if (md_2>0)
//		distwav[0,md_1][1,md_2][0]=(q<md_2?(1):(mw2_2))*distwav[p][0][0]		 // JDA Fix 14/04/11 mw2_2 was incorrectly mw2_1  // 1.51C
//	endif
//	distwav[0,md_1][0][0]*=mw1_2
//	
//	if (md_3>0)
//		distwav[0,md_1][0,md_2][1,md_3]=(r<md_3?(1):(mw2_3))*distwav[p][q][0]
//	endif
//	distwav[0,md_1][0,md_2][0]*=mw1_3
//
//End


