#pragma TextEncoding = "Windows-1252"
#pragma rtGlobals=3		// Use modern global access method.
//#pragma version=1.17d// decided to not use pragma but a string constant 

//HR_ms_conc Functions for High ToF-AMS analysis Toolkit
// Peak Integration by Key Analysis (PIKA)
//Copyright (C) 2005, 2006, 2007,2008, 2009,2010,2011, 2012, 2013, 2014, 2015, 2016, 2017
// Pete DeCarlo, Allison Aiken, Jose Jimenez, Donna Sueper, University of Colorado
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
//
///////////////////////////////////////////////////////////////////////////////
// CURRENT STATUS
// see HR read me file for details.


// ************************** PK_MS_concentration **************************
// Master function (stolen and modified from its squirrel origin) of stick data to end products.
// Different products identified by their product type:
// 0 = Speciated mass concentration (average)
// 1 = Speciated mass concentration vs time
// 2 = Speciated mass spectrum vs m/z (an averaged mass spectrum)
// 3 = Speciated mass spectrum vs time and m/z (what Qi's analysis uses)
//
//  Function outline is as follows:
//  Step 1. Identify & set global and local variables
// identify data waves to be worked on
// identify global waves 
// parse important flags, entities from paramsStr
// clean up, reset variables extracted from paramStr
// declare local waves possibly created in earlier acorns
// declare locall variables used in calculations
//
// Step 2. Create waves needed for calculations :
// destination waves,  weighting waves, error waves conversion waves
// if (acorn=0)
// 	for (each species) make one destination, weighting, error, conversion wave	
//
// Step 3. Do some preliminary data maniuplations
// generating the squared error matrix 
// prep the 'ugfac' data according to the operation and options 
// if (acorn==0)
// 	ugfac_wav_tmp for all product types has all the unit conversion factors except RIE, CE
//
// Step 4. Generate data product
// data product types are very similar to squirrel
// for (each species)
//	if (stick data)
// 		identify RIE, CE
// 	distribute the data to the destination wave accordingly.
//	for (each run in current acorn ...index_pos)
//		if (we have real data to work with)
//			p1 identifies point number for this run		
// 			find ugfac conversion for this run
//			find t_step for possible weighting
// 			switch (product type) .... assign result according to product type
// 			for each case consider possible combinations of d_base_flag, t_base_flag,  mz_base_flag
//			case 0: Speciated mass concentration (average, one point for entire time)
//			case 1: Speciated mass concentration vs time
//			case 2: Speciated mass spectrum vs m/z (an averaged mass spectrum)
//			case 3: Speciated mass spectrum vs time and m/z (get at the raw HR sticks)
//
// 			case 11: Elemental Analysis time series		// case numbers need to be bitwise "&2" compatible
//			case 14: Elemental Analysis average mass spec				
//  For average mass spectrum, we do NOT deal with summing to unit mass resolution within this function!

// Step 5. Finalize destination wave if at end of acorn
// Step 6. Delete temporary waves

CONSTANT ROUNDNUMHRIONCOLS = 200		// 1.14M

Function PK_MS_concentration(index_pos,paramstr,acorn,n_acorns)
	wave index_pos 	//& 1D wave of positions in 'run_index' that are being operated on
	string paramstr 	//& Keyword-identified parameter string
	variable acorn 	//& Number of the 'acorn' being passed
	variable n_acorns 	//& The total number of 'acorns' that will be passed
	
//  Step 1. Identify & set global and local variables
	// identify data waves to be worked on
	wave source=$stringbykey("acorn_0",paramstr)		//^ can be stick or raw spectra	
	wave /z source_err=$stringbykey("acorn_1",paramstr)			// added in 1.06H
	variable error_flag=waveexists(source_err)

	// identify global waves 
	wave series_index=root:index:series_index
	wave t_series=root:index:t_series
	wave imt_series=root:index:imt_series
	
	wave/wave TimeWaves=sq_GetModifiedTimes()
	wave TimeMSOpen = TimeWaves[0]
	wave TimeMSClosed = TimeWaves[1]
	wave TimePToF = TimeWaves[2]

	wave /z FastModeAvgTimeSec=root:diagnostics:FastModeAvgTimeSec // 1.07C
	wave /z corr_fact=root:diagnostics:corr_fact
	wave/z ioneff_wave=root:diagnostics:ioneff		//1.34
	if(waveexists(ioneff_wave)==0)
		wave ioneff_wave=root:diagnostics:ioneff_logged	//1.34//^ use the wave from the diagnostics, not the one calculated through the airbeam correction section 
	endif
	wave/z flowrate=root:diagnostics:flowrate
	if(waveexists(flowrate)==0)
		wave flowrate=root:diagnostics:flowrate_logged	//1.38C
	endif	
		
	// velocity calibration waves		//added in 1.20T
	wave/z vl=root:diagnostics:ptof_vl	
	wave/z vg=root:diagnostics:ptof_vg
	wave/z b=root:diagnostics:ptof_b	
	wave/z dx=root:diagnostics:ptof_dx
	if (!WaveExists(vl))
		wave vl=root:diagnostics:ptof_vl_logged	//^ 1.35
	endif
	if (!WaveExists(vg))
		wave vg=root:diagnostics:ptof_vg_logged	//^ 1.35
	endif
	if (!WaveExists(b))
		wave b=root:diagnostics:ptof_b_logged	//^ 1.35
	endif
	if (!WaveExists(dx))
		wave dx=root:diagnostics:ptof_dx_logged	//^ 1.35
	endif
	wave ptof_step=root:diagnostics:ptof_step
	wave ptof_t0=root:diagnostics:ptof_t0
	wave ChopperFreq = root:diagnostics:ChopperFreq		
	nvar chopper_duty=root:diagnostics:chopper_duty
	
	//  important product type variable
	variable product_type=numberbykey("product_type",paramstr) // The type of data being produced

	variable dimSource0=dimsize(source,0), dimSource1=dimsize(source,1), dimSource2=dimsize(source,2) 		
	// alternate time base
	wave/z t_base=$stringbykey("t_base",paramstr) // Time base for destination data (null if t_series used)
	wave/z d_base=$stringbykey("d_base",paramstr) // Diameter base for destination data (null if dsig used)
	variable dlog_op=numberbykey("dlog_op",paramstr)==1 // Whether to convert to dMdlogDp or not

	variable t_base_flag=waveexists(t_base) && (product_type&1)
	variable time_points=t_base_flag?(numpnts(t_base)-1):(numpnts(t_series))
	variable d_base_flag=waveexists(d_base) && (product_type&4) && dlog_op
	variable d_points=d_base_flag?(numpnts(d_base)-1):(dimSource1)		// diameter base
//  Note that the d_base_flag means something slightly different from the UMR case
//	wave/z PToFBinEdges=$stringbykey("PToFBinEdges",paramstr) // Diameter base for destination data (null if dsig used)
//	wave/z PToFTimeEdges=$stringbykey("PToFTimeEdges",paramstr) // Diameter base for destination data (null if dsig used)
	wave/z t_PToFCenters=$stringbykey("t_PToFCenters",paramstr) // Diameter base for destination data (null if dsig used)

	//1.19E  added for HR PToF... don't think I need the mz base but maybe??  for ePToF eventually?
	wave/z mz_base=$stringbykey("mz_base",paramstr) // m/z base for destination data (raw mass spec only)
	variable raw_flag=numberbykey("raw_flag",paramstr)==1 // Data being handled are raw (not stick) MS
	variable mz_base_flag=waveexists(mz_base) && raw_flag && (product_type&2)
	variable numMzSticks=dimsize(source, product_type<4?(1):(2)) // Assume fixed //  for MS OR PToF number of mz
	variable mz_points=mz_base_flag?(numpnts(mz_base)):(numMzSticks)	// raw spectra averaging

	// declare local waves possibly created in earlier fetch calls ...(acorns!=0)
	wave /z ms_conc_tmp
	wave /z mpos
	if ((!waveexists(ms_conc_tmp) || !waveexists(mpos)) && (!(product_type&1) || t_base_flag)  )
		make /o/n=(1,1,1) ms_conc_tmp
		make /o/n=(2,3) mpos
	endif

	variable hz_op=numberbykey("hz_op",paramstr)==1 // Convert raw data to Hz (eliminates need to preprocess)

	// parse important flags, entities from paramsStr	
	string speclist=stringbykey("speclist",paramstr) // Slash-seperated list of keywords
	string fraglist=stringbykey("fraglist",paramstr) // Slash-seperated list of destination waves
	string destlist=stringbykey("destlist",paramstr) // Slash-seperated list of destination waves
	string rie_list=stringbykey("rie_list",paramstr) // Slash-seperated list of RIEs
	string ce_list=stringbykey("ce_list",paramstr) // Slash-seperated list of CEs
	string spec_corr=stringbykey("spec_corr",paramstr) // Slash-seperated list of species-specific correction factors
	string frag_exclude=stringbykey("frag_exclude",paramstr) // Frag waves to exclude		// 1.41B by James
	variable timeweight_op=numberbykey("timeweight_op",paramstr) // Time weighting option		//1.48E addition by James
	string CE_corr=stringbykey("CE_corr",paramstr) // Slash-seperated list of species-specific CE correction factors

	variable PToFFlag=numberbykey("PToFFlag",paramstr) //
	// 1.19E
	String PToFExcludeAir=stringbykey("frag_exclude",paramstr)		//1.19E
	variable dtdlogdp_op=numberbykey("dtdlogdp_op",paramstr)		//1.19E
	variable OneBindlog_op=numberbykey("OneBindlog_op",paramstr)					//1.19E
//	variable OneBinSumNotAvg_op=numberbykey("OneBinSumNotAvg_op",paramstr)					//1.19E
	variable HRPToFMatrix_op=numberbykey("HRPToFMatrix_op",paramstr)					//1.20T  input is a matrix
	variable HRPToFMatrix_RunPos=numberbykey("HRPToFMatrix_RunPos",paramstr)					//1.20T
	variable lc = numberbykey("lc",paramstr) // Chamber length  //1.20T
	
	variable matrix_flag=product_type&2?(1):(0)
	string targsuff=selectstring(matrix_flag,"_vec","_sparse") // 1.46A  JDA edit

	// specific to HR
	variable HRSpec_flag=numberbykey("HRSpec_flag",paramstr)==1
	variable nanTo0_flag=numberbykey("nanTo0_flag",paramstr)==1
	variable negTo0_flag=numberbykey("negTo0_flag",paramstr)==1
	
	// still more flags
	variable corr_op=numberbykey("corr_op",paramstr) // Use correction factor or not
	variable aw=numberbykey("aw",paramstr) // Atomic mass of calibration material (nominally 62)
	variable ug_op=numberbykey("ug_op",paramstr) // Convert from Hz to g m-3

	//  clean up, reset variables extracted from paramStr
	aw=numtype(aw)?(62):(aw) // Defaults aw to 62 if omitted
	
	if (strlen(destlist)==0)
		destlist=speclist // Defaults the destination to the species name if omitted
	endif
				
	// declare local variables used in calculations
	variable idex, jdex, kdex, n_spec,numRunsInThisAcorn,n_HR, p2,p1, fragmentPos, numNF, pos, runDex
	variable rie,ce,t_step,ugfac
	string deststr, specstr, target
	variable n_dep, opfaclist_n
	variable numUserDef			//1.10K

	numRunsInThisAcorn=numpnts(index_pos)		// number of runs in this acorn
	n_spec=itemsinlist(speclist,"/")		// number of species
	n_HR = dimSource2==0 ? dimSource1 : dimSource2  // 1.21G

// A step not normally taken in squirrel.
// This is an important distinction with squirrel.  We only call this function with groups of results. 
//  Either the results group is all HRspecies like HROrg, HRChl, or every result is *not* HRSpecies, like familyCH, C2O3H4.
// One cannot mix a HROrg and the familyCH because of the redimensioning needed for HROrg.

	if (acorn==0)
		if (HRSpec_flag)
			HR_smartConvertFrags(ReplaceString("/", speclist, ";"), ug_op)		// identifies HR ions not fit, creates 
			// EA
			 if (product_type >=10 )
				 // there should be only one HR species
				 wave/t frag_wave= $"root:HR_frag:frag_"+stringfromlist(0, speclist,"/")  // i.e. frag_HRorg
				 HR_EAatomicMassSeparation(frag_wave)
			endif
		endif
	endif		//1.20M

	if (HRSpec_flag)
		svar HRMassesNotFitList = root:HR_frag:HRMassesNotFitList		// set by HR_getListNewHRAlgebraMass
		svar HRMassesNotFitPosList = root:HR_frag:HRMassesNotFitPosList // set by HR_getListNewHRAlgebraMass
		svar HRMassesNotFitPosListinAll = root:HR_frag:HRMassesNotFitPosListinAll // set by HR_getListNewHRAlgebraMass
		numNF = ItemsInList(HRMassesNotFitList)
//		n_HR =dimsize(source, 1)+numNF   //1.21G
		for (idex = 0;idex<numNF;idex +=1)
			pos = str2num(stringfromList(idex, HRMassesNotFitPosList))
			if (PToFFlag)
				insertpoints/m=2 pos, 1, source
			else
				insertpoints/m=1 pos, 1, source
				if (error_flag)
					insertpoints/m=1 pos, 1, source_err
				endif
			endif
		endfor
		n_HR +=numNF   //1.21G
		wave/t HRionText=root:HR_frag:SpeciesMassText
	else	
		wave/t/z HRionText=root:HR_Sticks:ExactMassText_FitTodo // 1.08E replaced with *_FitTodo
		if (!WaveExists(HRIonText))		// for the case of someone upgrading who has not yet done a fit for a todo wave
			wave/t HRionText=root:HR_Sticks:ExactMassText // 1.08E replaced with *_FitTodo
		endif
	endif
		
	if (nanTo0_flag)
		source = numtype(source[p][q])==0 ? source[p][q] : 0 
	endif
	
	if (negTo0_flag)
		source =  source[p][q] > 0 ? source[p][q] : 0 
	endif

// Step 2. Create waves needed for calculations:
	// destination waves,  weighting waves, error waves conversion waves
	if (acorn==0)
		sq_ams_stat_upd("Generating destination waves")
		gen_removeSpacesTextWaves("root:HR_frag", "HR_frag_")		// added in 1.47E

		for(idex=0;idex<n_spec;idex+=1)		// for each desired result, create output wave(s), frag waves, weighting waves
			deststr=stringfromlist(idex, destlist,"/")
			specstr=stringfromlist(idex, speclist,"/")
			switch(product_type)
				case 0:		// one value
					make /o/n=(1) $deststr=0
				break
				case 1:		// time series
					make /o/n=(time_points) $deststr=0
				break
				case 2:		// average mass spec
					make /o/n=(n_HR) $deststr=0
				break
				case 3:		// 2d matrix of time series, HR ions
					make /o/n=(time_points,n_HR) $deststr=0
					wave dest =  $deststr
//					print dimsize(dest, 0), dimsize(dest,1)
				break

				// added in 1.19E
				case 4:	// size distr
					make /o/n=(d_points) $deststr=0
				break
				case 5:	// regular PToF image plot
					make /o/n=(time_points,d_points) $deststr=0
				break
				case 6:	// 2D avg mass spectrum (with dimension)
					make /o/n=(n_HR,d_points ) $deststr=0
				break
				case 7:	// 3D time, mz, size
//					make /o/n=(time_points,mz_points,d_points) $deststr=0
					make /o/n=(time_points,n_HR,d_points) $deststr=0
				break
				// end added in 1.19E

				case 11:		// EA time series		// an unfortunate compromise
					// we have one 'species' HROrganic, in speclist, but a different number of output waves (destList).
					// these elementalMF_C etc waves are NOT the final results.  ratios will be calced after all acorns are finished.
					HR_EACreateElementalTSWaves(time_points, "")			
					make/o/n=(n_HR) root:ElementalAnalysis:OneHRSpectra_temp/WAVE=OneHRSpectra_temp =0
					make/o/n=(n_HR) root:ElementalAnalysis:ElementX_temp/WAVE=ElementX_temp =0
				break
				case 14:		// EA average mass spec
					HR_EACreateElementalMSWaves(n_HR, "")			
					make/o/n=(n_HR) root:ElementalAnalysis:OneHRSpectra_temp/WAVE=OneHRSpectra_temp =0
					make/o/n=(n_HR) root:ElementalAnalysis:ElementX_temp/WAVE=ElementX_temp =0
				break
			endswitch		// endswitch(product type)

			//& Create weighting wave, if needed
			if (!(product_type&1)  || (t_base_flag )  && product_type < 10)   // is NOT time series or mass spec time series
				duplicate/o $deststr, $("conc_weight_"+num2str(idex))
			elseif(!(product_type&1)  || (t_base_flag )  && product_type > 10) 
				make /o/n=(n_HR)  $("conc_weight_"+num2str(idex))	=0		
			elseif( ( product_type&1)  && product_type < 10)
				fastop $deststr=(nan)		// i. e. time series mass spec
			endif
									
			// this if statement moved to after previous statement /// in 1.20L  don't know why this was removed.		
			// Create error destination wave		// 1.06H
			if (error_flag)
				if (product_type<10 )		// for now no error waves for EA analysis
					duplicate /o $deststr,$(deststr+"_err")
//				elseif (product_type==11 )		// EA time series	
//					HR_EACreateElementalTSWaves(time_points, "_err")							
//				else	  // mass spec
//					HR_EACreateElementalMSWaves(n_HR, "_err")			
				endif			
			endif

			target="root:HR_ms_mats:"+stringfromlist(idex,speclist,"/")
			sq_ams_stat_upd("Generating "+target+" matrices")
			
			// specific to HR.  We need to prepare the HRspecies, family and HR ion multiplier waves somewhat differently
			if(stringMatch(upperstr(specStr[0,1]), "HR"))			//1.06A	// it is a HRspecies
				sq_ams_stat_upd("Generating "+target+" matrices")
				HR_fraglist_convert(stringfromlist(idex,fraglist,"/"),"vec_tmp",n_HR,frag_exclude)		//1.09E
				wave/t vec_tmp = $"vec_tmp"		//1.10K  a temporary wave   UserDefHz
				if (n_HR<dimsize(vec_tmp,0))			//1.58G
					vec_tmp[n_HR,]=""		//   UserDefHz  vec_tmp counts from zero. 
				endif
				HR_frag_matrix($"vec_tmp",target,0)	// generates 
			elseif (strsearch(specStr, "family", 0, 2)>=0 )		// we have a family
//				specStr = ReplaceString("family", specStr, "")
				wave/z familyMask = $"root:int_sticks:mask_"+ ReplaceString("family", specStr, "")+"_FitTodo"
				if (!waveexists(familyMask))
					abort "// This family mask wave root:int_sticks:mask_"+ReplaceString("family", specStr, "")+"_FitTodo doesn't exist - check spelling, i.e. CHO1 not CHO"
				endif
				make/o/n=(n_HR) $"root:HR_ms_mats:"+specStr+"_vec"/Wave=SpecVec
				SpecVec=familyMask[p]// selectnumber(familyMask[p]==1, 0, 1)
				make/o/n=(n_HR,3) $"root:HR_ms_mats:"+specStr+"_sparse"/Wave=SpecSparse
				SpecSparse[][0,1]=selectnumber(familyMask[p]==1, nan, p)
				SpecSparse[][2]=selectnumber(familyMask[p]==1, nan, 1)
				sq_RemoveNaNs2d(SpecSparse)		
			elseif(stringmatch("ALL", upperstr(specStr)))	//1.07C
				make/o/n=(n_HR,3) $"root:HR_ms_mats:"+specStr+"_sparse"/Wave=SpecSparse = 1 
				SpecSparse[][0,1]=p 
				make/o/n=(n_HR) $"root:HR_ms_mats:"+specStr+"_vec"/Wave=SpecVecWave = 1
			else		// single HR ion
				fragmentPos = sq_findStr(HRionText,specStr,1)
				if (fragmentPos<0)
					abort "Something is wrong with this ion "+specStr
				endif
				make/o/n=(1,3) $"root:HR_ms_mats:"+specStr+"_sparse" = { {fragmentPos}, {fragmentPos}, {1}} 
				make/o/n=(n_HR) $"root:HR_ms_mats:"+specStr+"_vec"/Wave=SpecVecWave = 0
				SpecVecWave[fragmentPos]=1
			endif

		// prepare our HRspecies, families, or HR ion "frag" waves		
									
		endfor	// endfor each species in spec_list
	
		if (product_type == 11 ||product_type == 14 )		// elemental analysis
			destList = "root:ElementalAnalysis:OneHRSpectra_temp"
		endif

		// 1.10K 
		// By default the sq_fragList_convert makes the frag waves to be 10 (=maxUserDefined) bigger than they need to be.
		// Once we know of all the userdefined waves, we trim all the converted frag waves back to the correct size
		for(idex=0;idex<n_spec;idex+=1)
			target=stringfromlist(idex, speclist,"/")
			wave/z spec_mat = $"root:HR_ms_mats:"+target+"_mat"
			wave/z spec_vec = $"root:HR_ms_mats:"+target+"_vec"
//			wave spec_sparse = $"root:ms_mats:"+target+"_sparse"
			if ( WaveExists(spec_mat) && WaveExists(spec_vec) )
				Deletepoints n_HR+numUserDef, (10-numUserDef), spec_vec
				Deletepoints/M=0 n_HR+numUserDef, (10-numUserDef), spec_mat
				Deletepoints/M=1 n_HR+numUserDef, (10-numUserDef), spec_mat 
			endif
// no need to fuss with the sparse matrix because it is empty after n_it
//			j=dimsize(spec_sparse,0)
//			make/o/FREE/n=(j)  tempWave = spec_sparse[p][1]
//			findLevel/q/r=(j-1, j-10) tempWave, n_it
//			if (V_flag==0)
//				Deletepoints/M=0 V_LevelX, (10-numUserDef), spec_sparse
//			endif

		endfor	// endfor each species in spec_list			

		make /o/n=(numpnts(t_series)) ugfac_wav_tmp
		if (ug_op)
			multithread ugfac_wav_tmp=(aw/AVOGADRO_NUM)*(1e12)/(ioneff_wave*flowrate)	  //1.16K 	// somehow added parathesis in 1.11F?? (amber)
		else // MS Hz or PToF dI/dPToF (Hz)
			multithread ugfac_wav_tmp=(1)  	// from FastOp  //1.57K 
		endif
		if (PToFFlag && product_type>=4)  //1.21G  we don't apply this for single bins
			ugfac_wav_tmp*=ChopperFreq/chopper_duty
		endif
		
		if (corr_op) //& apply correction factor
			ugfac_wav_tmp*=corr_fact[p]
		endif	

	else		// else not the first acorn, just get wave references for existing waves 

		wave ugfac_wav_tmp
		
		if (product_type >= 10)		// EA stuff		// an unfortunate compromise
			wave OneHRSpectra_temp = root:ElementalAnalysis:OneHRSpectra_temp
			wave ElementX_temp = root:ElementalAnalysis:ElementX_temp
		endif
//			
//		if (PToFFlag && HRPToFMatrix_op)
//			wave dtdlogdp_tmp
//			wave log_dsig_tmp
//		endif
	endif	// endif acorn ==0

// Step 3. Do some preliminary data manipulations
	// Prep custom time base
	if (hz_op)		//1.19E
		sq_Hz(index_pos,source)
	endif

	if (t_base_flag)
		make/free/o/d/n=(numRunsInThisAcorn,2) t_source= (index_pos[p]<0 ? 0 : imt_series[series_index[index_pos[p]]+q])		//1.58G
		sq_make_map(t_source,t_base,"t_map")
		wave t_map
	endif

	if (error_flag)
		matrixop /o source_err2=powr(source_err,2)
	endif

// Step 4. Generate data product
	for (idex=0;idex<n_spec;idex+=1)		// each species

		sq_ams_prog_upd(100*(acorn+(idex/n_spec))/n_acorns)
		specstr=stringfromlist(idex, speclist,"/")
		deststr=stringfromlist(idex, destlist,"/")
		wave/z dest=$deststr			//in the case of EA, our destination waves aren't in the list
		wave /z dest_n=$("conc_weight_"+num2str(idex))
		wave /z spec_corr_wav=$stringfromlist(idex,spec_corr,"/")
		wave /z CE_corr_wav=$stringfromlist(idex,CE_corr,"/")
		wave /z dest_err=$(deststr+"_err")		
		
		if (strlen(rie_list)>0 && strlen (ce_list)>0  && ug_op)			// 1.41B change to include ug_op
			rie=str2num(stringfromlist(idex, rie_list,"/"))
			ce=str2num(stringfromlist(idex, ce_list,"/"))
		else
			rie=1
			ce=1
		endif
		
		target=stringfromlist(idex,speclist,"/")					
		// apply frag-converted op wave to data
		sq_ams_stat_upd("Applying matrix to "+target)
		
//		if (error_flag)  //1.22A
//			wave/wave waveRefs = sq_ApplyFragWaves(source, idex, destlist, speclist, product_type, error_flag, "root:HR_ms_mats:", n_HR, numRunsInThisAcorn, dimSource1, index_pos = index_pos, source_err2 = source_err2)
//		else
//			wave/wave waveRefs = sq_ApplyFragWaves(source, idex, destlist, speclist, product_type, error_flag, "root:HR_ms_mats:", n_HR, numRunsInThisAcorn, dimSource1, index_pos = index_pos)
//		endif

		if (error_flag)
			wave/wave waveRefs = sq_ApplyFragWaves(source, idex, destlist, speclist, product_type, error_flag, "root:HR_ms_mats:", n_HR, numRunsInThisAcorn, dimSource1, index_pos = index_pos, source_err2 = source_err2)
		else
			wave/wave waveRefs = sq_ApplyFragWaves(source, idex, destlist, speclist, product_type, error_flag, "root:HR_ms_mats:", n_HR, numRunsInThisAcorn, dimSource1, index_pos = index_pos)
		endif

		wave mass_tmp = waveRefs[0]			// 	
		// mass_tmp  runs x mz x (PToF) for results with mz dimension
		// mass_tmp  runs x (PToF) for results without mz dimension	
		if (error_flag)
			wave mass_tmp_err = waveRefs[1]
		endif
			
		// Distribute the data to the destination wave accordingly.
		sq_ams_stat_upd("Assigning "+deststr+" data to destination")
		p2=-1
		
		for (jdex=0;jdex<numRunsInThisAcorn;jdex+=1)	// (each run in current acorn ...index_pos)
			sq_ams_prog_upd(100*(acorn+((idex+(jdex/numRunsInThisAcorn))/n_spec))/n_acorns)
			if (index_pos[jdex]>-1 && !numtype(source[jdex]))		//^ if(we have real data to work with) mass_tmp may be a 2-d wave.  But all is well because mass_tmp rows correspond to runs in all cases.
				// p1 identifies point number for this run
				p1=series_index[index_pos[jdex]]
					
				// find ugfac conversion for this run, species
				ugfac=ugfac_wav_tmp[p1]/(rie*ce)
				if (waveexists(spec_corr_wav))
					ugfac*=spec_corr_wav[p1]
				endif
					
				if (waveexists(CE_corr_wav))
					ugfac/=CE_corr_wav[p1]
				endif

				// find t_step for possible weighting
//				t_step=imt_series[p1+1]-imt_series[p1]
				// find t_step for possible weighting			//1.07C addition to mimic changes in squirrel .148E
				switch (timeweight_op)
					case 1: // Would be used for MSOpen data
						t_step=TimeMSOpen[p1]
						break
					case 2: // Would be used for MSClosed data
						t_step=TimeMSClosed[p1]
						break
					case 3: // Would be used for MSDiff data
						t_step=TimeMSOpen[p1]+TimeMSClosed[p1]
						break
					case 4: // Would be used for pToF data
						t_step=TimePToF[p1]
						break
					case 5: // Would be used for FMS data
						t_step=FastModeAvgTimeSec[p1]			//1.57J
						break
					default: // Old method
						t_step=imt_series[p1+1]-imt_series[p1]
				endswitch

// prepare to deal with averaging in any dimension, mz, DVa (not run time dimension)
				if (product_type>=4 && dlog_op) 	// (PToF data and results to be in DVa) make diameter map  // in this case we don't care what PostAvgFlag is....we always use the last one
					wave dtdlogdp_tmp = HR_Getdtdlogdp_tmp(d_points,p2,p1,vl,vg,b,dx,t_PToFCenters)
//					duplicate/o dtdlogdp_tmp root:dtdlogdp_tmp_HR
					if (dimsize(mass_tmp,2)==0)		// result does not have m/z dimension, just run and PToF
						mass_tmp[jdex][]*=dtdlogdp_tmp[q]
//						if (error_flag)
//							mass_tmp_err[j][]*=dtdlogdp_tmp[q]^2
//						endif
					else  // result retains m/z dimension... PToF is the [2] dimension
						mass_tmp[jdex][][]*=dtdlogdp_tmp[r]
//						if (error_flag)
//							mass_tmp_err[j][][]*=dtdlogdp_tmp[r]^2
//						endif
					endif
					p2=p1  // set p2 = p1 in prep for later potential averaging in PToF dim
				endif
				// assign result according to product type
				switch (product_type)
					case 0: // Speciated mass concentration (average)
						ugfac*=t_step
						dest[0]+=mass_tmp[jdex]*ugfac
						dest_n[0]+=t_step
						if (error_flag)
							dest_err[0]+=mass_tmp_err[jdex]*ugfac*ugfac
						endif
					break
					case 1: // Speciated mass concentration vs time
						if (t_base_flag)
							ugfac*=t_step
							sq_map2dist(jdex,0,0,ms_conc_tmp,mpos,map1=t_map)		// get map parameters for this time position
							dest[mpos[0][0],mpos[1][0]]+=mass_tmp[jdex]*ugfac*ms_conc_tmp[p-mpos[0][0]][0][0]
							dest_n[mpos[0][0],mpos[1][0]]+=t_step*ms_conc_tmp[p-mpos[0][0]][0][0]
							if (error_flag)
								dest_err[mpos[0][0],mpos[1][0]]+=mass_tmp_err[jdex]*((ugfac*ms_conc_tmp[p-mpos[0][0]][0][0])^2)
							endif
						else		// else !t_base_flag, use the DAQ time base
							dest[p1]=mass_tmp[jdex]*ugfac
							if (error_flag)
								dest_err[p1]=mass_tmp_err[jdex]*ugfac*ugfac
							endif
						endif  // endif(t_base_flag)
					break
					case 2:  // Speciated mass spectrum vs m/z (an averaged mass spectrum)
						ugfac*=t_step
						dest+=mass_tmp[jdex][p]*ugfac
						if (error_flag)
							dest_err+=mass_tmp_err[jdex][p]*ugfac*ugfac
						endif
						dest_n+=t_step
					break
					case 3: // Speciated mass spectrum vs time and m/z (what users may want to export for PMF)
						if (t_base_flag)
							ugfac*=t_step
							sq_map2dist(jdex,0,0,ms_conc_tmp,mpos,map1=t_map)
							dest[mpos[0][0],mpos[1][0]][]+=mass_tmp[jdex][q]*ugfac*ms_conc_tmp[p-mpos[0][0]][0][0]
							dest_n[mpos[0][0],mpos[1][0]][]+=t_step*ms_conc_tmp[p-mpos[0][0]][0][0]
							if (error_flag)
								dest_err[mpos[0][0],mpos[1][0]][]+=mass_tmp_err[jdex][q]*((ugfac*ms_conc_tmp[p-mpos[0][0]][0][0])^2)
							endif
						else			// use DAQ time base
							dest[p1][]=mass_tmp[jdex][q]*ugfac
							if (error_flag)
								dest_err[p1][]=mass_tmp_err[jdex][q]*ugfac*ugfac
							endif
						endif	// endif use DAQ time base
					break
					case 4: // avg size dist. At the moment we don't allow for users to select different 'custom' sizes. 
						ugfac*=t_step
						dest+=mass_tmp[jdex][p]*ugfac  // dest is 1D wave d_points  //1.21G
						dest_n+=t_step
					break
					case 5: // image plot
						if ( t_base_flag)
							ugfac*=t_step
							sq_map2dist(jdex,0,0,ms_conc_tmp,mpos,map1=t_map)		//1.20F
							dest[mpos[0][0],mpos[1][0]][]+=mass_tmp[jdex][q]*ugfac*ms_conc_tmp[p-mpos[0][0]][0][0]
							dest_n[mpos[0][0],mpos[1][0]]+=t_step*ms_conc_tmp[p-mpos[0][0]][0][0]
						else		//  else not t_base_flag
							dest[p1][]=mass_tmp[jdex][q]*ugfac
						endif  // endif ( t_base_flag)
					break
					case 6: // Speciated, different size mass spectra
							// handled a bit differently because this is the ony product where rows are mz
						ugfac*=t_step
						dest+= mass_tmp[jdex][p][q]*ugfac
						dest_n+=t_step
//						if (error_flag)
//							dest_err+= mass_tmp_err[j][p][q]*ugfac*ugfac  //    mass_tmp_err[j][p][q]*ugfac*ugfac
//						endif
					break
					case 7: // Speciated, banded mass spectrum vs Time, m/z and diameter
						if ( t_base_flag)
							sq_map2dist(jdex,0,0,ms_conc_tmp,mpos,map1=t_map)
							dest[mpos[0][0],mpos[1][0]][][]+=mass_tmp[jdex][q][r]*ugfac*ms_conc_tmp[p-mpos[0][0]][0][0]
							dest_n[mpos[0][0],mpos[1][0]][][]+=t_step*ms_conc_tmp[p-mpos[0][0]][0][0]
						else		//  else not t_base_flag
							dest[p1][][]=mass_tmp[jdex][q][r]*ugfac
						endif  // endif ( t_base_flag)
					break
					case 11: // EA time series
						// note that we deal with time base averaging in another place because it get's too darn confusing.
						OneHRSpectra_temp=mass_tmp[jdex][p]*ugfac
						HR_EAratioOneHRSpectra(OneHRSpectra_temp, ElementX_temp, p1, 0)
//						if (error_flag)
//							OneHRSpectra_temp=mass_tmp_err[jdex][p]*ugfac*ugfac
//							HR_EAratioOneHRSpectra(OneHRSpectra_temp, ElementX_temp, p1, 1)
//						endif
					break
					case 14: // EA average mass spec
//						ugfac*=t_step		// this is done in HR_EAparseOneHRSpectra
						OneHRSpectra_temp=mass_tmp[jdex][p]*ugfac		// HR organic matrix
						HR_EAparseOneHRSpectra(OneHRSpectra_temp, ElementX_temp, 1, 0)		// kinda goofy, we don't want to consider the time step within this function
//						if (error_flag)
//							OneHRSpectra_temp=mass_tmp_err[jdex][p]*ugfac*ugfac		// HR organic matrix
//							HR_EAparseOneHRSpectra(OneHRSpectra_temp, ElementX_temp, 1, 1)		// kinda goofy, we don't want to consider the time step within this function
//						endif
						dest_n+=t_step
					break

				endswitch	// product type
			endif		// if(we have real data to work with)
		endfor		// for each run in current acorn

	// Step 5. Finalize destination wave if at end of acorn... divide by any weighting factors
		if (acorn==n_acorns-1)
			sq_ams_stat_upd("Finalising data")
			if (!(product_type&1)  || t_base_flag )		// *Not time series, an average spectra, type 0 or 2, or non - DAQ time base  
				if (product_type<10)
					matrixop /o dest=dest/dest_n
					if (error_flag)
						matrixop /o dest_err=powr(dest_err,0.5)/dest_n  
					endif
					if (dimsize(dest_n,1)==1 && dimsize(dest_n,2)==0)
						Redimension/N=(-1,1) dest
						if (error_flag)
							redimension/n=(-1,1) dest_err
						endif
					elseif (dimsize(dest_n,2)==1)
						Redimension/N=(-1,-1,1) dest
						if (error_flag)
							redimension/n=(-1,-1,1) dest_err
						endif
					endif
					killwaves /z dest_n
				endif
			elseif (error_flag)
				if (product_type<4)		// don't do this for EA
					matrixop /o dest_err=powr(dest_err,0.5)	
					if (dimsize(dest,1)==1 && dimsize(dest,2)==0)
						redimension /n=(-1,1) dest_err
					elseif (dimsize(dest,2)==1)
						redimension /n=(-1,-1,1) dest_err
					endif
				endif
			endif
		endif

	endfor	// for (each species)
	
// Step 5. Finalize destination wave if at end of acorn... divide by any weighting factors
	if (acorn==n_acorns-1)
		sq_ams_stat_upd("Deleting temporary data")
		killwaves/z t_map, source_err2, TimeWaves, mass_tmp1, waveRefs
		killwaves /z mass_tmp, dest_n, m_volumetranspose,ugfac_wav_tmp, mass_tmp_err, HRdtdlogdp_tmp
		killwaves/z OneHRSpectra_temp, ElementX_temp
		killwaves /z vec_tmp, mpos, ms_conc_tmp
		killwaves /z op_dep1, op_dep2		//1.20M
		speclist= WaveList("conc_weight_*", ";", "")
		for (idex=0;idex<itemsinlist(speclist); idex+=1 )
			killwaves /z $stringfromList(idex, specList, ";")
		endfor
//		if (1)		// 1.10C		 for debugging
			string tempList
			SetDataFolder root:HR_ms_mats:
			tempList = WaveList("temp_*", ";", "")		// "true" HR species. air, ammonium, chloride, nitrate, organic, pco2, Pwater
			sq_KillWavesInList(tempList)
			nvar misc_KillAllInnms_mats = root:panel:misc_KillAllInnms_mats
			if (misc_KillAllInnms_mats)		//1.20M
				tempList = WaveList("*", ";", "")		// "true" HR species. air, ammonium, chloride, nitrate, organic, pco2, Pwater
				sq_KillWavesInList(tempList)		//1.20M
			endif
			SetDataFolder root:
//		endif
	endif

End


Function/wave HR_Getdtdlogdp_tmp(numPToFbins,p2,p1,vl,vg,b,dx, t_PToFCenters)
variable numPToFbins,p2,p1
wave vl,vg,b,dx,t_PToFCenters

	wave/z HRdtdlogdp_tmp = root:HR_ms_mats:HRdtdlogdp_tmp
	if(p1<0 || !WaveExists(HRdtdlogdp_tmp) || vl[p2]!=vl[p1] || vg[p2]!=vg[p1] || b[p2]!=b[p1] || dx[p2]!=dx[p1])
		nvar lc =root:panel:cor3_chamber
		make /o/n=(numPToFbins) HRdtdlogdp_tmp=sq_dtdlogdp(lc,vl[p1],vg[p1],b[p1],t_PToFCenters[p])	
	endif
	
	return HRdtdlogdp_tmp

end


// Is identical to sq_diffCalc and should be deleted in a later version
Function HR_MakeRawSpectra(index_pos,paramstr,acorn,n_acorns)
	wave index_pos
	string paramstr
	variable acorn 
	variable n_acorns

	string dest = stringbykey("dest",paramstr)
	wave MSOpen=$stringbykey("acorn_0",paramstr)				// could be MSePToFSum_p  1.57C
	wave MSClosed=$stringbykey("acorn_1",paramstr)
	variable DoGausFit_Op = numberbykey("DoGausFit_Op",paramstr)
	string GF_ParamStr = stringbykey("GF_ParamStr",paramstr)

	variable ePToFFlag=numberByKey("ePToFFlag",paramstr)
	ePToFFlag = numtype(ePToFFlag) == 0 ? ePToFFlag : 0
	if (acorn == 0)
		duplicate/o MSOpen MSOpenBk			// could be MSePToFSum_p  1.57C
		duplicate/o MSClosed MSClosedBk
	else
		wave MSOpenBk			// could be MSePToFSum_p  1.57C
		wave MSClosedBk
	endif
	
	// note that we do away with Closed and open m/z cal parameter distinctions here
	wave mzSlope=root:mzFitting:mz_FitSlope
	wave mzPower=root:mzFitting:mz_FitPower
	wave mzInt=root:mzFitting:mz_FitIntercept
//	nvar HR_UseSqBaseline = root:HR_panel:HR_UseSqBaseline
	
	sq_ams_prog_upd(100*acorn/n_acorns)

	// first create and save the raw difference spectra
	matrixop /o tempDiff = MSOpen - MSClosed  // from FastOp  //1.57K  
	//1.14M  Raw spectra
	squirrel_hdf_write(index_pos,tempDiff,dest)
	
	//1.57
	string MSOpenLessBaseL_pStr = selectString (ePToFFlag, "MSOpenLessBaseL_p", "MSePToFSumLessBaseL_p")
	string MSClosedLessBaseL_pStr =  "MSClosedLessBaseL_p"
	string MSDiffLessBaseL_pStr =  selectString (ePToFFlag, "MSDiffLessBaseL_p","MSePToFSumDiffLessBaseL_p")

	multithread MSOpenBk = MSOpen  // from FastOp  //1.57K
	multithread MSClosedBk = MSClosed  // from FastOp  //1.57K

	if (!ePToFFlag)
		bk_CreateBackgroundValues(index_pos, MSOpenBk,  mzSlope, mzInt, mzPower, "MSO")
	else
		bk_CreateBackgroundValues(index_pos, MSOpenBk,  mzSlope, mzInt, mzPower, "MSeS")
	endif
	
	bk_CreateBackgroundValues(index_pos, MSClosedBk,  mzSlope, mzInt, mzPower, "MSC")
	multithread MSOpen -= MSOpenBk
	multithread MSClosed -= MSClosedBk
	multithread tempDiff = MSOpen - MSClosed    // from FastOp  //1.57K

	//1.14M  Raw spectra
	squirrel_hdf_write(index_pos,MSOpen,MSOpenLessBaseL_pStr)
	squirrel_hdf_write(index_pos,MSClosed,MSClosedLessBaseL_pStr)
	squirrel_hdf_write(index_pos,tempDiff,MSDiffLessBaseL_pStr)

	if (DoGausFit_Op)
		string pstr =""
		string mzFolder=stringbykey("mzFolder",GF_ParamStr,"=", "/")
		string mz_InterceptStr=stringbykey("mz_Intercept",GF_ParamStr,"=", "/")
		string mz_SlopeStr=stringbykey("mz_Slope",GF_ParamStr,"=", "/")
		string mz_PowerStr=stringbykey("mz_Power",GF_ParamStr,"=", "/")
		variable openAlso=numberbyKey("OpenAlso",GF_ParamStr,"=", "/")
//		variable hz_op=numberbykey("hz_op",GF_ParamStr, "/")
		pstr+="specType:Closed;"
		pstr+="mzFolder:"+mzFolder+";"
		pstr+="mz_Intercept:"+mz_InterceptStr+";"
		pstr+="mz_Slope:"+mz_SlopeStr+";"
		pstr+="mz_Power:"+mz_PowerStr+";"
		pstr+="hz_op:0;"
		pstr+="acorn_0:root:hoard:acorn_1;"
		GF_fetch_GetGausCoefs(index_pos,pstr,acorn,n_acorns)
		if (openAlso)
			pstr = RemoveByKey("acorn_0", pstr)
			pstr+="acorn_0:root:hoard:acorn_0;"
			pstr = ReplaceString("Closed", pstr, "Open")
			duplicate/o MSOpen acorn_0
			GF_fetch_GetGausCoefs(index_pos,pstr,acorn,n_acorns)
		endif
	endif
	
	// cleanup
	if (acorn==n_acorns-1)
		Killwaves/z tempDiff, MSOpenBk, MSClosedBk
		sq_ams_prog_upd(100)
	endif

End


//& Quick and dirty difference calculator - can be used for sticks or raw
Function PK_diffCalc(index_pos,paramstr,acorn,n_acorns)
	wave index_pos
	string paramstr
	variable acorn,n_acorns

	string dest=stringbykey("dest",paramstr)
	wave MSSOpen=$stringbykey("acorn_0",paramstr)		// it really doesn't matter what is passed - it will subtract the data set in acorn 1 from acorn 0
	wave MSSClosed=$stringbykey("acorn_1",paramstr)

	variable idex,ColOpen, ColClosed, maxCol, diffCol, putInMemFlag, multipleFactor, PToFFlag, numHRCols, numMoreCols
	
	numHRCols = dimsize(MSSOpen,1)
	numMoreCols = mod(numHRCols, ROUNDNUMHRIONCOLS)

	multipleFactor =numberbykey("multipleFactor",paramstr)	
	multipleFactor = numtype(multipleFactor) == 0 ? multipleFactor : 1
	PToFFlag =numberbykey("PToFFlag",paramstr)	
	putInMemFlag = str2num(stringbykey("putInMem",paramstr))		// 1.46C
	
	ColOpen = dimsize(MSSOpen,1)
	ColClosed = dimsize(MSSClosed,1)
	
	if (PToFFlag)
		ColOpen = dimsize(MSSOpen,2)
		if (ColOpen!=ColClosed)		// 1.42E
			maxCol = max(ColOpen, ColClosed)
			diffCol = abs(ColOpen - ColClosed)		
			redimension/n=(dimsize(MSSOpen,0), maxCol) MSSClosed
			MSSClosed[][maxCol - diffCol-1, maxCol-1] = 0
			redimension/n=(dimsize(MSSOpen,0), dimsize(MSSOpen,1), maxCol) MSSClosed
			MSSOpen[][][maxCol - diffCol-1, maxCol-1] = 0
			print "// In find finding the difference spectra, there was a mismatch in the number of columns.  Open had "+num2str(ColOpen)+" and Closed had "+num2str(ColClosed)
		endif
		make/free/o/n=(dimsize(MSSOpen,0),dimsize(MSSOpen,1),dimsize(MSSOpen,2))  diffcalc_tmp
		for(idex=0;idex<dimsize(MSSOpen,1);idex+=1)
			ImageTransform/PTYP=1/P=(idex)  getPlane MSSOpen
			wave M_ImagePlane
			matrixop/free/o diffcalc_one = M_ImagePlane - MSSClosed
			multithread diffcalc_tmp[][idex][] = diffcalc_one[p][r]
//			ImageTransform/PTYP=1/P=(idex)/D=diffcalc_one  setPlane diffcalc_tmp  // run x PToF x HR
		endfor
		if (!putInMemFlag)		// 	
			InsertPoints/M=2 numHRCols, (ROUNDNUMHRIONCOLS-numMoreCols), diffcalc_tmp
		endif
	else
		if (ColOpen!=ColClosed)		// 1.42E
			maxCol = max(ColOpen, ColClosed)
			diffCol = abs(ColOpen - ColClosed)		
			redimension/n=(dimsize(MSSOpen,0), maxCol) MSSOpen, MSSClosed
			MSSOpen[][maxCol - diffCol-1, maxCol-1] = 0
			MSSClosed[][maxCol - diffCol-1, maxCol-1] = 0
			print "// In find finding the difference spectra, there was a mismatch in the number of columns.  Open had "+num2str(ColOpen)+" and Closed had "+num2str(ColClosed)
		endif
		matrixop /o diffcalc_tmp=MSSOpen - MSSClosed		// could also be MSOpen - MSClosed
		if (!putInMemFlag)		// 	
			InsertPoints/M=1 numHRCols, (ROUNDNUMHRIONCOLS-numMoreCols), diffcalc_tmp
		endif
	endif
	
	multithread diffcalc_tmp=(multipleFactor)*diffcalc_tmp

	if (putInMemFlag)		// 	
		squirrel_mem_write(index_pos,diffcalc_tmp,dest,dest)
	else 
		//1.14M 'expanded' columns to the next 200 HR spectra
		squirrel_hdf_write(index_pos,diffcalc_tmp,dest)
	endif
	
	if (acorn==n_acorns-1)
		killwaves /z diffcalc_tmp, M_ImagePlane
	endif

End


// Returns a string of the hold parameters for the gaussian fit (for one unit mass).
Function/S HR_MakeHoldString(numPeaks)
variable numPeaks

	string GaussHoldParams		//return string	
	variable iVar
	
	nvar HR_UseVaryingPeakWidth = root:HR_Panel:HR_UseVaryingPeakWidth
	
	if (HR_UseVaryingPeakWidth)
		GaussHoldParams = "10"		// hold background, don't hold peak width
	else
		GaussHoldParams = "11"		// hold background, AND peak width
	endif
//	GaussHoldParams = "00"		// let the baseline wiggle

	for(iVar = 2; iVar < (2+2*numPeaks); iVar += 2)
		GaussHoldParams += "01"		
	endfor	

	return GaussHoldParams			// for 3 peaks this would look like "11010101"

End



// fit function of a many-gauss spectra section using the peak shape
// It would be ideal if this could be sped up
// significantly redone in 1.04P to remove the fitting-to-gaussian-multiplying by ratio-type peak shape method.  Now just uses a simple look-up table
Function HR_ManyGaussWPS(wav,timepnt) : FitFunc
wave wav		// contains the baseline, sigma, then all peak centers, offset by timepnt
variable timepnt
	
	wave HR_finalPS = root:HR_PeakShape:HR_finalPS
	
	variable baseline, sigma, total, offset, num, GaussK
	 
	num = numpnts(wav)	
	baseline = wav[0]
	sigma = wav[1]
	total=0
	offset=2

	do
		GaussK =  wav[offset]==0? 0: wav[offset]*HR_finalPS((timepnt - wav[offset+1])/sigma)		// spectra point * peakshape scalar wave
		total += GaussK
		offset+=2		
	while(offset<num)
	
        total += baseline

	return total
	
End


// identical to HR_ManyGaussWPS using only gaussian shape.
Function HR_ManyGauss(wav,timepnt) : FitFunc
wave wav		// contains the baseline, sigma, then all peak centers, offset by timepnt
variable timepnt
	
	variable baseline, sigma, total, offset, num, GaussK
	 	
	num = numpnts(wav)
	baseline = wav[0]
	sigma = wav[1]
	total=0
	offset=2
	
	do
		GaussK =  wav[offset]==0? 0: wav[offset]*exp(-((timepnt-wav[offset+1])/sigma)^2)
		total += GaussK
		offset+=2
	while(offset<num)
	
        total += baseline

	return total
	
End


// Does sanity checking and sets values for the important HR_FinalPWIntercept, etc globals.
// Must be called before HR_CalculateSticksSQ_prep()
Function/s HR_PeakWidthSanity(OneRunFlag)
variable OneRunFlag

	wave HR_FinalPWInterceptWave = root:HR_PeakShape:HR_FinalPWInterceptWave
	wave HR_FinalPWSlopeWave = root:HR_PeakShape:HR_FinalPWSlopeWave
	wave HR_FinalPWPowerWave = root:HR_PeakShape:HR_FinalPWPowerWave 

	nvar HR_FinalPWIntercept = root:HR_PeakShape:HR_FinalPWIntercept
	nvar HR_FinalPWSlope = root:HR_PeakShape:HR_FinalPWSlope
	nvar HR_FinalPWPower = root:HR_PeakShape:HR_FinalPWPower

	nvar HR_RUnVar = root:HR_panel:HR_RUnVar

	svar sq_toDoWvNm = root:panel:sq_toDoWvNm
	wave todo = $"root:"+sq_toDoWvNm
	wave rn_series = root:index:rn_series

	variable pos, numTodo, myIntercept, mySlope,myPower
	string tempStr, returnStr=""
	
	if (OneRunFlag)
		pos = sq_findRun(rn_series,HR_RunVar)	
	else
		pos = sq_findRun(rn_series, todo[0])
	endif
	if (pos<0)
		abort "Please reselect run  Aborting from HR_PeakWidthSanity"
	endif
	HR_FinalPWIntercept = HR_FinalPWInterceptWave[pos]
	HR_FinalPWSlope = HR_FinalPWSlopeWave[pos]
	HR_FinalPWPower = HR_FinalPWPowerWave[pos]		// will be 1 if uing line for peak width formula

	if (numtype(HR_FinalPWIntercept)!=0 || numtype(HR_FinalPWSlope)!=0 || numtype(HR_FinalPWPower)!=0  )
		sprintf tempStr, "%d", rn_series[pos]
		abort "You do not have good peak width values for run "+tempStr+ " - aborting from HR_PeakWidthSanity"		
	endif

// Pika 1.10A  We do check that there are non-nan values in the waves
	if (!OneRunFlag)		// check to see that you have the same values for all runs in todo wave
		numTodo = numpnts(todo)
		make/o/n=(numpnts(rn_series)) mask_temp
		sq_makeMaskFromTodofindRun(mask_temp,todo)		
		duplicate/o HR_FinalPWInterceptWave temp_Wave
		
		temp_Wave = HR_FinalPWInterceptWave* (mask_temp/mask_temp)
		WaveStats/q/m=1 temp_Wave
		if (numtodo != V_npnts)		//  Now gives user more feedback as to what went wrong
			temp_Wave = numtype(mask_temp[p]) == 0 && numtype(temp_Wave[p]) != 0		//1.14G added to code to help users
			WaveStats/q/m=1 temp_Wave
			HR_PeakShapePanelWindow()
			PWPSTabProc("PWPS_tab",1) 
			PW_butt_PWParamTable("")			
			ModifyTable/W=PW_allParamTable selection=(V_maxloc , 0, V_maxloc , 0, V_maxloc,0 ), topLeftCell=(V_maxloc,0)
			sprintf tempStr, "%d", V_maxloc
			abort "There are some runs within the todo wave that do not have non-Nan values for the Peak Width intercept value; the first one being at row "+tempStr+" - aborting from HR_PeakWidthSanity"
		endif
		myIntercept=V_avg
			
		temp_Wave = HR_FinalPWSlopeWave* (mask_temp/mask_temp)
		WaveStats/q/m=1 temp_Wave
		if (numtodo != V_npnts )
			temp_Wave = numtype(mask_temp[p]) == 0 && numtype(temp_Wave[p]) != 0  //1.14G added to code to help users
			WaveStats/q/m=1 temp_Wave
			HR_PeakShapePanelWindow()
			PWPSTabProc("PWPS_tab",1) 
			PW_butt_PWParamTable("")			
			ModifyTable/W=PW_allParamTable selection=(V_maxloc , 0, V_maxloc , 0, V_maxloc,0 ), topLeftCell=(V_maxloc,0)
			sprintf tempStr, "%d", V_maxloc
			abort "There are some runs within the todo wave that do not have non-Nan values for the Peak Width intercept value; the first one being at row "+tempStr+" - aborting from HR_PeakWidthSanity"
		endif	
		mySlope=V_avg

		temp_Wave = HR_FinalPWPowerWave* (mask_temp/mask_temp)
		WaveStats/q/m=1 temp_Wave
		if (numtodo != V_npnts )
			temp_Wave = numtype(mask_temp[p]) == 0 && numtype(temp_Wave[p]) != 0  //1.14G added to code to help users
			WaveStats/q/m=1 temp_Wave
			HR_PeakShapePanelWindow()
			PWPSTabProc("PWPS_tab",1) 
			PW_butt_PWParamTable("")			
			ModifyTable/W=PW_allParamTable selection=(V_maxloc , 0, V_maxloc , 0, V_maxloc,0 ), topLeftCell=(V_maxloc,0)
			sprintf tempStr, "%d", V_maxloc
			abort "There are some runs within the todo wave that do not have non-Nan values for the Peak Width intercept value; the first one being at row "+tempStr+" - aborting from HR_PeakWidthSanity"
		endif	
		myPower=V_avg
		
		killwaves/z mask_temp, temp_Wave
	endif
		
	return "Intercept:"+num2str(myIntercept)+" Slope:"+num2str(mySlope)+" Power:"+num2str(mypower)
		
End

Function HR_PeakShapeSanity(OneRunFlag)
variable OneRunFlag

	nvar HR_UsePeakShapeVar = root:HR_panel:HR_UsePeakShapeVar
	wave/T HR_FinalPSNameWave = root:HR_PeakShape:HR_FinalPSNameWave
	nvar HR_FinalPSScalar = root:HR_PeakShape:HR_FinalPSScalar
	svar HR_FinalPSName = root:HR_PeakShape:HR_FinalPSName
	svar sq_toDoWvNm = root:panel:sq_toDoWvNm
	wave todo = $"root:"+sq_toDoWvNm
	wave rn_series = root:index:rn_series
	nvar HR_RUnVar = root:HR_panel:HR_RUnVar

	variable pos, tempVar, numTodo
	string tempStr
	
	if (OneRunFlag)
		pos = sq_findRun(rn_series,HR_RUnVar)	
	else
		pos = sq_findRun(rn_series, todo[0])
	endif

	tempStr = HR_FinalPSNameWave[pos]		// will be 1 if uing line for peak width formula
	wave/z finalPS = $"root:HR_PeakShape:"+tempStr

	numTodo = numpnts(todo)
	make/o/n=(numpnts(rn_series)) mask_temp, temp_Wave
	sq_makeMaskFromTodofindRun(mask_temp,todo)		

	if ( HR_UsePeakShapeVar)
		if( strlen(tempStr)==0 || !WaveExists(finalPS))
			HR_PeakShapePanelWindow()			//1.14G
			PW_butt_PWParamTable("")			
			ModifyTable/W=PW_allParamTable selection=(pos , 0, pos , 0, pos,0 ), topLeftCell=(pos,0)
			abort "You do not have good peak shape wave named "+TempStr+" - aborting from HR_PeakShapeSanity"		
		endif
		tempStr = Note(finalPS)
		tempVar = NumberByKey("PS_scalar", tempStr)
		if( numType(tempVar)!=0 || tempVar < 0.5 || tempVar >2)
			abort "You do not have good peak shape scalar for the peak shape named "+HR_FinalPSNameWave[pos]+" - aborting from HR_PeakShapeSanity"		
		endif
		HR_FinalPSScalar = tempVar
		duplicate/o finalPS root:HR_PeakShape:HR_finalPS
		HR_FinalPSName = NameofWave(finalPS)
	else
		HR_FinalPSScalar=1		
	endif

	if ( HR_UsePeakShapeVar)
		if(!OneRunFlag)
			temp_Wave = selectNumber(mask_temp[p]==1,  0, stringmatch(HR_FinalPSName, HR_FinalPSNameWave[p]))
			if (sum(temp_Wave) != numTodo )
				temp_Wave = mask_temp[p]==1 && temp_Wave[p]==0
				WaveStats/q/m=1 temp_Wave
				HR_PeakShapePanelWindow()
				PWPSTabProc("PWPS_tab",1) 
				PW_butt_PWParamTable("")			
				ModifyTable/W=PW_allParamTable selection=(V_maxloc , 0, V_maxloc , 0, V_maxloc,0 ), topLeftCell=(V_maxloc,0)
				sprintf tempStr, "%d", V_maxloc
				abort "There are some runs within the todo wave that do not have non-Nan values for the Peak Width intercept value; the first one being at row "+tempStr+" - aborting from HR_PeakShapeSanity"
				abort "You do not have the same Peak shape name for all runs in your todo wave - aborting from HR_PeakShapeSanity"		
			endif
		endif
	endif

	killwaves/z mask_temp, temp_Wave
End


// Gets things ready for an HR fit. This is called regardless as to whether we fit one spectra or many spectra
// After exact mass wave is created, we need to organize things by unit mass.
// Also makes waves & variables in HR_sticks data folder.
// Redimensions stuff.
Function HR_CalculateSticksSQ_prep()

	wave/t ExactMassText = root:HR_Sticks:ExactMassText		//1.12N
	wave ExactMassWave = root:HR_Sticks:ExactMassWave
	wave squirrel_index = root:index:squirrel_index
//	nvar HR_RunVar = root:HR_Panel:HR_RunVar
//	nvar HR_checkDoNotGrabRawSpectra = root:HR_Panel:HR_checkDoNotGrabRawSpectra	
	nvar HR_MaxNumGausFits = root:HR_Sticks:HR_MaxNumGausFits		
	nvar HR_FinalPWIntercept = root:HR_PeakShape:HR_FinalPWIntercept
	nvar HR_FinalPWSlope = root:HR_PeakShape:HR_FinalPWSlope
	nvar HR_FinalPWPower = root:HR_PeakShape:HR_FinalPWPower
	nvar HRSticksDisplayXResolution = root:HR_Sticks:HRSticksDisplayXResolution

	variable LowestUnitMass, HighestUnitMass, numInUnitMass, idex, numInExactMassWave, Var, numInRaw
	variable pos
	
	string paramStr=""
	string GaussConstraintsStr=""
	
	nvar HR_singleSpectraMSorPToF = root:HR_panel:HR_singleSpectraMSorPToF		//1.19F
	if (HR_singleSpectraMSorPToF==1 && finddimlabel(squirrel_index,2,"MSDiff_p")<0)  // User selected MS
		Abort "Please generate the MSDiff_p wave (in step 1) before proceeding."
	elseif(HR_singleSpectraMSorPToF==2) // User selected ePToFSum
		if(finddimlabel(squirrel_index,2,"PToF")<0)
			Abort "Raw PToF spectra was not indexed, so it is not possible to have generated MSePToFSum data and hence do HR peak fitting for ePToFSum data."		//1.20S
		elseif(finddimlabel(squirrel_index,2,"MSePToFSumDiffLessBaseL_p")<0)
			Abort "Please generate the MSePToFSumDiffLessBaseL_p wave (in step 1) before proceeding."
		endif
	elseif(HR_singleSpectraMSorPToF==0 && finddimlabel(squirrel_index,2,"PToF")<0)
		Abort "Raw PToF spectra was not indexed, so it is not possible to do HR peak fitting for HR PToF data."		//1.20S
	endif
	
	UM_MakeDutyCycleCorr()		// should be addressed when HR ions chosen?

	HR_MakeUnitMassWave()
// these are set once and don't change regardless if open, closed diff, or m/z changing with runs.
	wave MassStartPnt = root:HR_Sticks:MassStartPnt
	wave MassEndPnt = root:HR_Sticks:MassEndPnt
	wave UnitMass = root:HR_Sticks:UnitMass
	wave numOfPeaks = root:HR_Sticks:numOfPeaks	

	numInUnitMass = numpnts(UnitMass)
	numInExactMassWave = numpnts(ExactMassWave)

	make/o/n=(numInUnitMass) root:HR_Sticks:UnitSigma
//	make/o/n=(numInUnitMass) root:HR_Sticks:UnitSigmaByMass
	make/o/n=(numInUnitMass) root:HR_Sticks:UnitMassNumPeakSum
	make/o/n=(numInUnitMass)/t root:HR_Sticks:GaussConstraints
	make/o/n=(numInUnitMass)/t root:HR_Sticks:GaussHoldParams
	make/o/n=(numInUnitMass) root:HR_sticks:OneRunResidSumOpen
	make/o/n=(numInUnitMass) root:HR_sticks:OneRunResidSumClosed
	make/o/n=(numInUnitMass) root:HR_sticks:OneRunResidSumDiff
	make/o/n=(numInUnitMass) root:HR_sticks:OneRunResidSum_temp

// these are set once and don't change regardless if open, closed diff, or m/z changing with runs.
	wave MassStartPnt = root:HR_Sticks:MassStartPnt
	wave MassEndPnt = root:HR_Sticks:MassEndPnt	
	wave UnitMass = root:HR_Sticks:UnitMass	
	wave numOfPeaks = root:HR_Sticks:numOfPeaks		

//	wave UnitSigmaByMass = root:HR_Sticks:UnitSigmaByMass
	wave UnitSigma = root:HR_Sticks:UnitSigma
	wave UnitMassNumPeakSum = root:HR_Sticks:UnitMassNumPeakSum

	SetScale/P x 0,1,"", numOfPeaks 		// if by some chance the scaling gets chaned this will mess up the UnitMassPeakSum, calculated below.

	UnitMassNumPeakSum = sum(numOfPeaks, 0, p-1)
	UnitMassNumPeakSum[0]=0		//1.05A

	UnitSigma = HR_FinalPWIntercept + HR_FinalPWSlope*(UnitMass[p]^HR_FinalPWPower)		// peak width as function of mass
//	UnitSigmaByMass = nan													// placeholder for an individual run.  Will be filled with the V or W values later
			
	HR_CreateHoldConstraintWaves()

	// better in 1.04J
	make/o/n=(HRSticksDisplayXResolution, HR_MaxNumGausFits) root:HR_Sticks:GausFit_D_Y
	make/o/n=(HRSticksDisplayXResolution, HR_MaxNumGausFits) root:HR_Sticks:GausFit_O_Y
	make/o/n=(HRSticksDisplayXResolution, HR_MaxNumGausFits) root:HR_Sticks:GausFit_C_Y

	// for graphing in Sticks_Gr
	make/o/n=(HRSticksDisplayXResolution) root:HR_Sticks:GausFit_X
	make/o/n=(HRSticksDisplayXResolution) root:HR_Sticks:GausFit_D_Sum_Y
	make/o/n=(HRSticksDisplayXResolution) root:HR_Sticks:GausFit_O_Sum_Y
	make/o/n=(HRSticksDisplayXResolution) root:HR_Sticks:GausFit_C_Sum_Y

	// for graphing in Sticks_Gr
	sq_initializeGlobalNVar("root:HR_Sticks:HR_SameYScale", 1)				// in Sticks_gr
	sq_initializeGlobalNVar("root:HR_Sticks:HR_plotResidualInPercent", 0)				// in Sticks_gr
	sq_initializeGlobalNVar("root:HR_Sticks:HR_isW", 0)				// in Sticks_gr

	make/o/n=(numInExactMassWave) root:HR_sticks:OneRunMassTime = nan
	make/o/n=(numInExactMassWave) root:HR_sticks:OneRunStickOpen = nan
	make/o/n=(numInExactMassWave) root:HR_sticks:OneRunStickClosed = nan
	make/o/n=(numInExactMassWave) root:HR_sticks:OneRunStickDiff= nan
	make/o/n=(numInExactMassWave) root:HR_sticks:OneRunStick_temp = nan
	make/o/n=(numInExactMassWave) root:HR_sticks:OneRunStickArea_temp = nan		//1.12N
	make/o/n=(numInExactMassWave) root:HR_Sticks:ExactMassFit = 0					//1.12N

	make/o/n=(numInExactMassWave) root:HR_sticks:OneRunResidNoiseOpen = nan		//1.04E
	make/o/n=(numInExactMassWave) root:HR_sticks:OneRunResidNoiseClosed = nan		//1.04E
	make/o/n=(numInExactMassWave) root:HR_sticks:OneRunResidNoiseDiff = nan			//1.04E
	make/o/n=(numInExactMassWave) root:HR_sticks:OneRunResidNoise_temp = nan		//1.04E
		
	make/o/n=(numInExactMassWave) root:HR_sticks:OneRunDiffCountingError = nan		//1.04G

	make/o/n=(numInExactMassWave) root:HR_sticks:OneRunSqBkStickOpen = nan		//1.09H		// we want the ion counts of the baseline at each HR ion
	make/o/n=(numInExactMassWave) root:HR_sticks:OneRunSqBkStickClosed = nan		//1.09H
	make/o/n=(numInExactMassWave) root:HR_sticks:OneRunSqBkStickDiff = nan			//1.09H
	make/o/n=(numInExactMassWave) root:HR_sticks:OneRunSqBkStick_temp = nan		//1.09H

	make/o/n=(numInUnitMass) root:HR_sticks:SigmaOneRunOpen = nan
	make/o/n=(numInUnitMass) root:HR_sticks:SigmaOneRunClosed = nan
	make/o/n=(numInUnitMass) root:HR_sticks:SigmaOneRunDiff = nan
	make/o/n=(numInUnitMass) root:HR_sticks:SigmaOneRun_temp= nan
	
	make/o/n=(numInUnitMass) root:HR_sticks:HR_FitFail_temp= nan		//1.08E     // used only in non LLS HR fitting  1.11D
		
	//  The waves below were originally used for the calculation of Pete's method of finding the baseline
	// where one simply takes an average of the raw spectra x points away from the first and last HR ions in a unit group.
	//  functionality removed in 1.09H 
//	make/o/n=(numInUnitMass) root:HR_sticks:HR_BkOneRunOpen = nan
//	make/o/n=(numInUnitMass) root:HR_sticks:HR_BkOneRunClosed = nan
//	make/o/n=(numInUnitMass) root:HR_sticks:HR_BkOneRunDiff = nan
//	make/o/n=(numInUnitMass) root:HR_sticks:HR_BkOneRun_temp = nan	
    	
  	make/o/n=(numInUnitMass) root:HR_sticks:OneRunResidSumOpen=nan
	make/o/n=(numInUnitMass) root:HR_sticks:OneRunResidSumClosed=nan
	make/o/n=(numInUnitMass) root:HR_sticks:OneRunResidSumDiff=nan
	make/o/n=(numInUnitMass) root:HR_sticks:OneRunResidSum_temp=nan	

	make/o/n=(numInUnitMass) root:HR_Sticks:HRFitChiSqOpen			//1.04G		// eliminated in 1.11D
	make/o/n=(numInUnitMass) root:HR_Sticks:HRFitChiSqClosed		//1.04G		// eliminated in 1.11D
	make/o/n=(numInUnitMass) root:HR_Sticks:HRFitChiSqDiff			//1.04G		// eliminated in 1.11D
	make/o/n=(numInUnitMass) root:HR_Sticks:HRFitChiSq_temp			//1.04G		// eliminated in 1.11D

	wavestats/q/m=1 numOfPeaks
	for (idex=1; idex<=V_max; idex+=1)			// added in 1.03
		make/d/o/n=(2+2*idex) $"root:HR_sticks:GausFitParam_"+HR_gen_Num2Str2Char(idex)
	endfor
	//1.10E in Igor 6.22A we seem to be missing some global K values. Make them all every time and now put them in HR_sticks folder
	for (idex=0; idex<V_max; idex+=1)			// added in 1.03  global variables K0 - K19 we get for free
		variable/g $"root:HR_Sticks:myK"+num2str(2*idex+2)		// we need these extra globals
		variable/g $"root:HR_Sticks:myK"+num2str(2*idex+3)		// we need these extra globals
	endfor

	make/o/n=300 root:HR_sticks:absWave			// arbitrary length  we use to store the abs value of the residual
	
 	PK_convertUnitSigmaToEachSpecie()  	// added in 1.02A		we need the todo wave and the exact mass waves

	make/o/n=0 root:HR_Sticks:OneRunRawMz_temp		// used for HR fitting of todo... will be redimensioned once we know what wave we are working with
	
	make/o/n=(numInExactMassWave) root:HR_Sticks:ExactMassLeft, root:HR_Sticks:ExactMassRight, root:HR_Sticks:ExactMassFit
//	wave ExactMassLeft = root:HR_Sticks:ExactMassLeft  //1.53N
//	wave ExactMassRight = root:HR_Sticks:ExactMassRight
//	wave ExactMassFit = root:HR_Sticks:ExactMassFit
//	for(idex=0;idex<numInExactMassWave;idex+=1)
//		var =  BinarySearchInterp(OneRunMassTime, OneRunMassTime[idex] - 5*ExactMassSigma[idex])
//		ExactMassLeft[idex] = numtype(Var)==2 ? 0 : ceil(var)
//		var =  BinarySearchInterp(OneRunMassTime, OneRunMassTime[idex] + 5*ExactMassSigma[idex])
//		ExactMassRight[idex] =  numtype(Var)==2 ? numOfExactMass-1 : floor(var)		
//	endfor
	make/o/n=(numInExactMassWave) root:HR_Sticks:ConstrainedKey/wave=ConstrainedKey
	wave IsotopicMask = root:HR_Sticks:IsotopicMask
	wave/t IsotopicMassText = root:HR_Sticks:IsotopicMassText
	
	HR_findConstrainedFlag(IsotopicMassText, IsotopicMask,ExactMassText, ConstrainedKey)

End


// creates and fills in MassStartPnt, MassEndPnt, UnitMass, numOfPeaks
Function HR_MakeUnitMassWave()		

	wave exactMassWave = root:HR_Sticks:exactMassWave
	variable numInUnitMass, LowestUnitMass, HighestUnitMass, numInExactMass, currentRunIndex
	
	LowestUnitMass = round(exactMassWave[0])
	HighestUnitMass = round(exactMassWave[numpnts(exactMassWave)-1])		//1.58G
	
	numInExactMass = numpnts(exactMassWave)
	numInUnitMass = HighestUnitMass - LowestUnitMass+1		//1.03F

	make/o/n=(numInUnitMass) root:HR_Sticks:MassStartPnt, root:HR_Sticks:MassEndPnt
	make/o/n=(numInUnitMass) root:HR_Sticks:UnitMass, root:HR_Sticks:numOfPeaks

	wave MassStartPnt = root:HR_Sticks:MassStartPnt
	wave MassEndPnt = root:HR_Sticks:MassEndPnt
	wave UnitMass = root:HR_Sticks:UnitMass
	wave numOfPeaks = root:HR_Sticks:numOfPeaks

	UnitMass = LowestUnitMass+p
	MassStartPnt = binarysearch(exactMassWave, LowestUnitMass+p - 0.5) + 1
	MassEndPnt = binarysearch(exactMassWave, LowestUnitMass+p + 0.5)
	MassEndPnt[numInUnitMass-1] = numInExactMass-1		//1.03F

	numOfPeaks = MassEndPnt - MassStartPnt + 1 // this is a wave

	//1.11A check added   not sure why this was removed?? 
	wave/z mFinish = root:diagnostics:mFinish		// for now we don't know which run we are going to use.... 		//1.11A
	if (WaveExists(mFinish) && Wavemax(mFinish) < UnitMass[numpnts(UnitMass)-1] )		// we deal with PToF and Use Open params
		abort "Some HR ions that you selected have a m/z value of "+num2str(Wavemax(mFinish) )+", which is higher than the maximum mz range in the raw spectra."
	endif

End


// Constraint waves are used to tell the fits to have peak heights > 0.
// To make things faster, we make a wave with the same number of nominal mass groups as the chosen masses
Function HR_CreateHoldConstraintWaves() 

	wave/t GaussConstraints = root:HR_Sticks:GaussConstraints	
	wave/t GaussHoldParams = root:HR_Sticks:GaussHoldParams	

	wave UnitMass = root:HR_Sticks:UnitMass	
	wave numOfPeaks = root:HR_Sticks:numOfPeaks
	wave UnitSigma = root:HR_Sticks:UnitSigma

	nvar HR_UseFixedIsotopicVals = root:HR_Panel:HR_UseFixedIsotopicVals
	nvar HR_UseVaryingPeakWidth = root:HR_Panel:HR_UseVaryingPeakWidth
	nvar HR_MaxNumGausFits = root:HR_Sticks:HR_MaxNumGausFits

	variable LowestUnitMass, HighestUnitMass, numInUnitMass, idex, jdex, numInExactMassWave, Var, numInRaw
	variable thisAMU, orderedIndex, isotopesToConsider, isoItem, AMUthPt
	string isotopePairingStr, isoStr, GaussHoldParamsStr, GaussConstraintsStr, ConstraintStr

	numInUnitMass = numpnts(UnitMass)
	
	if (HR_UseFixedIsotopicVals)
 		HR_CheckIsotopeParents()
		HR_SortIsotopicValues()		// makes important MassFitOrder and IsotopePairings waves
	else
		duplicate/o UnitMass root:HR_Sticks:MassFitOrder
		make/t/o/n=(numInUnitMass) root:HR_Sticks:IsotopePairings = ""
	endif
	wave MassFitOrder=root:HR_Sticks:MassFitOrder
	wave/t IsotopePairings=root:HR_Sticks:IsotopePairings
	
	GaussConstraintsStr=""
	//  initialize GaussConstraintsStr and GaussHoldParams
	for (idex=0; idex<numInUnitMass; idex+=1)

		GaussConstraintsStr=""
		if (HR_UseVaryingPeakWidth)
			Var = UnitSigma[idex]		///sqrt(2)
			GaussConstraintsStr = "K1>"+num2str(var*.95)+";K1<"+num2str(var*1.05)+";"
		endif
		for (jdex=0;jdex<numOfPeaks[idex];jdex+=1)
			GaussConstraintsStr+="K" + num2str(jdex*2 + 2) + ">0;"
		endfor
		GaussConstraints[idex] = GaussConstraintsStr[0, strlen(GaussConstraintsStr)-2]
		
		GaussHoldParams[idex] =HR_MakeHoldString(numOfPeaks[idex])

	endfor
		
	LowestUnitMass = UnitMass[0]
	
	// now if insert extra hold values and remove constraint values if we are constraining isotopes
	if (HR_UseFixedIsotopicVals)
		for (idex=0; idex<numInUnitMass; idex+=1)
			
			thisAMU = MassFitOrder[idex]		// go in increasing MassFit Order, not AMU order
			isotopePairingStr=IsotopePairings[idex]
			orderedIndex = thisAMU - LowestUnitMass	
			isotopesToConsider =  ItemsinList(isotopePairingStr)
			GaussHoldParamsStr=GaussHoldParams[orderedIndex]
			ConstraintStr = GaussConstraints[orderedIndex]
			for (isoItem=0;isoItem <isotopesToConsider; isoItem+=1)
//			 	numConst=numpnts(ConstraintWave)
			 	isoStr = StringFromList(isoItem, isotopePairingStr)
//			 	childPt=str2num(StringFromList(0, isoStr,"/"))
//			 	parentPt=str2num(StringFromList(1, isoStr,"/"))
			 	AMUthPt=str2num(StringFromList(2, isoStr,"/"))
				GaussHoldParamsStr=GaussHoldParamsStr[0,2*(AMUthPt)+1]+"1"+GaussHoldParamsStr[2*(AMUthPt)+3, 41] 	// 41 = 2+20*2 counting from 0
				// we have to be careful about removing items "from the back"
				ConstraintStr = RemoveFromList("K"+num2str(2*AMUthPt+2)+">0", ConstraintStr)		// this notation needs Kx
//				deletepoints AMUthPt-isoItem, 1, ConstraintWave
			endfor
			GaussHoldParams[orderedIndex] = GaussHoldParamsStr
			GaussConstraints[orderedIndex] = ConstraintStr
		endfor
	endif

End


// Squirrel fetch exact mass stick calculating function
// Do we want to calc sticks for open, closed for all runs?
// kinda similar to HR_CalcSticksOneRun
//  IMPORTANT -- IN PRE 1.02 VERSIONS THESE RESULTS RETURN PEAK HEIGHT< NOT AREA, STICK values!!!
// Post 1.02 versions return areas, not heights!!!
// changed in 1.08E  to add fitfail info

//Function HR_CalculateSticksSQ(index_pos,paramstr,acorn,n_acorns)
// Squirrel fetch exact mass stick calculating function
// Do we want to calc sticks for open, closed for all runs?
// kinda similar to HR_CalcSticksOneRun
//  IMPORTANT -- IN PRE 1.02 VERSIONS THESE RESULTS RETURN PEAK HEIGHT< NOT AREA, STICK values!!!
// Post 1.02 versions return areas, not heights!!!
// changed in 1.08E  to add fitfail info

Function HR_CalculateSticksSQ(index_pos,paramstr,acorn,n_acorns)
	wave index_pos
	string paramstr
	variable acorn 
	variable n_acorns

	sq_ams_stat_upd("Calculating HR Sticks")

	wave MSSpectra_p=$stringbykey("acorn_0",paramstr)
	string spectraType = stringbykey("type",paramstr)	//1.06x
	string PToFdatasetStr = stringbykey("PToFdataset",paramstr)	

	wave HR_FinalPWInterceptWave = root:HR_PeakShape:HR_FinalPWInterceptWave		//added in 1.10A
	wave HR_FinalPWSlopeWave = root:HR_PeakShape:HR_FinalPWSlopeWave
	wave HR_FinalPWPowerWave = root:HR_PeakShape:HR_FinalPWPowerWave 

	wave mz_FitIntercept = $"root:mzFitting:"+stringbykey("intercept",paramstr)
	wave mz_FitSlope= $"root:mzFitting:"+stringbykey("slope",paramstr)
	wave mz_FitPower = $"root:mzFitting:"+stringbykey("power",paramstr)
	//  Note that the existence of these waves should be checked before the fetch function is called.
		
	wave ExactMassWave = root:HR_Sticks:ExactMassWave		// when we generate HR sticks for the current todo wave, this wave = ExactMassWave_fitTodo
	wave UnitMass = root:HR_Sticks:UnitMass
	
	wave/z OneRunRawMz_temp = root:HR_Sticks:OneRunRawMz_temp
	wave/z OneRunRawSpectra_temp = root:HR_Sticks:OneRunRawSpectra_temp
	
	wave OneRunResidSum_temp = root:HR_Sticks:OneRunResidSum_temp

	wave OneRunStickArea_temp = root:HR_Sticks:OneRunStickArea_temp		//1.12N
	wave OneRunMassTime = root:HR_Sticks:OneRunMassTime
	wave OneRunStick_temp = root:HR_Sticks:OneRunStick_temp
	wave SigmaOneRun_temp = root:HR_Sticks:SigmaOneRun_temp
	wave dutyCycleCorr = root:Int_Sticks:dutyCycleCorr

	wave/z nSecMSsamplingInterval =  root:diagnostics:nSecMSsamplingInterval		//1.14B
	if (!WaveExists(nSecMSsamplingInterval)   || !waveMax(nSecMSsamplingInterval) >0) 
		make/o/n=(numpnts(mz_FitIntercept)) root:diagnostics:nSecMSsamplingInterval=1		// default - old card always at 1ns
		wave nSecMSsamplingInterval =  root:diagnostics:nSecMSsamplingInterval		//1.14B
	endif

	wave HR_FitFail_temp = root:HR_Sticks:HR_FitFail_temp		//1.08E unsigned integer		//1.11D only used for old style non LLS fits
	wave OneRunSqBkStick_temp = root:HR_Sticks:OneRunSqBkStick_temp	//  used for the squirrel background		//1.09H
	
	// the same number of points as ExactMass	
	wave ExactMassSigma = root:HR_Sticks:ExactMassSigma	  // the same number of points as ExactMass	
	wave UnitSigma = root:HR_Sticks:UnitSigma				// the same number of points as UnitMass

	wave rn_series = root:index:rn_series
	wave series_index = root:index:series_index

	//1.12N
	wave/T HR_FinalPSNameWave = root:HR_PeakShape:HR_FinalPSNameWave
		
	nvar HR_isW = root:HR_sticks:HR_isW
	nvar HR_RunVar = root:HR_Panel:HR_RunVar
	
	nvar UsePeakShapeVar = root:HR_Panel:HR_UsePeakShapeVar
	nvar HR_FinalPSScalar = root:HR_PeakShape:HR_FinalPSScalar		// for use in calcs, not display HR_FinalPeakShapeScalar
	nvar HR_numFail = root:HR_Sticks:HR_numFail		//  1.08E
	
	variable numOfSpectra, numOfExactMass, numOfUnitMass, var, kdex
	variable jdex, PeakIndex, idex	
	variable TimeStartPnt, TimeEndPnt, background
	variable MassStartPntVar, MassEndPntVar, PeakNumVar, MassSigma
	variable currentRunIndex, intercept, slope, power, WhichSpectraType, numPToFBins=0, numRawSpectra, edex
	string GaussConstraintsStr, GaussHoldParamsStr, PToFStr
		
	if (acorn==0)
		HR_numFail = (0)			//1.08E
	endif

	numRawSpectra= dimsize(MSSpectra_p, 1)
	numOfSpectra = dimsize(MSSpectra_p, 0)
	numOfExactMass = numpnts(ExactMassWave)
	numOfUnitMass = numpnts(UnitMass)

	if (stringmatch(spectraType, "Diff"))				//  stringmatch is case insensitive
		spectraType="Diff"						// not necessary
	elseif (stringmatch(spectraType, "Closed"))	
		spectraType="Closed"	
	elseif(stringmatch(spectraType, "Open"))
		spectraType="Open"
	elseif(stringmatch(spectraType, "ePToFSum"))
		WhichSpectraType = 5	
	elseif(stringmatch(spectraType, "ePToFSumDiff"))
		WhichSpectraType = 6	
	elseif(stringmatch(spectraType[0,3], "PToF"))
		numRawSpectra= dimsize(MSSpectra_p, 2)
		numPToFBins = dimsize(MSSpectra_p,1)	  // number of new PToF bins
		if(numRawSpectra>0)
			WhichSpectraType = 7
		else
			WhichSpectraType = 4	
			numRawSpectra= numPToFBins
			numPToFBins = 0	  // number of new PToF bins
		endif
	else
		abort "There is something wrong - neither open, closed or diff spectra was identified - aborting from HR_CalculateSticksSQ"
	endif
			
	if (!WaveExists(OneRunRawMz_temp) || !WaveExists(OneRunRawSpectra_temp) )
		make/o/n=(numRawSpectra)  root:HR_Sticks:OneRunRawMz_temp/wave=OneRunRawMz_temp
		make/o/n=(numRawSpectra)  root:HR_Sticks:OneRunRawSpectra_temp/wave=OneRunRawSpectra_temp
	else
		Redimension/n=(numRawSpectra) OneRunRawSpectra_temp, OneRunRawMz_temp
	endif

	//  this destination waves for this acorn	
	if (WhichSpectraType != 7)
		make/o/n=(numOfSpectra,numOfExactMass) root:HR_Sticks:ExactTimeWave/wave=ExactTimeWave =nan
		make/o/n=(numOfSpectra,numOfExactMass) root:HR_Sticks:ExactStickWave/wave=ExactStickWave =nan
		make/o/n=(numOfSpectra,numOfUnitMass) root:HR_Sticks:Sigma/wave=Sigma	=nan
		make/o/n=(numOfSpectra,numOfUnitMass) root:HR_Sticks:ResidSum/wave=ResidSum 
		make/o/n=(numOfSpectra,numOfUnitMass) root:HR_Sticks:SqBkStick/wave=SqBkStick = nan //  used for the squirrel background  1.09H
		make/o/n=(numOfSpectra,numOfExactMass) root:HR_Sticks:FitSigErrorMatrix/wave=ExactMassFitAreaMatrix = 0
	else  //==7
		make/o/n=(numOfSpectra,numPToFBins,numOfExactMass) root:HR_Sticks:ExactTimeWave/wave=ExactTimeWave =nan
		make/o/n=(numOfSpectra,numPToFBins,numOfExactMass) root:HR_Sticks:ExactStickWave/wave=ExactStickWave =nan
		make/o/n=(numOfSpectra,numPToFBins,numOfUnitMass) root:HR_Sticks:Sigma/wave=Sigma	=nan
		make/o/n=(numOfSpectra,numPToFBins,numOfUnitMass) root:HR_Sticks:ResidSum/wave=ResidSum 
		make/o/n=(numOfSpectra,numPToFBins,numOfUnitMass) root:HR_Sticks:SqBkStick/wave=SqBkStick = nan //  used for the squirrel background  1.09H
		make/o/n=(numOfSpectra,numPToFBins,numOfExactMass) root:HR_Sticks:FitSigErrorMatrix/wave=ExactMassFitAreaMatrix = 0
	endif
	
	//1.10A added varying peak width
	make/o/n=(numOfSpectra,numOfUnitMass ) root:HR_Sticks:UMRPeakWidthInNs/Wave=UMRPeakWidthInNs
//	UMRPeakWidthInNs[][]=(index_pos[p]<0 ? 0 : HR_FinalPWInterceptWave[series_index[index_pos[p]]] + HR_FinalPWSlopeWave[series_index[index_pos[p]]] * (UnitMass[q]^HR_FinalPWPowerWave[series_index[index_pos[p]]]) )		// peak width as function of mass
	
	//1.12N
	wave ConstrainedKey = root:HR_Sticks:ConstrainedKey
	wave ExactMassLeft = root:HR_Sticks:ExactMassLeft
	wave ExactMassRight = root:HR_Sticks:ExactMassRight
	wave ExactMassFit = root:HR_Sticks:ExactMassFit
	wave ConstrainedKey = root:HR_Sticks:ConstrainedKey
	// end 1.12N
	
	variable numMoreCols = mod(numOfExactMass, ROUNDNUMHRIONCOLS)

	for (idex=0;idex<numOfSpectra;idex+=1)
	
		sq_ams_prog_upd(100*(acorn+(idex/numOfSpectra))/n_acorns)
		
		if (index_pos[idex]==-1)
			continue
		endif
		
		// first get all info that is constant for this run
		currentRunIndex = series_index[index_pos[idex]]		// Ken's prob
		HR_RunVar = rn_series[currentRunIndex]		// not necessary, just updates panel
				
		intercept = mz_FitIntercept[currentRunIndex]
		slope = mz_FitSlope[currentRunIndex]
		power = mz_FitPower[currentRunIndex]

		//1.20Y
		UMRPeakWidthInNs[idex][]= HR_FinalPWInterceptWave[currentRunIndex] + HR_FinalPWSlopeWave[currentRunIndex] * (UnitMass[q]^HR_FinalPWPowerWave[currentRunIndex])		// peak width as function of mass
		
		//  1.12N	
		wave/z finalPS = $"root:HR_PeakShape:"+HR_FinalPSNameWave[currentRunIndex]	
		if (!WaveExists(finalPS))
			abort "Could not find the peak shape wave root:HR_PeakShape:"+HR_FinalPSNameWave[currentRunIndex]+  " aborting from HR_CalculateSticksSQ"
		endif
		
		// 1.10A change for varying peak width
		UnitSigma = UMRPeakWidthInNs[idex][p]
		PK_convertUnitSigmaToEachSpecie()		// go from UnitSigma to ExactMassSigma // the same number of points as ExactMass	
		
		OneRunMassTime = intercept+slope*(ExactMassWave^power)
		// we don't care if  intercept is negative // 1.14I
		
		if (numPToFBins==0)		
			ImageTransform/G=(idex) getRow MSSpectra_p
			wave W_ExtractedRow
			multithread OneRunRawSpectra_temp = W_ExtractedRow   // 1.16K MSSpectra_p[idex][p]	 // need to think about this carefully.  The ith row needs to match index_pos[idex]
			HR_calcSticksOneRun(currentRunIndex, OneRunRawSpectra_temp, mz_FitIntercept, mz_FitSlope, mz_FitPower)//, BkOneRun, StartPtOneRun, EndPtOneRun)		
			FastOP OneRunStickArea_temp = ( HR_FinalPSScalar * sqrt(pi))*OneRunStick_temp*ExactMassSigma
			FastOP OneRunStickArea_temp = OneRunStickArea_temp*dutyCycleCorr
			OneRunStickArea_temp *= nSecMSsamplingInterval[currentRunIndex]	// 1.13D
		
			// I think these calcs are only needed for display!  // uncommented out in 1.21E
			for(edex=0;edex<numOfExactMass;edex+=1)
				var =  BinarySearchInterp(OneRunMassTime, OneRunMassTime[edex] - 5*ExactMassSigma[edex])
				ExactMassLeft[edex] = numtype(Var)==2 ? 0 : ceil(var)
				var =  BinarySearchInterp(OneRunMassTime, OneRunMassTime[edex] + 5*ExactMassSigma[edex])
				ExactMassRight[edex] =  numtype(Var)==2 ? numOfExactMass-1 : floor(var)		
			endfor		
			ExactMassFit=0			// this is the value of the intersection of the think blue line (sum of HR gaus-like fits) in the HR_PeakHeights graph and the vertical lines of the exact masses 
			for(edex=0;edex<numOfExactMass;edex+=1)
				ExactMassFit[ExactMassLeft[edex], ExactMassRight[edex]]+=OneRunStick_temp[edex] * finalPS((OneRunMassTime[p] - OneRunMassTime[edex])/ (ExactMassSigma[edex]))	
			endfor
			ExactMassFit = ConstrainedKey[p] == 1 ?  OneRunStick_temp[p] : ExactMassFit[p] // height of blue line
			FastOP ExactMassFit = ( HR_FinalPSScalar * sqrt(pi))*ExactMassFit*ExactMassSigma
			FastOP ExactMassFit = ExactMassFit*dutyCycleCorr		
			ExactMassFit *= nSecMSsamplingInterval[currentRunIndex]	// 1.13D			

			ImageTransform/G=(idex)/D=OneRunMassTime  putRow ExactTimeWave
			ImageTransform/G=(idex)/D=OneRunStickArea_temp  putRow ExactStickWave				
			ImageTransform/G=(idex)/D=OneRunResidSum_temp  putRow ResidSum
			ImageTransform/G=(idex)/D=ExactMassFit  putRow ExactMassFitAreaMatrix
		else
			for(kdex=0;kdex<numPToFBins;kdex+=1)
				ImageTransform/Beam={(idex),(kdex)} getBeam MSSpectra_p
				wave W_Beam
				multithread OneRunRawSpectra_temp = W_Beam   // 1.16K MSSpectra_p[idex][p]	 // need to think about this carefully.  The ith row needs to match index_pos[idex]
				HR_calcSticksOneRun(currentRunIndex, OneRunRawSpectra_temp, mz_FitIntercept, mz_FitSlope, mz_FitPower)//, BkOneRun, StartPtOneRun, EndPtOneRun)		
				FastOP OneRunStickArea_temp = ( HR_FinalPSScalar * sqrt(pi))*OneRunStick_temp*ExactMassSigma
				FastOP OneRunStickArea_temp = OneRunStickArea_temp*dutyCycleCorr
				OneRunStickArea_temp *= nSecMSsamplingInterval[currentRunIndex]	// 1.13D
			
					// I think these calcs are only needed for display!  // uncommented out in 1.21E
				for(edex=0;edex<numOfExactMass;edex+=1)
					var =  BinarySearchInterp(OneRunMassTime, OneRunMassTime[edex] - 5*ExactMassSigma[edex])
					ExactMassLeft[edex] = numtype(Var)==2 ? 0 : ceil(var)
					var =  BinarySearchInterp(OneRunMassTime, OneRunMassTime[edex] + 5*ExactMassSigma[edex])
					ExactMassRight[edex] =  numtype(Var)==2 ? numOfExactMass-1 : floor(var)		
				endfor		
				ExactMassFit=0			// this is the value of the intersection of the think blue line (sum of HR gaus-like fits) in the HR_PeakHeights graph and the vertical lines of the exact masses 
				for(edex=0;edex<numOfExactMass;edex+=1)
					ExactMassFit[ExactMassLeft[edex], ExactMassRight[edex]]+=OneRunStick_temp[edex] * finalPS((OneRunMassTime[p] - OneRunMassTime[edex])/ (ExactMassSigma[edex]))	
				endfor
				ExactMassFit = ConstrainedKey[p] == 1 ?  OneRunStick_temp[p] : ExactMassFit[p] // height of blue line
				FastOP ExactMassFit = ( HR_FinalPSScalar * sqrt(pi))*ExactMassFit*ExactMassSigma
				FastOP ExactMassFit = ExactMassFit*dutyCycleCorr		
				ExactMassFit *= nSecMSsamplingInterval[currentRunIndex]	// 1.13D			
				ImageTransform /Beam={(idex),(kdex)} /D=OneRunMassTime  setBeam ExactTimeWave
				ImageTransform /Beam={(idex),(kdex)} /D=OneRunStickArea_temp  setBeam ExactStickWave				
				ImageTransform /Beam={(idex),(kdex)} /D=OneRunResidSum_temp  setBeam ResidSum
				ImageTransform /Beam={(idex),(kdex)} /D=ExactMassFit  setBeam ExactMassFitAreaMatrix

			endfor
		endif

	endfor // each run	

	sq_ams_stat_upd("Writing HR stick data to hdf")

	//1.14M 'expanded' columns to the next 200 HR spectra
	if (WhichSpectraType<7)
		InsertPoints/M=1 numOfExactMass, (ROUNDNUMHRIONCOLS-numMoreCols),ExactStickWave, ExactTimeWave, ResidSum, ExactMassFitAreaMatrix
	else
		InsertPoints/M=2 numOfExactMass, (ROUNDNUMHRIONCOLS-numMoreCols),ExactStickWave, ExactTimeWave, ResidSum, ExactMassFitAreaMatrix
	endif
	
	if (WhichSpectraType==5 || WhichSpectraType==6)	// ePToFSum
		squirrel_hdf_write(index_pos,ExactStickWave,"HR"+SpectraType+"Stick")  //runs x exact mass
		squirrel_hdf_write(index_pos,ExactTimeWave,"HR"+SpectraType+"Time")  //runs x exact mass
		squirrel_hdf_write(index_pos,ResidSum,"ResidSum"+SpectraType) //runs x unit mass
		squirrel_hdf_write(index_pos,ExactMassFitAreaMatrix,"HRFitArea"+SpectraType[0]) //1.12N
	elseif(WhichSpectraType==7) 		// PToF single set  OR PToF matrix
		//1.14M 'expanded' columns to the next 200 HR spectra
		squirrel_hdf_write(index_pos,ExactStickWave,PToFdatasetStr+"Stick")  //runs x exact mass
		squirrel_hdf_write(index_pos,ExactTimeWave,PToFdatasetStr+"Time")  //runs x exact mass
		squirrel_hdf_write(index_pos,ResidSum,"ResidSum"+PToFdatasetStr) //runs x unit mass
//		squirrel_hdf_write(index_pos,ResidNoise,PToFdatasetStr+"ResidErr")  //1.12N
		squirrel_hdf_write(index_pos,ExactMassFitAreaMatrix,"HRFitArea"+PToFdatasetStr) //1.12N
//		squirrel_hdf_write(index_pos,HR_FitFail,"HR_FitFail"+PToFdatasetStr)	  //1.11D	
	elseif (WhichSpectraType!=4)		// MS Open, Closed, Diff
		//1.14M 'expanded' columns to the next 200 HR spectra
		squirrel_hdf_write(index_pos,ExactStickWave,"HR"+SpectraType+"Stick")  //runs x exact mass
		squirrel_hdf_write(index_pos,ExactTimeWave,"HR"+SpectraType+"Time")  //runs x exact mass
		squirrel_hdf_write(index_pos,ResidSum,"ResidSum"+SpectraType) //runs x unit mass
//		squirrel_hdf_write(index_pos,ResidNoise,"HRFit"+SpectraType[0]+"ResidErr") //1.04F		// changed name in 1.06I  //1.12N
		squirrel_hdf_write(index_pos,ExactMassFitAreaMatrix,"HRFitArea"+SpectraType[0]) //1.12N
//		squirrel_hdf_write(index_pos,HR_FitFail,"HR_FitFail"+SpectraType[0]) //1.04F		// changed name in 1.08E  //1.11D
	else  // WhichSpectraType==4	
		//1.14M 'expanded' columns to the next 200 HR spectra
		squirrel_hdf_write(index_pos,ExactStickWave,PToFdatasetStr+"Stick")  //runs x exact mass
		squirrel_hdf_write(index_pos,ExactTimeWave,PToFdatasetStr+"Time")  //runs x exact mass
		squirrel_hdf_write(index_pos,ResidSum,"ResidSum"+PToFdatasetStr) //runs x unit mass
//		squirrel_hdf_write(index_pos,ResidNoise,"HRFit"+SpectraType[0]+"ResidErr") //1.04F		// changed name in 1.06I  //1.12N
		squirrel_hdf_write(index_pos,ExactMassFitAreaMatrix,"HRFitArea"+PToFdatasetStr) //1.12N
//		squirrel_hdf_write(index_pos,HR_FitFail,"HR_FitFail"+SpectraType[0]) //1.04F		// changed name in 1.08E  //1.11D
	endif		

	if (acorn == n_acorns-1)
		killwaves/z W_ExtractedRow, W_beam
	endif
End



// ADDED IN 1.02A
// Function to fix ExactStickWave values that had reflected only height * duty correction, not the area - the real stick
Function PK_ConvertOldExactStickToArea()

	if ( exists("root:HR_Sticks:HeightToAreaConversionFlag") >0)
		Abort "The conversion from HRDiffStick as being heights to being areas has already been done! aborting from PK_ConvertOldExactStickToArea"
	endif
		
	wave all = root:all
	variable/g  root:HR_Sticks:HeightToAreaConversionFlag = 1
	
	PK_convertUnitSigmaToEachSpecie()
	
	// by default we do it for the all todo wave and the diff spectra
	print "// The correction to HRDiffSticks from pre Pika version 1.03 began at "+ time() + " " + date()+" for the all todo wave"
	squirrel_fetch(all, PK_HeightToAreaMatrix, "dest:HRDiffStick", "HRDiffStick")

	wave squirrel_index = root:index:squirrel_index
	if (FindDimlabel(squirrel_index,2,"HROpenStick")>=0)
		squirrel_fetch(all, PK_HeightToAreaMatrix, "dest:HROpenStick", "HROpenStick")
		squirrel_fetch(all, PK_HeightToAreaMatrix, "dest:HRClosedStick", "HRClosedStick")
	endif

	print "// The correction to HRDiffSticks from pre Pika version 1.03 finished at "+ time() + " " + date()+" for the all todo wave"  

End


// ADDED IN 1.02A
Function PK_HeightToAreaMatrix(index_pos,paramstr,acorn,n_acorns)
	wave index_pos
	string paramstr
	variable acorn 
	variable n_acorns
	
	nvar HR_FinalPSScalar = root:HR_PeakShape:HR_FinalPSScalar		// used in real calcs, unlike HR_FinalPeakShapeScalar
	wave series_index = root:index:series_index 
	wave ExactMassSigma = root:HR_PeakShape:ExactMassSigma

	string dest=stringbykey("dest",paramstr)	 
	wave ExactStickWave=$stringbykey("acorn_0",paramstr)

	variable currentRunIndex, idex, numRuns

	numRuns = dimsize(ExactStickWave,0)

	for (idex=0;idex<numRuns;idex+=1)
		sq_ams_prog_upd(100*(acorn+(idex/numRuns))/n_acorns)
		
		if (index_pos[idex]==-1)
			continue
		endif

		currentRunIndex = series_index[index_pos[idex]]

		ExactStickWave[idex][]  *= sqrt(pi) *ExactMassSigma[q]		// convert from stick height to area under gaussian
		ExactStickWave[idex][]  *= HR_FinalPSScalar					// convert from gaussian to custom peak shape

	endfor

	//1.14M 'expanded' columns to the next 200 HR spectra
	variable numHRCols = dimsize(ExactStickWave,1)
	variable numMoreCols = mod(numRuns, ROUNDNUMHRIONCOLS)

	InsertPoints/M=1 numHRCols, (ROUNDNUMHRIONCOLS-numMoreCols),  ExactStickWave
	//1.14M 'expanded' columns to the next 200 HR spectra
	squirrel_hdf_write(index_pos,ExactStickWave,dest)  // runs x exact mass

End




// ADDED IN 1.02A
// called from HR_CalculateSticksSQ_prep(), HR_butt_CalcSticksForThisRun(), PK_ConvertOldExactStickToArea()
Function PK_convertUnitSigmaToEachSpecie()

	wave ExactMassWave = root:HR_Sticks:ExactMassWave	
	wave numOfPeaks = root:HR_Sticks:numOfPeaks
	wave UnitSigma = root:HR_Sticks:UnitSigma
	
	variable idex, numUnitMzs, numExactMzs, counter, NumPeaksAtThisMz
	
	numUnitMzs = numpnts(UnitSigma)	
	numExactMzs = numpnts(ExactMassWave)
	
	make/o/n=(numExactMzs) root:HR_Sticks:ExactMassSigma
	wave ExactMassSigma = root:HR_Sticks:ExactMassSigma

	counter=0
	for (idex=0;idex<numUnitMzs;idex+=1)
		NumPeaksAtThisMz = numOfPeaks[idex]
		if (NumPeaksAtThisMz>0)
			ExactMassSigma[counter,counter+NumPeaksAtThisMz-1] = UnitSigma[idex]
			counter+=NumPeaksAtThisMz
		endif
	endfor

End


// IMPORTANT ! This function should really be called HR_CalcPEAKHEIGHTSOneRun
// Calculates exact mass values for one spectra (diff or open or closed).  Results are put in OneRunStick_temp
// Results get dumped into the same parameters ...  calling function is left to sort out what goes where (diff or open or closed)
// Not clear if we need to separate out background left and right values or could just use/save one background value.
Function HR_CalcSticksOneRun(currentRunIndex, OneRunRawSpectra_temp, mz_FitIntercept, mz_FitSlope, mz_FitPower)///, BkOneRun, StartPtOneRun, EndPtOneRun)
variable currentRunIndex
wave OneRunRawSpectra_temp// BkOneRun, StartPtOneRun, EndPtOneRun
wave mz_FitIntercept, mz_FitSlope, mz_FitPower

	wave ExactMassWave = root:HR_Sticks:ExactMassWave		// 1.08E not the same as ExactMassWave
	wave ExactMassIsotopeAbund = root:HR_Sticks:ExactMassIsotopeAbund

	wave run_series = root:index:rn_series

	wave OneRunMassTime = root:HR_Sticks:OneRunMassTime		// changes with each run, perhaps open closed.
	wave OneRunStick_temp = root:HR_Sticks:OneRunStick_temp	// placeholder for results
	wave SigmaOneRun_temp = root:HR_Sticks:SigmaOneRun_temp			// placeholder for sigma results if user chooses to vary this
 //	wave HR_BkOneRun_temp = root:HR_Sticks:HR_BkOneRun_temp			// placeholder for background results  used for original (Pete's), background, not the squirrel background

	wave HR_FitFail_temp = root:HR_Sticks:HR_FitFail_temp			// 1.08E  // 1.11D used on in old style non-LLS HR fitting

	wave/z OneRunResidSpectra_temp = root:HR_Sticks:OneRunResidSpectra_temp		// 1.36J
	wave/z OneRunResidPerSpectra_temp = root:HR_Sticks:OneRunResidPerSpectra_temp		// 1.36J
	wave/z OneRunResidSum_temp = root:HR_Sticks:OneRunResidSum_temp			// 1.36J
	if (!WaveExists(OneRunResidSpectra_temp) || !WaveExists(OneRunResidPerSpectra_temp) || !WaveExists(OneRunResidSum_temp))		//1.12F
		make/o/n=(numpnts(OneRunRawSpectra_temp)) root:HR_Sticks:OneRunResidSpectra_temp/wave=OneRunResidSpectra_temp
		make/o/n=(numpnts(OneRunRawSpectra_temp)) root:HR_Sticks:OneRunResidPerSpectra_temp/wave=OneRunResidPerSpectra_temp
		make/o/n=(numpnts(OneRunRawSpectra_temp)) root:HR_Sticks:OneRunResidSum_temp/wave=OneRunResidSum_temp
	endif

	wave MassStartPnt = root:HR_Sticks:MassStartPnt				 // the first exact mass near an integer amu (row number in Exact mass wave)
	wave MassEndPnt = root:HR_Sticks:MassEndPnt				 // the last exact mass near an integer amu (row number in Exact mass wave)
	wave UnitMass = root:HR_Sticks:UnitMass						// integer amu
//	wave UnitSigmaByMass= root:HR_Sticks:UnitSigmaByMass		// given sigma value (peak width) for integer amu
 	wave numOfPeaks = root:HR_Sticks:numOfPeaks				// number of exact masses at integer amu
	wave/t GaussConstraints = root:HR_Sticks:GaussConstraints
	wave/t GaussHoldParams = root:HR_Sticks:GaussHoldParams	// text wave (integer amu length) of parameters to hold
	
	wave MassFitOrder= root:HR_Sticks:MassFitOrder		//1.40A
	wave/t IsotopePairings= root:HR_Sticks:IsotopePairings		//1.40A
	
	wave ExactMassSigma = root:HR_Sticks:ExactMassSigma
	wave dutyCycleCorr = root:Int_Sticks:dutyCycleCorr
 					
 	wave absWave = root:HR_Sticks:absWave		// 1.03E .. an arbitrary length wave to put the absolute value of the residual
		
//	wave OneRunResidNoise_temp = root:HR_Sticks:OneRunResidNoise_temp			//1.04E
//	wave HRFitChiSq_temp = root:HR_Sticks:HRFitChiSq_temp			//1.04E

	wave rn_series = root:index:rn_series

	nvar HR_MaxNumGausFits = root:HR_Sticks:HR_MaxNumGausFits	// original default was 10
	
	nvar HR_UsePeakShapeVar = root:HR_Panel:HR_UsePeakShapeVar
	nvar HR_UseVaryingPeakWidth = root:HR_Panel:HR_UseVaryingPeakWidth

	nvar HR_UseFixedIsotopicVals = root:HR_Panel:HR_UseFixedIsotopicVals
	nvar HR_FitVerboseMode = root:HR_Panel:HR_FitVerboseMode
	nvar HR_IfFitNegConstrainToZero = root:HR_panel:HR_IfFitNegConstrainToZero		// added in 1.03C

	nvar AvgTodoFlag = root:HR_panel:AvgTodoFlag		// added in 1.10K

//	nvar HR_UseSqBaseline = root:HR_panel:HR_UseSqBaseline  //1.04B
	svar HR_FinalPSName = root:HR_PeakShape:HR_FinalPSName

	variable/g V_fitOptions = 4		// for use in FuncFit
	variable/g V_FitError			// for use in FuncFit
	variable/g V_FitTol =0.00001	// for use in FuncFit
	variable/g V_fitMaxIters = 100	// for use in FuncFit, added in 1.10D, default is 40
	variable/g V_FitQuitReason =nan	// for use in FuncFit, added in 1.10D
		
	variable num, thisAMU, firstAMU, orderedIndex
	variable slope, intercept, power, jdex, PeakIndex,sum_temp
	variable numOfExactMass, numInRawSpectra, numOfUnitMass, LowestMass, HighestMass 		
	variable PeakNumVar, MassStartPntVar, MassEndPntVar, MassSigmaVar, BackGroundVar
	variable TimeStartPnt, TimeEndPnt, BkLeftVar, BkRightVar, chisq_temp
	string GaussHoldParamsStr, GaussConstraintsStr	
	string NameOfConstraintWaveStr, isotopePairingStr
	
	variable childPt, ParentPt, ParentAbund, AMUthPt, isoItem, ParentVal, isotopesToConsider, reFitFlag, tempVar1, tempVar2, tempVar3  // numConst
	variable childVal,tempindex
	string isoStr, printStr, tempStr1, tempStr2
		
	nvar/z HR_UseLLS = root:HR_panel:HR_UseLLS		//1.11D
	if (!NVar_exists(HR_UseLLS))
		variable/g  root:HR_panel:HR_UseLLS=0
		nvar HR_UseLLS = root:HR_panel:HR_UseLLS		//1.11D
	endif
	
 	variable dutyCycleRatio, peakWidthRatio		//1.07C					

	if ( numtype(OneRunRawSpectra_temp[0]) !=0) 
		//abort "No Can do!"   1.04B change
		sprintf printstr, "Bad Raw spectra at run %d - aborting from HR_CalcSticksOneRun", rn_series[currentRunIndex]	// need to nicely format for big run numbers
//		abort printstr  	//1.09B change!  Don't abort, just print errors to history
		fastop HR_FitFail_temp = (nan)			//if this generates a compilation error commentize this line	  // used only in non-LLS fitting 1.11D
		sprintf printStr, "// Had problems at run %d with HR fitting because this spectra has nan values: ", run_series[currentRunIndex]
		 if (strSearch(NameOfWave(OneRunRawSpectra_temp), "pen", 0)>0 )
		 	printStr += "Open"
		 elseif(strSearch(NameOfWave(OneRunRawSpectra_temp), "losed", 0)>0 )
		 	printStr += "Closed"
		 else
		 	printStr += "Diff"
		 endif
		print printStr
//		HR_BkOneRun_temp = nan // used for original (Pete's), background, not the squirrel background
		fastop SigmaOneRun_temp = (nan)
		fastop OneRunResidSum_temp= (nan)
		fastop OneRunResidPerSpectra_temp= (nan)
		fastop OneRunStick_temp = (nan)	
//		HRFitChiSq_temp = nan	//1.09B   // eliminated in 1.11D
		return 0
	endif
	
	printstr=""
	
	numOfExactMass = numpnts(ExactMassWave)
	numInRawSpectra = numpnts(OneRunRawSpectra_temp)	
	numOfUnitMass =numpnts(UnitMass)
	LowestMass = UnitMass[0]
	HighestMass = UnitMass[numpnts(UnitMass)-1]

	intercept = mz_FitIntercept[currentRunIndex]
	slope = mz_FitSlope[currentRunIndex]
	power = mz_FitPower[currentRunIndex]
	
	OneRunMassTime = intercept+slope*(ExactMassWave^power)
	
	// we don't care if intercept is negative //1.14I
	
	redimension/n=(numInRawSpectra)	OneRunResidSpectra_temp, OneRunResidPerSpectra_temp		
	// OneRunResidSum_temp is the integral value
	// 1.1A ... residuals need to be resized for v and w mode switching

	fastop OneRunStick_temp = (nan)
//	HR_BkOneRun_temp=nan		// used for original (Pete's), background, not the squirrel background
	fastop SigmaOneRun_temp= (nan)
	fastop OneRunResidSum_temp= (nan)
	fastop OneRunResidSpectra_temp= (nan)		// wave length = original raw spectra wave
	fastop OneRunResidPerSpectra_temp= (nan)		// wave length = original raw spectra wave
//	fastop OneRunResidNoise_temp = (nan)		//  1.20Z 1.04E
//	HRFitChiSq_temp = nan		//1.04E  // eliminated in 1.11D
	
	fastop HR_FitFail_temp=0	//1.11D only used in non LLS HR fitting	//1.08E unsigned integer   assume all are success   here 0 is success 1 is fail
	
	wave UnitSigma = root:HR_Sticks:UnitSigma
	wave HR_finalPS = $"root:HR_PeakShape:"+HR_FinalPSName		//1.10K
	variable PSminX = DimOffset(HR_finalPS, 0)
	variable PSmaxX = PSminX+(numpnts(HR_finalPS)-1)*DimDelta(HR_finalPS, 0)
	
	wave/t GaussConstraints = root:HR_Sticks:GaussConstraints
	if (WaveExists(HR_finalPS)==0  || numpnts(HR_finalPS)==0)
		Abort "Cannot perform multi-gauss fitting without peak shape wave.  Please create and try again. Aborting from HR_CalcSticksOneRun"
	else
		wavestats/q/m=1 HR_finalPS
		if (V_numNans>0)
			Abort "Cannot perform multi-gauss fitting with nans in the peak shape wave.  Please create and try again. Aborting from HR_CalcSticksOneRun"
		endif
	endif

	firstAMU=UnitMass[0]
	
//	if (AvgTodoFlag)
//		wave todoPtHistMatrix = root:HR_sticks:todoPtHistMatrix		// 2D matrix rows  = mz, cols are normalized point histograms  1.10K
//		duplicate/o HR_finalPS root:HR_PeakShape:PeakShape_orig/Wave=PeakShape_orig
//		make/o/n=(dimSize(todoPtHistMatrix,1)) mzhist
//	endif

	if (HR_UseLLS) 	//1.11D
		SetScale/P x 0,10^(-09),  OneRunRawSpectra_temp
		duplicate/o OneRunMassTime root:HR_sticks:OneRunMassTimeNs/wave=OneRunMassTimeNs
		make/o/n=(numInRawSpectra) root:HR_sticks:OneRunMassMz/wave=OneRunMassMz
		multithread OneRunMassMz = ((p - intercept)/slope)^(1/power)			 //1.57K 
		if ( intercept>0)			//1.54A		//1.14I
			OneRunMassMz[0,ceil(intercept)] =0  // start at least at mz = 1			
		endif
		OneRunMassTimeNs*=10^(-9)
		variable unitSigmaNs
		wave HR_finalPS = root:HR_PeakShape:HR_finalPS
		variable negpeaks_flag=0
		variable err
		variable StartPt, EndPt
	endif

	// step through all mass integer (unit) values for this run	
	for(jdex=0; jdex<numOfUnitMass; jdex+=1)			//1.12N

		thisAMU = MassFitOrder[jdex]		// significant change in 1.40A	
		isotopePairingStr= IsotopePairings[jdex]	
		
		orderedIndex = thisAMU-firstAMU
				
		PeakNumVar = numOfPeaks[orderedIndex]

		// gauss function can only deal with HR_MaxNumGausFits or fewer peaks.
		if  (PeakNumVar > HR_MaxNumGausFits)
			printStr+=num2str(thisAMU)+","
			HR_FitFail_temp[orderedIndex]=1		//1.17c
			continue
		endif
		
//		if (thisAMU > 110)
//			print thisAMU
//		endif
		
		if (HR_UseLLS) 		//1.11D
			if (PeakNumVar  > 0)
				unitSigmaNs = UnitSigma[orderedIndex]*10^(-9)
				
				MassStartPntVar = MassStartPnt[orderedIndex]		// note that these values are dependent upon the exact masses selected
				MassEndPntVar = MassEndPnt[orderedIndex]
				StartPt = max(0,binarySearch(OneRunMassMz, ExactMassWave[MassStartPntVar]) - 50)		// original value was 10.  Changed to 50 in 1.10R
				EndPt = min(binarySearch(OneRunMassMz, ExactMassWave[MassEndPntVar]) + 50,numInRawSpectra-1)//1.22A			
				
				make/o/n=(PeakNumVar) PeakHeights, PeakCenters, peakIndexWave
				peakIndexWave = p
				PeakHeights = OneRunRawSpectra_temp(OneRunMassTimeNs[MassStartPntVar +p])		// initialize guesses for peak heights
				PeakCenters = OneRunMassTimeNs[MassStartPntVar +p]

				OneRunStick_temp[MassStartPntVar,MassStartPntVar+PeakNumVar-1]=0

				isotopesToConsider =  ItemsinList(isotopePairingStr)
				if (isotopesToConsider>0 && HR_UseFixedIsotopicVals )		// we need to constrain isotopes
					 for (isoItem=0;isoItem <isotopesToConsider; isoItem+=1)
//					 	numConst=numpnts(ConstraintWave)
					 	isoStr = StringFromList(isoItem, isotopePairingStr)
					 	childPt=str2num(StringFromList(0, isoStr,"/"))
					 	ParentPt=str2num(StringFromList(1, isoStr,"/"))
					 	AMUthPt=str2num(StringFromList(2, isoStr,"/"))
					 	ParentAbund = ExactMassIsotopeAbund[childPt]	
	 					dutyCycleRatio = dutyCycleCorr[ParentPt]/dutyCycleCorr[childPt]
	 					peakWidthRatio =  ExactMassSigma[ParentPt]/ExactMassSigma[childPt]				
					 	ParentVal = ParentAbund*OneRunStick_temp[ParentPt] * dutyCycleRatio * peakWidthRatio	//1.07C correction.  We need to consider the peak areas (duty cycle correction and peak width)
					 	ChildVal = ParentAbund*OneRunStick_temp[ParentPt] * dutyCycleRatio * peakWidthRatio	//1.07C correction.  We need to consider the peak areas (duty cycle correction and peak width)
	 					tempindex = childPt-(MassStartPntVar+isoItem)
	 					duplicate/free/o/r=[StartPt,EndPt] OneRunRawSpectra_temp mytempWave
	 					duplicate/free/o mytempWave HR_finalPSindex
	 					mytempWave = (x-PeakCenters[tempindex])/unitSigmaNs
	 					HR_finalPSindex = limit(mytempWave[p], PSminX, PSmaxX)
	 					mytempWave =ChildVal* HR_finalPS(HR_finalPSindex[p])   //1.57K 1.58G
                    // we can't do this if we press the redo fits button
	 					OneRunRawSpectra_temp[StartPt,EndPt]-=mytempWave[p-StartPt]	// subtract out peak height for this constrained HR ion                            		
                   	OneRunStick_temp[childPt] =  ChildVal  //* ExactMassSigma[ParentPt]/ExactMassSigma[childPt]		
                    DeletePoints tempindex,1, PeakHeights, PeakCenters, peakIndexWave // remove most negative peak
					endfor
				endif				
																
				do // loop until no negative heights or no peaks to fit are left, removing most negative peak first
                   if (numpnts(peakIndexWave)>0)
                   		err = tw_calc_peak_heights(PeakHeights, OneRunRawSpectra_temp, PeakCenters, HR_finalPS, StartPt, EndPt, unitSigmaNs, OneRunResidSpectra_temp, 0, 1)
	                    WaveStats/Q/M=1 PeakHeights
		                 // HS r47 don't delete if negative peak heights are allowed, also don't loop
	                		if (V_min <0 && !negpeaks_flag) // && sh.LLSnegPeaks_flag) // negative peak(s) found
	                      	DeletePoints V_minloc,1, PeakHeights, PeaKCenters, peakIndexWave // remove most negative peak
	                		endif
                    endif
  
            while (V_min<0 && numpnts(peakIndexWave)>0 && !negpeaks_flag) //  && sh.LLSnegPeaks_flag
           
            // for remaining HR ions (whether they have been removed due to heights being <0 *or* removed due to isotopic constraining,
            if (numpnts(PeakHeights)>0) // HS r14 assign remaining coefficients to correct positions in original coefw
                  for (PeakIndex=0; PeakIndex<numpnts(peakIndexWave); PeakIndex+=1)
                    	OneRunStick_temp[MassStartPntVar+peakIndexWave[PeakIndex]] = PeakHeights[PeakIndex]
                endfor
            endif
	
				TimeStartPnt = round(OneRunMassTime[MassStartPntVar]) - 10 		// original value was 10.  Changed to 50 in 1.10R  Changed back in 1.11H
				TimeEndPnt = round(OneRunMassTime[MassEndPntVar]) +10				
		
             sum_temp=sum(OneRunStick_temp, MassStartPntVar, MassStartPntVar+PeakNumVar-1)
				OneRunResidPerSpectra_temp[TimeStartPnt,TimeEndPnt]=OneRunResidSpectra_temp[p]/sum_temp
				
				fastop absWave = 0		// change in 1.16H    //1.57K 
				absWave[,TimeEndPnt - TimeStartPnt] = abs(OneRunResidSpectra_temp[p+TimeStartPnt])		// it is easier to put new values in a wave than to redimension it
				OneRunResidSum_temp[orderedIndex] = sum(absWave)	
			endif	// 			if (PeakNumVar  > 0)

		else	  // use old school fitting	
			if (PeakNumVar  > 0)		
										
				Wave GaussFitParams = $"root:HR_sticks:GausFitParam_"+HR_gen_Num2Str2Char(PeakNumVar)
						
				MassSigmaVar = UnitSigma[orderedIndex]			//   /(sqrt(2))
	
				GaussHoldParamsStr = GaussHoldParams[orderedIndex]
	
				GaussConstraintsStr = pk_List2textWave(GaussConstraints[orderedIndex], "ConstraintWave") // the input for the funcfit must be a text wave not a string
				wave/t ConstraintWave = $GaussConstraintsStr
							
				isotopesToConsider =  ItemsinList(isotopePairingStr)
				if (isotopesToConsider>0 && HR_UseFixedIsotopicVals )		// we need to constrain isotopes
					 for (isoItem=0;isoItem <isotopesToConsider; isoItem+=1)
//					 	numConst=numpnts(ConstraintWave)
					 	isoStr = StringFromList(isoItem, isotopePairingStr)
					 	childPt=str2num(StringFromList(0, isoStr,"/"))
					 	ParentPt=str2num(StringFromList(1, isoStr,"/"))
					 	AMUthPt=str2num(StringFromList(2, isoStr,"/"))
					 	ParentAbund = ExactMassIsotopeAbund[childPt]	
	 					dutyCycleRatio = dutyCycleCorr[ParentPt]/dutyCycleCorr[childPt]
	 					peakWidthRatio =  ExactMassSigma[ParentPt]/ExactMassSigma[childPt]				
					 	ChildVal = ParentAbund*OneRunStick_temp[ParentPt] * dutyCycleRatio * peakWidthRatio	//1.07C correction.  We need to consider the peak areas (duty cycle correction and peak width)
	 
					 	nvar myK=$"root:HR_Sticks:myK"+num2str(2*(AMUthPt)+2)		// 1.10D
					 	myK= ChildVal //  ParentVal		 					//1.40A
					endfor
	
				endif
				
				MassStartPntVar = MassStartPnt[orderedIndex]		// note that these values are dependent upon the exact masses selected
				MassEndPntVar = MassEndPnt[orderedIndex]
	
	// by default go out 10 points to the left and the right of the biggest and smallest exact masses at this amu
				TimeStartPnt = round(OneRunMassTime[MassStartPntVar]) - 10 		// original value was 10.  Changed to 50 in 1.10R  Changed back in 1.11H
				TimeEndPnt = round(OneRunMassTime[MassEndPntVar]) +10				
	
				GaussFitParams=0					// important
					
	//			if (HR_UseSqBaseline)		// keep this as the default in 1.09H
					GaussFitParams[0] = 0
	//			else
	//				BkLeftVar = sum(OneRunRawSpectra_temp, TimeStartPnt -20, TimeStartPnt -11) 
	//				BkLeftVar /= 10				
	//				BkRightVar = sum(OneRunRawSpectra_temp, TimeEndPnt +11, TimeEndPnt + 20) 
	//				BkRightVar /= 10
	//				BackGroundVar=  ( BkLeftVar +BkRightVar )/2			
	//				GaussFitParams[0] = BackGroundVar	// unit mass background
	//			endif
				
				GaussFitParams[1] = MassSigmaVar		//individual peak width for masses at this amu //V or W Mode Sigma
				
				for(PeakIndex=0; PeakIndex < PeakNumVar; PeakIndex +=1)
					// Giving initial guesses for individual HR peaks (center and height)
					GaussFitParams[PeakIndex*2 + 3] = OneRunMassTime[MassStartPntVar + PeakIndex]		// peak center in time space
					if (stringmatch(GaussHoldParamsStr[PeakIndex*2 + 2], "1") )
						nvar myKx = $"root:HR_Sticks:myK"+num2str(PeakIndex*2 +2)		//1.10D
						GaussFitParams[PeakIndex*2 + 2] =myKx
					else
						GaussFitParams[PeakIndex*2 + 2] = max(0, OneRunRawSpectra_temp[OneRunMassTime[MassStartPntVar + PeakIndex]])	// peak height	// notice that some interpolating is going on	
					endif
				endfor
	
				V_FitError=0
				// HEAR YE, HEAR YE, we are finally doing the pika fit.
				chisq_temp = HR_OneRunPerformFit(GaussHoldParamsStr, GaussFitParams, OneRunRawSpectra_temp, TimeStartPnt, TimeEndPnt, ConstraintWave,OneRunResidSpectra_temp, PeakNumVar )			
		
				if (V_FitError==0)		// we have a good fit
					
//					HRFitChiSq_temp[orderedIndex] = chisq_temp	//1.04G
//	
//	//				HR_BkOneRun_temp[orderedIndex] = GaussFitParams[0]
					SigmaOneRun_temp[orderedIndex] = GaussFitParams[1]
//					absWave = 0		// change in 1.03E
//					absWave[,TimeEndPnt - TimeStartPnt] = abs(OneRunResidSpectra_temp[p+TimeStartPnt])		// it is easier to put new values in a wave than to redimension it
//					OneRunResidSum_temp[orderedIndex] = sum(absWave)	
					for(PeakIndex=0; PeakIndex < PeakNumVar; PeakIndex +=1)		// for each HR ion at this integer mz
						OneRunStick_temp[MassStartPntVar + PeakIndex] = GaussFitParams[PeakIndex*2 + 2] 
//						tempVar1 =round(OneRunMassTime[MassStartPntVar + PeakIndex]) - TimeStartPnt
//						tempVar2= round(2*MassSigmaVar)
//						tempVar3 = sum(absWave, tempVar1-tempVar2, tempVar1+tempVar2)/(1+2*tempVar2)
//						OneRunResidNoise_temp[MassStartPntVar + PeakIndex] = tempVar3			// just to confirm peakheights
					endfor				
					sum_temp=sum(OneRunStick_temp, MassStartPntVar, MassStartPntVar+PeakNumVar-1)
					OneRunResidPerSpectra_temp[TimeStartPnt,TimeEndPnt]=OneRunResidSpectra_temp[p]/sum_temp
					
					// added in 1.10R			bailed!
					// OneRunStick_temp[MassStartPntVar, MassStartPntVar+PeakNumVar-1],  have all peak heights (constrained or no)
					// the section or raw spectra we are dealing with is at OneRunRawSpectra_temp from	
					//				TimeStartPnt = round(OneRunMassTime[MassStartPntVar]) - 50 		// original value was 10.  Changed to 50 in 1.10R
					//				TimeEndPnt = round(OneRunMassTime[MassEndPntVar]) +50	
					//  				gausFit_XTime = 	p* (TimeEndPnt-TimeStartPnt))HRSticksDisplayXResolution
					//				myHRStickStartIndex = UnitMassNumPeakSum[UnitMassIndex]+ q
					//				peakCentersInDeltaTIme=OneRunMassTime[myHRStickStartIndex+ q]
					//				GausFit_Y = 0	
					//				GausFit_Y[][0, numPeaksThisAmu-1] = HR stick heights[q] * PeakShape((gausFit_XTime[p] - peakCentersInDeltaTImeq])/ (PeakWidth))
					//				GausFit_Y[][0, numPeaksThisAmu-1] = OneRunStick_temp[myHRStickStartIndex+ q] * PeakShape((gausFit_XTime[p] - OneRunMassTime[myHRStickStartIndex+ q])/ (PeakWidth))	
	
				else
					HR_FitFail_temp[orderedIndex] =1			//1.08E   used only in old style, non LLS HR fitting
					if (HR_FitVerboseMode)
					 	sprintf printStr, "// Had problems at run %d with HR fitting at m/z %d between points %d and %d for this type of spectra: ", run_series[currentRunIndex], thisAMU, TimeStartPnt, TimeEndPnt
						 if (strSearch(NameOfWave(OneRunRawSpectra_temp), "pen", 0)>0 )
						 	printStr += "Open"
						 elseif(strSearch(NameOfWave(OneRunRawSpectra_temp), "losed", 0)>0 )
						 	printStr += "Closed"
						 else
						 	printStr += "Diff"
						 endif
						print printStr, "V_FitQuitReason:", V_FitQuitReason
					endif 
	//				HR_BkOneRun_temp[orderedIndex] = nan		// used for original (Pete's), background, not the squirrel background
					SigmaOneRun_temp[orderedIndex] = nan
					OneRunResidSum_temp[orderedIndex] = nan
					OneRunResidPerSpectra_temp[TimeStartPnt,TimeEndPnt]=nan
					for(PeakIndex=0; PeakIndex < PeakNumVar; PeakIndex +=1)
						OneRunStick_temp[MassStartPntVar + PeakIndex] =nan		// careful with indexing here
					endfor
//					HRFitChiSq_temp[orderedIndex] = nan	//1.04G   // eliminated in 1.11D
				endif
				
			endif 		// useLLS
			
		endif 
	
		MassSigmaVar = UnitSigma[orderedIndex]			//   /(sqrt(2))
		SigmaOneRun_temp[orderedIndex] = MassSigmaVar
	endfor  // unit mass index
		
	killwaves/z ConstraintWave, peakIndexWave,PeakHeights, PeakCenters, myTempTodo
		
End


Function HR_OneRunPerformFit(GaussHoldParamsStr, GaussFitParams, OneRunRawSpectra_temp, TimeStartPnt, TimeEndPnt, ConstraintWave,OneRunResidSpectra_temp, PeakNumVar )
wave/d GaussFitParams, OneRunRawSpectra_temp, OneRunResidSpectra_temp
string GaussHoldParamsStr
variable TimeStartPnt, TimeEndPnt, PeakNumVar
wave/t ConstraintWave

	nvar HR_UsePeakShapeVar = root:HR_Panel:HR_UsePeakShapeVar
	nvar HR_IfFitNegConstrainToZero = root:HR_Panel:HR_IfFitNegConstrainToZero
	nvar HR_MinimumPeakHeight = root:HR_panel:HR_MinimumPeakHeight
	
	variable/g V_FitError			// for use in FuncFit
//	variable/g V_chisq	// for use in FuncFit

	variable PeakIndex, refitFlag
		
	//  FuncFit doesn't allow one to pass the peak shape wave as a parameter... too bad
	//W=0 autodismisses curvefit window, /n surpresses updates  /h holds params identified by 1 in string, function, parameterwave, datawave		
	if (HR_UsePeakShapeVar==0)
		FuncFit  /Q/W=0/N/H=GaussHoldParamsStr HR_ManyGauss GaussFitParams  OneRunRawSpectra_temp[TimeStartPnt,TimeEndPnt]/C=ConstraintWave/R=OneRunResidSpectra_temp
	else
		FuncFit  /Q/W=0/N/H=GaussHoldParamsStr HR_ManyGaussWPS GaussFitParams  OneRunRawSpectra_temp[TimeStartPnt,TimeEndPnt]/C=ConstraintWave/R=OneRunResidSpectra_temp
	endif
							
	if (V_FitError!=0 && HR_IfFitNegConstrainToZero) // we did not successfully fit, but want to try once more to not get negs 

		// we need to remove the constraint Kx >0 and change the value in the hold string		
		reFitFlag = 0
		for(PeakIndex=PeakNumVar-1; PeakIndex >=0; PeakIndex -=1)
			if (GaussFitParams[PeakIndex*2 + 2] < HR_MinimumPeakHeight)		// just to confirm peakheights
				deletepoints PeakIndex, 1, ConstraintWave
				GaussHoldParamsStr[PeakIndex*2 + 2, PeakIndex*2 + 2]="1"
				GaussFitParams[PeakIndex*2 + 2] = 0 		// could possibly set to HR_MinimumPeakHeight
				reFitFlag +=1
			endif
		endfor

		if (refitFlag > 0)		// previous fit had negs
			
			if (refitFlag < PeakNumVar) // there were some parameters still left to fit
				if (HR_UsePeakShapeVar==0)
					FuncFit  /W=0/Q/N/H=GaussHoldParamsStr HR_ManyGauss GaussFitParams  OneRunRawSpectra_temp[TimeStartPnt,TimeEndPnt]/C=ConstraintWave/R=OneRunResidSpectra_temp
				else
					FuncFit  /W=0/Q/N/H=GaussHoldParamsStr HR_ManyGaussWPS GaussFitParams  OneRunRawSpectra_temp[TimeStartPnt,TimeEndPnt]/C=ConstraintWave/R=OneRunResidSpectra_temp
				endif
			else // force things to appear as if it was a bad fit 	
				V_fitError = 1	
				OneRunResidSpectra_temp[TimeStartPnt,TimeEndPnt]=nan	
			endif // try to refit 			
		
		endif	// we had negs
		
	endif
	return V_chisq
	
End


// Resizes and nans waves, sets raw spectra waves; all in anticipation for exact mass stick calc
Function HR_SticksOneRun_prep(UseExistingSpectraFlag)
variable UseExistingSpectraFlag

	wave ExactMassWave = root:HR_sticks:ExactMassWave		// 1.08E not the same as ExactMassWave
	wave rn_series = root:index:rn_series	
	wave run_index = root:index:run_index	
	wave UnitMass = root:HR_Sticks:UnitMass
	wave squirrel_index = root:index:squirrel_index	

	wave l_series=root:index:l_series
	wave squirrel_index=root:index:squirrel_index

	nvar HR_RunVar = root:HR_Panel:HR_RunVar
	nvar HR_checkDoNotGrabRawSpectra = root:HR_Panel:HR_checkDoNotGrabRawSpectra		// 1.09H variable name should be Single run or todo average
	nvar HR_UseOpenClosedMzParams = root:HR_Panel:HR_UseOpenClosedMzParams

	nvar HR_singleSpectraMSorPToF = root:HR_panel:HR_singleSpectraMSorPToF		// 1.10Q
  	svar sq_todoWvNm = root:panel:sq_todoWvNm
	wave todo = $sq_todoWvNm

	wave OneRunSqBkStickOpen = root:HR_Sticks:OneRunSqBkStickOpen
	wave OneRunSqBkStickClosed = root:HR_Sticks:OneRunSqBkStickClosed
	wave OneRunSqBkStickDiff = root:HR_Sticks:OneRunSqBkStickDiff
	wave OneRunSqBkStick_temp = root:HR_Sticks:OneRunSqBkStick_temp

	sq_initializeGlobalNVar("root:HR_Sticks:HR_ShowVerGrids", 0)	// added in 104J - in prep for display
	
	variable runPos, npnts, numInExactMassWave, numInUnitMass, numInRawSpectra, MSClosedIndex //109H
	variable RunIndex
	string paramStr
	string OparamStr, CparamStr, mzStringO, mzStringC		//1.15E
	
	numInExactMassWave = numpnts(ExactMassWave)
	numInUnitMass = numpnts(UnitMass)
	
	 if (HR_checkDoNotGrabRawSpectra)	//1.09H The name HR_checkDoNotGrabRawSpectra should be Single run or average	
		 HR_RunVar = todo[0]
	 endif

	if (HR_UseOpenClosedMzParams && HR_singleSpectraMSorPToF)  // only use separate open and closed if it is MS and checked
		wave mz_FitInterceptOpen = root:mzFitting:mz_FitInterceptOpen
		wave mz_FitSlopeOpen = root:mzFitting:mz_FitSlopeOpen
		wave mz_FitPowerOpen = root:mzFitting:mz_FitPowerOpen
		wave mz_FitInterceptClosed = root:mzFitting:mz_FitInterceptClosed
		wave mz_FitSlopeClosed = root:mzFitting:mz_FitSlopeClosed
		wave mz_FitPowerClosed = root:mzFitting:mz_FitPowerClosed
	else
		wave mz_FitInterceptOpen = root:mzFitting:mz_FitIntercept
		wave mz_FitSlopeOpen = root:mzFitting:mz_FitSlope
		wave mz_FitPowerOpen = root:mzFitting:mz_FitPower
		wave mz_FitInterceptClosed = root:mzFitting:mz_FitIntercept
		wave mz_FitSlopeClosed = root:mzFitting:mz_FitSlope
		wave mz_FitPowerClosed = root:mzFitting:mz_FitPower
	endif
	mzStringO = "mz_w_op:0;new_mz_cal:1;mzSlope:"+ReplaceString(":",GetWavesDataFolder(mz_FitSlopeOpen,2), "/") +";mzInt:"+ReplaceString(":",GetWavesDataFolder(mz_FitInterceptOpen,2), "/")+";mzPower:"+ReplaceString(":",GetWavesDataFolder(mz_FitPowerOpen,2), "/")+";" 
	mzStringC = "mz_w_op:0;new_mz_cal:1;mzSlope:"+ReplaceString(":",GetWavesDataFolder(mz_FitSlopeClosed,2), "/") +";mzInt:"+ReplaceString(":",GetWavesDataFolder(mz_FitInterceptClosed,2), "/")+";mzPower:"+ReplaceString(":",GetWavesDataFolder(mz_FitPowerClosed,2), "/")+";"
	
	UM_make3DColorWave()		// only needed when we do one spectra		//1.08E
 	
	//1.08E
	make/o/n=(numInExactMassWave) root:HR_Sticks:OneRunDisplayVertGrid=1	// these vertical lines will be used on plots with raw spectra and sticks
	
	RunPos =sq_FindRun(rn_series, HR_RunVar)
	RunIndex =sq_FindRun(run_index, HR_RunVar)

	// OneRunMassTime is a temporary wave to set the HR guides. can be changed later
	make/o/n=(numInExactMassWave)  root:HR_sticks:OneRunMassTime
	wave OneRunMassTime = root:HR_sticks:OneRunMassTime
	OneRunMassTime =  mz_FitInterceptOpen[runPos]+mz_FitSlopeOpen[runPos]*(ExactMassWave^mz_FitPowerOpen[runPos])

	redimension/n=(numInExactMassWave) OneRunSqBkStickOpen, OneRunSqBkStickClosed, OneRunSqBkStickDiff,  OneRunSqBkStick_temp	//1.09H
	
	sq_initializeGlobalNVar("root:HR_panel:AvgTodoFlag",0)	// 1.10K
	nvar  AvgTodoFlag = root:HR_panel:AvgTodoFlag 

//		WHERE HR PTOF is different 
 	// MS
 	if (HR_singleSpectraMSorPToF== 1 || HR_singleSpectraMSorPToF==2) // MS or ePToFSum
		MSClosedIndex = findDimLabel(squirrel_index, 2, "MSclosed_p")
	else	
		ControlInfo/W=HR_Analysis_Panel HRFit_popup_PToFClosed
		string ClosedStr = "PToF"+S_value		//1.15D
		MSClosedIndex = FindDimLabel(squirrel_index,2,ClosedStr)
		ControlInfo/W=HR_Analysis_Panel HRFit_popup_PToFOpen
		string OpenStr = "PToF"+S_value	//1.15D
		variable MSMxIndex = FindDimLabel(squirrel_index,2,OpenStr)
		ControlInfo/W=HR_Analysis_Panel HRFit_setvar_MatrixBin
		variable MxBin = V_value
		variable numInRawOSpectra=squirrel_index[RunIndex][4][MSMxIndex]	//1.10C fix  think I found a typo in 1.09H  series_index was l_series
		variable maxMxBin =squirrel_index[RunIndex][4][MSMxIndex] 
		if (maxMxBin<=0)
			abort "There doesn't appear to be raw PToF spectra for this run, this matrix."
		endif
		if (maxMxBin<=MxBin)
			DoAlert 1, "The maximum bin for this PToF matrix is "+num2istr(maxMxBin-1)+" not "+num2istr(MxBin)+". Change to "+num2istr(maxMxBin-1)+" and continue?"
			if (V_flag==2)
				abort "The user aborted"
			endif
			SetVariable HRFit_setvar_MatrixBin, value=_Num:(maxMxBin-1),Win=HR_Analysis_Panel
			MxBin=	maxMxBin-1
		endif
	endif
	
	//1.14G
	wave Diag_numPtsRawSpectra = root:diagnostics:numPtsRawSpectra
	variable FromDiagnosticNumPtsRawSpectra
	
	if (!UseExistingSpectraFlag)		// we need to grab a new spectra		//1.11G
	
// Perhaps one day will redo this to be a bit more streamlined.
//		1. Get mz wave
//		2. Get raw spectra without baseline removed
//		if using SQ baseline:
//			3. get raw spectra with baseline removed
//			4. calc baseline wave
//		else 
//			create a squirrel baseline wave using the existing parameters in the baseline panel 
//		5. calc baseline sticks
//	(even though these waves aren't used in the calcs of the HR sticks, we still need this wave to exist for the HR_peakHeights_gr
		if (!HR_checkDoNotGrabRawSpectra)	// onerun 	// HR_checkDoNotGrabRawSpectra not nicely named.  We want to do one run, not a todo wave

			AvgTodoFlag=0  //1.10K
			
			// added in 1.05A  These are waves that will only be shown if the SQ baseline was used and the checkbox "Show pika fits + squirrel baseline" is checked
			if (HR_singleSpectraMSorPToF==1)  // MS
				sq_GetThisRawMS(HR_RunVar, "HR_sticks", "OneRunRawSpectraClosedp", "MSClosed_p;")
				sq_GetThisRawMS(HR_RunVar, "HR_sticks", "OneRunRawSpectraOpenp", "MSOpen_p;")
				sq_GetThisRawMS(HR_RunVar, "HR_sticks", "OneRunRawSpectraDiffp", "MSDiff_p;")
				wave OneRunRawSpectraOpenp =root:HR_sticks:OneRunRawSpectraOpenp
				wave OneRunRawSpectraClosedp =root:HR_sticks:OneRunRawSpectraClosedp
				wave OneRunRawSpectraDiffp =root:HR_sticks:OneRunRawSpectraDiffp
			elseif (HR_singleSpectraMSorPToF==0) 		// PTOF							
				sq_GetThisRawMS(HR_RunVar, "HR_sticks", "OneRunRawSpectraClosedp", ClosedStr)
				sq_GetThisRawMS(HR_RunVar, "HR_sticks", "OneRunRawSpectraOpenp", OpenStr, PToFFlag=1, PToFBin = MxBin)
				wave OneRunRawSpectraOpenp =root:HR_sticks:OneRunRawSpectraOpenp
				wave OneRunRawSpectraClosedp =root:HR_sticks:OneRunRawSpectraClosedp
				//1.19E
				duplicate/o OneRunRawSpectraOpenp  root:HR_sticks:OneRunRawSpectraDiffp/wave=OneRunRawSpectraDiffp
				numInRawSpectra = numpnts(OneRunRawSpectraOpenp)
				if (numInRawSpectra !=numpnts(OneRunRawSpectraClosedp))
					numInRawSpectra = min(numpnts(OneRunRawSpectraClosedp),numpnts(OneRunRawSpectraOpenp) )
					redimension/n=(numInRawSpectra) OneRunRawSpectraOpenp, OneRunRawSpectraClosedp, OneRunRawSpectraDiffp
				endif	
				fastop OneRunRawSpectraDiffp = OneRunRawSpectraOpenp - OneRunRawSpectraClosedp
			elseif (HR_singleSpectraMSorPToF==2) 		// ePTOF							
				sq_GetThisRawMS(HR_RunVar, "HR_sticks", "OneRunRawSpectraClosedp", "MSClosed_p;")
				sq_GetThisRawMS(HR_RunVar, "HR_sticks", "OneRunRawSpectraOpenp", "MSePToFSum_p;")
				sq_GetThisRawMS(HR_RunVar, "HR_sticks", "OneRunRawSpectraDiffp", "MSePToFSumDiff_p;")
				wave OneRunRawSpectraOpenp =root:HR_sticks:OneRunRawSpectraOpenp
				wave OneRunRawSpectraClosedp =root:HR_sticks:OneRunRawSpectraClosedp
				wave OneRunRawSpectraDiffp =root:HR_sticks:OneRunRawSpectraDiffp
			endif
			
			if (!(WaveMax(OneRunRawSpectraOpenp)>0) )		//1.22A
				Abort "There is no raw 'open' spectra for this run.  Can you see the raw spectra in the baseline panel?"	
			elseif( !(WaveMax(OneRunRawSpectraOpenp)>0) )
				Abort "There is no raw 'closed' spectra for this run.  Can you see the raw spectra in the baseline panel?"	
			endif		
			
			// 1.14G
			FromDiagnosticNumPtsRawSpectra = Diag_numPtsRawSpectra[runPos]
			numInRawSpectra = numpnts(OneRunRawSpectraOpenp)
			
			if ( FromDiagnosticNumPtsRawSpectra != numInRawSpectra ||  FromDiagnosticNumPtsRawSpectra != numpnts(OneRunRawSpectraClosedp) || FromDiagnosticNumPtsRawSpectra != numpnts(OneRunRawSpectraDiffp) )
				numInRawSpectra = min(numInRawSpectra, FromDiagnosticNumPtsRawSpectra)
				redimension/n=(numInRawSpectra) OneRunRawSpectraOpenp, OneRunRawSpectraClosedp, OneRunRawSpectraDiffp
			endif

			make/o/n=(numInRawSpectra) root:HR_sticks:OneRunRawClosedMz/Wave=OneRunRawClosedMz
			make/o/n=(numInRawSpectra) root:HR_sticks:OneRunRawOpenMz/Wave=OneRunRawOpenMz
			make/o/n=(numInRawSpectra) root:HR_sticks:OneRunRawDiffMz/Wave=OneRunRawDiffMz
			OneRunRawOpenMz=( (p-mz_FitInterceptOpen[runPos])/mz_FitSlopeOpen[runPos])^(1/mz_FitPowerOpen[runPos])		// used for plotting purposes only!!
			OneRunRawClosedMz=( (p-mz_FitInterceptClosed[runPos])/mz_FitSlopeClosed[runPos])^(1/mz_FitPowerClosed[runPos])		// used for plotting purposes only!!
			if ( mz_FitInterceptClosed[runPos]>0)			//1.54A  //1.14I
				OneRunRawClosedMz[0,ceil(mz_FitInterceptClosed[runPos])] =0 		
			endif
			fastop OneRunRawDiffMz=OneRunRawOpenMz	// used for plotting purposes only!!

			// get the squirrel baseline fit waves ready.  For the moment, simply duplicate the preprocessed waves
			duplicate/o OneRunRawSpectraClosedp root:HR_sticks:OneRunRawSpectraClosedBkFt/Wave=OneRunRawSpectraClosedBkFt
			duplicate/o OneRunRawSpectraOpenp root:HR_sticks:OneRunRawSpectraOpenBkFt/Wave=OneRunRawSpectraOpenBkFt
			duplicate/o OneRunRawSpectraDiffp root:HR_sticks:OneRunRawSpectraDiffBkFt/Wave=OneRunRawSpectraDiffBkFt

			//  these are the waves that will be fit
			if (HR_singleSpectraMSorPToF==1)  // MS
				sq_GetThisRawMS(HR_RunVar, "HR_sticks", "OneRunRawSpectraClosed", "MSClosedLessBaseL_p;")
				sq_GetThisRawMS(HR_RunVar, "HR_sticks", "OneRunRawSpectraOpen", "MSOpenLessBaseL_p;")
				sq_GetThisRawMS(HR_RunVar, "HR_sticks", "OneRunRawSpectraDiff", "MSDiffLessBaseL_p;")
				wave OneRunRawSpectraClosed =root:HR_sticks:OneRunRawSpectraClosed
				wave OneRunRawSpectraOpen =root:HR_sticks:OneRunRawSpectraOpen
				wave OneRunRawSpectraDiff =root:HR_sticks:OneRunRawSpectraDiff
			elseif (HR_singleSpectraMSorPToF==0) 		// PTOF							
				sq_GetThisRawMS(HR_RunVar, "HR_sticks", "OneRunRawSpectraClosed", ClosedStr+"_BL")
//				sq_GetThisRawMS(HR_RunVar, "HR_sticks", "OneRunRawSpectraOpen", OpenStr+"_BL")
				sq_GetThisRawMS(HR_RunVar, "HR_sticks", "OneRunRawSpectraOpen", OpenStr+"_BL", PToFFlag=1, PToFBin = MxBin)
				wave OneRunRawSpectraClosed =root:HR_sticks:OneRunRawSpectraClosed
				wave OneRunRawSpectraOpen =root:HR_sticks:OneRunRawSpectraOpen
				wave/z OneRunRawSpectraDiff =root:HR_sticks:OneRunRawSpectraDiff			// might not exist yet if doing ptof first
				if (!WaveExists(OneRunRawSpectraDiff) || numpnts(OneRunRawSpectraDiff) != numpnts(OneRunRawSpectraOpen))
					duplicate/o OneRunRawSpectraOpen  root:HR_sticks:OneRunRawSpectraDiff/wave=OneRunRawSpectraDiff
					wave OneRunRawSpectraDiff =root:HR_sticks:OneRunRawSpectraDiff			// might not exist yet if doing ptof first
				endif
				OneRunRawSpectraDiff[0,MIN(numpnts(OneRunRawSpectraOpen), numpnts(OneRunRawSpectraClosed))-1] = OneRunRawSpectraOpen - OneRunRawSpectraClosed
			elseif (HR_singleSpectraMSorPToF==2) 		// ePTOF							
				sq_GetThisRawMS(HR_RunVar, "HR_sticks", "OneRunRawSpectraClosed", "MSClosedLessBaseL_p;")
				sq_GetThisRawMS(HR_RunVar, "HR_sticks", "OneRunRawSpectraOpen", "MSePToFSumLessBaseL_p;")
				sq_GetThisRawMS(HR_RunVar, "HR_sticks", "OneRunRawSpectraDiff", "MSePToFSumDiffLessBaseL_p;")
				wave OneRunRawSpectraClosed =root:HR_sticks:OneRunRawSpectraClosed
				wave OneRunRawSpectraOpen =root:HR_sticks:OneRunRawSpectraOpen
				wave OneRunRawSpectraDiff =root:HR_sticks:OneRunRawSpectraDiff
			endif
			
			// 1.14G
			if ( numInRawSpectra !=  numpnts(OneRunRawSpectraClosed) ||  numInRawSpectra != numpnts(OneRunRawSpectraOpen) || numInRawSpectra != numpnts(OneRunRawSpectraDiff) )
				redimension/n=(numInRawSpectra) OneRunRawSpectraClosed, OneRunRawSpectraOpen, OneRunRawSpectraDiff
			endif

		else		// we are getting raw spectra averages of a todo wave

			AvgTodoFlag=1  //1.10K

			//  Deal with the m/z axes first, because we want to use them for getting average mass spectra
			// Even if we have different m/zs for open and closed, we need to settle on a single mz wave
			
			FindValue/I=(todo[0]) root:index:run_index
			RunIndex=V_Value
			numInRawSpectra=squirrel_index[RunIndex][4][MSClosedIndex]	//1.10C fix  think I found a typo in 1.09H  series_index was l_series
			
			//1.15E  even though the parameter waves *fitOpen and *fitClosed may look different, they may be identical, hence mz_Obase_temp and mz_Cbase_temp will also
			make /o/n=(numInRawSpectra) $("mz_Obase_"+sq_todoWvNm)/wave = mz_Obase_temp			//1.15E
			mz_Obase_temp = ( (p-mz_FitInterceptOpen[runPos])/mz_FitSlopeOpen[runPos])^(1/mz_FitPowerOpen[runPos])	
			if ( mz_FitInterceptOpen[runPos]>0)			//1.54A
				mz_Obase_temp[0,ceil(mz_FitInterceptOpen[runPos])] =0 //1.14I // start at least at mz = 1			
			endif
			
			make /o/n=(numInRawSpectra) $("mz_Cbase_"+sq_todoWvNm)/wave = mz_Cbase_temp			//1.15E
			mz_Cbase_temp = ( (p-mz_FitInterceptClosed[runPos])/mz_FitSlopeClosed[runPos])^(1/mz_FitPowerClosed[runPos])	
			if ( mz_FitInterceptClosed[runPos]>0)			//1.54A
				mz_Cbase_temp[0,ceil(mz_FitInterceptClosed[runPos])] =0 //1.14I // start at least at mz = 1			
			endif
			
			duplicate/o mz_Obase_temp  root:HR_sticks:OneRunRawOpenMz/wave=OneRunRawOpenMz
			duplicate/o mz_Cbase_temp  root:HR_sticks:OneRunRawClosedMz/wave=OneRunRawClosedMz
			duplicate/o mz_Obase_temp  root:HR_sticks:OneRunRawDiffMz/wave=OneRunRawDiffMz

			paramStr="mz_w_op:1;mz_base:mz_base_"+sq_todoWvNm+";new_mz_cal:1;raw_flag:1;cache_op:0;filter_op:1;corr_op:0;product_type:2;ug_op:0;findBaseline_op:0;"
//			Also note that the average mass spectra is already converted to Hz - James applies a factor called dtdmz_tmp

			print "// During HR fits, began extracting averaged raw spectra at "+time()
			//1.15E
			OparamStr=mzStringO+"mz_base:mz_Obase_"+sq_todoWvNm+";raw_flag:1;cache_op:0;filter_op:1;corr_op:0;product_type:2;ug_op:0;findBaseline_op:0;"
			CparamStr=mzStringC+"mz_base:mz_Cbase_"+sq_todoWvNm+";raw_flag:1;cache_op:0;filter_op:1;corr_op:0;product_type:2;ug_op:0;findBaseline_op:0;"
			if (HR_singleSpectraMSorPToF==1)  // MS
				squirrel_fetch(todo,sq_MS_concentration,OparamStr+"destlist:RawSpectraOpen_temp", "MSOpenLessBaseL_p")
				squirrel_fetch(todo,sq_MS_concentration,CparamStr+"destlist:RawSpectraClosed_temp","MSClosedLessBaseL_p")
				squirrel_fetch(todo,sq_MS_concentration,OparamStr+"destlist:RawSpectraDiff_temp","MSDiffLessBaseL_p")			
				wave RawSpectraOpen_temp		 
				wave RawSpectraClosed_temp
				wave RawSpectraDiff_temp
				duplicate/o RawSpectraOpen_temp root:HR_sticks:OneRunRawSpectraOpen/Wave=OneRunRawSpectraOpen
				duplicate/o RawSpectraClosed_temp root:HR_sticks:OneRunRawSpectraClosed/Wave=OneRunRawSpectraClosed
				duplicate/o RawSpectraDiff_temp root:HR_sticks:OneRunRawSpectraDiff/Wave=OneRunRawSpectraDiff

				// 1.14G
				FromDiagnosticNumPtsRawSpectra = Diag_numPtsRawSpectra[runPos]
				numInRawSpectra = numpnts(OneRunRawSpectraOpen)
				if ( FromDiagnosticNumPtsRawSpectra != numInRawSpectra ||  FromDiagnosticNumPtsRawSpectra != numpnts(OneRunRawSpectraClosed) || FromDiagnosticNumPtsRawSpectra != numpnts(OneRunRawSpectraDiff) )
					numInRawSpectra = min(numInRawSpectra, FromDiagnosticNumPtsRawSpectra)
					redimension/n=(numInRawSpectra) OneRunRawSpectraOpen, OneRunRawSpectraClosed, OneRunRawSpectraDiff
				endif

				squirrel_fetch(todo,sq_MS_concentration,OparamStr+"destlist:RawSpectraOpen_temp", "MSOpen_p")
				squirrel_fetch(todo,sq_MS_concentration,CparamStr+"destlist:RawSpectraClosed_temp","MSClosed_p")
				squirrel_fetch(todo,sq_MS_concentration,OparamStr+"destlist:RawSpectraDiff_temp","MSDiff_p")			
				wave RawSpectraOpen_temp		 
				wave RawSpectraClosed_temp
				wave RawSpectraDiff_temp
				duplicate/o RawSpectraOpen_temp root:HR_sticks:OneRunRawSpectraOpenp/Wave=OneRunRawSpectraOpenp
				duplicate/o RawSpectraClosed_temp root:HR_sticks:OneRunRawSpectraClosedp/Wave=OneRunRawSpectraClosedp
				duplicate/o RawSpectraDiff_temp root:HR_sticks:OneRunRawSpectraDiffp/Wave=OneRunRawSpectraDiffp

				// 1.14G
				if ( numInRawSpectra != numpnts(OneRunRawSpectraOpenp) ||  numInRawSpectra != numpnts(OneRunRawSpectraClosedp) || numInRawSpectra != numpnts(OneRunRawSpectraDiffp) )
					redimension/n=(numInRawSpectra) OneRunRawSpectraOpenp, OneRunRawSpectraClosedp, OneRunRawSpectraDiffp
				endif

			elseif (HR_singleSpectraMSorPToF==0)  		// PToF
				string PToFparamStr = ReplaceStringByKey("product_type", paramStr, "6")	
				PToFparamStr	= ReplaceStringByKey("timeweight_op", PToFparamStr, "4")	+"min_cl:"+num2istr(MxBin)+";max_cl:"+num2istr(MxBin)+";ignoreChop:1;"
				squirrel_fetch(todo,sq_MS_concentration,PToFparamStr+"destlist:RawSpectraOpen_temp", OpenStr+"_BL")
				squirrel_fetch(todo,sq_MS_concentration,paramStr+"destlist:RawSpectraClosed_temp",ClosedStr+"_BL")
				wave RawSpectraOpen_temp
				Redimension/n=(dimsize(RawSpectraOpen_temp,0)) RawSpectraOpen_temp		 
				wave RawSpectraClosed_temp
				wave/z OneRunRawSpectraClosed =root:HR_sticks:OneRunRawSpectraClosed
				wave/z OneRunRawSpectraOpen =root:HR_sticks:OneRunRawSpectraOpen
				wave/z OneRunRawSpectraDiff =root:HR_sticks:OneRunRawSpectraDiff
				//  we need to be careful of wave length mismatches.  PToF data sets can be shorter than their MS counterparts
				if (!WaveExists(OneRunRawSpectraOpen) || numpnts(OneRunRawSpectraOpen) != numpnts(RawSpectraOpen_temp))// we only check one set of waves, assume the rest have the same number of points
					make/o/n=(numpnts(RawSpectraOpen_temp)) root:HR_sticks:OneRunRawSpectraClosed, root:HR_sticks:OneRunRawSpectraOpen, root:HR_sticks:OneRunRawSpectraDiff
					wave/z OneRunRawSpectraClosed =root:HR_sticks:OneRunRawSpectraClosed
					wave/z OneRunRawSpectraOpen =root:HR_sticks:OneRunRawSpectraOpen
					wave/z OneRunRawSpectraDiff =root:HR_sticks:OneRunRawSpectraDiff
				elseif (numpnts(OneRunRawSpectraOpen)!=numpnts(OneRunRawSpectraDiff))// we only check one set of waves, assume the rest have the same number of points
					redimension/n=(numpnts(RawSpectraOpen_temp)) OneRunRawSpectraOpen, OneRunRawSpectraClosed, OneRunRawSpectraDiff
				endif
				fastop OneRunRawSpectraOpen = RawSpectraOpen_temp
				fastop OneRunRawSpectraClosed = RawSpectraClosed_temp
				fastop OneRunRawSpectraDiff = RawSpectraOpen_temp - RawSpectraClosed_temp

				squirrel_fetch(todo,sq_MS_concentration,PToFparamStr+"destlist:RawSpectraOpen_temp", OpenStr)
				squirrel_fetch(todo,sq_MS_concentration,paramStr+"destlist:RawSpectraClosed_temp",ClosedStr)
				wave RawSpectraOpen_temp		 
				Redimension/n=(dimsize(RawSpectraOpen_temp,0)) RawSpectraOpen_temp		 
				wave RawSpectraClosed_temp
				wave/z OneRunRawSpectraClosedp =root:HR_sticks:OneRunRawSpectraClosedp
				wave/z OneRunRawSpectraOpenp =root:HR_sticks:OneRunRawSpectraOpenp
				wave/z OneRunRawSpectraDiffp =root:HR_sticks:OneRunRawSpectraDiffp				
				if (!WaveExists(OneRunRawSpectraOpenp) || numpnts(OneRunRawSpectraOpenp) != numpnts(RawSpectraOpen_temp))// we only check one set of waves, assume the rest have the same number of points
					make/o/n=(numpnts(RawSpectraOpen_temp)) root:HR_sticks:OneRunRawSpectraClosedp, root:HR_sticks:OneRunRawSpectraOpenp, root:HR_sticks:OneRunRawSpectraDiffp
					wave/z OneRunRawSpectraClosedp =root:HR_sticks:OneRunRawSpectraClosedp
					wave/z OneRunRawSpectraOpenp =root:HR_sticks:OneRunRawSpectraOpenp
					wave/z OneRunRawSpectraDiffp =root:HR_sticks:OneRunRawSpectraDiffp
				elseif (numpnts(OneRunRawSpectraOpenp)!=numpnts(RawSpectraOpen_temp))// we only check one set of waves, assume the rest have the same number of points
					redimension/n=(numpnts(RawSpectraOpen_temp)) OneRunRawSpectraOpenp, OneRunRawSpectraClosedp, OneRunRawSpectraDiffp
				endif
				fastop OneRunRawSpectraOpenp = RawSpectraOpen_temp
				fastop OneRunRawSpectraClosedp = RawSpectraClosed_temp
				fastop OneRunRawSpectraDiffp = OneRunRawSpectraOpenp - OneRunRawSpectraClosedp
			elseif (HR_singleSpectraMSorPToF==2)  		// ePToFSum
				squirrel_fetch(todo,sq_MS_concentration,OparamStr+"destlist:RawSpectraOpen_temp", "MSePToFSumLessBaseL_p")
				squirrel_fetch(todo,sq_MS_concentration,CparamStr+"destlist:RawSpectraClosed_temp","MSClosedLessBaseL_p")
				squirrel_fetch(todo,sq_MS_concentration,OparamStr+"destlist:RawSpectraDiff_temp","MSePToFSumDiffLessBaseL_p")			
				wave RawSpectraOpen_temp		 
				wave RawSpectraClosed_temp
				wave RawSpectraDiff_temp
				duplicate/o RawSpectraOpen_temp root:HR_sticks:OneRunRawSpectraOpen/Wave=OneRunRawSpectraOpen
				duplicate/o RawSpectraClosed_temp root:HR_sticks:OneRunRawSpectraClosed/Wave=OneRunRawSpectraClosed
				duplicate/o RawSpectraDiff_temp root:HR_sticks:OneRunRawSpectraDiff/Wave=OneRunRawSpectraDiff

				FromDiagnosticNumPtsRawSpectra = Diag_numPtsRawSpectra[runPos]
				numInRawSpectra = numpnts(OneRunRawSpectraOpen)
				if ( FromDiagnosticNumPtsRawSpectra != numInRawSpectra ||  FromDiagnosticNumPtsRawSpectra != numpnts(OneRunRawSpectraClosed) || FromDiagnosticNumPtsRawSpectra != numpnts(OneRunRawSpectraDiff) )
					numInRawSpectra = min(numInRawSpectra, FromDiagnosticNumPtsRawSpectra)
					redimension/n=(numInRawSpectra) OneRunRawSpectraOpen, OneRunRawSpectraClosed, OneRunRawSpectraDiff
				endif

				squirrel_fetch(todo,sq_MS_concentration,OparamStr+"destlist:RawSpectraOpen_temp", "MSePToFSum_p")
				squirrel_fetch(todo,sq_MS_concentration,CparamStr+"destlist:RawSpectraClosed_temp","MSClosed_p")
				squirrel_fetch(todo,sq_MS_concentration,OparamStr+"destlist:RawSpectraDiff_temp","MSePToFSumDiff_p")			
				wave RawSpectraOpen_temp		 
				wave RawSpectraClosed_temp
				wave RawSpectraDiff_temp
				duplicate/o RawSpectraOpen_temp root:HR_sticks:OneRunRawSpectraOpenp/Wave=OneRunRawSpectraOpenp
				duplicate/o RawSpectraClosed_temp root:HR_sticks:OneRunRawSpectraClosedp/Wave=OneRunRawSpectraClosedp
				duplicate/o RawSpectraDiff_temp root:HR_sticks:OneRunRawSpectraDiffp/Wave=OneRunRawSpectraDiffp
				if ( numInRawSpectra != numpnts(OneRunRawSpectraOpenp) ||  numInRawSpectra != numpnts(OneRunRawSpectraClosedp) || numInRawSpectra != numpnts(OneRunRawSpectraDiffp) )
					redimension/n=(numInRawSpectra) OneRunRawSpectraOpenp, OneRunRawSpectraClosedp, OneRunRawSpectraDiffp
				endif
			endif
			
			print "// During HR fits, finished extracting averaged raw spectra at "+time()
			
			Killwaves/z RawSpectraOpen_temp, RawSpectraClosed_temp, RawSpectraDiff_temp
		endif	// single run or todo avg

		// added 1.11M
		// get the squirrel baseline fit waves ready.  For the moment, simply duplicate the preprocessed waves
		duplicate/o OneRunRawSpectraClosedp root:HR_sticks:OneRunRawSpectraClosedBkFt/Wave=OneRunRawSpectraClosedBkFt
		duplicate/o OneRunRawSpectraOpenp root:HR_sticks:OneRunRawSpectraOpenBkFt/Wave=OneRunRawSpectraOpenBkFt
		duplicate/o OneRunRawSpectraDiffp root:HR_sticks:OneRunRawSpectraDiffBkFt/Wave=OneRunRawSpectraDiffBkFt

		wave OneRunRawSpectraOpenBkFt =root:HR_sticks:OneRunRawSpectraOpenBkFt
		wave OneRunRawSpectraClosedBkFt =root:HR_sticks:OneRunRawSpectraClosedBkFt
		wave OneRunRawSpectraDiffBkFt =root:HR_sticks:OneRunRawSpectraDiffBkFt				

		// added in 1.09H calc the baseline sticks
		// make the baseline fit waves
//		if (numpnts(OneRunRawSpectraOpenBkFt)!=numpnts(OneRunRawSpectraOpen))// we only check one set of waves, assume the rest have the same number of points
//			redimension/n=(numpnts(OneRunRawSpectraOpen)) OneRunRawSpectraOpenBkFt, OneRunRawSpectraClosedBkFt, OneRunRawSpectraDiffBkFt
//		endif
 		fastop OneRunRawSpectraOpenBkFt =OneRunRawSpectraOpenp - OneRunRawSpectraOpen
		fastop OneRunRawSpectraClosedBkFt =OneRunRawSpectraClosedp - OneRunRawSpectraClosed
		fastop OneRunRawSpectraDiffBkFt =OneRunRawSpectraDiffp - OneRunRawSpectraDiff

		killwaves/z MyOneRunToDoWave, tempPos
	
	else		// don't grab raw spectra
		// 1.10D
		wave OneRunRawOpenMz = root:HR_sticks:OneRunRawOpenMz
		wave OneRunRawClosedMz = root:HR_sticks:OneRunRawClosedMz
		wave OneRunRawDiffMz = root:HR_sticks:OneRunRawDiffMz
		wave OneRunRawSpectraOpenBkFt = root:HR_sticks:OneRunRawSpectraOpenBkFt
		wave OneRunRawSpectraClosedBkFt = root:HR_sticks:OneRunRawSpectraClosedBkFt
		wave OneRunRawSpectraDiffBkFt = root:HR_sticks:OneRunRawSpectraDiffBkFt
		
		wave OneRunRawSpectraClosed = root:HR_sticks:OneRunRawSpectraClosed
		numInRawSpectra = numpnts(OneRunRawSpectraClosed)	

		nvar/z HR_UseLLS = root:HR_panel:HR_UseLLS		//1.11D
		if (!NVar_exists(HR_UseLLS))
			variable/g  root:HR_panel:HR_UseLLS=0
			nvar HR_UseLLS = root:HR_panel:HR_UseLLS		//1.11D
		endif
		if (HR_UseLLS)		// duplicate original raw spectra in case user presses redo button.. Note that OneRunRawSpectraOpen gets changed in the LLS code
			wave OneRunRawSpectraOpenp = root:HR_sticks:OneRunRawSpectraOpenp
			wave OneRunRawSpectraClosedp = root:HR_sticks:OneRunRawSpectraClosedp
			wave OneRunRawSpectraDiffp = root:HR_sticks:OneRunRawSpectraDiffp
			wave OneRunRawSpectraOpen = root:HR_sticks:OneRunRawSpectraOpen
			wave OneRunRawSpectraClosed = root:HR_sticks:OneRunRawSpectraClosed
			wave OneRunRawSpectraDiff = root:HR_sticks:OneRunRawSpectraDiff
	 		fastop OneRunRawSpectraOpen = OneRunRawSpectraOpenp - OneRunRawSpectraOpenBkFt
			fastop OneRunRawSpectraClosed =OneRunRawSpectraClosedp - OneRunRawSpectraClosedBkFt
			fastop OneRunRawSpectraDiff = OneRunRawSpectraDiffp -  OneRunRawSpectraDiffBkFt
		endif
		
	endif
	
	OneRunSqBkStickOpen =interp(ExactMassWave[p], OneRunRawOpenMz, OneRunRawSpectraOpenBkFt)		//1.09H
	OneRunSqBkStickClosed=interp(ExactMassWave[p], OneRunRawClosedMz, OneRunRawSpectraClosedBkFt)		//1.09H
	OneRunSqBkStickDiff=interp(ExactMassWave[p], OneRunRawDiffMz, OneRunRawSpectraDiffBkFt)		//1.09H
	OneRunSqBkStickOpen=max(0, OneRunSqBkStickOpen[p])		//1.10R  these are for display only.  if we stack negative sticks they go missing
	OneRunSqBkStickClosed=max(0, OneRunSqBkStickClosed[p])
	OneRunSqBkStickDiff=max(0, OneRunSqBkStickDiff[p])

	make/o/n=(numInUnitMass) root:HR_sticks:BkOneRun = nan	// used for original (Pete's), background, not the squirrel background
	
	make/o/n=(numInRawSpectra) root:HR_Sticks:OneRunDisplayHRRegion=nan
	make/o/n=(numInRawSpectra) root:HR_Sticks:OneRunDisplayVertGrid	= 1		//1.10C

	make/o/n=(numInRawSpectra) root:HR_Sticks:OneRunRawSpectra_temp=nan			
	make/o/n=(numInRawSpectra) root:HR_Sticks:OneRunResidSpectraOpen=nan	
	make/o/n=(numInRawSpectra) root:HR_Sticks:OneRunResidSpectraClosed	=nan	
	make/o/n=(numInRawSpectra) root:HR_Sticks:OneRunResidSpectraDiff	=nan
	make/o/n=(numInRawSpectra) root:HR_Sticks:OneRunResidSpectra_temp=nan		
	make/o/n=(numInRawSpectra) root:HR_Sticks:OneRunResidPerSpectraOpen=nan	
	make/o/n=(numInRawSpectra) root:HR_Sticks:OneRunResidPerSpectraClosed=nan		
	make/o/n=(numInRawSpectra) root:HR_Sticks:OneRunResidPerSpectraDiff=nan
	make/o/n=(numInRawSpectra) root:HR_Sticks:OneRunResidPerSpectra_temp=nan		

//	make/o/n=(numInRawSpectra) root:HR_sticks:OneRunMassTime		// we only make shure that this has the correct # points

	// StartPt and EndPt are the point numbers in the raw spectra indicating the region
	// we are considering when we do a many-guass fit at an integer amu.
	// They also are a visual guide to determine the background.
	// They are not used in the calculation of HR sticks.
	// Technically, if one uses different m/zs for open and closed, they are different values,
	// but I thought it unneccessary to make separate ones (Open and closed) for this case.
	make/o/n=(numInUnitMass) root:HR_sticks:StartPt,  root:HR_sticks:EndPt
	wave StartPt = root:HR_sticks:StartPt
	wave EndPt = root:HR_sticks:EndPt
	wave MassStartPnt = root:HR_Sticks:MassStartPnt
	wave MassEndPnt = root:HR_Sticks:MassEndPnt	
	StartPt = round(OneRunMassTime[MassStartPnt[p]]) - 10 
	EndPt = round(OneRunMassTime[MassEndPnt[p]]) +10
		
End


/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// sample usage squirrel_fetch(todo, pk_GetErrors,"mass:Ar,CHNO,C2H3O,C2H5N,C3H7;specType:Open","HRResidNoiseDiff")
// sample usage squirrel_fetch(todo, pk_GetErrors,"mass:Ar,CHNO,C2H3O,C2H5N,C3H7;specType:Open","HRFitDResidErr)
//Function PK_GetErrors(index_pos,paramstr,acorn,n_acorns)
//	wave index_pos
//	string paramstr
//	variable acorn,n_acorns
//
//	wave series_index = root:index:series_index
//	wave OneRunResidNoiseA=$stringbykey("acorn_0",paramstr)		// it really doesn't matter what is passed - it will subtract the data set in acorn 1 from acorn 0
//	wave/z OneRunResidNoiseB=$stringbykey("acorn_1",paramstr)
//
//	wave/z/t ExactMassText_todo = root:HR_sticks:ExactMassText_todo		// 1.08E replaced with *_FitTodo
//	if (!WaveExists(ExactMassText_todo))		// for the case of someone upgrading who has not yet done a fit for a todo wave
//		wave/t ExactMassText_todo=root:HR_Sticks:ExactMassText // 1.08E replaced with *_FitTodo
//	endif
//
//	string massList, massStr, badList, specTypeStr
//	variable idex, jdex, numMass, massPos, numRuns, numAcorn, diffFlag
//	
//	massList=stringbykey("mass",paramstr)
//	specTypeStr =stringbykey("specType",paramstr)
//	badList = ""		
//	numRuns = numpnts(series_index)
//	numMass = ItemsInList(masslist, ",")
//	numAcorn = numpnts(index_pos)
//	
//	diffFlag = stringMatch(lowerStr(specTypeStr), "diff")
//	
//	if (acorn == 0)
//		for (idex = 0;idex<numMass; idex+=1)
//			massStr = stringFromList(idex, masslist, ",")
//			massPos = sq_findStr(ExactMassText_todo,massStr,1)	
//			if (massPos == -1)
//				badList +=massStr+";"	
//				massList = RemoveFromList(massStr, massList,  ",")
//				continue	
//			endif 
//			if (!exists("root:PikaError_"+massStr))
//				make/o/n=(numRuns) $"root:PikaError"+specTypeStr+"_"+massStr =nan
//			endif
//		endfor
//	endif
//	
//	numMass = ItemsInList(masslist, ",")
//	for (idex = 0;idex<numMass; idex+=1)
//		massStr = stringFromList(idex, masslist, ",")
//		wave PikaErrorWave =  $"root:PikaError"+specTypeStr+"_"+massStr
//		massPos = sq_findStr(ExactMassText_todo,massStr,1)	
//		for (jdex=0;jdex<numAcorn;jdex+=1)
//			if(index_pos[jdex]==-1)
//				continue
//			endif
//			if (!diffFlag)
//				pikaErrorWave[series_index[index_pos[jdex]]] = OneRunResidNoiseA[jdex][massPos]
//			else
//				pikaErrorWave[series_index[index_pos[jdex]]] = OneRunResidNoiseA[jdex][massPos] - OneRunResidNoiseB[jdex][massPos]
//			endif
//		endfor
//	endfor
//	
//	if (strlen(badList)>0)
//		print "//  In trying to retrieve errors, could not find these masses "+BadList
//	endif
//	
//End


// can be used for HR sticks or HR errors
// sample usage squirrel_fetch(todo, pk_AvgPikaSticks,"Name:todo;specType:Closed","HRClosedStick")
// sample usage squirrel_fetch(todo, pk_AvgPikaSticks,"Name:todo;specType:Diff","OneRunResidNoiseDiff")
Function PK_AvgPikaSticks(index_pos,paramstr,acorn,n_acorns)
	wave index_pos
	string paramstr
	variable acorn,n_acorns

	wave series_index = root:index:series_index
	wave source=$stringbykey("acorn_0",paramstr)		// it really doesn't matter what is passed - it will subtract the data set in acorn 1 from acorn 0

	string specTypeStr, NameStr
	variable jdex, numAcorn, num2avg, numCols
	
	NameStr=stringbykey("Name",paramstr)
	specTypeStr =stringbykey("specType",paramstr)

	numAcorn = numpnts(index_pos)
	numCols = dimSize(source,1)
	
	if (acorn == 0)
		num2avg=0
		make/o/n=(numCols) $"root:PikaAvg"+specTypeStr+"_"+NameStr = 0
	endif
	
	wave PikaAvg =  $"root:PikaAvg"+specTypeStr+"_"+NameStr
	
	for (jdex=0;jdex<numAcorn;jdex+=1)
		if(index_pos[jdex]==-1)
			continue
		endif
		num2avg+=1
		PikaAvg+=source[jdex][p]
	endfor
		
	if (acorn==n_acorns-1)
		PikaAvg/=num2Avg
		print "// The average of  HR sticks named :PikaAvg"+specTypeStr+"_"+NameStr+" was calculated at "+time()
	endif

End


// sample usage squirrel_fetch(todo, pk_AvgDiffSquarPikaSticks,"Name:todo;dataType:Errors;","HROpenStick;HRClosedStick")
// can be used for HR sticks or HR errors
Function PK_AvgDiffSquarPikaSticks(index_pos,paramstr,acorn,n_acorns)
	wave index_pos
	string paramstr
	variable acorn,n_acorns

	wave series_index = root:index:series_index
	wave sourceOpen=$stringbykey("acorn_0",paramstr)		// it really doesn't matter what is passed - it will subtract the data set in acorn 1 from acorn 0
	wave sourceClosed=$stringbykey("acorn_1",paramstr)		// it really doesn't matter what is passed - it will subtract the data set in acorn 1 from acorn 0

	string dataTypeStr, specTypeStr, NameStr
	variable jdex, numRuns, numAcorn, num2avg, numCols
	
	NameStr=stringbykey("Name",paramstr)

	numRuns = numpnts(series_index)
	numAcorn = numpnts(index_pos)
	numCols = dimSize(sourceOpen,1)
	
	if (acorn == 0)
		num2avg=0
		make/o/n=(numCols) $"root:PikaSqrtSumSq"+"_"+NameStr = 0
	endif
	
	wave PikaAvg =  $"root:PikaSqrtSumSq"+"_"+NameStr
	
	for (jdex=0;jdex<numAcorn;jdex+=1)
		if(index_pos[jdex]==-1)
			continue
		endif
		num2avg+=1
		PikaAvg+=sqrt(sourceOpen[jdex][p]*sourceOpen[jdex][p]+sourceClosed[jdex][p]*sourceClosed[jdex][p])
	endfor
		
	if (acorn==n_acorns-1)
		PikaAvg/=num2Avg
		print "// The square root of the sum of the squares of HR sticks named PikaSqrtSumSq"+"_"+NameStr+" was calculated at "+time()
	endif

End


// sample usage squirrel_fetch(allW, pk_SqrtSumSquarPikaSticks,"dest:HRFitChiSqDiff;","HRFitChiSqOpen;HRFitChiSqClosed")  // 1.11D
// can be used for HR sticks or HR errors
Function PK_SqrtSumSquarPikaSticks(index_pos,paramstr,acorn,n_acorns)
	wave index_pos
	string paramstr
	variable acorn,n_acorns

	wave sourceOpen=$stringbykey("acorn_0",paramstr)		// it really doesn't matter what is passed - it will subtract the data set in acorn 1 from acorn 0
	wave sourceClosed=$stringbykey("acorn_1",paramstr)		// it really doesn't matter what is passed - it will subtract the data set in acorn 1 from acorn 0

	string destStr
	variable jdex, numAcorn, numCols
	
	destStr=stringbykey("dest",paramstr)

	numAcorn = numpnts(index_pos)
	numCols = dimSize(sourceOpen,1)
	
	make/o/n=(numAcorn, numCols) $destStr
	wave destWave =$destStr
		
	for (jdex=0;jdex<numAcorn;jdex+=1)
		if(index_pos[jdex]==-1)
			continue
		endif
		destWave[jdex][]=sqrt(sourceOpen[jdex][p]*sourceOpen[jdex][p]+sourceClosed[jdex][p]*sourceClosed[jdex][p])
	endfor
		
	// 1.14M not used
	squirrel_hdf_write(index_pos,destWave,destStr)
	
End


// Used for HR sticks counting, "Pete's", errors
// sample usage squirrel_fetch(todo, pk_CountingErrors,"","HROpenStick;HRClosedStick")
//NO LONGER USED!  See pk_TotalCountingErrors below  //1.09H
//Function PK_CountingErrors(index_pos,paramstr,acorn,n_acorns)
//	wave index_pos
//	string paramstr
//	variable acorn,n_acorns
//
//	wave series_index = root:index:series_index
//	wave sourceOpen=$stringbykey("acorn_0",paramstr)		
//	wave sourceClosed=$stringbykey("acorn_1",paramstr)
//	
//	wave samptimeOpen = root:diagnostics:TimeMSOpen
//	wave samptimeClosed = root:diagnostics:TimeMSClosed
//	wave dutyCycleCorr=root:Int_Sticks:dutyCycleCorr		// created and set in UM_MakeDutyCycleCorr()
//		
//	// simplified in1.08C
//	nvar error_sigma=root:panel:error_sigma
//	variable sigma = error_sigma^2		//  this is the way James did it in Squirrel
//	
//	matrixop /o err_tmpClosed=abs(sourceClosed)		
//	wave err_tmpClosed
//	matrixop /o err_tmpOpen=abs(sourceOpen)	
//	wave err_tmpOpen
//	matrixop /o err_tmpOMinusC=abs(sourceOpen)// just a placeholder
//	wave err_tmpOMinusC
//	
//	err_tmpClosed*=sigma/samptimeClosed[series_index[index_pos[p]]]
//	err_tmpOpen*=sigma/samptimeOpen[series_index[index_pos[p]]]
//
//	err_tmpClosed*=sqrt(28/(q+1)) 	//	// we start our m/z matrix dimension at m/z 1.  When q = 0, m/z = 1 
//	err_tmpOpen*=sqrt(28/(q+1))
//	err_tmpOMinusC=sqrt(err_tmpOpen+err_tmpClosed)	// more officially,  it is sqrt(  sqrt(err_tmpOpen)^2 + sqrt(err_tmpClosed)^2 ) 
//		
//	err_tmpClosed=sqrt(err_tmpClosed[p][q]) 
//	err_tmpOpen=sqrt(err_tmpOpen[p][q])
//
//	squirrel_hdf_write(index_pos,err_tmpClosed,"HRCCountErr") 		// 1.08B
//	squirrel_hdf_write(index_pos,err_tmpOpen,"HROCountErr") 		// 1.08B
//	squirrel_hdf_write(index_pos,err_tmpOMinusC,"HROMinusCIonCountErr") 		// note name change in 1.06H
//	
//	if (acorn == n_acorns - 1)
//		killwaves/z err_tmpClosed, err_tmpOpen, err_tmpOMinusC
//	endif
//		
//End
//
//

//  What we now use, in 1.50K.  Includes baseline errors based on discussions in Feb 2011.
// Also now inlcudes electronic noise -integration window dependence, similar to the unit rex
Function PK_TotalCountingErrors(index_pos,paramstr,acorn,n_acorns)
	wave index_pos
	string paramstr
	variable acorn,n_acorns

	wave series_index = root:index:series_index
	wave sourceOpen=$stringbykey("acorn_0",paramstr)			// units of sticks = Hz	
	wave sourceClosed=$stringbykey("acorn_1",paramstr)		// units of sticks = Hz
	wave sourceOpenBase=$stringbykey("acorn_2",paramstr)		// units of sticks = Hz		// this will have UMR number of columns
	wave sourceClosedBase=$stringbykey("acorn_3",paramstr)		// units of sticks = Hz  // this will have UMR number of columns

//	string sampTimeClosedStr=stringbykey("sampTimeClosed",paramstr)	// Closed or PToF 	// timeMSClosed
//	string sampTimeOpenStr=stringbykey("sampTimeOpen",paramstr)	// Open or nothing 	// timeMSOpen

	variable TotCountErrFlag =numberbykey("TotCountErrFlag",paramstr)
	variable countSigErrFlag =numberbykey("countSigErrFlag",paramstr)
	string OMinusCErrName = SelectString(!TotCountErrFlag ,"HROMinusCSigErr", "HROMinusCTotCountErr")
	string OpenErrName =SelectString(!TotCountErrFlag ,"HROTotSigErr", "HROTotCountErr")
	string ClosedErrName =SelectString(!TotCountErrFlag ,"HRCTotSigErr", "HRCTotCountErr")
	if (countSigErrFlag)		//1.21E
		OMinusCErrName = "HROMinusCCountSigErr"
		OpenErrName = "HROCountSigErr"
		ClosedErrName = "HRCCountSigErr"
	endif

//	string OMinusCErrName =  "HROMinusCSigErr" // SelectString(ErrorNamePrefixFlag ,"HROMinusCSigErr", "HROMinusCTotCountErr")
//	string OpenErrName ="HROTotSigErr" //SelectString(ErrorNamePrefixFlag ,"HROTotSigErr", "HROTotCountErr")
//	string ClosedErrName ="HRCTotSigErr" //SelectString(ErrorNamePrefixFlag ,"HRCTotSigErr", "HRCTotCountErr")
	variable PreviousHRFit=numberbykey("PreviousHRFit",paramstr)

//	wave UMRPeakWidthInNs=$stringbykey("UMRPeakWidthInNs",paramstr)		// units of sticks = Hz. //changed in 1.10A to accomodate varying peak width
	//  UMRPeakWidthInNs is the HR peak width of every integer(starting from m/z=0)  We use the same peak width for the entire set of HR ions at one nominal mass
//	variable ErrorNamePrefixFlag = numberByKey("TotCountErrFlag",paramstr)
//	string OMinusCErrName =SelectString(ErrorNamePrefixFlag ,"HROMinusCSigErr", "HROMinusCTotCountErr")
//	string OpenErrName =SelectString(ErrorNamePrefixFlag ,"HROTotSigErr", "HROTotCountErr")
//	string ClosedErrName =SelectString(ErrorNamePrefixFlag ,"HRCTotSigErr", "HRCTotCountErr")
// 1.69Z  do away with HROMinusCTotCountErr
//	variable ErrorNamePrefixFlag = numberByKey("TotCountErrFlag",paramstr)

	string ExactMassWaveStr = stringbykey("ExactMassWave",paramstr)		//1.12N
	wave ExactMassWave = $ReplaceString("/", ExactMassWaveStr, ":" )
	
//	wave samptimeOpen=$ReplaceString("/", sampTimeOpenStr, ":")
//	wave samptimeClosedTemp = $ReplaceString("/", sampTimeClosedStr, ":")
//	Wave samptimeClosed = sq_AdjustClosedTimeForErrCalcs(sampTimeClosedTemp, sampTimeOpen)

	wave/wave TimeWaves=sq_GetModifiedTimes()
	wave samptimeOpen = TimeWaves[0]
	wave samptimeClosed = TimeWaves[1]
	wave TimePToF = TimeWaves[2]

	wave /z corr_fact= root:diagnostics:corr_fact
	wave /z ion_wave=root:diagnostics:ionSingleStr
	if ( !waveexists(ion_wave) ||  numpnts(ion_wave)==0 )
		wave ion_wave=root:diagnostics:ionSingleStr_logged
	endif
	wave ToFPulserInHz=root:diagnostics:ToFPulserInHz

	variable sigma=numberbykey("sigma",paramstr)^2		//Sigma is the sqrt(1+dSI2) from the formula in James' Word doc
	variable noise=numberbykey("noise",paramstr)

	variable numUnitRes = dimsize(sourceOpenBase,1)
	variable numRows =dimsize(index_pos,0)
	variable numHRions =dimsize(ExactMassWave,0)
	
	sq_findIntegrationWidthNs(numRows, numUnitRes, 0, index_pos)		// does not consider stick complement or mass defect functions (yet) in 1.09K.
	wave IntegrationWidthNs = root:IntegrationWidthNs  // a 2D wave (rows=run x cols=mz)

	//1.19F
	if (WaveMin(IntegrationWidthNs)==0)	//  case if user's mz min gets to be < min HR value  will result in errors of nan
		IntegrationWidthNs = 	IntegrationWidthNs[p][q]==0 ? nan :IntegrationWidthNs[p][q]
		variable val = abs(WaveMin(IntegrationWidthNs)) // 1.12B
		IntegrationWidthNs = 	numtype(IntegrationWidthNs[p][q])==0 ? IntegrationWidthNs[p][q] : val
		// print "// The minimum UMR integration width may have a small problem. -- printing from PK_TotalCountingErrors"
	endif

//	make/o/n=(numRows) UMR_temp=p+1	// HR_ExpandUMR2HR will overwrite		//1.12N bug! in the calculation of HR sticks we use the exact mass, not the umr equivalent
	
	wave HR_FinalPWInterceptWave = root:HR_PeakShape:HR_FinalPWInterceptWave
	wave HR_FinalPWSlopeWave = root:HR_PeakShape:HR_FinalPWSlopeWave
	wave HR_FinalPWPowerWave = root:HR_PeakShape:HR_FinalPWPowerWave 
	make/o/n=(numRows, numUnitRes) UMRPeakWidthInNs
	UMRPeakWidthInNs[][0,8]=0		// presume we don't fit anything < m/z 9
	UMRPeakWidthInNs[][9,]=(index_pos[p]<0 ? 0 : HR_FinalPWInterceptWave[series_index[index_pos[p]]] + HR_FinalPWSlopeWave[series_index[index_pos[p]]]*q^HR_FinalPWPowerWave[series_index[index_pos[p]]]	)	// peak width as function of mass
		
	// all unit resolution 2-d waves at this point must have column 0 = m/z 1.  Now expand them to have columns matching the HR stick nominal mass values
	HR_ExpandUMR2HR(IntegrationWidthNs, PreviousHRFit)		// these values are unit res sticks - we need to 'expand' them to match the columns of the HR ions that were fit
	HR_ExpandUMR2HR(sourceOpenBase, PreviousHRFit)		// these values are unit res sticks - we need to 'expand' them to match the columns of the HR ions that were fit
	HR_ExpandUMR2HR(sourceClosedBase, PreviousHRFit)
	HR_ExpandUMR2HR(UMRPeakWidthInNs, PreviousHRFit)	// changed in 1.10A to accomodate varying peak width  // we use the same PW values for all HR ions fit at some nominal mass
//	HR_ExpandUMR2HR(UMR_temp, PreviousHRFit)	// will use for duty cycle
	
	// we use the fraction of the unit resolution baseline stick value
	sourceClosedBase  *= 8*UMRPeakWidthInNs[p][q] / IntegrationWidthNs[p][q]	// 8 comes from +-4*peak widths = 8 total peak widths, 8 is arbitrary
	sourceOpenBase *= 8*UMRPeakWidthInNs[p][q] / IntegrationWidthNs[p][q]	// 8 comes from +-4*peak widths = 8 total peak widths,  8 is arbitrary
	// we are still in UMR columns here
	sourceClosedBase = numtype(sourceClosedBase) ?  0 : sourceClosedBase
	sourceOpenBase = numtype(sourceOpenBase) ?  0 : sourceOpenBase
	
	matrixop /o err_tmpClosed=abs(sourceClosed)	
	wave err_tmpClosed
	matrixop /o err_tmpOpen=abs(sourceOpen) 	
	wave err_tmpOpen
	matrixop /o err_tmpOMinusC=abs(sourceOpen) // just a placeholder
	wave err_tmpOMinusC
	
	//1.09K addition begin
	matrixop /o err_tmpClosedBase=abs(sourceClosedBase)	
	wave err_tmpClosedBase
	matrixop /o err_tmpOpenBase=abs(sourceOpenBase)	
	wave err_tmpOpenBase
	//1.09K addition end

	err_tmpClosed*=(index_pos[p]<0 ? 0 : sigma/samptimeClosed[series_index[index_pos[p]]])
	err_tmpOpen*=(index_pos[p]<0 ? 0 : sigma/samptimeOpen[series_index[index_pos[p]]])

	//1.09K addition begin
	err_tmpClosedBase*=(index_pos[p]<0 ? 0 : sigma/samptimeClosed[series_index[index_pos[p]]])
	err_tmpOpenBase*=(index_pos[p]<0 ? 0 : sigma/samptimeOpen[series_index[index_pos[p]]])
	//1.09K addition end

	if (waveexists(corr_fact))
		err_tmpClosed/=(index_pos[p]<0 ? 0 : corr_fact[series_index[index_pos[p]]])
		err_tmpOpen/=(index_pos[p]<0 ? 0 : corr_fact[series_index[index_pos[p]]])
		// 1.50K addition begin
		err_tmpClosedBase/=(index_pos[p]<0 ? 0 : corr_fact[series_index[index_pos[p]]])
		err_tmpOpenBase/=(index_pos[p]<0 ? 0 : corr_fact[series_index[index_pos[p]]])
		// 1.50K addition end
	endif

//	err_tmpClosed*=sqrt(28/UMR_temp[q]) 	// we start our m/z matrix dimension at m/z 1.  When q = 0, m/z = 1 
//	err_tmpOpen*=sqrt(28/UMR_temp[q]) 
//		
//	//1.09K addition begin
//	err_tmpClosedBase*=sqrt(28/UMR_temp[q]) 
//	err_tmpOpenBase*=sqrt(28/UMR_temp[q])
// bug fix 1.12N 
	err_tmpClosed[][0,numHRions-1]*=sqrt(28/ExactMassWave[q]) 	// we start our m/z matrix dimension at m/z 1.  When q = 0, m/z = 1 
	err_tmpOpen[][0,numHRions-1]*=sqrt(28/ExactMassWave[q]) 		//1.58G
	err_tmpClosedBase[][0,numHRions-1]*=sqrt(28/ExactMassWave[q]) 
	err_tmpOpenBase[][0,numHRions-1]*=sqrt(28/ExactMassWave[q]) 

//  	I really don't think these matter for HR (we've got bigger fish to fry), but include here to be consistent with UMR 
	duplicate/o err_tmpClosed err_tmp		// simply get a wave of the correct size
	wave err_tmp
	err_tmp[][0,numHRions-1] = (index_pos[p]<0 ? 0 : noise*noise*IntegrationWidthNs[p][q]*28/((ion_wave[series_index[index_pos[p]]]^2)*ToFPulserInHz[series_index[index_pos[p]]]*(q+1)) )		//1.12N mistake, do not use q+1
	
	err_tmpClosed+=err_tmp
	err_tmpOpen+= err_tmp

	//1.09K addition begin  I don't include here because for open we should only add the error once, ditto for closed
	err_tmpClosedBase+=err_tmp
	err_tmpOpenBase+=err_tmp
	//1.09K addition end

	err_tmpOMinusC[][0,numHRions-1]=sqrt(err_tmpOpen+err_tmpClosed+err_tmpClosedBase+err_tmpOpenBase)	// the squared elements in each term cancel out

	//1.14M 'expanded' columns to the next 200 HR spectra
	variable numHRCols = dimsize(err_tmpOMinusC,1)
	variable numMoreCols = mod(numHRCols, ROUNDNUMHRIONCOLS)

	//1.14M 'expanded' columns to the next 200 HR spectra
	if (numMoreCols>0)			//1.16F
		InsertPoints/M=1 numHRCols,  (ROUNDNUMHRIONCOLS-numMoreCols), err_tmpOMinusC
		err_tmpOMinusC[][numHRCols,]=nan
		InsertPoints/M=1 numHRCols,  (ROUNDNUMHRIONCOLS-numMoreCols), err_tmpClosed
		err_tmpClosed[][numHRCols,]=nan
		InsertPoints/M=1 numHRCols, (ROUNDNUMHRIONCOLS-numMoreCols),  err_tmpOpen
		err_tmpOpen[][numHRCols,]=nan
	endif
	
	//1.14M 'expanded' columns to the next 200 HR spectra
	squirrel_hdf_write(index_pos,err_tmpOMinusC,OMinusCErrName) 	
	
	err_tmpClosed[][0,numHRions-1]=sqrt(err_tmpClosed+err_tmpClosedBase)	
	squirrel_hdf_write(index_pos,err_tmpClosed,ClosedErrName)
	
	err_tmpOpen[][0,numHRions-1]=sqrt(err_tmpOpen+err_tmpOpenBase) 
	squirrel_hdf_write(index_pos,err_tmpOpen,OpenErrName) 	
	
	if (acorn == n_acorns - 1)
		killwaves/z UMR_temp, UMRPeakWidthInNs, samptimeClosedModified,TimeWaves
		killwaves/z err_tmp, err_tmpClosed, err_tmpOpen, err_tmpOMinusC, expanded_temp, UMR_Mz,IntegrationWidthNs, err_tmpClosedBase, err_tmpOpenBase
	endif
		
End


// creates UnitSigma_temp which has the 1 peak width values for each Mz.
// added variable peak width 1.10A
//Function PK_getHRPeakWidths(PreviousHRFit)		// added pos value for variable peak width 1.10A
//variable PreviousHRFit
//	
//	HR_PeakWidthSanity(0) // 0 flag means do for a todo wave
//	 // sets global variables HR_FinalPWIntercept, HR_FinalPWSlope, HR_FinalPWPower// will be 1 if uing line for peak width formula
////	nvar HR_FinalPWIntercept = root:HR_PeakShape:HR_FinalPWIntercept
////	nvar HR_FinalPWSlope = root:HR_PeakShape:HR_FinalPWSlope
////	nvar HR_FinalPWPower = root:HR_PeakShape:HR_FinalPWPower
//
//	if (PreviousHRFit)
//		wave UnitMass_FitTodo = root:HR_Sticks:UnitMass_FitTodo
//	else
//		wave UnitMass_FitTodo = root:HR_Sticks:UnitMass
//	endif
//	
//	wave HR_FinalPWInterceptWave = root:HR_PeakShape:HR_FinalPWInterceptWave
//	wave HR_FinalPWSlopeWave = root:HR_PeakShape:HR_FinalPWSlopeWave
//	wave HR_FinalPWPowerWave = root:HR_PeakShape:HR_FinalPWPowerWave 
//
//	variable firstmz=UnitMass_FitTodo[0]
//	variable lastmz=UnitMass_FitTodo[numpnts(UnitMass_FitTodo)-1]
//	
//	make/o/n=(lastmz) root:UnitSigma_temp/wave=UnitSigma_temp
//	UnitSigma_temp[0,firstmz-2]=0
//	UnitSigma_temp[firstmz-1,]=HR_FinalPWInterceptWave[pos] + HR_FinalPWSlopeWave[pos]*(UnitMass_FitTodo[p-(firstmz-1)]^HR_FinalPWPowerWave[pos])		// peak width as function of mass
//
//End
	

//1.09K
// give a 2d matrix with the columns in umr, copy UMR column values so that the matrix corresponds to the HR-UMR equivalent
// The UMR stick columns must correspond to the usual m/z 1 = column 0, etc. 
//  If performing on a previously fit set of runs, assume that the *_FitTodo waves have already been correctly put in place.
// Overwrites UMRStick!
Function	HR_ExpandUMR2HR(UMRStick, TodoFlag)
wave UMRStick
variable TodoFlag

	if (TodoFlag)
		wave ExactMassText = root:HR_Sticks:ExactMassText_FitTodo
		wave MassEndPnt = root:HR_Sticks:MassEndPnt_FitTodo
		wave MassStartPnt = root:HR_Sticks:MassStartPnt_FitTodo
		wave UnitMass = root:HR_Sticks:UnitMass_FitTodo
	else
		wave ExactMassText = root:HR_Sticks:ExactMassText
		wave MassEndPnt = root:HR_Sticks:MassEndPnt
		wave MassStartPnt = root:HR_Sticks:MassStartPnt
		wave UnitMass = root:HR_Sticks:UnitMass
	endif	
	
	variable idex, rows, UMRCols, numHR
	rows=dimsize(UMRStick,0)
	UMRCols=dimsize(UMRStick,1)
	numHR = numpnts(ExactMassText)
	
	// first get wave that indicates the m/z column values
	make/o/n=(numHR) UMRColWave
//	for(idex=0;idex<UMRCols;idex+=1)		//1.58G
	for(idex=0;idex<numpnts(MassStartPnt);idex+=1)
		UMRColWave[MassStartPnt[idex], MassEndPnt[idex]]=UnitMass[idex]
	endfor
	
	if (UMRCols>1)	// original data was 2D
		Make/o/n=(rows,numHR) expanded_temp=0
		expanded_temp[][]=UMRStick[p][UMRColWave[q]-1]		// bug in 1.12N  this was	  UMRColWave[q]
		redimension/n=(rows,numHR) UMRStick
		fastop UMRStick = expanded_temp
	else		// original data was 1D
		Make/o/n=(numHR) expanded_temp=0
		expanded_temp[]=UMRStick[UMRColWave[p]]	
		redimension/n=(numHR) UMRStick
		fastop UMRStick = expanded_temp
	endif
	
	killwaves/z expanded_temp, UMRColWave 
End

//1.09H Previous to 1.09H the ion count did not include the ions from the squirrel baseline.  This rectifies this. 
// It will only have a small effect on small ions above a genuine background like at 29, 32, etc.

////////////////////////////////////////////////////////////////////////////////////////////
// Modified from 2006GeneralMacros
// Should later be dumped to sq panel ipf?

//  Returns the name of a text wave containing elements in the list; overwrites contents of text wave textWaveNameStr if it exists
// more stringent than gen_List2textWave
Function/S PK_List2textWave(ListStr, textWaveNameStr)
	String ListStr, textWaveNameStr

	string returnStr = textWaveNameStr
	variable idex, num
	
	num=ItemsinList(ListStr)
//	if (strlen(ListStr)==0)			// we want to allow the null case for sq
//		abort "There were no items in this list"
//	endif
	
	if (Exists(textWaveNameStr)==1 )
		wave/T tw = $textWaveNameStr
		Redimension/N=(num)  tw
	else
		returnStr = CleanupName(textWaveNameStr, 0 )
		if (CheckName(returnStr, 1)!=0)
			 returnStr = UniqueName(textWaveNameStr[0, min(20, strlen(returnStr)-1)], 1, 0 )
		endif
		Make/o/t/n=(num) $returnStr
		wave/T tw = $returnStr
	endif
	
	for (idex=0; idex<num; idex+=1)
		tw[idex]  = StringFromList(idex, ListStr, ";")
	endfor
	
	return returnStr
	
End



// RemoveNaNs(theWave)
//	Removes all points in the wave with the value NaN.
//	A NaN represents a blank or missing value.
//	Returns the number of points removed.
Function sq_RemoveNaNs2D(theWave)
	Wave theWave

	Variable idex, numRows
	
	numRows = dimsize(theWave,0)				// number of times to loop

	for(idex=numRows-1;idex>=0;idex-=1)
		if (numtype( theWave[idex])==2)					// is this NaN?
			DeletePoints/m=0 idex, 1, theWave
		endif
	endfor
		
End


//1.49E New
// creates element waves for checking of exact mass.
//  If an element has been commented out, that means it cannot be used in the name of an HRion, just that
Function PK_CreateElementWaves()
// values taken from J. R. de LAETER et al.ATOMIC WEIGHTS OF THE ELEMENTS: REVIEW 2000 (IUPAC Technical Report)

	wave/t/z ElementIonName = root:HR:ElementIonName
	wave/d/z ElementIonMass = root:HR:ElementIonName
	wave/t/z ElementIsotopParentList = root:HR:ElementIsotopParentList
	wave/t/z ElementIsotopParentIndexList = root:HR:ElementIsotopParentIndexList
	wave/d/z ElementIsotopAbund = root:HR:ElementIsotopAbund
	
	variable idex, jdex, num
	string str
	if (WaveExists(ElementIonName) && WaveExists(ElementIonMass) && WaveExists(ElementIsotopParentList) && WaveExists(ElementIsotopParentIndexList)  && WaveExists(ElementIsotopAbund)  )
		num=numpnts(ElementIonMass)
		if (num == numpnts(ElementIonName) && num == numpnts(ElementIsotopParentList) && num == numpnts(ElementIsotopParentIndexList)  && num == numpnts(ElementIsotopAbund))
			str =WaveInfo(ElementIonName,0)
			if(str2num(StringByKey("MODTIME",str)) < date2secs(2018, 9,1)  )
			else
				return 0
			endif
		endif
	endif
	
	make/t/o/n=500 root:HR:ElementIonName/Wave=ElementIonName=""
	make/d/o/n=500 root:HR:ElementIonMass/Wave=ElementIonMass=nan		// double precision
	make/t/o/n=500 root:HR:ElementIsotopParentList/Wave=ElementIsotopParentList=""	 
	make/t/o/n=500 root:HR:ElementIsotopParentIndexList/Wave=ElementIsotopParentIndexList=""	 
	make/d/o/n=500 root:HR:ElementIsotopAbund/Wave=ElementIsotopAbund=nan	// double precision
	
	//  The exact mass for each element has a slightly varying amount of digits of precision (I think) .
	// Table copied from Qi Chen who created it at the users mtg in 2009 and was implemented for all in 2010..
	// Note that the current input waves (ElementIonName and ElementIonMass, data from NIST website) do not include all the possible elements and isotopes, and will be updated.
	
	variable n=0		// use a variable as an index for when we want to add more elements
	
//  version 1.10I.  A couple of folks wanted the other child isotopes in this table for use with the mass calculator
	ElementIonName[n]="H";		ElementIonMass[n]= 1.0078250319	;ElementIsotopParentList[n]="j2H"				;n+=1
	ElementIonName[n]="j2H";		ElementIonMass[n]= 2.0141017779	;ElementIsotopAbund[n] = 0.000115/0.999885	;n+=1		// in 1.08 we changed from D to j2H

	ElementIonName[n]="j3He";	ElementIonMass[n]= 3.0160293094	;ElementIsotopAbund[n] = 0.00000134/0.99999866;n+=1
	ElementIonName[n]="He";		ElementIonMass[n]= 4.0026032497	;ElementIsotopParentList[n]="j3He"				;n+=1	

	ElementIonName[n]="j6Li";	ElementIonMass[n]= 6.0151223	;ElementIsotopAbund[n] = 0.0759/0.9241		;n+=1
	ElementIonName[n]="Li";		ElementIonMass[n]= 7.0160041	;ElementIsotopParentList[n]="j6Li"				;n+=1	

	ElementIonName[n]="Be";		ElementIonMass[n]= 9.0121822	;n+=1

	ElementIonName[n]="j10B";	ElementIonMass[n]=10.0129371	;ElementIsotopAbund[n] = 0.199/0.801			;n+=1
	ElementIonName[n]="B";		ElementIonMass[n]=11.0093055	;ElementIsotopParentList[n]="j10B"				;n+=1

	ElementIonName[n]="C";		ElementIonMass[n]=12.000000000	;ElementIsotopParentList[n]="j13C"				;n+=1
	ElementIonName[n]="j13C";	ElementIonMass[n]=13.003354838	;ElementIsotopAbund[n] = 0.0107/0.9893		;n+=1

	ElementIonName[n]="N";		ElementIonMass[n]=14.0030740074;ElementIsotopParentList[n]="j15N"				;n+=1
	ElementIonName[n]="j15N";	ElementIonMass[n]=15.000108973	;ElementIsotopAbund[n] = 0.00364/0.99636		;n+=1

	ElementIonName[n]="O";		ElementIonMass[n]=15.9949146223;ElementIsotopParentList[n]="j17O;j18O"		;n+=1
	ElementIonName[n]="j17O";	ElementIonMass[n]=16.99913150	;ElementIsotopAbund[n] = 0.00038/0.99757		;n+=1
	ElementIonName[n]="j18O";	ElementIonMass[n]=17.9991604	;ElementIsotopAbund[n] = 0.00205/0.99757		;n+=1

	ElementIonName[n]="F";		ElementIonMass[n]=18.99840320	;n+=1

	ElementIonName[n]="Ne";		ElementIonMass[n]=19.992440176	;ElementIsotopParentList[n]="j21Ne;j22Ne"		;n+=1
	ElementIonName[n]="j21Ne";	ElementIonMass[n]=20.99384674	;ElementIsotopAbund[n] = 0.0027/0.9048		;n+=1		
	ElementIonName[n]="j22Ne";	ElementIonMass[n]=21.99138550	;ElementIsotopAbund[n] = 0.0925/0.9048		;n+=1

	ElementIonName[n]="Na";		ElementIonMass[n]=22.98976966	;n+=1

	ElementIonName[n]="Mg";		ElementIonMass[n]=23.98504187	;ElementIsotopParentList[n]="j25Mg;j26Mg"		;n+=1
	ElementIonName[n]="j25Mg";	ElementIonMass[n]=24.98583700	;ElementIsotopAbund[n] = 0.1000/0.78990		;n+=1
	ElementIonName[n]="j26Mg";	ElementIonMass[n]=25.98259300	;ElementIsotopAbund[n] = 0.1101/0.78990		;n+=1

	ElementIonName[n]="Al";		ElementIonMass[n]=26.98153841	;n+=1

	ElementIonName[n]="Si";		ElementIonMass[n]=27.97692649	;ElementIsotopParentList[n]="j29Si;j30Si"		;n+=1
	ElementIonName[n]="j29Si";	ElementIonMass[n]=28.97649468	;ElementIsotopAbund[n] = 0.04685/0.92223		;n+=1	
	ElementIonName[n]="j30Si";	ElementIonMass[n]=29.97377018	;ElementIsotopAbund[n] = 0.03092/0.92223		;n+=1

	ElementIonName[n]="P";		ElementIonMass[n]=30.97376149	;n+=1

	ElementIonName[n]="S";		ElementIonMass[n]=31.97207073	;ElementIsotopParentList[n]="j33S;j34S;j36S"	;n+=1
	ElementIonName[n]="j33S";	ElementIonMass[n]=32.97145854	;ElementIsotopAbund[n] = 0.0075/0.9499		;n+=1
	ElementIonName[n]="j34S";	ElementIonMass[n]=33.96786687	;ElementIsotopAbund[n] = 0.0425/0.9499		;n+=1
	ElementIonName[n]="j36S";	ElementIonMass[n]=35.96708088	;ElementIsotopAbund[n] = 0.0001/0.9499		;n+=1

	ElementIonName[n]="Cl";		ElementIonMass[n]=34.96885271	;ElementIsotopParentList[n]="j37Cl"			;n+=1
	ElementIonName[n]="j37Cl";	ElementIonMass[n]=36.96590260	;ElementIsotopAbund[n] = 0.2424/0.7576		;n+=1

	ElementIonName[n]="j36Ar";	ElementIonMass[n]=35.96754626	;ElementIsotopAbund[n] = 0.003365/0.996003	;n+=1
	ElementIonName[n]="j38Ar";	ElementIonMass[n]=37.9627322	;ElementIsotopAbund[n] = 0.000632/0.996003	;n+=1
	ElementIonName[n]="Ar";		ElementIonMass[n]=39.962383124	;ElementIsotopParentList[n]="j36Ar;j38Ar"	;n+=1

	ElementIonName[n]="K";		ElementIonMass[n]=38.9637069	;ElementIsotopParentList[n]="j40K;j41K"				;n+=1
	ElementIonName[n]="j40K";	ElementIonMass[n]=39.96399867	;ElementIsotopAbund[n] = 0.000117/0.932581	;n+=1
	ElementIonName[n]="j41K";	ElementIonMass[n]=40.96182597	;ElementIsotopAbund[n] = 0.067302/0.932581	;n+=1

	ElementIonName[n]="Ca";		ElementIonMass[n]=39.9625912	;ElementIsotopParentList[n]="j42Ca;j43Ca;j44Ca;j46Ca;j48Ca";n+=1
	ElementIonName[n]="j42Ca";	ElementIonMass[n]=41.9586183	;ElementIsotopAbund[n] = 0.00647/0.96941		;n+=1
	ElementIonName[n]="j43Ca";	ElementIonMass[n]=42.9587668	;ElementIsotopAbund[n] = 0.00135/0.96941		;n+=1
	ElementIonName[n]="j44Ca";	ElementIonMass[n]=43.9554811	;ElementIsotopAbund[n] = 0.02086/0.96941		;n+=1
	ElementIonName[n]="j46Ca";	ElementIonMass[n]=45.9536927	;ElementIsotopAbund[n] = 0.00004/0.96941		;n+=1
	ElementIonName[n]="j48Ca";	ElementIonMass[n]=47.952533		;ElementIsotopAbund[n] = 0.00187/0.96941		;n+=1

	ElementIonName[n]="Sc";		ElementIonMass[n]=44.9559102	;n+=1

	ElementIonName[n]="j46Ti";	ElementIonMass[n]=45.9526295	;ElementIsotopAbund[n] = 0.0825/0.7372		;n+=1
	ElementIonName[n]="j47Ti";	ElementIonMass[n]=46.9517637	;ElementIsotopAbund[n] = 0.0744/0.7372		;n+=1
	ElementIonName[n]="Ti";		ElementIonMass[n]=47.9479470	;ElementIsotopParentList[n]="j46Ti;j47Ti;j49Ti;j50Ti"	;n+=1
	ElementIonName[n]="j49Ti";	ElementIonMass[n]=48.9478707	;ElementIsotopAbund[n] = 0.0541/0.7372		;n+=1
	ElementIonName[n]="j50Ti";	ElementIonMass[n]=49.9447920	;ElementIsotopAbund[n] = 0.0518/0.7372		;n+=1

	ElementIonName[n]="j50V";	ElementIonMass[n]=49.9471627	;ElementIsotopAbund[n] = 0.00250/0.99750		;n+=1
	ElementIonName[n]="V";		ElementIonMass[n]=50.9439635	;ElementIsotopParentList[n]="j50V"	;n+=1

	ElementIonName[n]="j50Cr";	ElementIonMass[n]=49.9460495	;ElementIsotopAbund[n] = 0.04345/0.83789		;n+=1
	ElementIonName[n]="Cr";		ElementIonMass[n]=51.9405115	;ElementIsotopParentList[n]="j50Cr;j53Cr;j54Cr"	;n+=1
	ElementIonName[n]="j53Cr";	ElementIonMass[n]=52.9406534	;ElementIsotopAbund[n] = 0.09501/0.83789		;n+=1
	ElementIonName[n]="j54Cr";	ElementIonMass[n]=53.9388846	;ElementIsotopAbund[n] = 0.02365/0.83789		;n+=1

	ElementIonName[n]="Mn";		ElementIonMass[n]=54.9380493	;n+=1

	ElementIonName[n]="j54Fe";	ElementIonMass[n]=53.9396147	;ElementIsotopAbund[n] = 0.05845/0.91754		;n+=1
	ElementIonName[n]="Fe";		ElementIonMass[n]=55.9349418	;ElementIsotopParentList[n]="j54Fe;j57Fe;j58Fe";n+=1
	ElementIonName[n]="j57Fe";	ElementIonMass[n]=56.9353983	;ElementIsotopAbund[n] = 0.02119/0.91754		;n+=1
	ElementIonName[n]="j58Fe";	ElementIonMass[n]=57.9332801	;ElementIsotopAbund[n] = 0.00282/0.91754		;n+=1

	ElementIonName[n]="Co";		ElementIonMass[n]=58.9331999	;n+=1

	ElementIonName[n]="Ni";		ElementIonMass[n]=57.9353477	;ElementIsotopParentList[n]="j60Ni;j61Ni;j62Ni;j64Ni"	;n+=1  //1.21B
	ElementIonName[n]="j60Ni";	ElementIonMass[n]=59.9307903	;ElementIsotopAbund[n] = 0.262231/0.680769	;n+=1
	ElementIonName[n]="j61Ni";	ElementIonMass[n]=60.9310601	;ElementIsotopAbund[n] = 0.011399/0.680769	;n+=1
	ElementIonName[n]="j62Ni"	;ElementIonMass[n]=61.9283484	;ElementIsotopAbund[n] = 0.036345/0.680769	;n+=1
	ElementIonName[n]="j64Ni"	;ElementIonMass[n]=63.9279692	;ElementIsotopAbund[n] = 0.009256/0.680769	;n+=1

	ElementIonName[n]="Cu"		;ElementIonMass[n]=62.9296007	;ElementIsotopParentList[n]="j65Cu"		;n+=1
	ElementIonName[n]="j65Cu"	;ElementIonMass[n]=64.9277938	;ElementIsotopAbund[n] = 0.3085/0.6915		;n+=1

	ElementIonName[n]="Zn"		;ElementIonMass[n]=63.9291461	;ElementIsotopParentList[n]="j66Zn;j67Zn;j68Zn;j70Zn";n+=1
	ElementIonName[n]="j66Zn"	;ElementIonMass[n]=65.9260364	;ElementIsotopAbund[n] = 0.27975/0.48268		;n+=1
	ElementIonName[n]="j67Zn"	;ElementIonMass[n]=66.9271305	;ElementIsotopAbund[n] = 0.04102/0.48268		;n+=1
	ElementIonName[n]="j68Zn"	;ElementIonMass[n]=67.9248473	;ElementIsotopAbund[n] = 0.19024/0.48268		;n+=1
	ElementIonName[n]="j70Zn"	;ElementIonMass[n]=69.925325	;ElementIsotopAbund[n] = 0.00631/0.48268		;n+=1

	ElementIonName[n]="Ga"		;ElementIonMass[n]=68.925581	;ElementIsotopParentList[n]="j71Ga"			;n+=1
	ElementIonName[n]="j71Ga"	;ElementIonMass[n]=70.9247073	;ElementIsotopAbund[n] = 0.39892/0.60108		;n+=1

	ElementIonName[n]="j70Ge"	;ElementIonMass[n]=69.9242500	;ElementIsotopAbund[n] = 0.2038/0.3672		;n+=1
	ElementIonName[n]="j72Ge"	;ElementIonMass[n]=71.9220763	;ElementIsotopAbund[n] = 0.2731/0.3672		;n+=1
	ElementIonName[n]="j73Ge"	;ElementIonMass[n]=72.9234595	;ElementIsotopAbund[n] = 0.0776/0.3672		;n+=1
	ElementIonName[n]="Ge"		;ElementIonMass[n]=73.9211784	;ElementIsotopParentList[n]="j70Ge;j72Ge;j73Ge;j76Ge;"		;n+=1
	ElementIonName[n]="j76Ge"	;ElementIonMass[n]=75.9214029	;ElementIsotopAbund[n] = 0.0783/0.3672		;n+=1

	ElementIonName[n]="As";		ElementIonMass[n]=74.9215966	;ElementIsotopParentList[n]=""			;n+=1

	ElementIonName[n]="j74Se";	ElementIonMass[n]=73.9224767	;ElementIsotopAbund[n] = 0.0089/0.4961		;n+=1
	ElementIonName[n]="j76Se";	ElementIonMass[n]=75.9192143	;ElementIsotopAbund[n] = 0.0937/0.4961		;n+=1
	ElementIonName[n]="j77Se";	ElementIonMass[n]=76.9199148	;ElementIsotopAbund[n] = 0.0763/0.4961		;n+=1
	ElementIonName[n]="j78Se";	ElementIonMass[n]=77.9173097	;ElementIsotopAbund[n] = 0.2377/0.4961		;n+=1
	ElementIonName[n]="Se";		ElementIonMass[n]=79.9165221	;ElementIsotopParentList[n]="j74Se;j76Se;j77Se;j78Se;j82Se;"		;n+=1			//1.16E
	ElementIonName[n]="j82Se";	ElementIonMass[n]=81.9167003	;ElementIsotopAbund[n] = 0.0873/0.4961		;n+=1

	ElementIonName[n]="Br";		ElementIonMass[n]=78.9183379	;ElementIsotopParentList[n]="j81Br"			;n+=1
	ElementIonName[n]="j81Br";	ElementIonMass[n]=80.916291		;ElementIsotopAbund[n] = 0.4931/0.5069		;n+=1
	
	ElementIonName[n]="j78Kr";	ElementIonMass[n]=77.920388		;ElementIsotopAbund[n] = 0.00355/0.56987		;n+=1
	ElementIonName[n]="j80Kr";	ElementIonMass[n]=79.916379		;ElementIsotopAbund[n] = 0.02286/0.56987		;n+=1
	ElementIonName[n]="j82Kr";	ElementIonMass[n]=81.9134850	;ElementIsotopAbund[n] = 0.11593/0.56987		;n+=1
	ElementIonName[n]="j83Kr";	ElementIonMass[n]=82.914137		;ElementIsotopAbund[n] = 0.11500/0.56987		;n+=1
	ElementIonName[n]="Kr";		ElementIonMass[n]=83.911508		;ElementIsotopParentList[n]="j78Kr;j80Kr;j82Kr;j83Kr;j86Kr";n+=1
	ElementIonName[n]="j86Kr";	ElementIonMass[n]=85.910615		;ElementIsotopAbund[n] = 0.17279/0.56987		;n+=1
	
	//  fyi Rb is atomic number 37.... new line in periodic table
	ElementIonName[n]="Rb";		ElementIonMass[n]=84.9117924	;ElementIsotopParentList[n]="j87Rb"			;n+=1
	ElementIonName[n]="j87Rb";	ElementIonMass[n]=86.9091858	;ElementIsotopAbund[n] = 0.2783/0.7217		;n+=1
	
	// added in 1.11M
	ElementIonName[n]="j84Sr";	ElementIonMass[n]=83.913426	;	ElementIsotopAbund[n]= 0.0056/0.8258	;n+=1		//1.20E
	ElementIonName[n]="j86Sr";	ElementIonMass[n]=85.9092647	;ElementIsotopAbund[n]= 0.0986/0.8258	;n+=1
	ElementIonName[n]="j87Sr";	ElementIonMass[n]=86.9088816	;ElementIsotopAbund[n]= 0.0700/0.8258	;n+=1
	ElementIonName[n]="Sr";		ElementIonMass[n]=87.9056167	;ElementIsotopParentList[n]="j84Sr;j86Sr;j87Sr"	;n+=1  //1.20E

	ElementIonName[n]="Y";		ElementIonMass[n]=88.9058485	;ElementIsotopParentList[n]=""	;n+=1

	ElementIonName[n]="Zr";		ElementIonMass[n]=89.9047022	;ElementIsotopParentList[n]="j91Zr;j92Zr;j94Zr;j96Zr"	;n+=1
	ElementIonName[n]="j91Zr";	ElementIonMass[n]=90.9056434	;ElementIsotopAbund[n]= 0.1122/0.5145;n+=1
	ElementIonName[n]="j92Zr";	ElementIonMass[n]=91.9050386	;ElementIsotopAbund[n]= 0.1715/0.5145	;n+=1
	ElementIonName[n]="j94Zr";	ElementIonMass[n]=93.9063144	;ElementIsotopAbund[n]= 0.1738/0.5145	;n+=1
	ElementIonName[n]="j96Zr";	ElementIonMass[n]=95.908275		;ElementIsotopAbund[n]= 0.0280/0.5145	;n+=1

	ElementIonName[n]="Nb";		ElementIonMass[n]=92.9063762	;ElementIsotopParentList[n]=""	;n+=1

	ElementIonName[n]="j92Mo";	ElementIonMass[n]=91.906810	;	;ElementIsotopAbund[n]= 0.1477/0.2419;n+=1
	ElementIonName[n]="j94Mo";	ElementIonMass[n]=93.9050867	;ElementIsotopAbund[n]= 0.0923/0.2419;n+=1
	ElementIonName[n]="j95Mo";	ElementIonMass[n]=94.9058406	;ElementIsotopAbund[n]= 0.1590/0.2419	;n+=1
	ElementIonName[n]="j96Mo";	ElementIonMass[n]=95.9046780	;ElementIsotopAbund[n]= 0.1668/0.2419	;n+=1
	ElementIonName[n]="j97Mo";	ElementIonMass[n]=96.9060201	;ElementIsotopAbund[n]= 0.0956/0.2419	;n+=1
	ElementIonName[n]="Mo";		ElementIonMass[n]=97.9054069	;ElementIsotopParentList[n]="j92Mo;j94Mo;j95Mo;j96Mo;j97Mo;j100Mo"	;n+=1
	ElementIonName[n]="j100Mo";	ElementIonMass[n]=99.907476		;ElementIsotopAbund[n]= 0.0967/0.2419	;n+=1

	//  Tc  		// not done
	//1.15 additions
		// Ru
	ElementIonName[n]="j96Ru";	ElementIonMass[n]= 95.907604	;ElementIsotopAbund[n]= 0.0554/0.3155		;n+=1
	ElementIonName[n]="j98Ru";	ElementIonMass[n]= 97.905287	;ElementIsotopAbund[n]= 0.0187/0.3155		;n+=1
	ElementIonName[n]="j99Ru";	ElementIonMass[n]= 98.9059385	;ElementIsotopAbund[n]=0.1276/0.3155		;n+=1
	ElementIonName[n]="j100Ru";	ElementIonMass[n]= 99.9042189	;ElementIsotopAbund[n]=0.1260/0.3155		;n+=1
	ElementIonName[n]="j101Ru";	ElementIonMass[n]= 100.9055815	;ElementIsotopAbund[n]=0.1706/0.3155		;n+=1
	ElementIonName[n]="Ru";		ElementIonMass[n]= 101.9043488	;ElementIsotopParentList[n]="j96Ru;j98Ru;j99Ru;j100Ru;j101Ru;j104Ru;"	;n+=1
	ElementIonName[n]="j104Ru";	ElementIonMass[n]= 103.905430	;ElementIsotopAbund[n]=0.1862/0.3155		;n+=1

	ElementIonName[n]="Rh";	ElementIonMass[n]= 102.905504	;ElementIsotopParentList[n]=""	;n+=1

	ElementIonName[n]="j102Pd";	ElementIonMass[n]= 101.905607	;ElementIsotopAbund[n]=0.0102/0.2733	;n+=1
	ElementIonName[n]="j104Pd";	ElementIonMass[n]= 103.904034	;ElementIsotopAbund[n]=0.1114/0.2733	;n+=1
	ElementIonName[n]="j105Pd";	ElementIonMass[n]= 104.905083	;ElementIsotopAbund[n]=0.2233/0.2733	;n+=1
	ElementIonName[n]="Pd";		ElementIonMass[n]= 105.903484	;ElementIsotopParentList[n]="j102Pd;j104Pd;j105Pd;j108Pd;j110Pd;"	;n+=1
	ElementIonName[n]="j108Pd";	ElementIonMass[n]= 107.903895	;ElementIsotopAbund[n]=0.2646/0.2733	;n+=1
	ElementIonName[n]="j110Pd";	ElementIonMass[n]= 109.905153	;ElementIsotopAbund[n]=0.1172/0.2733	;n+=1
	//1.15 additions  end

	ElementIonName[n]="Ag"		;ElementIonMass[n]=106.905093	;ElementIsotopParentList[n]="j109Ag"	;n+=1
	ElementIonName[n]="j109Ag"	;ElementIonMass[n]=108.904756	;ElementIsotopAbund[n]= 0.481/0.518	;n+=1

	//1.15 additions  begin
	ElementIonName[n]="j106Cd";	ElementIonMass[n]=  105.9064587	;ElementIsotopAbund[n]=0.0125/0.2873	;n+=1
	ElementIonName[n]="j108Cd";	ElementIonMass[n]=  107.9041837	;ElementIsotopAbund[n]=0.0089/0.2873	;n+=1
	ElementIonName[n]="j110Cd";	ElementIonMass[n]=  109.9030067	;ElementIsotopAbund[n]=0.1249/0.2873	;n+=1
	ElementIonName[n]="j111Cd";	ElementIonMass[n]=  110.9041827	;ElementIsotopAbund[n]= 0.1280/0.2873	;n+=1
	ElementIonName[n]="j112Cd";	ElementIonMass[n]=  111.90275777	;ElementIsotopAbund[n]= 0.2413/0.2873	;n+=1
	ElementIonName[n]="j113Cd";	ElementIonMass[n]=  112.90440147	;ElementIsotopAbund[n]= 0.1222/0.2873	;n+=1
	ElementIonName[n]="Cd";		ElementIonMass[n]=  113.90335867	;ElementIsotopParentList[n]="j106Cd;j108Cd;j110Cd;j111Cd;j112Cd;j113Cd;j116Cd"	;n+=1
	ElementIonName[n]="j116Cd";	ElementIonMass[n]=  115.9047567	;ElementIsotopAbund[n]= 0.0749/0.2873	;n+=1

	ElementIonName[n]="j113In"; 	ElementIonMass[n]= 112.904062	;ElementIsotopAbund[n]=0.0429/0.9571	;n+=1
	ElementIonName[n]="In";		ElementIonMass[n]= 114.903879	;ElementIsotopParentList[n]="j113In"	;n+=1
	//1.15 additions  end

	ElementIonName[n]="j112Sn";	ElementIonMass[n]=111.904822	;ElementIsotopAbund[n]= 0.0097/0.3258	;n+=1
	ElementIonName[n]="j114Sn";	ElementIonMass[n]=113.902783	;ElementIsotopAbund[n]= 0.0066/0.3258	;n+=1
	ElementIonName[n]="j115Sn";	ElementIonMass[n]=114.903347	;ElementIsotopAbund[n]= 0.0034/0.3258	;n+=1
	ElementIonName[n]="j116Sn";	ElementIonMass[n]=115.901745	;ElementIsotopAbund[n]= 0.1454/0.3258	;n+=1
	ElementIonName[n]="j117Sn";	ElementIonMass[n]=116.902955	;ElementIsotopAbund[n]= 0.0768/0.3258	;n+=1
	ElementIonName[n]="j118Sn";	ElementIonMass[n]=117.901608	;ElementIsotopAbund[n]= 0.2422/0.3258	;n+=1  //1.21B
	ElementIonName[n]="j119Sn";	ElementIonMass[n]=118.903311	;ElementIsotopAbund[n]= 0.0859/0.3258	;n+=1
	ElementIonName[n]="Sn";		ElementIonMass[n]=119.9021985	;ElementIsotopParentList[n]="j112Sn;j114Sn;j115Sn;j116Sn;j117Sn;j118Sn;j119Sn;j122Sn;j124Sn;"	;n+=1
	ElementIonName[n]="j122Sn";	ElementIonMass[n]=121.9034411	;ElementIsotopAbund[n]= 0.0463/0.3258	;n+=1
	ElementIonName[n]="j124Sn";	ElementIonMass[n]=123.9052745	;ElementIsotopAbund[n]= 0.0579/0.3258	;n+=1

	ElementIonName[n]="Sb";		ElementIonMass[n]=120.903822	;ElementIsotopParentList[n]="j122Sb"	;n+=1
	ElementIonName[n]="j122Sb";	ElementIonMass[n]=122.9042160	;ElementIsotopAbund[n]=0.4279/0.5721	;n+=1

	//1.15 additions  begin
	ElementIonName[n]="j120Te"; 	ElementIonMass[n]=  119.904026	;ElementIsotopAbund[n]=0.0009/0.3408	;n+=1
	ElementIonName[n]="j122Te"; 	ElementIonMass[n]=  121.9030558	;ElementIsotopAbund[n]=0.0255/0.3408	;n+=1
	ElementIonName[n]="j123Te"; 	ElementIonMass[n]=  122.9042711	;ElementIsotopAbund[n]=0.0089/0.3408	;n+=1
	ElementIonName[n]="j124Te"; 	ElementIonMass[n]=  123.9028188	;ElementIsotopAbund[n]=0.0474/0.3408	;n+=1
	ElementIonName[n]="j125Te"; 	ElementIonMass[n]=  124.9044241	;ElementIsotopAbund[n]=0.0707/0.3408	;n+=1
	ElementIonName[n]="j126Te"; 	ElementIonMass[n]=  125.9033049	;ElementIsotopAbund[n]=0.1884/0.3408	;n+=1
	ElementIonName[n]="j128Te"; 	ElementIonMass[n]=  127.9044615	;ElementIsotopAbund[n]=0.3174/0.3408	;n+=1
	ElementIonName[n]="Te"	; 	ElementIonMass[n]=  129.9062229	;ElementIsotopParentList[n]="j120Te;j122Te;j123Te;j124Te;j125Te;j126Te;j128Te;"	;n+=1

	ElementIonName[n]="I"	; 	ElementIonMass[n]= 126.904468	;ElementIsotopParentList[n]=""; n+=1

	ElementIonName[n]="j124Xe"; 	ElementIonMass[n]=  123.9058954	;ElementIsotopAbund[n]=0.000952/0.269086	;n+=1
	ElementIonName[n]="j126Xe"; 	ElementIonMass[n]=  125.904268 	;ElementIsotopAbund[n]=0.000890/0.269086	;n+=1
	ElementIonName[n]="j128Xe"; 	ElementIonMass[n]=  127.9035305 ;ElementIsotopAbund[n]=0.019102/0.269086	;n+=1
	ElementIonName[n]="j129Xe"; 	ElementIonMass[n]=  128.9047799 ;ElementIsotopAbund[n]=0.264006/0.269086	;n+=1
	ElementIonName[n]="j130Xe"; 	ElementIonMass[n]=  129.9035089 ;ElementIsotopAbund[n]=0.040710/0.269086	;n+=1
	ElementIonName[n]="j131Xe"; 	ElementIonMass[n]=  130.9050828 ;ElementIsotopAbund[n]=0.212324/0.269086	;n+=1
	ElementIonName[n]="Xe"; 		ElementIonMass[n]=  131.9041546 ;ElementIsotopParentList[n]="j124Xe;j126Xe;j128Xe;j129Xe;j130Xe;j131Xe;j134Xe;j136Xe;";n+=1
	ElementIonName[n]="j134Xe"; 	ElementIonMass[n]=  133.9053945 ;ElementIsotopAbund[n]=0.104357/0.269086	;n+=1
	ElementIonName[n]="j136Xe"; 	ElementIonMass[n]=  135.907220 	;ElementIsotopAbund[n]=0.088573/0.269086	;n+=1

	//  Cs has atomic number 55, new line in periodic table
	ElementIonName[n]="Cs"	; 	ElementIonMass[n]= 132.905447	;ElementIsotopParentList[n]=""; n+=1

	ElementIonName[n]="j130Ba"; 	ElementIonMass[n]=   129.906311 	;ElementIsotopAbund[n]=0.00106/0.71698	;n+=1
	ElementIonName[n]="j132Ba"; 	ElementIonMass[n]=   131.905056 	;ElementIsotopAbund[n]=0.00101/0.71698	;n+=1
	ElementIonName[n]="j134Ba"; 	ElementIonMass[n]=   133.904504 	;ElementIsotopAbund[n]=0.02417/0.71698	;n+=1
	ElementIonName[n]="j135Ba"; 	ElementIonMass[n]=   134.905684 	;ElementIsotopAbund[n]=0.06592/0.71698	;n+=1
	ElementIonName[n]="j136Ba"; 	ElementIonMass[n]=   135.904571 	;ElementIsotopAbund[n]=0.07854/0.71698	;n+=1
	ElementIonName[n]="j137Ba"; 	ElementIonMass[n]=   136.905822 	;ElementIsotopAbund[n]=0.11232/0.71698	;n+=1
	ElementIonName[n]="Ba"; 	ElementIonMass[n]=   137.905242 	;ElementIsotopParentList[n]="j130Ba;j132Ba;j134Ba;j135Ba;j136Ba;j137Ba;"	;n+=1  //1.21B

	ElementIonName[n]="Au"	; 	ElementIonMass[n]= 196.966551	;ElementIsotopParentList[n]=""; n+=1

	ElementIonName[n]="j180W";	ElementIonMass[n]=179.946706	;ElementIsotopAbund[n] = 0.0012/0.3064		;n+=1
	ElementIonName[n]="j182W";	ElementIonMass[n]=181.948205	;ElementIsotopAbund[n] = 0.2650/0.3064		;n+=1
	ElementIonName[n]="j183W";	ElementIonMass[n]=182.9502242	;ElementIsotopAbund[n] = 0.1431/0.3064		;n+=1
	ElementIonName[n]="W";		ElementIonMass[n]=183.9509323	;ElementIsotopParentList[n]="j180W;j182W;j183W;j186W";n+=1
	ElementIonName[n]="j186W";	ElementIonMass[n]=185.954362	;ElementIsotopAbund[n] = 0.2843/0.3064		;n+=1
	
	ElementIonName[n]="j196Hg";	ElementIonMass[n]=195.965814	;ElementIsotopAbund[n] = 0.0015/0.2986		;n+=1
	ElementIonName[n]="j198Hg";	ElementIonMass[n]=197.966752	;ElementIsotopAbund[n] = 0.0997/0.2986		;n+=1
	ElementIonName[n]="j199Hg";	ElementIonMass[n]=198.968262	;ElementIsotopAbund[n] = 0.1687/0.2986		;n+=1
	ElementIonName[n]="j200Hg";	ElementIonMass[n]=199.968309	;ElementIsotopAbund[n] = 0.2310/0.2986		;n+=1
	ElementIonName[n]="j201Hg";	ElementIonMass[n]=200.970285	;ElementIsotopAbund[n] = 0.1318/0.2986		;n+=1
	ElementIonName[n]="Hg";		ElementIonMass[n]=201.970625	;ElementIsotopParentList[n]="j196Hg;j198Hg;j199Hg;j200Hg;j201Hg;j204Hg";n+=1
	ElementIonName[n]="j204Hg";	ElementIonMass[n]=203.973475	;ElementIsotopAbund[n] = 0.0687/0.2986		;n+=1
	
	ElementIonName[n]="j204Pb";	ElementIonMass[n]=203.973028	;ElementIsotopAbund[n] = 0.014/0.524			;n+=1
	ElementIonName[n]="j206Pb";	ElementIonMass[n]=205.974449 	;ElementIsotopAbund[n] = 0.241/0.524			;n+=1
	ElementIonName[n]="j207Pb";	ElementIonMass[n]=206.975880	;ElementIsotopAbund[n] = 0.221/0.524			;n+=1
	ElementIonName[n]="Pb";		ElementIonMass[n]=207.976636	;ElementIsotopParentList[n]="j204Pb;j206Pb;j207Pb"	;n+=1
	
	ElementIonName[n]="j203Tl"	; 	ElementIonMass[n]=   202.972329 	;ElementIsotopAbund[n]=0.2952/0.7048	;n+=1
	ElementIonName[n]="Tl"	; 		ElementIonMass[n]=   204.974412 	;ElementIsotopParentList[n]="j203Tl;";n+=1  //1.21B

	ElementIonName[n]="Bi"	; 	ElementIonMass[n]=  208.980384	;ElementIsotopParentList[n]=""; n+=1

	deletepoints n, 500, ElementIonName, ElementIonMass, ElementIsotopParentList, ElementIsotopAbund, ElementIsotopParentIndexList

	for(idex=0;idex<n;idex+=1)
		if (strlen(ElementIsotopParentList[idex])>0)
			str=ElementIsotopParentList[idex]
			for(jdex=0;jdex<ItemsInList(str);jdex+=1)
				ElementIsotopParentIndexList[idex]+=num2str(sq_TextIntextWave(StringFromList(jdex,str), ElementIonName))+";"
			endfor
		endif
	endfor
	
	PK_AtomOrder()  // 1.21B

End


// Code for calculating ion masses (QC, Nov 2, 2009)
// This code reads IonName and uses the corresponding masses stored in the wave "ElementExactMasses" to recalculate the ion mass.
// Note that the current input waves (ElementIons and ElementExactMasses, data from NIST website) do not include all the possible elements and isotopes, and will be updated.
// usage:	   recalc_IonMasses(root:HR:ExactMassText_All, root:tempMass)
Function PK_recalc_IonMasses(IonName, resultMassTemp, multiChargedtemp, inVorWnameTemp)
wave/T IonName, inVorWnameTemp //Ion name list
wave resultMassTemp, multiChargedTemp

	wave/T ElementIonName = root:HR:ElementIonName		// created in CreateElementWaves
	wave ElementIonMass = root:HR:ElementIonMass		// created in CreateElementWaves
	
	string CheckIonName, IonNameStr, ResidIonName
	string str1, str2
	variable stepPos, startPos, elementIndex, numAfterElement
	variable ionIndex, massTemp, plusPos, inPos, numElements
	string infoStr
	variable verboseON = 0	
		
	redimension/D/N=(numpnts(IonName)) resultMassTemp //double-precision; single-precision will give a wrong answer. 
	redimension/N=(numpnts(IonName)) multiChargedtemp		
	redimension/N=(numpnts(IonName)) inVorWnameTemp			// masses of these beast not precise
	
	fastop multiChargedtemp = (nan)
	inVorWnameTemp=""
	
	numElements = numpnts(ElementIonName)
	multiChargedtemp=1
	resultMassTemp=0
		
	for (ionIndex=0;ionIndex<numpnts(IonName);ionIndex+=1)
		CheckIonName=IonName[ionIndex]
		infoStr= num2str(ionIndex) + "\t"+CheckIonName+"\t\t" //note out when run the code for a long ion list
		startPos=0
		massTemp=0
		plusPos=strsearch(CheckIonName, "plus", 0, 2)
		inPos=strsearch(CheckIonName, "inV", 0, 2)		// as in inV or inW  1.14M
		if(inPos>=0	)		//1.15
			multiChargedtemp[ionIndex] = 4		//The exact mass values will likely not work out exactly.
			CheckIonName=CheckIonName[0,strlen(CheckIonName)-4]
		endif
		inPos=strsearch(CheckIonName, "inW", 0, 2)		// as in inV or inW
		if(inPos>=0	)		//1.15
			multiChargedtemp[ionIndex] = 4/9	// The exact mass values will likely not work out exactly.
			CheckIonName=CheckIonName[0,strlen(CheckIonName)-4]
		endif
	
		// check for multiple-charged ions first; reformat their ionNames by removing "plus*" 
		if (plusPos>=0) 
			splitstring/E="(plus)(\\d{1,3})" CheckIonName, str1, str2
			if (strlen(str2) ==0) //we do not have numbers after the letter
				numAfterElement = 1
			else	
				numAfterElement = str2num(str2)
			endif
			multiChargedtemp[ionIndex] =numAfterElement
			CheckIonName=CheckIonName[0,plusPos-1]+CheckIonName[plusPos+4+strlen(str2),strlen(CheckIonName)-1]
		endif
	 	// read ions for all the elements and isotopes
	 	str2=""
	 	do 
	 		ResidIonName=CheckIonName[startPos, startPos+6]
			for (stepPos=6;stepPos>=0;stepPos-=1) //maximum string length for the element name = 6: e.g., j207Pb
				IonNameStr=CheckIonName[startPos, startPos+stepPos]
				//print IonName
				for (elementIndex=0;elementIndex<numElements;elementIndex+=1)
//					if (stringmatch(ElementIonName[elementIndex],IonNameStr))		// not case sensitive!!		//1.11K
					if (cmpStr(ElementIonName[elementIndex],IonNameStr,1)==0)
						splitstring/E=("("+ElementIonName[elementIndex]+")"+"(\\d{1,3})") ResidIonName, str1, str2
						if (strlen(str2)==0) //we do not have numbers after the letter
							numAfterElement= 1
							infoStr+= "Read Ions of " + ElementIonName[elementIndex] + " Num: 1"+"\t\t" //note out when run the code for a long ion list
						else	
							numAfterElement = str2num(str2)
							infoStr+= "Read Ions of " + str1 + " Num:"+ str2 +"\t\t"//note out when run the code for a long ion list
						endif
						massTemp += (numAfterElement*ElementIonMass[elementIndex])
						break
					endif	// we found the element
				endfor
				if (elementIndex<numElements && cmpStr(ElementIonName[elementIndex],IonNameStr,1)==0)		//1.58G
				// if (stringmatch(ElementIonName[elementIndex],IonNameStr))  // not case sensitive!!		//1.11K
					startPos+=(stepPos+strlen(str2))
				break
				endif
			endfor
			startPos+=1	
		while(startPos<strlen(CheckIonName))
	 	resultMassTemp[ionIndex]=massTemp/multiChargedtemp[ionIndex]
	 	infoStr+= "multiCharged: "+num2str(multiChargedtemp[ionIndex]) //note out when run the code for a long ion list
		if (verboseON)
			print infoStr
		endif
	endfor

End


//1.08G
Function PK_checkMassWithChemFormula(discrepancyVal)
variable discrepancyVal

	variable idex, num, numBad
	string str, resultStr
	wave/t ExactMassText_All = root:HR:ExactMassText_All	
	wave/d ExactMassWave_All = root:HR:ExactMassWave_All	
	
	num = numpnts(ExactMassWave_All)

	make/o/n=(num) resultMassTemp, multiChargedtemp, exactMassDiff

	make/o/t/n=(num) inVorWnameTemp
	
	pk_recalc_IonMasses(ExactMassText_All, resultMassTemp, multiChargedtemp, inVorWnameTemp)
	
	exactMassDiff = abs(ExactMassWave_All - resultMassTemp)
	str=""
	resultStr=""
	numBad=0
	for (idex=0;idex<num;idex+=1)
		if (exactMassDiff[idex]>discrepancyVal)
			sprintf str, "from the formula mass is: %1.5f ; from the ExactMassWave_All: %1.5f  \r", ExactMassWave_All[idex], resultMassTemp[idex]
			resultStr +="// At point number "+num2str(idex) + " = HR mass "+ExactMassText_All[idex]+" there is a discrepancy with the mass value; "+str
			numBad+=1
		endif	
	endfor
	
	killwaves/z  resultMassTemp, multiChargedtemp, inVorWnameTemp, exactMassDiff
	
	print resultStr
	sprintf str, "%1.1e", discrepancyVal
	DoAlert  0, "Using the ElementIonMass and ExactMassWave_all waves "+num2str(numbad)+" many ions showed a mass discrepancy between the mass estimated from the chemical formula and the mass value in the ExactMassWave_all within "+str+" mass units.  Check the history window for details."	
	
End


//1.11D  change the name of thewaves to have a number prefix.  this makes sure waves don't get overwritten (todo waves can be changed)
//1.08E
// After a sucessful HR fit of a todo wave we make copies root:HR_sticks:ExactMassText_FitTodo
// We duplicate all the waves that are needed by any function that calls the results of a todo fit
Function PK_duplicate_FitTodoWaves(str,[PToFSubtractFlag])  //1.21G  we may be doing this at a later time, subtracting PToF matricies, not attime of fit!
string str
variable PToFSubtractFlag

	svar sq_toDoWvNm = root:panel:sq_toDoWvNm		//1.08G
	nvar HR_singleSpectraMSorPToF = root:HR_panel:HR_singleSpectraMSorPToF
//	nvar HR_CalcManyRunsOpenClosed = root:HR_panel:HR_CalcManyRunsOpenClosed	//1.11M  eliminated in 1.14M

	//  the main players
	duplicate/o root:HR_sticks:ExactMassText 		root:HR_sticks:ExactMassText_FitTodo	
	duplicate/o root:HR_sticks:ExactMassWave		root:HR_sticks:ExactMassWave_FitTodo	
	duplicate/o root:HR_sticks:ExactMassIsotopeRef 	root:HR_sticks:ExactMassIsotopeRef_FitTodo		// not sure if these are needed, but we do them here
	duplicate/o root:HR_sticks:ExactMassIsotopeAbund	root:HR_sticks:ExactMassIsotopeAbund_FitTodo
	duplicate/o root:HR_sticks:ExactMassFamilyText	root:HR_sticks:ExactMassFamilyText_FitTodo

//added in  1.09K
	duplicate/o/t root:HR_sticks:IsotopicMassText	 	root:HR_sticks:IsotopicMassText_FitTodo		
	duplicate/o root:HR_sticks:IsotopicMask		root:HR_sticks:IsotopicMask_FitTodo

	wave ExactMassText_FitTodo = root:HR_sticks:ExactMassText_FitTodo	
	wave ExactMassWave_FitTodo = root:HR_sticks:ExactMassWave_FitTodo	
	wave ExactMassIsotopeRef_FitTodo = root:HR_sticks:ExactMassIsotopeRef_FitTodo		// not sure if these are needed, but we do them here
	wave ExactMassIsotopeAbund_FitTodo = root:HR_sticks:ExactMassIsotopeAbund_FitTodo
	wave ExactMassFamilyText_FitTodo = root:HR_sticks:ExactMassFamilyText_FitTodo

//added in  1.09K
	wave/t IsotopicMassText_FitTodo = root:HR_sticks:IsotopicMassText_FitTodo
	wave IsotopicMask_FitTodo = root:HR_sticks:IsotopicMask_FitTodo

	Note/K ExactMassText_FitTodo, "Todo:"+sq_toDoWvNm		// notes are not really necessary
	Note/K ExactMassWave_FitTodo, "Todo:"+sq_toDoWvNm
	Note/K ExactMassIsotopeRef_FitTodo, "Todo:"+sq_toDoWvNm
	Note/K ExactMassIsotopeAbund_FitTodo, "Todo:"+sq_toDoWvNm
	Note/K ExactMassFamilyText_FitTodo, "Todo:"+sq_toDoWvNm

	Note/K IsotopicMassText_FitTodo, "Todo:"+sq_toDoWvNm
	Note/K IsotopicMask_FitTodo, "Todo:"+sq_toDoWvNm

//added in  1.09K
	// parts needed for UMR summing of HR sticks
	duplicate/o root:HR_Sticks:MassStartPnt	root:HR_Sticks:MassStartPnt_FitTodo
	duplicate/o root:HR_Sticks:MassEndPnt	root:HR_Sticks:MassEndPnt_FitTodo
	duplicate/o root:HR_Sticks:UnitMass		root:HR_Sticks:UnitMass_FitTodo
	duplicate/o root:HR_Sticks:numOfPeaks	root:HR_Sticks:numOfPeaks_FitTodo	

	wave MassStartPnt_FitTodo = root:HR_sticks:MassStartPnt_FitTodo
	wave MassEndPnt_FitTodo = root:HR_sticks:MassEndPnt_FitTodo
	wave UnitMass_FitTodo = root:HR_sticks:UnitMass_FitTodo
	wave numOfPeaks_FitTodo = root:HR_sticks:numOfPeaks_FitTodo

	Note/K ExactMassText_FitTodo, "Todo:"+sq_toDoWvNm		// notes are not really necessary
	Note/K MassEndPnt_FitTodo, "Todo:"+sq_toDoWvNm		
	Note/K UnitMass_FitTodo, "Todo:"+sq_toDoWvNm			
	Note/K numOfPeaks_FitTodo, "Todo:"+sq_toDoWvNm
	
	// family identifiers
	wave/t HR_familyName = root:HR:HR_familyName
	variable kdex, numFamily, iterationNum
	
	numFamily = numpnts(HR_familyName)
	for (kdex = 0;kdex<numFamily;kdex+=1)	// go through all the families to find its match
		wave ThisFamily = $"root:Int_Sticks:mask_"+HR_familyName[kdex]			
		duplicate/o ThisFamily $"root:Int_Sticks:mask_"+HR_familyName[kdex]	+"_FitTodo"
		wave mask =  $"root:Int_Sticks:mask_"+HR_familyName[kdex]+"_FitTodo"
		Note/K mask, "Todo:"+sq_toDoWvNm		// notes are not really necessary
	endfor		

	if (PToFSubtractFlag || HR_singleSpectraMSorPToF==0)			//PToF
//		ControlInfo/W=HR_Analysis_Panel HRFit_popup_PToFOpen
		string OpenStr =str  //"PToF"+S_value
		newdatafolder/o root:HR_sticks:HRIonWaveCopies	//1.13B
		newdatafolder/o root:HR_sticks:HRIonWavePToF
		newdatafolder/o $"root:HR_sticks:HRIonWavePToF:"+OpenStr
		wave/t/z PToF_bAxB_seriesKey = $"root:index:"+OpenStr+"_seriesKey"
		iterationNum = WaveExists(PToF_bAxB_seriesKey) ?  numpnts(PToF_bAxB_seriesKey) : 0
		
		PK_CopyHRIonWavesFitTodo("root:HR_sticks:", "root:HR_sticks:HRIonWavePToF:"+OpenStr+":"+"I"+num2str(iterationNum)+"_"+sq_toDoWvNm+":","root:Int_Sticks:", "root:HR_sticks:HRIonWaveCopies:"+"I"+num2str(iterationNum)+"_"+sq_toDoWvNm+":")
	elseif (HR_singleSpectraMSorPToF==1 || HR_singleSpectraMSorPToF==2)		// MS or ePToFSum
		//1.11D
		wave/t/z HRstick_seriesKey = root:index:HRstick_seriesKey
		iterationNum = WaveExists(HRstick_seriesKey) ? numpnts(HRstick_seriesKey) : 0 // 1.14G

		newdatafolder/o root:HR_sticks:HRIonWaveCopies
 		PK_CopyHRIonWavesFitTodo("root:HR_sticks:", "root:HR_sticks:HRIonWaveCopies:"+"I"+num2str(iterationNum)+"_"+sq_toDoWvNm+":","root:Int_Sticks:", "root:HR_sticks:HRIonWaveCopies:"+"I"+num2str(iterationNum)+"_"+sq_toDoWvNm+":")
	endif

//		if (HR_CalcManyRunsOpenClosed)	// do both PToF 'open' and 'closed'   //1.14M  Changed how PToF HR sticks operate.
//			ControlInfo/W=HR_Analysis_Panel HRFit_popup_PToFClosed
//			string ClosedStr = "PToF_"+S_value
//			newdatafolder/o root:HR_sticks:HRIonWavePToF
//			newdatafolder/o $"root:HR_sticks:HRIonWavePToF:"+OpenStr+"M"+ClosedStr 		// from ECB  1.14G
//			newdatafolder/o $"root:HR_sticks:HRIonWavePToF:"+ClosedStr
//			PK_CopyHRIonWavesFitTodo("root:HR_sticks:", "root:HR_sticks:HRIonWavePToF:"+ClosedStr+":"+"I"+num2str(iterationNum)+"_"+sq_toDoWvNm+":","root:Int_Sticks:", "root:HR_sticks:HRIonWaveCopies:"+"I"+num2str(iterationNum)+"_"+sq_toDoWvNm+":")			
//			PK_CopyHRIonWavesFitTodo("root:HR_sticks:", "root:HR_sticks:HRIonWavePToF:"+OpenStr+"M"+ClosedStr+":"+"I"+num2str(iterationNum)+"_"+sq_toDoWvNm+":","root:Int_Sticks:", "root:HR_sticks:HRIonWaveCopies:"+"I"+num2str(iterationNum)+"_"+sq_toDoWvNm+":")
//		endif
 
 End
 
 
// after a successful HR fit of a todo wave save info
// enters values in  root:index:HRstick_series and root:index:HRstick_seriesKey
// to keep track of which runs have been fit with a todo wave.
Function PK_IndexHRResultsWavesFitTodo(str, [PToFSubtractFlag])
string str //1.15D
variable PToFSubtractFlag

	svar sq_toDoWvNm = root:panel:sq_toDoWvNm		//1.08G
	nvar HR_singleSpectraMSorPToF=root:HR_panel:HR_singleSpectraMSorPToF
	
	wave todo = $"root:"+sq_toDoWvNm
	wave rn_series = root:index:rn_series

	variable num, numHRstick_seriesKey
	
	if(PToFSubtractFlag || HR_singleSpectraMSorPToF==0)					//PTOF   for the same run we may have several PToF data sets... each PToF data set gets its own *_series and *_seriesKey waves.
//		ControlInfo/W=HR_Analysis_Panel HRFit_popup_PToFOpen
		string OpenStr =  str // "PToF_"+S_value

		wave/z HRstick_series = $"root:index:"+OpenStr+"_series"
		wave/t/z HRstick_seriesKey = $"root:index:"+OpenStr+"_seriesKey"
		if (!WaveExists(HRstick_series))
			make/o/n=(numpnts(rn_series))$"root:index:"+OpenStr+"_series"=nan
			wave HRstick_series = $"root:index:"+OpenStr+"_series"
			make/t/o/n=(0) $"root:index:"+OpenStr+"_seriesKey"
			wave/t HRstick_seriesKey =$"root:index:"+OpenStr+"_seriesKey"
		endif
	elseif (HR_singleSpectraMSorPToF==1 || HR_singleSpectraMSorPToF==2)		// MS  we have one set of *_series and *_seriesKey waves.
		wave/z HRstick_series = root:index:HRstick_series
		wave/t/z HRstick_seriesKey = root:index:HRstick_seriesKey
		if (!WaveExists(HRstick_series))
			make/o/n=(numpnts(rn_series)) root:index:HRstick_series=nan
			wave HRstick_series = root:index:HRstick_series
			make/t/o/n=(0) root:index:HRstick_seriesKey
			wave/t HRstick_seriesKey = root:index:HRstick_seriesKey
		endif
	endif

	if (numpnts(HRstick_series)!= numpnts(rn_series))		//1.10L
		InsertPoints numpnts(HRstick_series), (numpnts(rn_series)!= numpnts(HRstick_series)), HRstick_series
		HRstick_series[num,]=nan
	endif
	
	numHRstick_seriesKey = numpnts(HRstick_seriesKey)		

	insertPoints numHRstick_seriesKey, 1, HRstick_seriesKey
	HRstick_seriesKey[numHRstick_seriesKey]="I"+num2str(numHRstick_seriesKey)+"_"+sq_toDoWvNm	//1.11D change to include iteration number so wave copies don't get overwritten.
	
	make/o/n=(numpnts(rn_series)) tempMask
	sq_makeMaskFromTodofindRun(tempMask,todo)
	HRstick_series = tempMask[p]==1? numHRstick_seriesKey : HRstick_series[p] 
	
	killwaves/z tempMask

End

//1.08G
// from the currently selected todo wave returns string of folder containing ExactMassWave et al waves from a todo fitting
Function/s PK_FindHRResultsWavesFitTodo()

	ControlInfo/W=HR_Analysis_Panel HRResults_pop_dataSet
	if (stringMatch("Get List", S_value))		// 1.21B
		abort "Please select a data set from the Pika HR Reuslts tab, Data set"
	endif
	return PK_FindHRResultsWavesFitTodoDS(S_value)

End

//1.08G
// from the currently selected todo wave returns string of folder containing ExactMassWave et al waves from a todo fitting
Function/s PK_FindHRResultsWavesFitTodoDS(dataSet)
string dataSet

	svar sq_toDoWvNm = root:panel:sq_toDoWvNm		//1.08G
	wave todo = $"root:"+sq_toDoWvNm

	variable MSorPToFFlag
	string dataStr=""
	
	MSorPToFFlag = stringMatch(dataSet[0,1],"HR")
	
	if (MSorPToFFlag)
		wave/z HRstick_series = root:index:HRstick_series
		wave/t/z HRstick_seriesKey = root:index:HRstick_seriesKey
		if (!WaveExists(HRstick_series)  || !WaveExists(HRstick_seriesKey))
			return ""
		endif
	else
		dataStr = dataSet
		dataStr=ReplaceString("Stick", dataStr, "")
		wave/z HRstick_series = $"root:index:"+dataStr+"_series"
		wave/t/z HRstick_seriesKey = $"root:index:"+dataStr+"_seriesKey"
		if (!WaveExists(HRstick_series)  || !WaveExists(HRstick_seriesKey))
			//return ""
			abort "Could not find the series waves root:index:"+dataStr+"_series   aborting from PK_FindHRResultsWavesFitTodo"
		endif
	endif

	wave rn_series = root:index:rn_series		// 1.11A
	if (numpnts(rn_series) != numpnts(HRstick_series)	)
		DoAlert 2, "The number of runs has changed.  If more runs were added to the end of the previous runs, simple change the number of points in these waves."
		Edit HRstick_series,HRstick_seriesKey, rn_series 		
	endif

	variable myindex
	string thistodo, TodoPlusIterationNum, FolderStr, tempTodoPlusIterationNum
	
	make/FREE/o/n=(numpnts(HRstick_series)) tempMask
	sq_makeMaskFromTodofindRun(tempMask,todo)
	tempMask/=tempMask		// make nans and 1s
	tempMask*=HRstick_series		// will have nans an numerical values. numerical values are indexes in HRstick_seriesKey

	WaveStats/q/M=1 tempMask
	if (V_npnts==0)
		abort "There doesn't seem to be any HR sticks that were generated in this todo wave.  Aborting from  PK_FindHRResultsWavesFitTodo()"
	endif

	myindex = Wavemax(tempMask)		// could use max or min
	TodoPlusIterationNum = HRstick_seriesKey[myindex]		// as of 1.11D this also has the iteration number
//	TodoPlusIterationNum = replaceString(stringFromList(0, TodoPlusIterationNum, "_"), TodoPlusIterationNum,"I0")
	if (MSorPToFFlag)
		wave/Z ExactMassText_FitTodo = $"root:HR_sticks:HRIonWaveCopies:"+TodoPlusIterationNum+":ExactMassText_FitTodo"		// one of many waves could be used here	
	else
		wave/Z ExactMassText_FitTodo = $"root:HR_sticks:HRIonWavePToF:"+dataStr+":"+TodoPlusIterationNum+":ExactMassText_FitTodo"		// one of many waves could be used here	
	endif
	if (!WaveExists(ExactMassText_FitTodo)	)		// for backwards compatibility
		abort "Something seems to be wrong with the wave root:HR_sticks:HRIonWaveCopies:"+TodoPlusIterationNum+":ExactMassText_FitTodo -  Aborting from  PK_FindHRResultsWavesFitTodo()"
	endif

	do
		tempMask = tempMask[p] == 	myindex ? nan : tempMask[p]
		myindex = Wavemax(tempMask)		// could use max or min   when the runs in the todo wave are exhaused (tempMask  is all nans and myindex is nan.  then tempTodoPlusIterationNum becomes garbage
		if (myindex>=0)
			tempTodoPlusIterationNum= HRstick_seriesKey[myindex] //1.19D  now inside if statement // we no longer use only the todo wave Name because this can change alot.
			FolderStr=selectString(MSorPToFFlag, "root:HR_sticks:HRIonWavePToF:"+dataStr+":"+tempTodoPlusIterationNum, "root:HR_sticks:HRIonWaveCopies:"+tempTodoPlusIterationNum)
			
			wave/Z ExactMassText_FitTodoTemp = $FolderStr+":ExactMassText_FitTodo"		// one of many waves could be used here	
			if (!WaveExists(ExactMassText_FitTodoTemp)	)		// for backwards compatibility
				abort "Something seems to be wrong with the wave "+FolderStr+":ExactMassText_FitTodo -  Aborting from  PK_FindHRResultsWavesFitTodo()"
			endif
			if (!gen_TextWavestheSame(ExactMassText_FitTodo, ExactMassText_FitTodoTemp))
				abort " You have chosen different HR ions to fit between different todo waves. Cannot merge results -  Aborting from PK_FindHRResultsWavesFitTodo()"				
			endif
			tempMask = tempMask[p] == myindex ? nan : tempMask[p]
			myindex = Wavemax(tempMask)		// could use max or min
		endif
	while(myindex>=0)  // go through the most recent fits.  Check to see if HRIonWaveCopies waves match
	
	killwaves/z tempMask
	if (MSorPToFFlag)	
		return "HRIonWaveCopies:"+TodoPlusIterationNum
	else
		return "HRIonWavePToF:"+dataStr+":"+TodoPlusIterationNum	
	endif
End


//1.09E
// compares two text waves, returns 1 if they are the same, else 0
Function gen_TextWavesTheSame(tw1, tw2)
wave/t tw1, tw2
	
	variable idex, num1, num2
	
	num1=numpnts(tw1)
	num2=numpnts(tw2)
	
	if ( num1 != num2 )
		return 0
	endif
	
	for(idex=0;idex<num1;idex+=1)
		if (! (cmpstr(tw1[idex], tw2[idex],1)==0) )
			return 0
		endif
	endfor
	
	return 1

End


//1.08G		// PK_CopyHRIonWavesFitTodo changed in 1.10C toreset family groupings
// When the user requests any kind of HR product (time series, avg mass spec, etc) we need to copy the saved waves
//  ExactMassText_FitTodo to their place in the HR_Sticks folder.
Function PK_WavesForHRResults()

	ControlInfo/W=HR_Analysis_Panel HRResults_pop_dataSet
	return PK_WavesForHRResultsDataSet(S_value)
	
//	return 0
End

//1.08G		// PK_CopyHRIonWavesFitTodo changed in 1.10C toreset family groupings
// When the user requests any kind of HR product (time series, avg mass spec, etc) we need to copy the saved waves
//  ExactMassText_FitTodo to their place in the HR_Sticks folder.
Function PK_WavesForHRResultsDataSet(DataSet)
string DataSet

	string HRtodoFolder, MSPToFStr
	HRtodoFolder = PK_FindHRResultsWavesFitTodoDS(DataSet)	  //1.22A	// HRtodoFolder = name of todo wave
//	
//	ControlInfo/W=HR_Analysis_Panel HRResults_pop_dataSet
	variable MSorPToFFlag = stringMatch(DataSet[0,1],"HR")
	
//	MSPToFStr = selectString(  !MSorPToFFlag, "root:HR_sticks:", "root:HR_sticks:HRIonWavePToF:"+S_value+":")		//1.14M
	MSPToFStr = "root:HR_sticks:" //1.19E
		
	wave/Z ExactMassText_FitTodo = $MSPToFStr+HRtodoFolder+":ExactMassText_FitTodo"		// one of many waves could be used here	

	if (WaveExists(ExactMassText_FitTodo)	)		// for backwards compatibility
		// PK_CopyHRIonWavesFitTodo(fromString1, toString1,fromString2, toString2)
		PK_CopyHRIonWavesFitTodo(MSPToFStr+HRtodoFolder+":", "root:HR_sticks:",MSPToFStr+HRtodoFolder+":", "root:Int_Sticks:")
		return numpnts(ExactMassText_FitTodo)
	else		//1.21B
		abort "Had trouble finding the wave " +MSPToFStr+HRtodoFolder+":ExactMassText_FitTodo.  Aborting from PK_WavesForHRResults"	
	endif
	
	return 0
End

//1.08G
// used to copy/move waves from root:HR_sticks:HRIonWaveCopies:myTodoWave: to root:HR_sticks: or visa versa
Function PK_CopyHRIonWavesFitTodo(fromString1, toString1,fromString2, toString2)
string fromString1, toString1,fromString2, toString2

	NewDataFolder/o $(toString1[0, strlen(toString1) -2])		// don't use trailing colon
	NewDataFolder/o $(toString2[0, strlen(toString2) -2])
	
	wave ExactMassText_FitTodo = $fromString1+"ExactMassText_FitTodo"	
	wave ExactMassWave_FitTodo = $fromString1+"ExactMassWave_FitTodo"	
	wave ExactMassIsotopeRef_FitTodo = $fromString1+"ExactMassIsotopeRef_FitTodo"		// not sure if these are needed, but we do them here
	wave ExactMassIsotopeAbund_FitTodo = $fromString1+"ExactMassIsotopeAbund_FitTodo"
	wave ExactMassFamilyText_FitTodo = $fromString1+"ExactMassFamilyText_FitTodo"

	// added in 1.09K
	wave/z/t IsotopicMassText_FitTodo = $fromString1+"IsotopicMassText_FitTodo"
	wave/z IsotopicMask_FitTodo = $fromString1+"IsotopicMask_FitTodo"

		//  the main players
	duplicate/o ExactMassText_FitTodo		$toString1+"ExactMassText_FitTodo"	
	duplicate/o ExactMassWave_FitTodo		$toString1+"ExactMassWave_FitTodo"	
	duplicate/o ExactMassIsotopeRef_FitTodo 	$toString1+"ExactMassIsotopeRef_FitTodo"		// not sure if these are needed, but we do them here
	duplicate/o ExactMassIsotopeAbund_FitTodo	$toString1+"ExactMassIsotopeAbund_FitTodo"
	duplicate/o ExactMassFamilyText_FitTodo	$toString1+"ExactMassFamilyText_FitTodo"

	// added in 1.09K
	if (!WaveExists(IsotopicMassText_FitTodo) || !WaveExists(IsotopicMask_FitTodo)) 
		make/t/o/n=0 $toString1+"IsotopicMassText_FitTodo"
		make/o/n=0 $toString1+"IsotopicMask_FitTodo"
	else
		duplicate/o/t IsotopicMassText_FitTodo	$toString1+"IsotopicMassText_FitTodo"
		duplicate/o IsotopicMask_FitTodo	$toString1+"IsotopicMask_FitTodo"
	endif
	
	// parts needed for UMR summing of HR sticke
	wave MassStartPnt_FitTodo = $fromString1+"MassStartPnt_FitTodo"	
	wave MassEndPnt_FitTodo = $fromString1+"MassEndPnt_FitTodo"	
	wave UnitMass_FitTodo = $fromString1+"UnitMass_FitTodo"		// not sure if these are needed, but we do them here
	wave numOfPeaks_FitTodo = $fromString1+"numOfPeaks_FitTodo"

	duplicate/o MassStartPnt_FitTodo	$toString1+"MassStartPnt_FitTodo"
	duplicate/o MassEndPnt_FitTodo	$toString1+"MassEndPnt_FitTodo"
	duplicate/o UnitMass_FitTodo		$toString1+"UnitMass_FitTodo"
	duplicate/o numOfPeaks_FitTodo	$toString1+"numOfPeaks_FitTodo"	

	wave/t HR_familyName = root:HR:HR_familyName
	variable kdex, numFamily
	string lists	
	numFamily = numpnts(HR_familyName)

//	reset families
	UM_GenerateAxByMaskWaves(ExactMassText_FitTodo, "_FitTodo")
// 
//  1.09A, 1,10C  we recalculate families now
//	wave/t useExactMassText_FitTodo = $toString1+"ExactMassText_FitTodo"
//	wave/t useExactMassFamilyText_FitTodo = $toString1+"ExactMassFamilyText_FitTodo"
//	lists = HR_GenerateFamilyExceptionLists()
//	useExactMassFamilyText_FitTodo=HR_returnGrouping(useExactMassText_FitTodo[p], stringFromList(0, lists, "/"), stringFromList(1, lists, "/"))
//	// family identifiers
//	for (kdex = 0;kdex<numFamily;kdex+=1)	// go through all the families to find its match
//		make/o/n=(numpnts(useExactMassText_FitTodo)) $toString2+"mask_"+HR_familyName[kdex]+"_FitTodo"=0
//		wave FamilyMask =  $toString2+"mask_"+HR_familyName[kdex]+"_FitTodo"
//		FamilyMask = stringmatch(useExactMassFamilyText_FitTodo[p],HR_familyName[kdex] ) ? 1 : 0
//	endfor		

End


Function HR_butt_totalIonCountErr(ctrlName) : ButtonControl
	String ctrlName

	PK_TotalCountErr_CheckFor(1)
	
End


Function PK_TotalCountErr_CheckFor(PreviousHRFit)
variable PreviousHRFit

	nvar HR_UseOpenClosedMzParams = root:HR_Panel:HR_UseOpenClosedMzParams
	svar todoStr=root:panel:sq_toDoWvNm
	wave/z todo=$(todoStr)
	string paramStr, paramStrOpen, paramStrClosed
	nvar error_sigma=root:panel:error_sigma
	nvar error_noise=root:panel:error_noise

	//  first check to see if it was already performed.  If so, so only for those runs for which it hasn't been calced yet.  
	// Create a todo wave called HRTotIonCount_temp which is a subset of todo
	if (PreviousHRFit)
		PK_WavesForHRResults()		//1.08G		// get correct list of HR ions  the exactMassWave that was used for the fits in this todo gets put in its correct place 
		//  changed in 1.10C to now also recalculate family designations
	endif
	
 	sq_checkSamplingTimeVals(todo)   // 1.14F

// deal with unit mass squirrel baseline sticks
	sq_preproc_check("MSSOpenBaseL_p","MSSOpenBaseL_todoTemp",todo)
	wave/z MSSOpenBaseL_todoTemp = root:MSSOpenBaseL_todoTemp
	sq_preproc_check("MSSClosedBaseL_p","MSSClosedBaseL_todoTemp",todo)
	wave/z MSSClosedBaseL_todoTemp = root:MSSClosedBaseL_todoTemp	
	sq_preproc_check("MSSDiffBaseL_p","MSSDiffBaseL_todoTemp",todo)
	wave/z MSSDiffBaseL_todoTemp = root:MSSDiffBaseL_todoTemp	

	if ( numpnts(MSSOpenBaseL_todoTemp)!=0 || numpnts(MSSClosedBaseL_todoTemp)!=0 ) 	//1.10K  
		// don't fuss with separate open and closed params here, it won't make any difference
		variable m = sq_FindMaxMinMz(todo,0,0)		//1.20L
		if (numtype(m) !=0 || m<0 || m>1e8)		// 1e8 is arbitrary
			abort "The minimum Mz of "+num2str(m)+" seems incorrect.  Aborting from PK_TotalCountErr_CheckFor" 
		endif		//1.60L
		paramStr="MinMz:"+num2str(m)+";"
 		m = sq_findMaxMinMz(todo, 1, 1)	//1.20L
		if (numtype(m) !=0 || m<0 || m>1e8)		// 1e8 is arbitrary
			abort "The maximum Mz of "+num2str(m)+" seems incorrect.  Aborting from PK_TotalCountErr_CheckFor" 
		endif		//1.60L
		paramStr+="MaxMz:"+num2str(m)+";"
		paramStr+="MzSlope:root/mzFitting/mz_FitSlope;MzIntercept:root/mzFitting/mz_FitIntercept;MzPower:root/mzFitting/mz_FitPower;"
		squirrel_fetch(MSSOpenBaseL_todoTemp, sq_UMRBaselineSticks, paramStr+"dest:MSSOpenBaseL_p;", "MSOpen_p;MSOpenLessBaseL_p")
		squirrel_fetch(MSSClosedBaseL_todoTemp, sq_UMRBaselineSticks, paramStr+"dest:MSSClosedBaseL_p;","MSClosed_p;MSClosedLessBaseL_p")
		squirrel_fetch(MSSDiffBaseL_todoTemp, sq_UMRBaselineSticks, paramStr+"dest:MSSClosedBaseL_p;","MSDiff_p;MSDiffLessBaseL_p")

		killwaves/z MSSOpenBaseL_todoTemp, MSSClosedBaseL_todoTemp, MSSDiffBaseL_todoTemp
	endif
// end of deal with unit mass squirrel baseline sticks
	
//	PK_getHRPeakWidths(PreviousHRFit)
	paramStr= "PreviousHRFit:"+Num2str(PreviousHRFit)+";"+ "sigma:"+num2str(error_sigma)+";noise:"+num2str(error_noise)+";countSigErrFlag:1;"		// 1.12N  UMRPeakWidthInNs:UnitSigma_temp"
	paramStr+= "ExactMassWave:root/HR_sticks/ExactMassText_FitTodo;"		//  1.12N

//	wave/wave TimeWaves=sq_GetModifiedTimes()
//	wave TimeMSOpen = TimeWaves[0]
//	wave TimeMSClosed = TimeWaves[1]
//	wave TimePToF = TimeWaves[2]
		
//	paramstr+="sampTimeClosed:"+ReplaceString(":", GetWavesDataFolder(timeMSClosed,2),"/")+";"		//1.58E
//	paramstr+="sampTimeOpen:"+ReplaceString(":", GetWavesDataFolder(timeMSOpen,2),"/")+";"  	//1.58E	

	squirrel_fetch(todo, pk_TotalCountingErrors,paramStr, "HROpenStick;HRClosedStick;MSSOpenBaseL_p;MSSClosedBaseL_p")		// paramStrOpen=paramStrClosed in this case	
	
	killwaves/z UnitSigma_temp, MSSOpenBaseL_todoTemp, MSSClosedBaseL_todoTemp,MSSDiffBaseL_todoTemp,TimeWaves
End
	

// 1.09H Calc baseline HR sticks - will use for total ion count
Function sq_UMRBaselineSticks(index_pos,paramstr,acorn,n_acorns)
	wave index_pos
	string paramstr
	variable acorn,n_acorns

	string dest=stringbykey("dest",paramstr)
	wave MSRawSpectraPlusBaseline=$stringbykey("acorn_0",paramstr)
	wave MSRawSpectraLessBaseline=$stringbykey("acorn_1",paramstr)
	wave MzSlope=$ReplaceString("/", stringbykey("MzSlope",paramstr), ":")
	wave MzIntercept=$ReplaceString("/", stringbykey("MzIntercept",paramstr), ":")
	wave MzPower=$ReplaceString("/", stringbykey("MzPower",paramstr), ":")
	variable maxMz = numberbykey("maxMz",paramstr)
	variable minMz = numberbykey("minMz",paramstr)
	wave series_index =root:index:series_index
	wave/z nSecMSsamplingInterval =  root:diagnostics:nSecMSsamplingInterval		//1.53H  1.12H
	if (!WaveExists(nSecMSsamplingInterval)   || !waveMax(nSecMSsamplingInterval) >0) 
		make/o/n=(numpnts(series_index)) root:diagnostics:nSecMSsamplingInterval=1		// default - old card always at 1ns
		wave nSecMSsamplingInterval =  root:diagnostics:nSecMSsamplingInterval		//1.53H  1.12H
	endif

	variable idex, numRow, numInRawSpectra, numOfSpectra
	variable run_int, run_sl, run_pow, nSecMSsampling
	
	variable SamplingInterval = sq_nSecMSsamplingIntervalConst()			//1.57K

	numOfSpectra=dimsize(MSRawSpectraPlusBaseline,0)
	numInRawSpectra = dimsize(MSRawSpectraPlusBaseline,1)
	
	make/o/n=(numInRawSpectra) RawSpectra_temp	, axis_temp
	make/o/n=(numOfSpectra,maxMz) MatrixStick_temp=nan
	make/o/n=(maxMz) Stick_temp
	make/o/n=(maxMz) pt_Start, pt_End			//^ contains point numbers

	sq_stickCalcGetRegions(maxMz)	
	wave StickRegion_startmz = root:baselineFitting:StickRegion_startmz
	wave StickRegion_stopmz = root:baselineFitting:StickRegion_stopmz

	matrixop /o RawSpectraBaseline = MSRawSpectraPlusBaseline - MSRawSpectraLessBaseline

	for(idex=0;idex<numOfSpectra;idex+=1)
		sq_ams_prog_upd(100*(acorn+(idex/numOfSpectra))/n_acorns)
		if(index_pos[idex]==-1)
			continue
		endif
	 	run_int=MzIntercept[series_index[index_pos[idex]]]
		run_sl=MzSlope[series_index[index_pos[idex]]]
		run_pow=MzPower[series_index[index_pos[idex]]]
		multithread axis_temp=((p-run_int)/run_sl)^(1/run_pow)			//1.16K
		if ( run_int>0)			//1.54A		//1.14I
			axis_temp[0,ceil(run_int)] =0  // start at least at mz = 0			
		endif
		RawSpectra_temp=RawSpectraBaseline[idex][p]
		
		nSecMSsampling= nSecMSsamplingInterval[series_index[index_pos[idex]]]		//1.12H
		
		sq_StickCalcPrepRegion(RawSpectra_temp, axis_temp, minMz, maxMz, StickRegion_startmz, StickRegion_stopmz, pt_start, pt_end)
		sq_stickCalc(RawSpectra_temp,Stick_temp, minMz, acorn==n_acorns-1, pt_start, pt_end, nSecMSsampling=nSecMSsampling)	
		MatrixStick_temp[idex][]=Stick_temp[q]
	endfor
	
	sq_dutycyclecorr(MatrixStick_temp,acorn==n_acorns-1)
	
	sq_correctSamplingTimeUnit(index_pos,MatrixStick_temp, nSecMSsamplingIntervalVal=SamplingInterval)		//1.13D

	// 1.14M  data is UMR here
	squirrel_hdf_write(index_pos,MatrixStick_temp,dest)		
	
	if (acorn==n_acorns-1)
		killwaves /z RawSpectra_temp, axis_temp, MatrixStick_temp,pt_Start, pt_End, Stick_temp,pt_Start, pt_End, RawSpectraBaseline	
	endif

End


//1.10K
// Get the delta, in tof space (not mz space) between the first run and other runs within a todo at a given m/z.
// HR_GetMzDist(44, someV, mz44_ptWave, 5)		 // where mz44_ptWave is numpnts(someV) long and the first run in someV is the 5th run overall. 				
Function HR_GetMzDist(mz, todoWave, resultWave, firstRunPos)
variable mz,firstRunPos
wave todoWave, resultWave
	
	wave mzintercept = root:mzfitting:mz_FitIntercept
	wave mzslope = root:mzfitting:mz_FitSlope
	wave mzpower = root:mzfitting:mz_FitPower
	wave rn_series = root:index:rn_series
	variable idex, num, pt, mzexact, ptDelta, pos
	num = numpnts(todowave)
	
	// first point number nearest to the mz..  i.e. at pt 2000 we are at m/z 40
	pt = round(mzintercept[firstRunPos] + mzslope[firstRunPos]*(mz^mzpower[firstRunPos])) // point number of first run closest to m/z
	// the exact mass at this integer point value
	mzexact = ((pt - mzintercept[firstRunPos])/mzslope[firstRunPos] )^(1/mzpower[firstRunPos])	// at point 2000 we are at exactly 39.9999
//	if (mzintercept[firstRunPos]>0)		//1.14I
//		mzexact[0,ceil(mzintercept[firstRunPos])]=0
//	endif
	
	for(idex=0;idex<num;idex+=1)	// for each run in todo wave
		pos = sq_FindRun(rn_series, todowave[idex])
		ptDelta = mzintercept[pos]+mzslope[pos]*mzexact^(mzpower[pos])  //ptDelta is the (non-integer except for idex=0) point value corresponding to the exact mass
		ptDelta = ptDelta-round(ptDelta) // the difference in point space between the point closest to m/z 39.99999
		resultWave[idex]=ptDelta		// just returns the wave containing point delta values
	endfor
	
End


//1.10K
// Presumes that the drift is the same across all masses at nominal mass (i.e. C2H3O and C3H7 at 43)
// Generates todoPtHistMatrix which will be convolved with the canonical peak shape
Function HR_GetPointDeltaHistogram(todoWave)
wave todoWave

	wave rn_series = root:index:rn_series
	wave HR_FinalPWIntercept = root:HR_peakShape:HR_FinalPWInterceptWave
	wave HR_FinalPWSlope = root:HR_peakShape:HR_FinalPWSlopeWave
	wave HR_FinalPWPower = root:HR_peakShape:HR_FinalPWPowerWave
	wave UnitMass = root:HR_sticks:UnitMass
	variable idex, num, minMz, maxMz
	variable firstRunPos, int, slope, pow, tempSum
	
	minMz = UnitMass[0]
	maxMz = UnitMass[numpnts(UnitMass)-1]
	make/FREE/o/n=( numpnts(todoWave)) tempWave
	Make/FREE/o/n=21 tempHist		// peakshape is in units of 0.05 peak width units
	make/o/n=(maxMz - MinMz + 1, 21) root:HR_sticks:todoPtHistMatrix/Wave=todoPtHistMatrix
	firstRunPos = sq_FindRun(rn_series, todowave[0])
	for(idex=minMz;idex<=maxMz;idex+=1)
		HR_GetMzDist(idex, todoWave, tempWave, firstRunPos)
		Histogram/C/B={-0.525,0.05,21} tempWave,tempHist		// /c makes bins centered in x scale starting at -.5
		tempSum =  sum(tempHist)
		tempHist/=tempSum	// normalize... so that the convolution is really a weighted average 
		todoPtHistMatrix[idex-minMz][]=tempHist[q]	// todoPtHistMatrix is 2dmatrix. Rows=mz histograms for a todo
	endfor
	
End




/////////////////////////////////////
// HS r9 Driver for linear least squares calculation (LLS)
// This function is based on the analytical solution of the linear least squares problem using matrices.
// It does not iterate but calculates the peak heights directly through a matrixop command
// For a description of the problem and its solution, see, e.g.:
// http://en.wikipedia.org/wiki/Linear_least_squares_(mathematics)#Inverting_the_matrix_of_the_normal_equations
// The solution can not be constrained to only positive peak heights, therefore, additional statements may be required in the calling function.
// This function performs about 30 times faster than the above (tw_HR_doManyIonFit) using funcfit
// A jitter optimization can not be performed, however, a peak offset can be specified

Function tw_calc_peak_heights(coefw, signal_all, peak_centers, PeakShape, sp, ep, sigma, residw, peaks_offset, print_flag)
Wave coefw            // output wave for fit coefficients
    // Mass spectrum input, all rank1 and of equal length
Wave signal_all        // mass spectrum: must be scaled to TOF time (seconds) if mz_flag=0
Wave peak_centers    // x0 positions of ions (TOF time or exact masses)
Wave residw            // residual wave
Wave PeakShape        // Peak shape wave. x-scaling = standard deviations from centre. amplitude=0->1
Variable sp            // Fit start point. Corresponding to signal
Variable ep            // Fit end points. Corresponding to signal
Variable sigma        // Fit width: defines how wide one PSw standard deviation is
Variable peaks_offset    // offset for all peaks in points
Variable print_flag

    // make sub_signal wave for fit, have to check if it's faster to just pass subwave
    Duplicate/R=[sp,ep]/FREE signal_all, signal //FREE
    Variable x_start, x_stop, n_peaks, n_points, width, minPS, maxPS
    minPS = pnt2x(PeakShape, 0) // get Peakshape x limits for avoiding indexing error in threadsafe mode
    maxPS = pnt2x(PeakShape, numpnts(PeakShape)-1)
    n_peaks = numpnts(peak_centers)
    n_points = numpnts(signal)
    if (n_points<n_peaks) // need at least as many points as peaks
        Return -1
    endif
    x_start = pnt2x(signal, 0)
    x_stop = pnt2x(signal, n_points-1)
    Make/N=(n_points, n_peaks)/FREE M_peaks
    SetScale/I x x_start+peaks_offset,x_stop+peaks_offset,"", M_peaks
    // from tw (Pika) fit function tw_HR_manyIonFitFunc_x_noX0
    // s.yw += s.coefw[n] * s.PSw((x - s.x0[n]*(1-1e-3*s.coefw[s.numions]))/s.sigma)
    // s.coefw[n]: amplitude of nth peak
    // s.PSw: Peak shape wave. x-scaling = standard deviations from centre. amplitude=0->1
    // s.x0[n]: center of nth peak
    // s.coefw[s.numions]: jitter
    // s.sigma: peak width
    // equation for normalized peak and jitter = 0:
    // PS((x - currcent/s.sigma)
    //possible to calculate individual peaks in one line?
    // HS r21: x limits in PeakShape have to be taken into account, then it seems to work without indexing error
    M_peaks = (x-peak_centers[q])/sigma>minPS && (x-peak_centers[q])/sigma<maxPS ? PeakShape((x-peak_centers[q])/sigma) : 0// Multithread would be possible, but it's slower!
    // formula: b = (XT X)-1 XT y
    // X: M_peaks, T: transpose, -1: inverse, y: signal
    // question: how to constrain peak heights to >0?
    // Probably not easy or fast, so better to do the calculation, check for negative values in b
    // and do funcfit if any parameters <0
    Matrixop/FREE b = Inv(M_peaks^t x M_peaks) x M_peaks^t x signal
//    coefw = p<n_peaks? b[p] : coefw[p]
    // HS r21 try to avoid index out of range error
    coefw[0,n_peaks-1] = b[p]			//1.58G
//    Matrixop/FREE simsig = M_peaks x b

    //Fastop residuals = signal - simsig
    Matrixop/FREE residuals = signal - M_peaks x b
    residw[sp,ep] = residuals[p-sp]
// optional: calculate sum of square of residuals
//    Duplicate/FREE signal, residuals_sq
//    fastop residuals_sq = residuals * residuals
//    Redimension/N=(n_peaks+1) coefw
//    sumval = sum(residuals_sq)
//    print sumval / n_points
//    coefw[n_peaks] = sumval / n_points // add avg residual square as additional parameter

    Return 0 // fit will always succeed if enough points
End Function


// creates HR data sets of errors interpRawError,
//  the data point where exact mass lies (interpolated bewteen ns raw points) Harald likes these
//1.11D
Function PK_interpRawErrorFetch(index_pos,paramstr,acorn,n_acorns)
	wave index_pos
	string paramstr
	variable acorn,n_acorns

	wave RawMinusBaseline=$stringbykey("acorn_0",paramstr)		
	wave mzintercept=$(ReplaceString("/",stringbykey("intercept",paramstr), ":")	)
	wave mzslope=$(ReplaceString("/",stringbykey("slope",paramstr), ":")	)	
	wave mzpower=$(ReplaceString("/",stringbykey("power",paramstr), ":")	)	
	wave ExactMassWave =$(ReplaceString("/",stringbykey("ExactMassWave",paramstr), ":")	)
	wave samplingTime =$(ReplaceString("/",stringbykey("samplingTime",paramstr), ":")	)
//	wave RawBaseline=$stringbykey("acorn_1",paramstr)

	string OpenOrClosedStr=stringbykey("OpenOrClosed",paramstr)

	wave series_index = root:index:series_index
	// we want our errors to be in units of Hz.
		
	variable numHRSticks, numRuns, idex, numHRRaw, currentRunIndex
	variable intercept, slope, power
	
	numRuns = dimsize(RawMinusBaseline,0)
	numHRRaw = dimsize(RawMinusBaseline,1)
	numHRSticks = numpnts(ExactMassWave)
	
	make/free/o/n=(numRuns, numHRSticks)  interpRawErrorMatrix		// the result
	make/free/o/n=(numHRSticks)  interpRawErrorRun, tempRun				// result for single run
	make/free/o/n=(numHRRaw)  RawSpectraRun, RawMzRun
	
//	RawBaseline+=RawMinusBaseline
	
	for(idex=0;idex<numRuns;idex+=1)
		currentRunIndex = series_index[index_pos[idex]]			// set things up
		intercept = mzIntercept[currentRunIndex]
		slope = mzSlope[currentRunIndex]
		power = mzPower[currentRunIndex]
		RawSpectraRun=RawMinusBaseline[idex][p]
		RawMzRun = ((p - intercept)/slope)^(1/(power))		//
		if (intercept>0)		//1.14I
			RawMzRun[0,ceil(intercept)]=0		//
		endif
		interpRawErrorRun = interp(ExactMassWave[p], RawMzRun, RawSpectraRun )
		interpRawErrorRun/=samplingTime[currentRunIndex]
		// save results
		interpRawErrorMatrix[idex][]=interpRawErrorRun[q]
	endfor
	
	// 1.14M not used
	squirrel_hdf_write(index_pos,interpRawErrorMatrix,OpenOrClosedStr+"interpRawError") 		// 1.08B

End


//1.12I
Function HR_butt_SigIonCountErr(ctrlName) : ButtonControl
	String ctrlName

	HR_SignalErrors_CheckFor(1)
	
End


//1.12I
Function HR_SignalErrors_CheckFor(PreviousHRFit)
variable PreviousHRFit

	svar sq_toDoWvNm = root:panel:sq_toDoWvNm
	wave todo = $sq_toDoWvNm
	nvar error_sigma=root:panel:error_sigma
	
//	HR_CalculateSticksSQ_prep()
//	PK_convertUnitSigmaToEachSpecie()

	nvar HR_UseOpenClosedMzParams = root:HR_Panel:HR_UseOpenClosedMzParams
	svar todoStr=root:panel:sq_toDoWvNm
	wave/z todo=$(todoStr)
	string paramStr="", paramStrOpen, paramStrClosed
	nvar error_sigma=root:panel:error_sigma
	nvar error_noise=root:panel:error_noise

 	sq_checkSamplingTimeVals(todo)   // 1.14F
 	
 	//  first check to see if it was already performed.  If so, so only for those runs for which it hasn't been calced yet.  
	// Create a todo wave called HRTotIonCount_temp which is a subset of todo
	if (PreviousHRFit)
		PK_WavesForHRResults()		//1.08G		// get correct list of HR ions  the exactMassWave that was used for the fits in this todo gets put in its correct place 
		//  changed in 1.10C to now also recalculate family designations
	endif
	
// deal with unit mass squirrel baseline sticks
	sq_preproc_check("MSSOpenBaseL_p","MSSOpenBaseL_todoTemp",todo)
	wave/z MSSOpenBaseL_todoTemp = root:MSSOpenBaseL_todoTemp
	sq_preproc_check("MSSClosedBaseL_p","MSSClosedBaseL_todoTemp",todo)
	wave/z MSSClosedBaseL_todoTemp = root:MSSClosedBaseL_todoTemp	
	sq_preproc_check("MSSDiffBaseL_p","MSSDiffBaseL_todoTemp",todo)
	wave/z MSSDiffBaseL_todoTemp = root:MSSDiffBaseL_todoTemp	

	if ( numpnts(MSSOpenBaseL_todoTemp)!=0 || numpnts(MSSClosedBaseL_todoTemp)!=0 ) 	//1.10K  
		// don't fuss with separate open and closed params here, it won't make any difference
		variable m = sq_FindMaxMinMz(todo,0,0)		//1.20L
		if (numtype(m) !=0 || m<0 || m>1e8)		// 1e8 is arbitrary
			abort "The minimum Mz of "+num2str(m)+" seems incorrect.  Aborting from HR_SignalErrors_CheckFor" 
		endif		//1.60L
		paramStr+="MinMz:"+num2str(m)+";"
 		m = sq_findMaxMinMz(todo, 1, 1)	//1.20L
		if (numtype(m) !=0 || m<0 || m>1e8)		// 1e8 is arbitrary
			abort "The maximum Mz of "+num2str(m)+" seems incorrect.  Aborting from HR_SignalErrors_CheckFor" 
		endif		//1.60L
		paramStr+="MaxMz:"+num2str(m)+";"
//		paramStr="MinMz:"+num2str(sq_FindMaxMinMz(todo,0,0))+";"
//		paramStr+="MaxMz:"+num2str(sq_findMaxMinMz(todo, 1, 1))+";"
		paramStr+="MzSlope:root/mzFitting/mz_FitSlope;MzIntercept:root/mzFitting/mz_FitIntercept;MzPower:root/mzFitting/mz_FitPower;"
		squirrel_fetch(MSSOpenBaseL_todoTemp, sq_UMRBaselineSticks, paramStr+"dest:MSSOpenBaseL_p;", "MSOpen_p;MSOpenLessBaseL_p")
		squirrel_fetch(MSSClosedBaseL_todoTemp, sq_UMRBaselineSticks, paramStr+"dest:MSSClosedBaseL_p;","MSClosed_p;MSClosedLessBaseL_p")
		squirrel_fetch(MSSDiffBaseL_todoTemp, sq_UMRBaselineSticks, paramStr+"dest:MSSClosedBaseL_p;","MSDiff_p;MSDiffLessBaseL_p")

		killwaves/z MSSOpenBaseL_todoTemp, MSSClosedBaseL_todoTemp, MSSDiffBaseL_todoTemp
	endif
// end of deal with unit mass squirrel baseline sticks
	
//	PK_getHRPeakWidths(PreviousHRFit)
//	paramStr= "PreviousHRFit:"+Num2str(PreviousHRFit)+";"+ "sigma:"+num2str(error_sigma)+";noise:"+num2str(error_noise)+";"+"TotCountErrFlag:0;"		// 1.10A  UMRPeakWidthInNs:UnitSigma_temp"
//1.21E
	paramStr= "PreviousHRFit:"+Num2str(PreviousHRFit)+";"+ "sigma:"+num2str(error_sigma)+";noise:"+num2str(error_noise)+";"+"countSigErrFlag:1;"		// 1.10A  UMRPeakWidthInNs:UnitSigma_temp"
	paramStr+= "ExactMassWave:root/HR_sticks/ExactMassWave_FitTodo;"		//  1.21G

	// 1.55D
	sq_preProc_check("HRFitAreaO","temp_todoHRFItArea",todo)			// do for Open only, and assume the same is true for closed
	wave/z temp_todoHRFItArea = root:temp_todoHRFItArea
	if (WaveExists(temp_todoHRFItArea)  && numpnts(temp_todoHRFItArea)>0 )  //1.14E
		abort "Some calculations that are performed and saved during step 5 need to be done and the TotSigErr errors can't be calculated.  The best way to proceed is to redo Step 5 on your todo wave.  Aborting from HR_SignalErrors_CheckFor. "
	endif	

	killwaves/z MSSOpenBaseL_todoTemp, MSSClosedBaseL_todoTemp, MSSDiffBaseL_todoTemp

	squirrel_fetch(todo, pk_TotalCountingErrors,paramStr, "HRFitAreaO;HRFitAreaC;MSSOpenBaseL_p;MSSClosedBaseL_p")		// paramStrOpen=paramStrClosed in this case	
	
	killwaves/z UnitSigma_temp, MSSOpenBaseL_todoTemp, MSSClosedBaseL_todoTemp,MSSDiffBaseL_todoTemp,TimeWaves
	
	HR_pop_getStickErrSet("",nan,"Get List")

End

